@@ -37,7 +37,11 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
  Added 19.15/20.1:  `-i site-packages' allow installer to dump extra packages
  without modifying Makefiles, etc.
- */
+
+ This file is stupid. The entire point of XEmacs is handling text, this is far
+ easier in Lisp than in C. And C evolves, meaning this file needs revision as
+ the years go on. As an old comment in make-docfile.el says, this should just
+ be moved completely to Lisp. */
 
 #include <config.h>
 #include <sysfile.h>
@@ -84,6 +88,12 @@ char *progname;
 /* XEmacs addition: set to 1 if this was invoked by ellcc */
 int ellcc = 0;
 
+/* For Lisp objects created in modules, as well as the declaration provided by
+   DECLARE_MODULE_LISP_OBJECT(), lrecord_type_##c_name needs a
+   definition. This table saves c_name for that definition. */
+static char *definedlispobjects[32];
+static char **definedlispobjects_ptr = definedlispobjects;
+
 /* Print error message.  `s1' is printf control string, `s2' is arg for it. */
 
 static void
@@ -105,10 +115,10 @@ fatal (const char *s1, const char *s2)
 
 /* Like malloc but get fatal error if memory is exhausted.  */
 
-static long *
-xmalloc (unsigned int size)
+static void *
+xmalloc (size_t size)
 {
-  long *result = (long *) malloc (size);
+  void *result = malloc (size);
   if (result == NULL)
     fatal ("virtual memory exhausted", 0);
   return result;
@@ -263,7 +273,20 @@ main (int argc, char **argv)
 
   putc ('\n', outfile);
   if (ellcc)
-    fprintf (outfile, "}\n\n");
+    {
+      fprintf (outfile, "}\n\n");
+
+      definedlispobjects_ptr = definedlispobjects;
+      while (*definedlispobjects_ptr)
+        {
+          fprintf (outfile, "int lrecord_type_%s;\n",
+                   *definedlispobjects_ptr);
+          definedlispobjects_ptr++;
+        }
+
+      fprintf (outfile, "const char *emodule_coding = \"%s\";\n",
+               "no-conversion");
+    }
   /* End XEmacs addition */
 
 #ifndef VMS
@@ -298,6 +321,21 @@ put_filename (const char *filename)
   fprintf (outfile, "%s\n", filename);
 }
 
+/* Add a source file name in the output file, escaping as a C string.  */
+static void
+put_filename_as_c_string (const char *filename)
+{
+  while (*filename)
+    {
+      if (*filename == '\\')
+        {
+          fputc ('\\', outfile);
+        }
+      fputc (*filename, outfile);
+      filename++;
+    }
+}
+
 /* Read file FILENAME and output its doc strings to outfile.  */
 /* Return 1 if file is not found, 0 if it is found.  */
 
@@ -737,6 +775,7 @@ scan_c_file (const char *filename, const char *mode)
   register int defunflag;
   register int defvarperbufferflag = 0;
   register int defvarflag;
+  int definelispobjectflag = 0;
   int minargs, maxargs;
   int l = strlen (filename);
   char f[QXE_PATH_MAX];
@@ -779,6 +818,8 @@ scan_c_file (const char *filename, const char *mode)
 	  c = getc (infile);
 	  continue;
 	}
+      defunflag = defvarperbufferflag = defvarflag = definelispobjectflag
+        = 0;
       c = getc (infile);
       if (c == ' ')
 	{
@@ -793,27 +834,106 @@ scan_c_file (const char *filename, const char *mode)
 	  if (c != 'F')
 	    continue;
 	  c = getc (infile);
-	  if (c != 'V')
-	    continue;
-	  c = getc (infile);
-	  if (c != 'A')
-	    continue;
-	  c = getc (infile);
-	  if (c != 'R')
-	    continue;
-	  c = getc (infile);
-	  if (c != '_')
-	    continue;
+	  if (c == 'V')
+	    {
+              /* XEmacs; handle both DEFVAR*, DEFINE.*LISP_OBJECT. */
+	      c = getc (infile);
+	      if (c != 'A')
+		continue;
+	      c = getc (infile);
+	      if (c != 'R')
+		continue;
+	      c = getc (infile);
+	      if (c != '_')
+		continue;
 
-	  defvarflag = 1;
-	  defunflag = 0;
+	      defvarflag = 1;
+	      defunflag = 0;
 
-	  c = getc (infile);
-	  /* Note that this business doesn't apply under XEmacs.
-	     DEFVAR_BUFFER_LOCAL in XEmacs behaves normally. */
-	  defvarperbufferflag = (c == 'P');
+	      c = getc (infile);
+	      /* Note that this business doesn't apply under XEmacs.
+		 DEFVAR_BUFFER_LOCAL in XEmacs behaves normally. */
+	      defvarperbufferflag = (c == 'P');
+	    }
+	  else if (c == 'I' && ellcc)
+	    {
+	      /* XEmacs change; parse DEFINE_[A-Z_]+_LISP_OBJECT() to allow
+		 definition of the lrecord_type for that Lisp object in the
+		 generated code. */
+	      int last_underscore = 0;
 
-	  c = getc (infile);
+	      c = getc (infile);
+	      if (c != 'N')
+		continue;
+	      c = getc (infile);
+	      if (c != 'E')
+		continue;
+	      c = getc (infile);
+	      if (c != '_')
+		{
+		break_from_match:
+		  continue;
+#define REALLY_CONTINUE goto break_from_match
+		}
+	      last_underscore = 1;
+	      while (!feof (infile))
+		{
+		  c = getc (infile);
+		  if (c == 'L' && last_underscore)
+		    {
+		      last_underscore = 0;
+		      c = getc (infile);
+		      if (c != 'I')
+			REALLY_CONTINUE;
+		      c = getc (infile);
+		      if (c != 'S')
+			REALLY_CONTINUE;
+		      c = getc (infile);
+		      if (c != 'P')
+			REALLY_CONTINUE;
+		      c = getc (infile);
+		      if (c != '_')
+			REALLY_CONTINUE;
+		      c = getc (infile);
+		      if (c != 'O')
+			REALLY_CONTINUE;
+		      c = getc (infile);
+		      if (c != 'B')
+			REALLY_CONTINUE;
+		      c = getc (infile);
+		      if (c != 'J')
+			REALLY_CONTINUE;
+		      c = getc (infile);
+		      if (c != 'E')
+			REALLY_CONTINUE;
+		      c = getc (infile);
+		      if (c != 'C')
+			REALLY_CONTINUE;
+		      c = getc (infile);
+		      if (c != 'T')
+			REALLY_CONTINUE;
+
+		      definelispobjectflag = 1;
+                      break;
+		    }
+		  else if (c >= 'A' && c <= 'Z')
+		    {
+		      last_underscore = 0;
+		    }
+		  else if (c == '_')
+		    {
+		      last_underscore = 1;
+		    }
+		  else
+		    {
+		      REALLY_CONTINUE;
+		    }
+		}
+	    }
+	  else
+	    {
+	      continue;
+	    }
 	}
       else if (c == 'D')
 	{
@@ -846,7 +966,7 @@ scan_c_file (const char *filename, const char *mode)
 	commas = 4;
       else if (defvarperbufferflag)
 	commas = 2;
-      else if (defvarflag)
+      else if (defvarflag || definelispobjectflag)
 	commas = 1;
       else  /* For DEFSIMPLE and DEFPRED ... which now don't exist! */
 	commas = 2;
@@ -867,7 +987,8 @@ scan_c_file (const char *filename, const char *mode)
 		  if (commas == 2) /* pick up minargs */
 		    fscanf (infile, "%d", &minargs);
 		  else /* pick up maxargs */
-		    if (c == 'M' || c == 'U') /* MANY || UNEVALLED */
+                    /* MANY || UNEVALLED || KEYWORDS */
+		    if (c == 'M' || c == 'U' || c == 'K') 
 		      maxargs = -1;
 		    else
 		      fscanf (infile, "%d", &maxargs);
@@ -896,6 +1017,31 @@ scan_c_file (const char *filename, const char *mode)
 	  while (c == '*')
 	    c = getc (infile);
 	}
+      else if (definelispobjectflag)
+	{
+          if (ellcc)
+            {
+              char *p = globalbuf;
+
+              while (C_IDENTIFIER_CHAR_P (c))
+                {
+                  *p++ = c;
+                  c = getc (infile);
+                  assert (p - globalbuf < (ssize_t) (sizeof (globalbuf)));
+                }
+              *p++ = '\0';
+              ungetc (c, infile);
+              *definedlispobjects_ptr = (char *) xmalloc (p - globalbuf);
+              memcpy (*definedlispobjects_ptr, globalbuf, p - globalbuf);
+              ++definedlispobjects_ptr;
+              assert (definedlispobjects_ptr - definedlispobjects
+                      < (ssize_t) (sizeof (definedlispobjects) /
+                                   sizeof (definedlispobjects[0])));
+              *definedlispobjects_ptr = NULL;
+            }
+
+          continue;
+	}
       else
 	{
 	  while (c != ',')
@@ -917,11 +1063,11 @@ scan_c_file (const char *filename, const char *mode)
       if (defunflag || defvarflag || c == '"')
 	{
 	  /* XEmacs change: the original code is in the "else" clause */
-	  /* XXX Must modify the documentation file name code to handle
-	     ELLCCs */
 	  if (ellcc)
-	    fprintf (outfile, "  CDOC%s(\"%s\", \"\\\n",
-		     defvarflag ? "SYM" : "SUBR", globalbuf);
+            {
+              fprintf (outfile, "  CDOC%s (\"%s\", \"\\\n",
+                       defvarflag ? "SYM" : "SUBR", globalbuf);
+            }
 	  else
 	    {
 	      put_filename (filename);	/* XEmacs addition */
@@ -977,7 +1123,11 @@ scan_c_file (const char *filename, const char *mode)
 	      write_c_args (outfile, globalbuf, argbuf, minargs, maxargs);
 	    }
 	  if (ellcc)
-	    fprintf (outfile, "\\n\");\n\n");
+            {
+              fprintf (outfile, "\\n\", \"");
+              put_filename_as_c_string (filename);
+              fprintf (outfile, "\");\n\n");
+            }
 	}
     }
  eof:
@@ -1120,6 +1270,11 @@ scan_lisp_file (const char *filename, const char *mode)
                     }
 		}
 
+              if (length == 0)
+                {
+                  fatal ("dynamic docstring length %s", "not supplied");
+                }
+
 	      /* The next character is a space that is counted in the length
 		 but not part of the doc string.
 		 We already read it, so just ignore it.  */
@@ -1141,8 +1296,14 @@ scan_lisp_file (const char *filename, const char *mode)
                 }
 	      /* The last character is a ^_.
 		 That is needed in the .elc file
-		 but it is redundant in DOC.  So get rid of it here.  */
-	      saved_string[length - 1] = 0;
+		 but it is redundant in DOC.  So get rid of it here.
+                 The length > 0 check is redundant, but quiets a GCC 15
+                 warning.  */
+              if (length > 0)
+                {
+                  saved_string[length - 1] = 0;
+                }
+
 	      /* Skip the line break.  */
 	      while (c == '\n')
 		c = getc_skipping_iso2022 (infile);
