@@ -26,10 +26,9 @@
 
 ;;; Commentary:
 
-;; The ideas for this package were derived from the C code in
-;; src/keymap.c and elsewhere.  The functions in this file should
-;; always be byte-compiled for speed.  Someone should rewrite this in
-;; C (as part of src/keymap.c) for speed.
+;; The ideas for this package were derived from C code in src/keymap.c
+;; and elsewhere.  The functions in this file should always be
+;; byte-compiled for speed, as should Lisp code in general.
 
 ;; The idea for super-apropos is based on the original implementation
 ;; by Lynn Slater <lrs@esl.com>.
@@ -177,7 +176,7 @@ variables."
 					  (documentation symbol t)
 					(void-function "(aliased to undefined function)")
 					(error "(unexpected error from `documention')"))))
-			    (substring doc 0 (string-match "\n" doc))
+			    (subseq doc 0 (position ?\n doc))
 			  "(not documented)"))
 		    (and do-all
 			 (user-variable-p symbol)
@@ -188,8 +187,7 @@ variables."
 				    (variable-obsoleteness-doc symbol)
 				    (documentation-property
 				     symbol 'variable-documentation t)))
-			     (substring doc 0
-					    (string-match "\n" doc))))))
+			     (subseq doc 0 (position ?\n doc))))))
 	 (setq p (cdr p)))))
    nil))
 
@@ -230,7 +228,7 @@ Returns list of symbols and documentation found."
 					  (documentation symbol t)
 					(void-function "(aliased to undefined function)")
 					(error "(unexpected error from `documention')"))))
-			    (substring doc 0 (string-match "\n" doc))
+			    (subseq doc 0 (position ?\n doc))
 			  "(not documented)"))
 		    (if (boundp symbol)
 			(if (setq doc
@@ -240,8 +238,7 @@ Returns list of symbols and documentation found."
 				   (variable-obsoleteness-doc symbol)
 				   (documentation-property
 				    symbol 'variable-documentation t)))
-			    (substring doc 0
-				       (string-match "\n" doc))
+			    (subseq doc 0 (position ?\n doc))
 			  "(not documented)"))
 		    (if (setq doc (symbol-plist symbol))
 			(if (eq (/ (length doc) 2) 1)
@@ -250,19 +247,16 @@ Returns list of symbols and documentation found."
 		    (if (get symbol 'widget-type)
 			(if (setq doc (documentation-property
 				       symbol 'widget-documentation t))
-			    (substring doc 0
-				       (string-match "\n" doc))
+			    (subseq doc 0 (position ?\n doc))
 			  "(not documented)"))
 		    (if (find-face symbol)
 			(if (setq doc (face-doc-string symbol))
-			    (substring doc 0
-				       (string-match "\n" doc))
+			    (subseq doc 0 (position ?\n doc))
 			  "(not documented)"))
 		    (when (get symbol 'custom-group)
 		      (if (setq doc (documentation-property
 				     symbol 'group-documentation t))
-			  (substring doc 0
-				     (string-match "\n" doc))
+			  (subseq doc 0 (position ?\n doc))
 			"(not documented)"))))
 	 (setq p (cdr p)))))
    nil))
@@ -332,6 +326,26 @@ Returns list of symbols and documentation found."
       (kill-buffer standard-input))))
 
 
+;;;###autoload
+(defun* apropos-internal (regexp &optional predicate (package obarray))
+  "Return a list of interned symbols whose names match REGEXP.
+If optional second arg PREDICATE is non-nil, only symbols for which
+\(funcall PREDICATE SYMBOL) returns non-nil are returned.
+Optional third argument PACKAGE describes the Lisp package to search, and
+defaults to the value of `obarray'."
+  ;; XEmacs; this is in subr.el in GNU.
+  (let (found)
+    (mapatoms (if predicate
+                  #'(lambda (symbol)
+                      (when (and (string-match-p regexp (symbol-name symbol))
+                                 (funcall predicate symbol))
+                        (push symbol found)))
+                #'(lambda (symbol)
+                    (when (string-match-p regexp (symbol-name symbol))
+                      (push symbol found))))
+                    package)
+    (sort found #'string-lessp)))
+
 (defun apropos-value-internal (predicate symbol function)
   (if (funcall predicate symbol)
       (progn
@@ -397,7 +411,7 @@ Returns list of symbols and documentation found."
 	    (setq start (match-beginning 0)
 		  end (point))
 	    (goto-char (1+ sepa))
-	    (or (setq type (if (eq ?F (preceding-char))
+	    (or (setq type (if (eq ?F (char-before))
 			       1	; function documentation
 			     2)		; variable documentation
 		      symbol (read)
