@@ -120,7 +120,10 @@ surely saves the buffer with CODING-SYSTEM.  From a program, if you
 don't want to mark the buffer modified, specify t for NOMODIFY.
 If you know exactly what coding system you want to use,
 just set the variable `buffer-file-coding-system' directly."
-  (interactive "zCoding system for saving file (default nil): \nP")
+  (interactive (list
+                (read-coding-system "Coding system for saving file: "
+                                    (car coding-system-history))
+                current-prefix-arg))
   (check-coding-system coding-system)
   (if (and coding-system buffer-file-coding-system (null force))
        (setq coding-system
@@ -159,7 +162,7 @@ This is equivalent to setting the variable
 `file-coding-system-alist' to specify the coding system for
 particular files."
   (interactive "zFile coding system for read: ")
-  (get-coding-system coding-system) ;; correctness check
+  (check-coding-system coding-system) ;; correctness check
   (setq buffer-file-coding-system-for-read coding-system))
 
 (define-obsolete-function-alias
@@ -189,7 +192,7 @@ object (the entry specified a coding system)."
     (let ((case-fold-search nil))
       (setq filename (file-name-sans-versions filename))
       (while (and (not found) alist)
-	(if (string-match (car (car alist)) filename)
+	(if (string-match-p (car (car alist)) filename)
 	    (setq codesys (cdr (car alist))
 		  found t))
 	(setq alist (cdr alist))))
@@ -215,7 +218,7 @@ object (the entry specified a coding system)."
     (let ((case-fold-search nil))
       (setq filename (file-name-sans-versions filename))
       (while (and (not found) alist)
-	(if (string-match (car (car alist)) filename)
+	(if (string-match-p (car (car alist)) filename)
 	    (setq codesys (cdr (car alist))
 		  found t))
 	(setq alist (cdr alist))))
@@ -248,54 +251,103 @@ If optional fourth arg NOSUFFIX is non-nil, don't try adding suffixes
  .elc, .el, or elements of `module-extensions' to the specified name FILE.
 Return t if file exists."
   (declare (special load-modules-quietly))
-  (let ((handler (find-file-name-handler filename 'load))
-	(path nil))
-    (if handler
-	(funcall handler 'load filename noerror nomessage nosuffix)
-      ;; First try to load a Lisp file
-      (if (and (> (length filename) 0)
-	       (setq path (locate-file filename load-path
-				       (and (not nosuffix)
-					    '(".elc" ".el" "")))))
-	  ;; now use the internal load to actually load the file.
-	  (load-internal
-	   filename noerror nomessage nosuffix
-	   (let ((elc ; use string= instead of string-match to keep match-data.
-		  (equalp ".elc" (substring path -4))))
-	     (or (and (not elc) coding-system-for-read)	;prefer for source file
-		 ;; find magic-cookie
-		 (let ((codesys
-			(find-coding-system-magic-cookie-in-file path)))
-		   (when codesys
-		     (setq codesys (intern codesys))
-		     (if (find-coding-system codesys) codesys)))
-		 (if elc
-		     ;; if reading a byte-compiled file and we didn't find
-		     ;; a coding-system magic cookie, then use `binary'.
-		     ;; We need to guarantee that we never do autodetection
-		     ;; on byte-compiled files because confusion here would
-		     ;; be a very bad thing.  Pre-existing byte-compiled
-		     ;; files are always in the `binary' coding system.
-		     ;; Also, byte-compiled files always use `lf' to terminate
-		     ;; a line; don't risk confusion here either.
-		     'binary
-		   (or (find-file-coding-system-for-read-from-filename path)
-		       ;; looking up in `file-coding-system-alist'.
-		       ;; otherwise use `buffer-file-coding-system-for-read',
-		       ;; as normal
-		       buffer-file-coding-system-for-read)
-		   ))))
-	;; The file name is invalid, or we want to load a binary module
-	(if (and (> (length filename) 0)
-		 (locate-file filename module-load-path
-			      (and (not nosuffix) module-extensions)))
-	    (if (featurep 'modules)
-		(let ((load-modules-quietly nomessage))
-		  (declare-fboundp (load-module filename)))
-	      (signal 'file-error '("This XEmacs does not support modules")))
-	  (and (null noerror)
-	       (signal 'file-error (list "Cannot open load file" filename))))
-	))))
+  (let (handler path elc old (spaces ""))
+    (cond
+      ;; Maybe there's a handler.
+      ((setq handler (find-file-name-handler filename 'load))
+       (funcall handler 'load filename noerror nomessage nosuffix))
+      ;; Nope, no handler. Try to load a Lisp file.
+      ((setq path
+             (locate-file (setq filename (substitute-in-file-name filename))
+                          load-path (and (not nosuffix) '(".elc" ".el" ""))))
+       (setq elc
+             (not (mismatch
+                   ".elc" path :test (and (file-system-ignore-case-p path)
+                                          #'equalp)
+                   :start2 (max 0 (- (length path) (length ".elc"))))))
+       ;; Maybe the .elc is out of date with regard to the .el file. If so we
+       ;; may need to prefer the .el file, or just to warn.
+       (when (and elc (or load-ignore-out-of-date-elc-files
+                          load-warn-when-source-newer)
+                  (file-newer-than-file-p (subseq path 0 -1) path))
+         (setq old (if load-show-full-path-in-messages
+                       path
+                     (file-name-nondirectory path))
+               nomessage (or nomessage t))
+               ;; Considered binding this within this function, but decided
+               ;; against it, since it's most used for debugging, and
+               ;; encountering an out-of-date-ELC file should happen rarely
+               ;; enough that the infelicity of the double-loading message is
+               ;; outweighed by the improvement in debugging:
+               ;load-always-display-messages nil)
+         (if (and noninteractive load-in-progress)
+             (setq spaces "  "))
+         (when load-ignore-out-of-date-elc-files
+           (setq filename
+                 (if (not (mismatch ".elc" filename
+                                    :start2 (max 0 (- (length filename)
+                                                      (length ".elc")))))
+                     (subseq filename 0 -1)
+                   (concat filename ".elc"))
+                 path (locate-file filename load-path)))
+         (message "%s%s %s... (file %s is out-of-date)" spaces
+                  (if (eq nomessage 'require) "Requiring" "Loading")
+                  (if load-show-full-path-in-messages path filename) old))
+       (when (and (not nosuffix) (position ?. filename)
+                  (> (count ?. path) (count ?. filename))
+                  (not (equal (file-name-nondirectory
+			       (file-name-sans-extension filename))
+                              (file-name-nondirectory
+			       (file-name-sans-extension path)))))
+         (lwarn 'locate-file 'critical
+           (format "Attempting to %s:
+
+Filename `%s' already has an extension, locate-file gave us an existing file \
+with a further extension, `%s'. This is unlikely to be intended.
+
+Our caller likely should pass the NOSUFFIX argument to `load'."
+                   (if (eq nomessage 'require) "require" "load")
+                   filename path)))
+       ;; Now use #'load-internal to actually load the file.
+       (prog1
+           (load-internal
+            filename noerror nomessage nosuffix
+            (or (and (not elc) coding-system-for-read) ;; Prefer for source
+                ;; find magic-cookie
+                (let ((codesys
+                       (find-coding-system-magic-cookie-in-file path)))
+                  (when codesys
+                    (setq codesys (intern codesys))
+                    (if (find-coding-system codesys) codesys)))
+                (if elc
+                    ;; If reading a byte-compiled file and we didn't find a
+                    ;; coding-system magic cookie, then use `binary'.  We need
+                    ;; to guarantee that we never do autodetection on
+                    ;; byte-compiled files because confusion here would be a
+                    ;; very bad thing.  Pre-existing byte-compiled files are
+                    ;; always in the `binary' coding system.  Also,
+                    ;; byte-compiled files always use `lf' to terminate a
+                    ;; line; don't risk confusion here either.
+                    'binary
+                  (or (find-file-coding-system-for-read-from-filename path)
+                      ;; looking up in `file-coding-system-alist'.  otherwise
+                      ;; use `buffer-file-coding-system-for-read', as normal
+                      buffer-file-coding-system-for-read))))
+         (and old
+              (message "%s%s %s... done (file %s is out-of-date)" spaces
+                       (if (eq nomessage 'require) "Requiring" "Loading")
+                       (if load-show-full-path-in-messages path filename)
+                       old))))
+      ;; Maybe there's a module.
+      ((locate-file filename module-load-path
+                    (and (not nosuffix) module-extensions))
+       (let ((load-modules-quietly nomessage))
+         ;; This will just error if we don't have module support, no need to
+         ;; error specially ourselves.
+         (declare-fboundp (load-module filename))))
+      ;; Maybe we can't find the file.
+      (t (and (not noerror)
+              (signal 'file-error (list "Cannot open load file" filename)))))))
 
 (defvar insert-file-contents-access-hook nil
   "A hook to make a file accessible before reading it.
@@ -502,12 +554,12 @@ and `insert-file-contents-post-hook'."
 			  ;; convention of post-read-conversion.  We try to
 			  ;; support the old way.  #### Should we kill this?
 			  (funcall func (point) (marker-position endmark))
-			(funcall func (- (marker-position endmark) (point))))))
+			(funcall func (- endmark (point))))))
 		(if visit
 		    (progn
 		      (set-buffer-auto-saved)
 		      (set-buffer-modified-p nil)))))
-	  (setcar (cdr return-val) (- (marker-position endmark) (point))))
+	  (setcar (cdr return-val) (- endmark (point))))
 	;; now finally set the buffer's `buffer-file-coding-system' ...
 	(if (run-hook-with-args-until-success 'insert-file-contents-post-hook
 					      filename visit return-val)
@@ -526,7 +578,7 @@ and `insert-file-contents-post-hook'."
 	;; of set-buffer-file-coding-system cause the actual coding system
 	;; object to be stored, so do that here, too.
 	(setq buffer-file-coding-system-when-loaded 
-	      (get-coding-system coding-system))
+	      (check-coding-system coding-system))
 	return-val))))
 
 (defvar write-region-pre-hook nil
@@ -563,8 +615,8 @@ With a prefix arg, prompts for a coding system as well.
 
 When called from a program, takes three required arguments:
 START, END and FILENAME.  START and END are buffer positions.
-APPEND, if non-nil, means append to existing file contents (if any), else
-  the file's existing contents are replaced by the specified region.
+APPEND, if non-nil, means append to existing file contents (if any). If it is
+a byte file offset, seek to that point before writing.
 VISIT, if non-nil, should be a string naming a file.  The buffer is marked
                  as  visiting VISIT.  VISIT is also the file name to lock
                  and unlock for clash detection.
