@@ -41,13 +41,8 @@
 ;; prevent require/provide loop with custom and cus-face.
 (provide 'custom)
 
-(eval-when-compile
-  ;; To elude warnings.
-  (require 'cus-face))
-
 (autoload 'custom-declare-face "cus-face")
-
-(require 'widget)
+(autoload 'custom-theme-reset-internal-face "cus-face")
 
 (defvar custom-define-hook nil
   ;; Customize information for this option is in `cus-edit.el'.
@@ -178,6 +173,10 @@ keyword in your own programs.  "
 		 (put symbol 'custom-get value))
 		((eq keyword :require)
 		 (push value requests))
+		((eq keyword :risky)
+		 (put symbol 'risky-local-variable value))
+		((eq keyword :safe)
+		 (put symbol 'safe-local-variable value))
 		((eq keyword :type)
 		 (put symbol 'custom-type value))
 		((eq keyword :options)
@@ -264,10 +263,23 @@ The following keywords are meaningful:
 	VALUE should be a feature symbol.  If you save a value
 	for this option, then when your custom init file loads the value,
 	it does (require VALUE) first.
+:risky  Set SYMBOL's `risky-local-variable' property to VALUE.
+:safe   Set SYMBOL's `safe-local-variable' property to VALUE.
 :version
         VALUE should be a string specifying that the variable was
         first introduced, or its default value was changed, in Emacs
         version VERSION.
+:package-version
+        VALUE should be a list with the form (PACKAGE . VERSION),
+        where PACKAGE and VERSION are strings.
+
+        This specifies that the variable was first introduced, or its
+        default value was changed, in PACKAGE version VERSION.  This
+        keyword takes priority over :version.
+
+        Since PACKAGE must be unique and the user might see it in an
+        error message, a good choice is the official name of the
+        package, such as MH-E or Gnus.
 :tag LABEL
         Use LABEL, a string, instead of the item's name, to label the item
         in customization menus and buffers.
@@ -336,7 +348,7 @@ information."
 ;;; The `defgroup' Macro.
 
 (defun custom-current-group ()
-  (cdr (assoc load-file-name custom-current-group-alist)))
+  (cdr (assoc* load-file-name custom-current-group-alist :test #'equal)))
 
 (defun custom-declare-group (symbol members doc &rest args)
   "Like `defgroup', but SYMBOL is evaluated as a normal argument."
@@ -360,7 +372,7 @@ information."
 	       (custom-handle-keyword symbol keyword value
 				      'custom-group))))))
   ;; Record the group on the `current' list.
-  (let ((elt (assoc load-file-name custom-current-group-alist)))
+  (let ((elt (assoc* load-file-name custom-current-group-alist :test #'equal)))
     (if elt (setcdr elt symbol)
       (push (cons load-file-name symbol) custom-current-group-alist)))
   (run-hooks 'custom-define-hook)
@@ -421,9 +433,9 @@ If there already is an entry for OPTION and WIDGET, nothing is done."
 If no such group is found, return nil."
   (or (get mode 'custom-mode-group)
       (if (or (get mode 'custom-group)
-	      (and (string-match "-mode\\'" (symbol-name mode))
-		   (get (setq mode (intern (substring (symbol-name mode)
-						      0 (match-beginning 0))))
+	      (and (string-match-p "-mode\\'" (symbol-name mode))
+		   (get (setq mode (intern (substring (symbol-name mode) 0
+                                                      (- (length "-mode")))))
 			'custom-group)))
 	  mode)))
 
@@ -452,6 +464,8 @@ Fourth argument TYPE is the custom option type."
 	 (custom-add-to-group value symbol type))
 	((eq keyword :version)
 	 (custom-add-version symbol value))
+	((eq keyword :package-version)
+	 (custom-add-package-version symbol value))
 	((eq keyword :link)
 	 (custom-add-link symbol value))
 	((eq keyword :load)
@@ -501,6 +515,14 @@ For other types variables, the effect is undefined."
   "To the custom option SYMBOL add the version VERSION."
   (put symbol 'custom-version version))
 
+(defun custom-add-package-version (symbol version)
+  "To the custom option SYMBOL add the package version VERSION."
+  (unless (and (consp version)
+	       (symbolp (car version))
+	       (stringp (cdr version)))
+        (error "Invalid package version `%s'" version))
+  (put symbol 'custom-package-version version))
+
 (defun custom-add-load (symbol load)
   "To the custom option SYMBOL add the dependency LOAD.
 LOAD should be either a library file name, or a feature name."
@@ -540,7 +562,7 @@ LOAD should be either a library file name, or a feature name."
 		     (found nil))
 		 (dolist (loaded load-history)
 		   (and (stringp (car loaded))
-			(string-match regexp (car loaded))
+			(string-match-p regexp (car loaded))
 			(setq found t)))
 		 found))
 	      ;; Without this, we would load cus-edit recursively.
