@@ -39,6 +39,13 @@
 ;; yes-or-no-p-dialog-box, but such code is difficult to write and test.
 
 ;;; Code:
+
+(globally-declare-fboundp
+ (unless (featurep 'dialog) '(make-dialog-box-internal)))
+
+(globally-declare-boundp
+ (unless (featurep 'dialog) '(dialog-frame-plist)))
+
 (defun yes-or-no-p-dialog-box (prompt)
   "Ask user a yes-or-no question with a popup dialog box.
 Return t if the answer is \"yes\".
@@ -689,7 +696,7 @@ This is meant to be executed from a dialog box callback function."
 is equivalent to
 
 \(make-dialog-box 'question :question QUESTION :buttons BUTTONS)"
-  (check-argument-type 'stringp (car desc))
+  (check-type (car desc) string)
   (or (consp (cdr desc))
       (error 'syntax-error
 	     "Dialog descriptor must supply at least one button"
