@@ -268,7 +268,7 @@ A directory name is ok too; it means file TAGS in that directory."
 				     default-directory
 				     (expand-file-name "TAGS" default-directory)
 				     t)))
-  (if (string-equal file "") 
+  (if (equal file "") 
       (setq tags-file-name nil)
     (setq file (expand-file-name file))
     (when (file-directory-p file)
@@ -325,7 +325,7 @@ If appropriate, reverting the buffer, and possibly build a completion-table."
       ;; Make the TAGS buffer invisible.
       (when (and check-name
 		 make-tags-files-invisible
-		 (string-match "\\`[^ ]" (buffer-name)))
+		 (string-match-p "\\`[^ ]" (buffer-name)))
 	(rename-buffer (generate-new-buffer-name
 			(concat " " (buffer-name)))))
       (or (verify-visited-file-modtime buf)
@@ -555,13 +555,13 @@ work with xemacs etags."
 	    ;; file-type.  This was way too slow, as it had to process
 	    ;; an enormous amount of regexps for each time.  Now we
 	    ;; use the shotgun approach with only two regexps.
-	    file-type (cond ((string-match "\\.\\([cC]\\|cc\\|cxx\\)\\'"
+	    file-type (cond ((string-match-p "\\.\\([cC]\\|cc\\|cxx\\)\\'"
 					   filename)
 			     'c-mode)
-			    ((string-match "\\.\\(el\\|cl\\|lisp\\)\\'"
-					   filename)
+			    ((string-match-p "\\.\\(el\\|cl\\|lisp\\)\\'"
+					     filename)
 			     'lisp-mode)
-			    ((string-match "\\.scm\\'" filename)
+			    ((string-match-p "\\.scm\\'" filename)
 			     'scheme-mode)
 			    (t nil)))
       (defvar c-mode-syntax-table)
@@ -983,7 +983,7 @@ Variables of note:
     (cond ((eq completion t))
 	  ((null completion)
 	   (error "Can't find completion for \"%s\"" pattern))
-	  ((not (string-equal pattern completion))
+	  ((not (equal pattern completion))
 	   (delete-region beg end)
 	   (insert completion))
 	  (t
@@ -1285,7 +1285,7 @@ If this is a C-defined elisp function, it does something more clever."
 		 (setq info (format "Elisp: %s, C: %s %s, #args: %s"
 				    lname
 				    fname args
-				    (if (string-equal min max)
+				    (if (equal min max)
 					min
 				      (format "from %s to %s" min max)))))
 		(t
