@@ -17,6 +17,7 @@
 ;; You should have received a copy of the GNU General Public License
 ;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
-(require 'x-compose)
+(require 'compose)
 
 (provide 'gtk-compose)
+;;; gtk-compose.el ends here
