@@ -53,46 +53,57 @@ return faces created with `make-face' or `copy-face', just those created
 with `defface' and `custom-declare-face'."
   (interactive "SFind source file for symbol: ") ; XEmacs
   (let (built-in-file autoload-cons symbol-details)
-    (cond ((and (eq 'autoload
-                    (car-safe
-                     (setq autoload-cons
-                           (and (fboundp sym) (symbol-function sym)))))
-                (or (and (or (null type) (eq 'defvar type))
-                         (eq (fifth autoload-cons) 'keymap))
-                    (and (or (null type) (eq 'defun type))
-                         (memq (fifth autoload-cons) '(nil macro)))))
-           (return-from symbol-file (locate-library (second autoload-cons))))
-          ((eq 'defvar type)
-           ;; Load history entries corresponding to variables are just
-           ;; symbols.
-           (dolist (entry load-history)
-             (when (memq sym (cdr entry))
-               (return-from symbol-file (car entry)))))
-           ((not (null type))
-            ;; Non-variables have the type stored as the car of the entry. 
-            (dolist (entry load-history)
-              (when (and (setq symbol-details (rassq sym (cdr entry)))
-                         (eq type (car symbol-details)))
-                (return-from symbol-file (car entry)))))
-          (t
-           ;; If TYPE hasn't been specified, we need to check both for
-           ;; variables and other symbols.
-           (dolist (entry load-history)
-             (when (or (memq sym (cdr entry))
-                       (rassq sym (cdr entry)))
-               (return-from symbol-file (car entry))))))
-    (when (setq built-in-file (built-in-symbol-file sym type))
-      (if (equal built-in-file (file-truename built-in-file))
-          ;; Probably a full path name:
-          built-in-file
-        ;; This is a bit heuristic, but shouldn't realistically be a
-        ;; problem:
-        (if (string-match #r"\.elc?$" built-in-file)
-            (concat (if (file-readable-p source-lisp)
-			source-lisp
-		      lisp-directory)
-		    built-in-file)
-          (concat source-directory "/src/" built-in-file))))))
+    (labels
+        ((handle-built-in ()
+           (when (setq built-in-file (built-in-symbol-file sym type))
+             (if (equal built-in-file (file-truename built-in-file))
+                 ;; Probably a full path name:
+                 built-in-file
+               ;; This is a bit heuristic, but shouldn't realistically be a
+               ;; problem:
+               (if (string-match-p #r"\.elc?$" built-in-file)
+                   (concat (if (file-readable-p source-lisp)
+                               source-lisp
+                             lisp-directory)
+                           built-in-file)
+                 (concat source-directory "/src/" built-in-file)))))
+         (handle-module-file (entry)
+           (when (equal (cadr entry) '(module))
+             (return-from symbol-file (handle-built-in)))))
+      (cond ((and (eq 'autoload
+                      (car-safe
+                       (setq autoload-cons
+                             (and (fboundp sym) (symbol-function sym)))))
+                  (or (and (or (null type) (eq 'defvar type))
+                           (eq (fifth autoload-cons) 'keymap))
+                      (and (or (null type) (eq 'defun type))
+                           (memq (fifth autoload-cons) '(nil macro)))))
+             (return-from symbol-file (locate-library (second autoload-cons))))
+            ((eq 'defvar type)
+             ;; Load history entries corresponding to variables are just
+             ;; symbols.
+             (dolist (entry load-history)
+               (when (memq sym (cdr entry))
+                 (handle-module-file entry)
+                 (return-from symbol-file (car entry)))))
+            ((not (null type))
+             ;; Non-variables have the type stored as the car of the entry. 
+             (dolist (entry load-history)
+               (when (and (setq symbol-details (rassq sym (cdr entry)))
+                          (eq type (car symbol-details)))
+                 (if (equal (cadr entry) '(module))
+                     (handle-module-file entry))
+                 (return-from symbol-file (car entry)))))
+            (t
+             ;; If TYPE hasn't been specified, we need to check both for
+             ;; variables and other symbols.
+             (dolist (entry load-history)
+               (when (or (memq sym (cdr entry))
+                         (rassq sym (cdr entry)))
+                 (if (equal (cadr entry) '(module))
+                     (handle-module-file entry))
+                 (return-from symbol-file (car entry))))))
+      (handle-built-in))))
 
 (defun feature-symbols (feature)
   "Return the file and list of symbols associated with a given FEATURE."
@@ -107,17 +118,18 @@ Actually, return the load argument, if any; this is sometimes the name of a
 Lisp file without an extension.  If the feature came from an eval-buffer on
 a buffer with no associated file, or an eval-region, return nil."
   (unless (featurep feature)
-    (error "%s is not a currently loaded feature" (symbol-name feature)))
+    (error 'invalid-argument "Not a currently loaded feature" feature))
   (car (feature-symbols feature)))
 
 (defun file-symbols (file)
   "Return the file and list of symbols associated with FILE.
 The file name in the returned list is the string used to load the file,
 and may not be the same string as FILE, but it will be equivalent."
-  (or (assoc file load-history)
-      (assoc (file-name-sans-extension file) load-history)
-      (assoc (concat file ".el") load-history)
-      (assoc (concat file ".elc") load-history)))
+  (let ((test (if (file-system-ignore-case-p file) #'equalp #'equal)))
+    (or (assoc* file load-history :test test)
+        (assoc* (file-name-sans-extension file) load-history :test test)
+        (assoc* (concat file ".el") load-history :test test)
+        (assoc* (concat file ".elc") load-history :test test))))
 
 (defun file-provides (file)
   "Return the list of features provided by FILE."
@@ -165,45 +177,46 @@ If the feature is required by any other loaded code, and optional FORCE
 is nil, raise an error."
   (interactive "SFeature: ")
   (unless (featurep feature)
-    (error "%s is not a currently loaded feature" (symbol-name feature)))
+    (error 'invalid-argument "Not a currently loaded feature" feature))
   (when (not force)
     (let* ((file (feature-file feature))
-	   (dependents (delete file (copy-sequence (file-dependents file)))))
+	   (dependents (remove* file (file-dependents file)
+				:test (if default-file-system-ignore-case
+					  #'equalp
+					#'equal))))
       (when dependents
-	(error "Loaded libraries %s depend on %s"
-	       (prin1-to-string dependents) file))))
+	(error 'invalid-state "Loaded libraries depend on feature"
+	       dependents feature))))
   (let* ((flist (feature-symbols feature))
 	 (file (car flist))
 	 (unloading-module nil))
-    (flet ((reset-aload (x)
-	     (let ((aload (get x 'autoload)))
-	       (if aload (fset x (cons 'autoload aload))))))
-    (mapc
-     #'(lambda (x)
-	 (cond ((stringp x) nil)
-	       ((consp x)
-		;; Remove any feature names that this file provided.
-		(if (eq (car x) 'provide)
-		    (setq features (delq (cdr x) features))
-		  (if (eq (car x) 'module)
-		      (setq unloading-module t))))
-	       ((and (boundp x)
-		     (fboundp x))
-		(makunbound x)
-		(fmakunbound x)
-		(reset-aload x))
-	       ((boundp x)
-		(makunbound x))
-	       ((fboundp x)
-		(fmakunbound x)
-		(reset-aload x))))
-     (cdr flist)))
+    (labels ((reset-aload (x)
+               (let ((aload (get x 'autoload)))
+                 (if aload (fset x (cons 'autoload aload))))))
+      (mapc
+       #'(lambda (x)
+           (cond ((stringp x) nil)
+                 ((consp x)
+                  (case (car x)
+                    (provide
+                     ;; Remove any feature names that this file provided.
+                     (setq features (delete* (cdr x) features)))
+                    (module
+                     (setq unloading-module t))
+                    (defun
+                     (when (fboundp (cdr x))
+                       (fmakunbound (cdr x))
+                       (reset-aload (cdr x))))))
+                 ((boundp x)
+                  (makunbound x))))
+       (cdr flist)))
     ;; Delete the load-history element for this file.
-    (let ((elt (assoc file load-history)))
-      (setq load-history (delq elt load-history)))
-    ;; If it is a module, really unload it.
-    (if unloading-module
-	(declare-fboundp (unload-module (symbol-name feature))))))
+    (setq load-history (delete* file load-history :key #'car :test 
+                                (if (file-system-ignore-case-p file)
+                                    #'equalp
+                                  #'equal)))
+    ;; If it is a module, really (attempt to) unload it.
+    (if unloading-module (unload-module file))))
 
 (provide 'loadhist)
 
