@@ -52,6 +52,14 @@
 
 ;;; Code:
 
+(globally-declare-fboundp
+ (unless (featurep 'menubar)
+   '(menu-split-long-menu-and-sort submenu-generate-accelerator-spec
+     menu-split-long-menu add-menu-button menu-item-generate-accelerator-spec
+     set-menubar)))
+
+(globally-declare-boundp (unless (featurep 'menubar) '(current-menubar)))
+
 (defun Menubar-items-truncate-history (list count label-length)
   "Truncate a history LIST to first COUNT items.
 Return a list of (label value) lists with labels truncated to last
@@ -70,9 +78,9 @@ LABEL-LENGTH characters of value."
   "Filter for menu entries with a submenu listing all coding systems.
 This is for operations that take a coding system as an argument.  FUN
 should be a function of one argument, which will be a coding system symbol.
-ACTIVE should be a function one argument (again, a coding system symbol),
-indicating whether the entry is active.  If DOTS is given, the menu entries
-will have three dots appended.
+ACTIVE should be a function of one argument (again, a coding system symbol),
+returning a boolean describing whether the entry is active.  If DOTS is given,
+the menu entries will have three dots appended.
 
 Write your filter like this:
 
@@ -986,24 +994,22 @@ Write your filter like this:
 	     (lambda (entry)
 	       (set-buffer-process-coding-system entry entry))
 	     (lambda (entry) (get-buffer-process (current-buffer)))))))
-     ,@(when (featurep 'mule)
-	 `(("Internationa%_l"
-	    ("Set %_Language Environment"
-	     :filter
-	     ,#'(lambda (menu)
-		  (menu-split-long-menu-and-sort
-		   (mapcar #'(lambda (entry)
-			       `[ ,(car entry)
-				  (set-language-environment ',(car entry))
-				  :style radio
-				  :selected
-				  ,(equal (car entry)
-					  current-language-environment)])
-			   language-info-alist)
-		   )))
-	    ["%_Toggle Input Method" toggle-input-method]
-	    ["Select %_Input Method" set-input-method]
-	    )))
+      ("Internationa%_l"
+       ("Set %_Language Environment"
+	:filter
+	,#'(lambda (menu)
+	     (menu-split-long-menu-and-sort
+	      (mapcar #'(lambda (entry)
+			  `[ ,(car entry)
+			    (set-language-environment ',(car entry))
+			    :style radio
+			    :selected
+			    ,(equal (car entry)
+				    current-language-environment)])
+		      language-info-alist)
+	      )))
+       ["%_Toggle Input Method" toggle-input-method]
+       ["Select %_Input Method" set-input-method])
      "-----"
      ("%_Display"
       ,@(if (featurep 'scrollbar)
@@ -1439,11 +1445,13 @@ Write your filter like this:
      ["Edit I%_nit File"
       ;; #### there should be something that holds the name that the init
       ;; file should be created as, when it's not present.
-      (let ((el-file (or user-init-file "~/.xemacs/init.el"))
-            el-file-directory)
-	(if (string-match "\\.elc$" el-file)
+      (let* ((el-file (or user-init-file "~/.xemacs/init.el"))
+             (position (position ?. el-file :from-end t))
+             el-file-directory)
+	(if (not (mismatch el-file ".elc"
+                           :start1 (or position (length el-file))))
 	    (setq el-file
-		  (substring user-init-file 0 (1- (length el-file)))))
+		  (subseq user-init-file 0 -1)))
         (unless (file-directory-p
                  (setq el-file-directory (file-name-directory el-file)))
           (message "Creating %s... " el-file-directory)
@@ -1479,7 +1487,7 @@ Write your filter like this:
       ["%_Home Page (www.xemacs.org)" xemacs-www-page
        :active (fboundp 'browse-url)])
      ["B%_eta Info" describe-beta
-      :included (string-match "beta" emacs-version)]
+      :included (search "beta" emacs-version)]
      "-----"
      ("%_Info (Online Docs)"
       ["%_Info Contents" (Info-goto-node "(dir)")]
@@ -1536,30 +1544,29 @@ Write your filter like this:
       ["Describe %_Function..." describe-function]
       ["Describe %_Variable..." describe-variable]
       ["%_Locate Command in Keymap..." where-is])
-     ,@(when (featurep 'mule)
-	 `(("Internationa%_l"
-	    ("Describe %_Language Support"
-	     :filter
-	     ,#'(lambda (menu)
-		  (menu-split-long-menu-and-sort
-		   (mapcar #'(lambda (entry)
-			       `[ ,(car entry)
-				  (describe-language-environment
-				   ',(car entry))
-				  :style radio
-				  :selected
-				  ,(equal (car entry)
-					  current-language-environment)])
-			   language-info-alist)
-		   )))
-	    ["Describe %_Input Method" describe-input-method]
-	    ["Describe Current %_Coding Systems"
-	     describe-current-coding-system]
-	    ["Show Character %_Table" view-charset-by-menu]
-	    ;; not implemented yet
-	    ["Show %_Diagnosis for MULE" mule-diag :active nil]
-	    ["Show \"%_hello\" in Many Languages" view-hello-file]
-	    )))
+      ("Internationa%_l"
+       ("Describe %_Language Support"
+	:filter
+	,#'(lambda (menu)
+	     (menu-split-long-menu-and-sort
+	      (mapcar #'(lambda (entry)
+			  `[ ,(car entry)
+			    (describe-language-environment
+			     ',(car entry))
+			    :style radio
+			    :selected
+			    ,(equal (car entry)
+				    current-language-environment)])
+		      language-info-alist)
+	      )))
+       ["Describe %_Input Method" describe-input-method]
+       ["Describe Current %_Coding Systems"
+	describe-current-coding-system]
+       ["Show Character %_Table" view-charset-by-menu]
+       ;; not implemented yet
+       ["Show %_Diagnosis for MULE" mule-diag :active nil]
+       ["Show \"%_hello\" in Many Languages" view-hello-file]
+       )
      ["%_Unix Manual" manual-entry]
      "-----"
      ["%_Current Installation Info" describe-installation
@@ -1653,7 +1660,7 @@ begins with a space)."
 (defcustom buffers-menu-format-buffer-line-function 'format-buffers-menu-line
   "*The function to call to return a string to represent a buffer in
 the buffers menu.  The function is passed a buffer and a number
-(starting with 1) indicating which buffer line in the menu is being
+\(starting with 1) indicating which buffer line in the menu is being
 processed and should return a string containing an accelerator
 spec. (Check out `menu-item-generate-accelerator-spec' as a convenient
 way of generating the accelerator specs.) The default value
@@ -1662,11 +1669,8 @@ uses the number as the accelerator.  Also check out
 `slow-format-buffers-menu-line' which returns a whole bunch of info
 about a buffer.
 
-Note: Gross Compatibility Hack: Older versions of this function prototype
-only expected one argument, not two.  We deal gracefully with such
-functions by simply calling them with one argument and leaving out the
-line number.  However, this may go away at any time, so make sure to
-update all of your functions of this type."
+Very old versions of this function prototype only expected one argument, not
+two.  This implementation will fail when supplied with only one argument."
   :type 'function
   :group 'buffers-menu)
 
@@ -1711,10 +1715,12 @@ Sorts the buffers in alphabetical order by name, but puts buffers beginning
 with a star at the end of the list."
   (let* ((nam1 (buffer-name buf1))
 	 (nam2 (buffer-name buf2))
-	 (inv1p (not (null (string-match "\\` " nam1))))
-	 (inv2p (not (null (string-match "\\` " nam2))))
-	 (star1p (not (null (string-match "\\`*" nam1))))
-	 (star2p (not (null (string-match "\\`*" nam2)))))
+         (len1 (length nam1))
+         (len2 (length nam2))
+	 (inv1p (eql ?\ (and (> len1 0) (aref nam1 0))))
+	 (inv2p (eql ?\ (and (> len2 0) (aref nam2 0))))
+	 (star1p (eql ?* (and (> len1 0) (aref nam1 0))))
+         (star2p (eql ?* (and (> len2 0) (aref nam2 0)))))
     (cond ((not (eq inv1p inv2p))
 	   (not inv1p))
 	  ((not (eq star1p star2p))
@@ -1728,23 +1734,37 @@ Sorts first by major mode and then alphabetically by name, but puts buffers
 beginning with a star at the end of the list."
   (let* ((nam1 (buffer-name buf1))
 	 (nam2 (buffer-name buf2))
-	 (inv1p (not (null (string-match "\\` " nam1))))
-	 (inv2p (not (null (string-match "\\` " nam2))))
-	 (star1p (not (null (string-match "\\`*" nam1))))
-	 (star2p (not (null (string-match "\\`*" nam2))))
-	 (mode1 (symbol-value-in-buffer 'major-mode buf1))
-	 (mode2 (symbol-value-in-buffer 'major-mode buf2)))
+	 (first1 (elt nam1 0))
+	 (first2 (elt nam2 0))
+	 (inv1p (eql ?\  first1))
+	 (inv2p (eql ?\  first2))
+	 (star1p (eql ?* first1))
+         (star2p (eql ?* first2))
+	 compare-strings)
     (cond ((not (eq inv1p inv2p))
 	   (not inv1p))
 	  ((not (eq star1p star2p))
 	   (not star1p))
-	  ((and star1p star2p (string-lessp nam1 nam2)))
-	  ((string-lessp mode1 mode2)
-	   t)
-	  ((string-lessp mode2 mode1)
-	   nil)
-	  (t
-	   (string-lessp nam1 nam2)))))
+	  ((and star1p star2p (prog2 
+				  (setq compare-strings
+					(compare-strings nam1 0 nil
+							 nam2 0 nil t))
+				  (and (fixnump compare-strings)
+				       (minusp compare-strings)))))
+	  ((eq (setq compare-strings
+                     (compare-strings (symbol-name
+                                       (symbol-value-in-buffer
+                                        'major-mode buf1))
+                                      0 nil
+                                      (symbol-name
+                                       (symbol-value-in-buffer
+                                        'major-mode buf2))
+                                      0 nil t))
+	       t)
+	   (setq compare-strings (compare-strings nam1 0 nil
+						  nam2 0 nil t))
+	   (and (fixnump compare-strings) (minusp compare-strings)))
+	  (t (minusp compare-strings)))))
 
 ;; this version is too slow on some machines.
 ;; (vintage 1990, that is)
@@ -1772,14 +1792,18 @@ This just returns the buffer's name."
 This groups buffers by major mode.  It only really makes sense if
 `buffers-menu-sorting-function' is
 `sort-buffers-menu-by-mode-then-alphabetically'."
-  (cond ((string-match "\\`*" (buffer-name buf1))
-	 (and (null buf2) "*Misc*"))
-	((or (null buf2)
-	     (string-match "\\`*" (buffer-name buf2))
-	     (not (eq (symbol-value-in-buffer 'major-mode buf1)
-		      (symbol-value-in-buffer 'major-mode buf2))))
-	 (symbol-value-in-buffer 'mode-name buf1))
-	(t nil)))
+  (let* ((nam1 (buffer-name buf1))
+	 (nam2 (buffer-name buf2))
+         (len1 (length nam1))
+         (len2 (length nam2)))
+    (cond ((eql ?* (and (> len1 0) (aref nam1 0)))
+           (and (null buf2) "*Misc*"))
+          ((or (null buf2)
+               (eql ?* (and (> len2 0) (aref nam2 0)))
+               (not (eq (symbol-value-in-buffer 'major-mode buf1)
+                        (symbol-value-in-buffer 'major-mode buf2))))
+           (symbol-value-in-buffer 'mode-name buf1))
+          (t nil))))
 
 (defun buffer-menu-save-buffer (buffer)
   (save-excursion
@@ -1793,53 +1817,6 @@ This groups buffers by major mode.  It only really makes sense if
 		 (format "Write %s to file: "
 			 (buffer-name (current-buffer)))))))
 
-(defsubst build-buffers-menu-internal (buffers)
-  (let (name line (n 0))
-    (mapcar
-     #'(lambda (buffer)
-	 (if (eq buffer t)
-	     "---"
-	   (setq n (1+ n))
-	   (setq line
-		 ; #### a truly Kyle-friendly hack.
-		 (let ((fn buffers-menu-format-buffer-line-function))
-		   (if (= (function-max-args fn) 1)
-		       (funcall fn buffer)
-		     (funcall fn buffer n))))
-	   (if complex-buffers-menu-p
-	       (delete* nil
-		     (list line
-			   (vector "S%_witch to Buffer"
-				   (list buffers-menu-switch-to-buffer-function
-					 (setq name (buffer-name buffer)))
-				   t)
-			   (if (eq buffers-menu-switch-to-buffer-function
-				   'switch-to-buffer)
-			       (vector "Switch to Buffer, Other %_Frame"
-				       (list 'switch-to-buffer-other-frame
-					     (setq name (buffer-name buffer)))
-				       t)
-			     nil)
-			   (if (and (buffer-modified-p buffer)
-				    (buffer-file-name buffer))
-			       (vector "%_Save Buffer"
-				       (list 'buffer-menu-save-buffer name) t)
-			     ["%_Save Buffer" nil nil]
-			     )
-			   (vector "Save %_As..."
-				   (list 'buffer-menu-write-file name) t)
-			   (vector "%_Delete Buffer" (list 'kill-buffer name)
-				   t)))
-	     ;; #### We don't want buffer names to be translated,
-	     ;; #### so we put the buffer name in the suffix.
-	     ;; #### Also, avoid losing with non-ASCII buffer names.
-	     ;; #### We still lose, however, if complex-buffers-menu-p. --mrb
-	     (vector ""
-		     (list buffers-menu-switch-to-buffer-function
-			   (buffer-name buffer))
-		     t line))))
-     buffers)))
-
 (defun buffers-menu-filter (menu)
   "This is the menu filter for the top-level buffers \"Buffers\" menu.
 It dynamically creates a list of buffers to use as the contents of the menu.
@@ -1847,62 +1824,102 @@ Only the most-recently-used few buffers will be listed on the menu, for
 efficiency reasons.  You can control how many buffers will be shown by
 setting `buffers-menu-max-size'.  You can control the text of the menu
 items by redefining the function `format-buffers-menu-line'."
-  (let ((buffers (delete-if buffers-menu-omit-function (buffer-list))))
-    (and (integerp buffers-menu-max-size)
-	 (> buffers-menu-max-size 1)
-	 (> (length buffers) buffers-menu-max-size)
-	 ;; shorten list of buffers (not with submenus!)
+  (let ((buffers (delete-if buffers-menu-omit-function (buffer-list)))
+        (n 0) tail)
+    (and buffers-menu-max-size (> buffers-menu-max-size 1)
 	 (not (and buffers-menu-grouping-function
-		   buffers-menu-submenus-for-groups-p))
-	 (setcdr (nthcdr buffers-menu-max-size buffers) nil))
-    (if buffers-menu-sort-function
-	(setq buffers (sort buffers buffers-menu-sort-function)))
-    (if (and buffers-menu-grouping-function
-	     buffers-menu-submenus-for-groups-p
-	     (or (not (integerp buffers-menu-submenus-for-groups-p))
-		 (> (length buffers) buffers-menu-submenus-for-groups-p)))
-	(let (groups groupnames current-group)
-	  (mapl
-	   #'(lambda (sublist)
-	       (let ((groupname (funcall buffers-menu-grouping-function
-					 (car sublist) (cadr sublist))))
-		 (setq current-group (cons (car sublist) current-group))
-		 (if groupname
-		     (progn
-		       (setq groups (cons (nreverse current-group)
-					  groups))
-		       (setq groupnames (cons groupname groupnames))
-		       (setq current-group nil)))))
-	   buffers)
-	  (setq buffers
-		(mapcar*
-		 #'(lambda (groupname group)
-		     (cons groupname (build-buffers-menu-internal group)))
-		 (nreverse groupnames)
-		 (nreverse groups))))
-      (if buffers-menu-grouping-function
-	  (progn
-	    (setq buffers
-		  (mapcon
-		   #'(lambda (sublist)
-		       (cond ((funcall buffers-menu-grouping-function
-				       (car sublist) (cadr sublist))
-			      (list (car sublist) t))
-			     (t (list (car sublist)))))
-		   buffers))
-	    ;; remove a trailing separator.
-	    (and (>= (length buffers) 2)
-		 (let ((lastcdr (nthcdr (- (length buffers) 2) buffers)))
-		   (if (eq t (cadr lastcdr))
-		       (setcdr lastcdr nil))))))
-      (setq buffers (build-buffers-menu-internal buffers)))
-    (append menu buffers)
-    ))
+                   buffers-menu-submenus-for-groups-p))
+	 ;; Shorten list of buffers (not with submenus!)
+         (setf tail (nthcdr buffers-menu-max-size buffers))
+         (setf (cdr tail) nil)) ;; Only set tail's cdr if tail is non-nil.
+    (labels
+        ((build-buffers-menu-internal-mapper (buffer)
+           (if (eq buffer t)
+               "---"
+             (let ((buffer-name (buffer-name buffer))
+                   (line (funcall buffers-menu-format-buffer-line-function
+                                  buffer (setf n (1+ n)))))
+               (if complex-buffers-menu-p
+                   `(,line
+                     ["S%_witch to Buffer"
+                      (,buffers-menu-switch-to-buffer-function ,buffer-name) t]
+                     ,@(if (eq buffers-menu-switch-to-buffer-function
+                               'switch-to-buffer)
+                           `(["Switch to Buffer, Other %_Frame"
+                              (switch-to-buffer-other-frame ,buffer-name) t]))
+                     ["%_Save Buffer"
+                      ,@(if (and (buffer-modified-p buffer)
+                                 (buffer-file-name buffer))
+                            `((buffer-menu-save-buffer ,buffer-name) t)
+                            (list nil nil))]
+                     ["Save %_As..." (buffer-menu-write-file ,buffer-name) t]
+                     ["%_Delete Buffer" (kill-buffer ,buffer-name) t])
+                 ;; We don't want buffer names to be translated, so we put the
+                 ;; buffer name in the suffix.  Also, avoid losing with
+                 ;; non-ASCII buffer names.
+                 ;; #### We still lose, however, if complex-buffers-menu-p.
+                 ;; --mrb
+                 `["" (,buffers-menu-switch-to-buffer-function ,buffer-name) t
+                   ,line]))))
+         (build-buffers-menu-internal (buffers)
+           (setq n 0) ;; Wouldn't it be nice to have a good implementation of
+		      ;; lexical scope.
+           (mapcar #'build-buffers-menu-internal-mapper buffers)))
+      (declare (inline build-buffers-menu-internal))
+      (if buffers-menu-sort-function
+          (setq buffers (sort buffers buffers-menu-sort-function)))
+      (if (and buffers-menu-grouping-function
+               buffers-menu-submenus-for-groups-p
+               (or (not (fixnump buffers-menu-submenus-for-groups-p))
+                   ;; (> (length buffers)
+                   ;; buffers-menu-submenus-for-groups-p)))
+                   (nthcdr buffers-menu-submenus-for-groups-p buffers)))
+          (let (groups groupnames current-group)
+            (mapl
+             #'(lambda (sublist)
+                 (let ((groupname (funcall buffers-menu-grouping-function
+                                           (car sublist) (cadr sublist))))
+                   (setq current-group (cons (car sublist) current-group))
+                   (if groupname
+                       (progn
+                         (setq groups (cons (nreverse current-group)
+                                            groups))
+                         (setq groupnames (cons groupname groupnames))
+                         (setq current-group nil)))))
+             buffers)
+            (setq buffers
+                  ;; Do build-buffers-menu-internal on each entry, and return
+                  ;; a reversed list with the group name.
+                  (loop for groupname in groupnames
+                        for group in groups
+                        with result = nil
+                        do (push (cons groupname
+                                       (build-buffers-menu-internal group))
+                                 result)
+                        finally return result)))
+          (if buffers-menu-grouping-function
+              (progn
+                (setq buffers
+                      (mapcon
+                       #'(lambda (sublist)
+
+                           (cond ((funcall buffers-menu-grouping-function
+                                           (car sublist) (cadr sublist))
+                                  (list (car sublist) t))
+                                 (t (list (car sublist)))))
+                       buffers))
+                ;; Remove any trailing separator.
+                (when (nthcdr 1 buffers) ;; (>= (length buffers) 2)
+                  (setq tail (nthcdr (- (length buffers) 2) buffers))
+                  (if (eq t (cadr tail))
+                      (setf (cdr tail) nil)))))
+          (setq buffers (build-buffers-menu-internal buffers)))
+      (append menu buffers))))
 
 (defun list-all-buffers ()
   "Display a list of buffers.  Calls `list-all-buffers-function'."
   (interactive)
-  (funcall (if (fboundp list-all-buffers-function)
+  (funcall (if (functionp list-all-buffers-function)
 	       list-all-buffers-function
 	     'list-buffers)))
 
@@ -1910,40 +1927,27 @@ items by redefining the function `format-buffers-menu-line'."
 ;;; The Help menu
 
 (defun tutorials-menu-filter (menu-items)
-  (declare (special language-info-alist
-		    current-language-environment
-		    tutorial-supported-languages))
   (append
-   (if (featurep 'mule)
-       (if (assq 'tutorial
-		 (assoc current-language-environment language-info-alist))
-	   `([,(concat "%_Default (" current-language-environment ")")
-	      help-with-tutorial]))
-     '(["%_English" help-with-tutorial]))
+   (if (assq 'tutorial
+	     (assoc current-language-environment language-info-alist))
+       `([,(concat "%_Default (" current-language-environment ")")
+	  help-with-tutorial]))
    (submenu-generate-accelerator-spec
-    (if (featurep 'mule)
-	;; Mule tutorials.
-	(mapcan #'(lambda (lang)
-		    (let ((tut (assq 'tutorial lang)))
-		      (and tut
-			   (not (string= (car lang) "ASCII"))
-			   ;; skip current language, since we already
-			   ;; included it first
-			   (not (string= (car lang)
-					 current-language-environment))
-                           ;; Hackish approach; if a language environment
-                           ;; doesn't have associated locale information,
-                           ;; it's not the preferred implementation for that
-                           ;; language. Don't use it.
-                           (assq 'locale lang)
-			   `([,(car lang)
-			      (help-with-tutorial nil ,(car lang))]))))
-		language-info-alist)
-      ;; Non mule tutorials.
-      (mapcar #'(lambda (lang)
-		  `[,(car lang)
-		    (help-with-tutorial nil ,(car lang))])
-	      tutorial-supported-languages)))))
+    (mapcan #'(lambda (lang)
+		(let ((tut (assq 'tutorial lang)))
+		  (and tut
+		       (not (equal (car lang) "ASCII"))
+		       ;; skip current language, since we already
+		       ;; included it first
+		       (not (equal (car lang) current-language-environment))
+		       ;; Hackish approach; if a language environment
+		       ;; doesn't have associated locale information,
+		       ;; it's not the preferred implementation for that
+		       ;; language. Don't use it.
+		       (assq 'locale lang)
+		       `([,(car lang)
+			  (help-with-tutorial nil ,(car lang))]))))
+	    language-info-alist))))
 
 (set-menubar default-menubar)
 
