@@ -40,11 +40,19 @@
 
 (require 'cl)
 
+(globally-declare-fboundp
+ (unless (featurep 'window-system)
+   '(mode-motion-highlight-internal default-mouse-track-beginning-of-word
+     default-mouse-track-end-of-word make-dialog-frame insert-gui-button
+     make-gui-button get-dialog-box-response)))
+
+(globally-declare-boundp
+ (unless (featurep 'window-system) '(mode-motion-hook)))
+
 (defgroup minibuffer nil
   "Controlling the behavior of the minibuffer."
   :group 'environment)
 
-
 (defcustom insert-default-directory t
  "*Non-nil means when reading a filename start with default dir in minibuffer."
  :type 'boolean
@@ -407,7 +415,8 @@ See also the variable `completion-highlight-first-word-only' for
 	  (mconfig (if (eq frame (selected-frame))
 		       nil (current-window-configuration frame)))
 	  (oconfig (current-window-configuration))
-	  (minibuffer-default default))
+	  (minibuffer-default default)
+          (help-form minibuffer-help-form))
      (unwind-protect
          (progn
            (set-buffer (reset-buffer buffer))
@@ -443,9 +452,7 @@ See also the variable `completion-highlight-first-word-only' for
 		 (insert initial-contents)
 		 (setq current-minibuffer-contents initial-contents
 		       current-minibuffer-point (point))))
-           (use-local-map (help-keymap-with-help-key
-			   (or keymap minibuffer-local-map)
-			   minibuffer-help-form))
+           (use-local-map (or keymap minibuffer-local-map))
            (let ((mouse-grabbed-buffer
 		  (and minibuffer-smart-completion-tracking-behavior
 		       (current-buffer)))
@@ -489,7 +496,7 @@ See also the variable `completion-highlight-first-word-only' for
 					   default)
 				      default
 				    (buffer-string))))
-		      (histval (if (and default (string= val ""))
+		      (histval (if (and default (equal val ""))
 				   default
 				 val))
                       (err nil))
@@ -629,7 +636,7 @@ See also the variable `completion-highlight-first-word-only' for
          'unique)
         (t
          ;; It did find a match.  Do we match some possibility exactly now?
-         (let ((completedp (not (string-equal completion buffer-string))))
+         (let ((completedp (not (equal completion buffer-string))))
            (if completedp
                (progn
                  ;; Some completion happened
@@ -671,7 +678,7 @@ See also the variable `completion-highlight-first-word-only' for
            )
           (t
            ;; It did find a match.  Do we match some possibility exactly now?
-           (if (not (string-equal completion buffer-string))
+           (if (not (equal completion buffer-string))
                (progn
                  ;; Some completion happened
                  (erase-buffer)
@@ -741,7 +748,7 @@ Completion ignores case if the ambient value of
 				    history
 				    nil
 				    default))
-    (if (and (string= ret "")
+    (if (and (equal ret "")
 	     default)
 	default
       ret)))
@@ -805,7 +812,7 @@ a repetition of this command will exit."
     ;; Short-cut -- don't call minibuffer-do-completion if we already
     ;;  have an (possibly nonunique) exact completion.
     (if (test-completion buffer-string minibuffer-completion-table
-                                minibuffer-completion-predicate)
+			 minibuffer-completion-predicate)
         (throw 'exit nil))
     (let ((status (minibuffer-do-completion buffer-string)))
       (if (or (eq status 'unique)
@@ -837,7 +844,7 @@ the character in question must be typed again)."
       (throw 'exit nil))
   (let ((buffer-string (buffer-string)))
     (if (test-completion buffer-string minibuffer-completion-table
-                                minibuffer-completion-predicate)
+			 minibuffer-completion-predicate)
         (throw 'exit nil))
     (let ((completion (if (not minibuffer-completion-table)
                           t
@@ -846,7 +853,7 @@ the character in question must be typed again)."
                                           minibuffer-completion-predicate))))
       (if (or (eq completion 't)
               ;; Crockishly allow user to specify null string
-              (string-equal buffer-string ""))
+              (equal buffer-string ""))
           (throw 'exit nil))
       (if completion ;; rewritten for I18N3 snarfing
 	  (temp-minibuffer-message " [incomplete; confirm]")
@@ -889,31 +896,27 @@ Return nil if there is no valid completion, else t."
           (t
            (cond ((or (eq status 'uncompleted)
                       (eq status 'exact))
-                  (let ((foo #'(lambda (s)
-				 (condition-case nil
-				     (if (try-completion
-					  (concat buffer-string s)
-					  minibuffer-completion-table
-					  minibuffer-completion-predicate)
-					 (progn
-					   (goto-char (point-max))
-					   (insert s)
-					   t)
-                                       nil)
-                                   (error nil))))
-                        (char last-command-char))
-                    ;; Try to complete by adding a word-delimiter
-                    (or (and (characterp char) (> char 0)
-                             (funcall foo (char-to-string char)))
-                        (and (not (eq char ?\ ))
-                             (funcall foo " "))
-                        (and (not (eq char ?\-))
-                             (funcall foo "-"))
+                  (labels ((foo (c)
+                             (ignore-errors
+                               (when (try-completion
+                                      (concat buffer-string `(,c))
+                                      minibuffer-completion-table
+                                      minibuffer-completion-predicate)
+                                 (goto-char (point-max))
+                                 (insert c)
+                                 t))))
+                    (declare (inline foo))
+                    (or (and (characterp last-command-char)
+                             (> last-command-char 0)
+                             (foo last-command-char))
+                        ;; Try to complete by adding a word-delimiter
+                        (and (not (eql last-command-char ?\ )) (foo ?\ ))
+                        (and (not (eql last-command-char ?\-)) (foo ?-))
                         (progn
                           (if completion-auto-help
                               (minibuffer-completion-help)
-                              ;; New message, only in this new Lisp code
-			    ;; rewritten for I18N3 snarfing
+                            ;; New message, only in this new Lisp code
+                            ;; rewritten for I18N3 snarfing
 			    (if (eq status 'exact)
 				(temp-minibuffer-message
 				 " [Complete, but not unique]")
@@ -1003,10 +1006,9 @@ This is not enabled by default because
   ;; the way filename completion works is funny.  Possibly there's some
   ;; more general way this could be dealt with...
   ;;
-  ;; We do some further voodoo when reading a pathname that is an
-  ;; ange-ftp or efs path, because causing FTP activity as a result of
-  ;; mouse motion is a really bad time.
-  ;;
+  ;; We do some further voodoo when reading a pathname that is a remote path,
+  ;; because causing FTP activity as a result of mouse motion is a really bad
+  ;; time.
   (and minibuffer-smart-completion-tracking-behavior
        (event-point event)
        ;; avoid conflict with display-completion-list extents
@@ -1031,53 +1033,18 @@ This is not enabled by default because
 		  (if filename-kludge-p
 		      (setq string (minibuffer-smart-select-kludge-filename
 				    string)))
-		  ;; try-completion bogusly returns a string even when
-		  ;; that string is complete if that string is also a
-		  ;; prefix for other completions.  This means that we
-		  ;; can't just do the obvious thing, (eq t
-		  ;; (try-completion ...)).
-                  ;; 
-                  ;; Could be reasonable to use #'test-completion
-                  ;; instead. Aidan Kehoe, Mo 14 Mai 2012 08:17:10 IST
-		  (let (comp)
-		    (if (and filename-kludge-p
-			     ;; #### evil evil evil evil
-			     (or (and (fboundp 'ange-ftp-ftp-path)
-				      (declare-fboundp
-				       (ange-ftp-ftp-path string)))
-				 (and (fboundp 'efs-ftp-path)
-				      (declare-fboundp
-				       (efs-ftp-path string)))))
-			(setq comp t)
-		      (setq comp
-			    (try-completion string
-					    minibuffer-completion-table
-					    minibuffer-completion-predicate)))
-		    (or (eq comp t)
-			(and (equal comp string)
-			     (or (null minibuffer-completion-predicate)
-				 (stringp
-				  minibuffer-completion-predicate) ; ???
-				 (funcall minibuffer-completion-predicate
-					  (if (vectorp
-					       minibuffer-completion-table)
-					      (intern-soft
-					       string
-					       minibuffer-completion-table)
-					    string))))
-			(goto-char p))))))))))
+                  (or (and filename-kludge-p (file-remote-p string))
+                      (test-completion string
+                                       minibuffer-completion-table
+                                       minibuffer-completion-predicate)
+		      (goto-char p)))))))))
 
 (defun minibuffer-smart-select-kludge-filename (string)
   (save-excursion
     (set-buffer mouse-grabbed-buffer) ; the minibuf
-    (let ((kludge-string (concat (buffer-string) string)))
-      (if (or (and (fboundp 'ange-ftp-ftp-path)
-		   (declare-fboundp (ange-ftp-ftp-path kludge-string)))
-	       (and (fboundp 'efs-ftp-path)
-		    (declare-fboundp (efs-ftp-path kludge-string))))
-	  ;; #### evil evil evil, but more so.
-	  string
-	(append-expand-filename (buffer-string) string)))))
+    (if (file-remote-p (concat (buffer-string) string))
+	string
+      (append-expand-filename (buffer-string) string))))
 
 (defun minibuffer-smart-select-highlighted-completion (event)
   "Select the highlighted text under the mouse as a minibuffer response.
@@ -1186,6 +1153,7 @@ list is specified.")
 (defvar buffer-history '())
 (defvar shell-command-history '())
 (defvar file-name-history '())
+(defvar coding-system-history '())
 
 (defvar read-expression-history nil)
 
@@ -1467,7 +1435,9 @@ If DEFAULT-VALUE is non-nil, return that if user enters an empty
 		  (input-error nil)
 		  (invalid-read-syntax nil)
 		  (end-of-file nil)))
-      (or (funcall pred num) (beep)))
+      ;; undefined-key is not theoretically ideal as the sound, but close
+      ;; enough.
+      (or (funcall pred num) (ding nil 'undefined-key)))
     num))
 
 (defun read-shell-command (prompt &optional initial-input history default-value)
@@ -1483,15 +1453,11 @@ If DEFAULT-VALUE is non-nil, return that if user enters an empty
 
 ;; Quote "$" as "$$" to get it past substitute-in-file-name
 (defun un-substitute-in-file-name (string)
-  (let ((regexp "\\$")
+  (let ((n (count ?$ string))
         (olen (length string))
-        new
-        n o ch)
-    (if (not (string-match regexp string))
+        new o ch)
+    (if (eql n 0)
 	string
-      (setq n 1)
-      (while (string-match regexp string (match-end 0))
-	(setq n (1+ n)))
       (setq new (make-string (+ olen n) ?$))
       (setq n 0 o 0)
       (while (< o olen)
@@ -1503,7 +1469,6 @@ If DEFAULT-VALUE is non-nil, return that if user enters an empty
 	    (setq n (1+ n))))
       new)))
 
-
 ;; Wrapper for `directory-files' for use in generating completion lists.
 ;; Generates output in the same format as `file-name-all-completions'.
 ;;
@@ -1669,7 +1634,7 @@ This will prompt with a dialog box if appropriate, according to
 Value is not expanded---you must call `expand-file-name' yourself.
 Value is subject to interpretation by `substitute-in-file-name' however.
 Default name to DEFAULT if user enters a null string.
- (If DEFAULT is omitted, the visited file name is used,
+ \(If DEFAULT is omitted, the visited file name is used,
   except that if INITIAL-CONTENTS is specified, that combined with DIR is
   used.)
 Fourth arg MUST-MATCH non-nil means require existing file's name.
@@ -1703,7 +1668,7 @@ This will prompt with a dialog box if appropriate, according to
 Value is not expanded---you must call `expand-file-name' yourself.
 Value is subject to interpreted by substitute-in-file-name however.
 Default name to DEFAULT if user enters a null string.
- (If DEFAULT is omitted, the current buffer's default directory is used.)
+ \(If DEFAULT is omitted, the current buffer's default directory is used.)
 Fourth arg MUST-MATCH non-nil means require existing directory's name.
  Non-nil and non-t means also require confirmation after completion.
 Fifth arg INITIAL-CONTENTS specifies text to start with.
@@ -1730,25 +1695,22 @@ DIR defaults to current buffer's directory default."
              (name    (if orig (file-name-nondirectory sstring) string))
              (direct  (if specdir (expand-file-name specdir dir) dir)))
         ;; ~username completion
-        (if (and (fboundp 'user-name-completion-1)
-                 (string-match "^[~]" name))
+        (if (and (> (length name) 0) (eql ?~ (aref name 0)))
             (let ((user (substring name 1)))
               (cond ((eq action 'lambda)
                      (file-directory-p name))
-                    ((eq action 't)
+                    ((eq action t)
                      ;; all completions
                      (mapcar #'(lambda (p) (concat "~" p))
-                             (declare-fboundp
-			      (user-name-all-completions user))))
+                             (user-name-all-completions user)))
                     (t;; 'nil
                      ;; complete
-                     (let* ((val+uniq (declare-fboundp
-				       (user-name-completion-1 user)))
-                            (val  (car val+uniq))
-                            (uniq (cdr val+uniq)))
+                     (multiple-value-bind (val uniq)
+                         (user-name-completion user)
                        (cond ((stringp val)
                               (if uniq
-                                  (file-name-as-directory (concat "~" val))
+                                  (file-name-as-directory
+                                   (concat "~" val))
                                 (concat "~" val)))
                              ((eq val t)
                               (file-name-as-directory name))
@@ -1775,9 +1737,7 @@ DIR defaults to current buffer's directory default."
                                     start))))
              (head (substring string 0 (1- start)))
              (alist #'(lambda ()
-                        (mapcar #'(lambda (x)
-                                    (cons (substring x 0 (string-match "=" x))
-                                          nil))
+                        (mapcar #'(lambda (x) (subseq x 0 (position ?= x)))
                                 process-environment))))
 
 	(cond ((eq action 'lambda)
@@ -1785,10 +1745,7 @@ DIR defaults to current buffer's directory default."
               ((eq action 't)
                ;; all completions
                (mapcar #'(lambda (p)
-			   (if (and (> (length p) 0)
-				    ;;#### Unix-specific
-				    ;;####  -- need absolute-pathname-p
-				    (/= (aref p 0) ?/))
+			   (if (not (file-name-absolute-p p))
 			       (concat "$" p)
                              (concat head "$" p)))
                        (all-completions env (funcall alist))))
@@ -1827,7 +1784,7 @@ DIR defaults to current buffer's directory default."
             ((eq action 't)
              ;; all completions
              (mapcar #'un-substitute-in-file-name
-                     (if (string= name "")
+                     (if (equal name "")
                          (delete "./" (file-name-all-completions "" dir))
                        (file-name-all-completions name dir))))
             (t;; nil
@@ -1856,19 +1813,15 @@ DIR defaults to current buffer's directory default."
   (read-file-name-internal-1
    string dir action
    #'(lambda (action orig string specdir dir name)
-      (let* ((dirs #'(lambda (fn)
-		       (let ((l (if (equal name "")
-				    (minibuf-directory-files
-				     dir
-				     ""
-				     'directories)
-				  (minibuf-directory-files
-				   dir
-				   (concat "\\`" (regexp-quote name))
-				   'directories))))
-			 (mapcar fn
-				 ;; Wretched unix
-				 (delete "." l))))))
+      (labels ((dirs (fn)
+                 (let ((l (if (equal name "")
+                              (minibuf-directory-files dir "" 'directories)
+                            (minibuf-directory-files
+                             dir (concat "\\`" (regexp-quote name))
+                             'directories))))
+                   (mapcar fn
+                           ;; Wretched unix
+                           (delete "." l)))))
         (cond ((eq action 'lambda)
                ;; complete?
                (if (not orig)
@@ -1876,17 +1829,14 @@ DIR defaults to current buffer's directory default."
 		 (file-directory-p string)))
               ((eq action 't)
                ;; all completions
-               (funcall dirs #'(lambda (n)
-				 (un-substitute-in-file-name
-				  (file-name-as-directory n)))))
+               (dirs #'(lambda (n) (un-substitute-in-file-name
+                                    (file-name-as-directory n)))))
               (t
                ;; complete
                (let ((val (try-completion
                            name
-                           (funcall dirs
-                                    #'(lambda (n)
-					(list (file-name-as-directory
-					       n)))))))
+                           (dirs #'(lambda (n)
+                                     (list (file-name-as-directory n)))))))
                  (if (stringp val)
                      (un-substitute-in-file-name (if specdir
                                                      (concat specdir val)
@@ -1905,9 +1855,8 @@ environment variable or username expanded and resolved to indicate
 whether it is a file(/result) or a directory (/result/)."
   (let ((file
 	 (cond ((string-match "\\([~$]\\)\\([^~$/]*\\)$" file-string)
-		(cond ((string= (substring file-string
-					   (match-beginning 1)
-					   (match-end 1)) "~")
+		(cond ((and (match-beginning 1)
+                            (eql ?~ (aref file-string (match-beginning 1))))
 		       (concat (substring file-string 0 (match-end 1))
 			       string))
 		      (t (substitute-in-file-name
@@ -1926,18 +1875,6 @@ whether it is a file(/result) or a directory (/result/)."
 	   result)
 	  (t file))))
 
-(defun mouse-rfn-setup-vars (prompt)
-  ;; a specifier would be nice.
-  (set (make-local-variable 'frame-title-format)
-       (capitalize-string-as-title
-	;; Kludge: Delete ": " off the end.
-	(replace-in-string prompt ": $" "")))
-  ;; ensure that killing the frame works right,
-  ;; instead of leaving us in the minibuffer.
-  (add-local-hook 'delete-frame-hook
-		  #'(lambda (frame)
-		      (abort-recursive-edit))))
-
 (defun mouse-file-display-completion-list (window dir minibuf user-data)
   (let ((standard-output (window-buffer window)))
     (condition-case nil
@@ -2011,7 +1948,28 @@ whether it is a file(/result) or a directory (/result/)."
 	 user-data
 	 (window-min-height 1)) ; allow button window to be height 2
     (unwind-protect
-	(progn
+	(labels
+            ((mouse-rfn-setup-vars (prompt)
+               ;; a specifier would be nice.
+               (set (make-local-variable 'frame-title-format)
+                    (capitalize-string-as-title
+                     ;; Kludge: Delete ": " off the end.
+                     (replace-in-string prompt ": $" "")))
+               ;; ensure that killing the frame works right,
+               ;; instead of leaving us in the minibuffer.
+               (add-local-hook 'delete-frame-hook
+                               #'(lambda (frame) (abort-recursive-edit))))
+             (rfcshookfun ()
+               ;; kludge!
+               ;; #### I really need to flesh out the object
+               ;; hierarchy better to avoid these kludges.
+               ;; (?? I wrote this comment above some time ago,
+               ;; and I don't understand what I'm referring to
+               ;; any more. --ben
+               (mouse-rfn-setup-vars prompt)
+               (when-boundp 'scrollbar-width
+                 (set-specifier scrollbar-width 0 (current-buffer)))
+               (setq truncate-lines t)))
 	  (reset-buffer filebuf)
 
 	  ;; set up the frame.
@@ -2030,24 +1988,11 @@ whether it is a file(/result) or a directory (/result/)."
 	  (set-window-buffer (frame-lowest-window frame) butbuf)
 
 	  ;; set up completion buffers.
-	  (let ((rfcshookfun
-		 ;; kludge!
-		 ;; #### I really need to flesh out the object
-		 ;; hierarchy better to avoid these kludges.
-		 ;; (?? I wrote this comment above some time ago,
-		 ;; and I don't understand what I'm referring to
-		 ;; any more. --ben
-		 (lambda ()
-		   (mouse-rfn-setup-vars prompt)
-		   (when-boundp 'scrollbar-width
-		     (set-specifier scrollbar-width 0 (current-buffer)))
-		   (setq truncate-lines t))))
-	    
-	    (set-buffer filebuf)
-	    (add-local-hook 'completion-setup-hook rfcshookfun)
-	    (when file-p
-	      (set-buffer dirbuf)
-	      (add-local-hook 'completion-setup-hook rfcshookfun)))
+          (set-buffer filebuf)
+          (add-local-hook 'completion-setup-hook #'rfcshookfun)
+          (when file-p
+            (set-buffer dirbuf)
+            (add-local-hook 'completion-setup-hook #'rfcshookfun))
 
 	  ;; set up minibuffer.
 	  (add-one-shot-hook
@@ -2139,28 +2084,60 @@ On mswindows devices, this uses `mswindows-color-list'."
 
 
 (defun read-coding-system (prompt &optional default-coding-system)
-  "Read a coding-system (or nil) from the minibuffer.
-Prompting with string PROMPT.
+  "Read a coding system name from the minibuffer, using PROMPT.
+
 If the user enters null input, return second argument DEFAULT-CODING-SYSTEM.
-DEFAULT-CODING-SYSTEM can be a string, symbol, or coding-system object."
-  (intern (completing-read prompt obarray 'find-coding-system t nil nil 
-			   (cond ((symbolp default-coding-system)
-				  (symbol-name default-coding-system))
-				 ((coding-system-p default-coding-system)
-				  (symbol-name (coding-system-name default-coding-system)))
-				 (t
-				  default-coding-system)))))
+DEFAULT-CODING-SYSTEM can be a string, symbol, or coding-system object.
+
+XEmacs coding system names are usually lowercase, but there are exceptions.
+Case is ignored by this function and by `read-non-nil-coding-system', though
+other XEmacs functions related to coding systems require that case be
+preserved.
+
+The return value will be a symbol which can usefully be passed to
+`find-coding-system'."
+  (let* ((default-coding-system
+          (cond ((coding-system-p default-coding-system)
+                 (symbol-name (coding-system-name default-coding-system)))
+                (default-coding-system
+                    (if (symbolp default-coding-system)
+                        (symbol-name default-coding-system)
+                      default-coding-system))
+                (t "nil")))
+         (prompt
+          (if (and (not (equal default-coding-system "nil"))
+                   (eql (mismatch prompt ": " :from-end t)
+                        (- (length prompt) (length ": ")))
+                   (not (eql (mismatch prompt "): " :from-end t)
+                             (- (length prompt) (length "): ")))))
+              (format "%.*s (default `%s'): " (- (length prompt) (length ": "))
+                      prompt default-coding-system)
+            prompt))
+         (completion-ignore-case t)
+         (result
+          (completing-read prompt obarray 'find-coding-system t nil
+                           'coding-system-history default-coding-system)))
+    (or (and (find-coding-system (intern result)) (intern result))
+        (find result (coding-system-list) :key #'symbol-name :test #'equalp)
+        result)))
 
 (defun read-non-nil-coding-system (prompt)
-  "Read a non-nil coding-system from the minibuffer.
-Prompt with string PROMPT."
-  (let ((retval (intern "")))
-    (while (eql 0 (length (symbol-name retval)))
-      (setq retval (intern (completing-read prompt obarray
-					    'find-coding-system
-					    t))))
-    retval))
+  "Read a non-nil coding system name from the minibuffer, using PROMPT.
 
+Case is ignored by this function and by `read-coding-system', though other
+XEmacs functions related to coding systesm require that case be preserved.
+XEmacs coding system name are usually lowercase, but there are exceptions.
+
+The return value will be a non-nil symbol which can usefully be passed to
+`find-coding-system'."
+  (let ((result '##) (completion-ignore-case t))
+    (while (or (eq '## result) (not result))
+      (setq result (intern (completing-read prompt obarray 'find-coding-system
+                                            t nil 'coding-system-history))))
+    (or (and (find-coding-system result) result)
+        (find (symbol-name result)
+              (coding-system-list) :key #'symbol-name :test #'equalp)
+        result)))
 
 
 (defcustom force-dialog-box-use nil
@@ -2230,12 +2207,12 @@ in the minibuffer."
       (let* ((answers (remove-if-not #'consp answers))
 	     (possible
 	      (gettext
-	       (labels ((car-to-string-if (x)
+	       (labels ((car-to-sequence-if (x)
                           (setq x (car x))
-                          (if (stringp x)  x (char-to-string x))))
-		 (concat (mapconcat #'car-to-string-if
+                          (if (stringp x) x (list x))))
+		 (concat (mapconcat #'car-to-sequence-if
 			   (butlast answers) ", ") " or "
-			   (car-to-string-if (car (last answers)))))))
+			   (car-to-sequence-if (car (last answers)))))))
 	     (question (gettext question))
 	     (p (format "%s(%s) " question possible)))
 	(block nil
@@ -2268,15 +2245,8 @@ in the minibuffer."
 				 (single-key-description event))
 			(setq quit-flag nil)
 			(signal 'quit '())))
-		  (let* ((keys (events-to-keys (vector event)))
-			 (def (lookup-key query-replace-map keys)))
+		  (let ((def (lookup-key query-replace-map (vector event))))
 		    (cond
-; 		     ((eq def 'skip)
-; 		      (message "%s%sNo" question possible)
-; 		      (return nil))
-; 		     ((eq def 'act)
-; 		      (message "%s%sYes" question possible)
-; 		      (return t))
 		     ((eq def 'recenter)
 		      (recenter))
 		     ((or (eq def 'quit) (eq def 'exit-prefix))
