@@ -1,11 +1,11 @@
-;;; mule-coding.el --- Coding-system functions for Mule. -*- coding: iso-2022-7bit; -*-
+;;; mule-coding.el --- Coding-system functions for Mule.
 
 ;; Copyright (C) 1995 Electrotechnical Laboratory, JAPAN.
 ;; Licensed to the Free Software Foundation.
 ;; Copyright (C) 1995 Amdahl Corporation.
 ;; Copyright (C) 1995 Sun Microsystems.
 ;; Copyright (C) 1997 MORIOKA Tomohiko
-;; Copyright (C) 2001 Ben Wing.
+;; Copyright (C) 2001, 2005, 2010 Ben Wing.
 
 ;; This file is part of XEmacs.
 
@@ -99,7 +99,6 @@ The allowable range of REGISTER is 0 through 3."
  '(charset-g0 ascii
    charset-g1 latin-iso8859-1
    eol-type nil
-   safe-charsets t ;; Reasonable
    mnemonic "CText"))
 
 (make-coding-system
@@ -109,9 +108,6 @@ The allowable range of REGISTER is 0 through 3."
    charset-g1 latin-iso8859-1
    charset-g2 t ;; unspecified but can be used later.
    short t
-   safe-charsets (ascii katakana-jisx0201 japanese-jisx0208-1978
-                  japanese-jisx0208 japanese-jisx0212 japanese-jisx0213-1
-                  japanese-jisx0213-2)
    mnemonic "ISO8/SS"
    documentation "ISO 2022 based 8-bit encoding using SS2 for 96-charset"
    ))
@@ -123,7 +119,6 @@ The allowable range of REGISTER is 0 through 3."
    charset-g2 t ;; unspecified but can be used later.
    seven t
    short t
-   safe-charsets t
    mnemonic "ISO7/SS"
    documentation "ISO 2022 based 7-bit encoding using SS2 for 96-charset"
    eol-type nil))
@@ -136,7 +131,6 @@ The allowable range of REGISTER is 0 through 3."
    charset-g2 t ;; unspecified but can be used later.
    seven t
    short t
-   safe-charsets t
    mnemonic "ISO7/SS"
    eol-type nil))
 
@@ -146,7 +140,6 @@ The allowable range of REGISTER is 0 through 3."
  '(charset-g0 ascii
    seven t
    short t
-   safe-charsets t
    mnemonic "ISO7"
    documentation "ISO-2022-based 7-bit encoding using only G0"
    ))
@@ -154,13 +147,26 @@ The allowable range of REGISTER is 0 through 3."
 ;; compatibility for old XEmacsen
 (define-coding-system-alias 'iso-2022-7 'iso-2022-7bit)
 
+(make-coding-system
+ 'iso-2022-8bit-preserve 'iso2022
+ "ISO 2022 8-bit, ISO-2022-preserving"
+ '(charset-g0 ascii
+   charset-g1 latin-iso8859-1
+   short t
+   iso2022-preserve t
+   mnemonic "ISO8-Preserve"
+   documentation "ISO-2022-based 8-bit encoding with I/O preservation.
+This uses private Unicode characters, as necessary, to preserve the particular
+ISO-2022 charset upon output.  This will make such characters unusable
+in normal editing."
+   ))
+
 (make-coding-system
  'iso-2022-8 'iso2022
  "ISO-2022 8-bit"
  '(charset-g0 ascii
    charset-g1 latin-iso8859-1
    short t
-   safe-charsets t
    mnemonic "ISO8"
    documentation "ISO-2022 eight-bit coding system.  No single-shift or locking-shift."
    ))
@@ -172,7 +178,6 @@ The allowable range of REGISTER is 0 through 3."
    charset-g1 latin-iso8859-1
    eol-type lf
    escape-quoted t
-   safe-charsets t
    mnemonic "ESC/Quot"
    documentation "ISO-2022 eight-bit coding system with escape quoting; used for .ELC files."
    ))
@@ -184,7 +189,6 @@ The allowable range of REGISTER is 0 through 3."
    charset-g1 t ;; unspecified but can be used later.
    seven t
    lock-shift t
-   safe-charsets t
    mnemonic "ISO7/Lock"
    documentation "ISO-2022 coding system using Locking-Shift for 96-charset."
    ))
@@ -220,13 +224,10 @@ Analogous to `define-translation-table', but updates
 ;; Ideally this would be in latin.el, but code-init.el uses it.
 (make-coding-system
  'iso-8859-1 
- 'fixed-width
+ 'multibyte
  "ISO-8859-1 (Latin-1)"
- (eval-when-compile
-   `(unicode-map 
-     ,(loop
-        for i from #x80 to #xff
-        collect (list i (int-char i))) ;; Identical to Latin-1.
-     mnemonic "Latin 1"
-     documentation "The most used encoding of Western Europe and the Americas."
-     aliases (iso-latin-1 latin-1))))
+ '(charsets (ascii control-1 latin-iso8859-1)
+   mnemonic "Latin 1"
+   documentation "The most used encoding of Western Europe and the Americas."
+   aliases (iso-latin-1 latin-1)))
+;;; mule-coding.el ends here
