@@ -44,6 +44,7 @@ as obsolete.
 Optional WHEN is for GNU compatibility.  XEmacs ignores it.  \(In Emacs, WHEN
 is a string indicating the version where OLDFUN was first marked obsolete.)
 Optional DOCSTRING describes any changes in semantics users should be aware of."
+  (declare (special when))
   (define-function oldfun newfun)
   (make-obsolete oldfun (or docstring newfun) when))
 
@@ -56,6 +57,7 @@ Optional WHEN is for consistency with `define-obsolete-function-alias'.  XEmacs
 ignores it.  \(If present, it is a string indicating the version where OLDFUN
 was first marked as a compatibility API.)
 Optional DOCSTRING describes any changes in semantics users should be aware of."
+  (declare (special when))
   (define-function oldfun newfun)
   (make-compatible oldfun (or docstring newfun)))
 
@@ -70,6 +72,7 @@ Note: Use this before any other references (defvar/defcustom) to NEWVAR.
 Optional WHEN is for GNU compatibility.  XEmacs ignores it.  \(In Emacs, WHEN
 is a string indicating the version where OLDVAR was first marked obsolete.)
 Optional DOCSTRING describes any changes in semantics users should be aware of."
+  (declare (special when))
   (let ((needs-setting (and (boundp oldvar) (not (boundp newvar))))
         (value (and (boundp oldvar) (symbol-value oldvar))))
      (defvaralias oldvar newvar docstring)
@@ -85,6 +88,7 @@ Optional WHEN is for consistency with `define-obsolete-variable-alias'.  XEmacs
 ignores it.  \(If present, it is a string indicating the version where OLDFUN
 was first marked as a compatibility API.)
 Optional DOCSTRING describes any changes in semantics users should be aware of."
+  (declare (special when))
   (defvaralias oldvar newvar docstring)
   (make-compatible-variable oldvar newvar))
 
@@ -228,9 +232,74 @@ If FRAME is omitted or nil, use the selected frame."
 (make-obsolete-variable 'suspend-hooks 'suspend-hook)
 (make-obsolete-variable 'first-change-function 'first-change-hook)
 (make-obsolete-variable 'before-change-function
-  "use before-change-functions; which is a list of functions rather than a single function.")
-(make-obsolete-variable 'after-change-function
-  "use after-change-functions; which is a list of functions rather than a single function.")
+  "Use before-change-functions; which is a list of functions rather than a \
+single function.")
+
+(labels
+    ((make-local-handler (sym args fun harg handler)
+       (error 'protected-field
+              (format "Minimal compatibility provided for `%s'; \
+use `%ss'" sym sym)
+              fun)))
+
+  (dontusethis-set-symbol-value-handler
+   'before-change-function
+   'set-value
+   #'(lambda (sym args fun harg handler)
+       (if (car args)
+           (unless (function-allows-args (car args) 2)
+             (error 'invalid-argument "Not a valid before-change-function"
+                    (car args))))
+       (setq before-change-functions
+             (delete*
+              '#22222=#:before-change-function-secret
+              before-change-functions :test #'member*
+              :key #'(lambda (object)
+                       (and (compiled-function-p object)
+                            (compiled-function-arglist object))))
+             before-change-functions (append before-change-functions
+                                             (if (car args)
+                                                 (list
+                                                  (apply-partially
+                                                   #'(lambda (f &rest #22222#)
+                                                       (apply f #22222#))
+                                                   (car args))))))))
+
+  (dontusethis-set-symbol-value-handler
+   'before-change-function
+   'make-local #'make-local-handler)
+
+  (make-obsolete-variable 'after-change-function
+   "Use after-change-functions; which is a list of functions rather than a \
+single function.")
+
+  (dontusethis-set-symbol-value-handler
+   'after-change-function
+   'set-value
+   #'(lambda (sym args fun harg handler)
+       (if (car args)
+           (unless (function-allows-args (car args) 3)
+             (error 'invalid-argument "Not a valid after-change-function"
+                    (car args))))
+       (setq after-change-functions
+             (delete*
+              '#44444=#:after-change-function-secret
+              after-change-functions :test #'member*
+              :key #'(lambda (object)
+                       (and (compiled-function-p object)
+                            (compiled-function-arglist object))))
+             after-change-functions (append after-change-functions
+                                            (if (car args)
+                                                (list
+                                                 (apply-partially
+                                                  #'(lambda (f &rest #44444#)
+                                                      (apply f #44444#))
+                                                  (car args))))))))
+
+  (dontusethis-set-symbol-value-handler
+   'after-change-function
+   'make-local
+   #'make-local-handler))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;; insertion, deletion, movement
 
@@ -266,15 +335,6 @@ we no longer encounter bytecode from 21.4."
 (make-obsolete 'cl-float-limits "this is called at dump time in 21.5 and \
 later, no need to call it in user code.")
 
-;; XEmacs; old compiler macros meant that this was called directly
-;; from compiled code, and we need to provide a version of it for a
-;; couple of years at least because of that. Aidan Kehoe, Mon Oct 4
-;; 12:06:41 IST 2010
-(defun cl-delete-duplicates (cl-seq cl-keys cl-copy)
-  (apply (if cl-copy #'remove-duplicates #'delete-duplicates) cl-seq cl-keys))
-
-(make-obsolete 'cl-delete-duplicates 'delete-duplicates)
-
 ;; This occurs because of setf expansion by 21.4, changing the affected
 ;; packages won't fix that.
 (define-obsolete-function-alias 'cl-puthash 'puthash)
@@ -381,6 +441,12 @@ Predefined tests are `eq', `eql', and `equal'.  Default is `eql'."
   (cdr (mouse-position (frame-device frame))))
 (make-obsolete 'read-mouse-position 'mouse-position)
 
+(define-obsolete-function-alias 'mwheel-scroll 'mouse-wheel-scroll)
+(define-obsolete-variable-alias 'mwheel-scroll-amount
+    'mouse-wheel-scroll-amount nil "Now a list with modifiers specified")
+(define-obsolete-variable-alias 'mwheel-follow-mouse
+    'mouse-wheel-follow-mouse)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;; paths
 
 (defvar Info-default-directory-list nil
@@ -446,9 +512,17 @@ because its `find-charset-string' ignores ASCII charset."
 (define-obsolete-function-alias 'string-to-char-list 'string-to-list)
 
 ;; Two loser functions which shouldn't be used.
+(defun following-char (&optional buffer)
+  "Return the result of calling `char-after' with buffer BUFFER.
+If that is nil, return integer zero."
+  (or (char-after nil buffer) 0))
 (make-obsolete 'following-char 'char-after)
-(make-obsolete 'preceding-char 'char-before)
 
+(defun preceding-char (&optional buffer)
+  "Return the result of calling `char-before' with buffer BUFFER.
+If that is nil, return integer zero."
+  (or (char-before nil buffer) 0))
+(make-obsolete 'preceding-char 'char-before)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;; misc
 
@@ -485,5 +559,87 @@ because its `find-charset-string' ignores ASCII charset."
 (make-obsolete 'remrassoc "use delete* with :test #'equal, :key #'cdr")
 (make-obsolete 'remrassq "use delete* with :test #'eq, :key #'cdr")
 
+(defun max-char ()
+  "Return a fixnum one less than `char-code-limit'.
+
+In contrast with GNU Emacs, this is not necessarily a valid character, but it
+is an inclusive upper bound on the possible values returned by `char-int'."
+  (1- char-code-limit))
+(make-compatible
+ 'max-char
+ "consider `char-code-limit', which gives an exclusive upper bound")
+
+(autoload 'xemacs-intern-in-vector "obarray")
+(autoload 'xemacs-intern-soft-in-vector "obarray")
+(autoload 'xemacs-unintern-in-vector "obarray")
+(autoload 'xemacs-mapatoms-in-vector "obarray")
+
+(labels ((canonicalize-string-equal-argument (string)
+           (check-type string (or string symbol))
+           (if (symbolp string)
+               (symbol-name string)
+             string)))
+
+  (defun string-equal (string1 string2)
+    "Return t if two strings have identical contents.
+
+Case is significant.  Text properties are ignored.
+\(Under XEmacs, `equal' also ignores text properties and extents in
+strings, but this is not the case under FSF Emacs 19.  In FSF Emacs 20
+`equal' is the same as in XEmacs, in that respect.)
+Symbols are also allowed; their print names are used instead.
+
+This function is usually not appropriate, for the following reasons:
+
+-- STRING1 and STRING2 can be strings or symbols, indifferently; but any other
+   type provokes an error.
+   If you don't want type-based error-checking, you should be using `equal'
+   If you do want type-based error-checking, it's very easy to get the calling
+   convention of this function wrong and let a symbol slip through when you
+   don't intend to.
+
+-- You may be thinking \"ah, a string equal predicate, surely this defaults to
+   case-insensitive comparison, or takes account of `case-fold-search'\".
+   No, neither is true.  For the former, see `equalp'. For the latter, you're
+   stuck with something like:
+     `(string-match-p (concat \"\\\\`\" (regexp-quote NEEDLE) \"\\\\'\")
+                      HAYSTACK)'.
+
+-- It is rare that you want something like `(string-equal buffer-file-name
+   filename)' to give `t' when `filename' is the string \"nil\", if the buffer
+   doesn't have an associated file.
+
+-- `equal' has a bytecode and so most of the time will be fast, even if you
+   do your own type-checking (since `stringp' and `symbolp' both have
+   bytecodes too).
+
+It is included for compatibility with old versions of GNU Emacs, see above."
+    (equal (canonicalize-string-equal-argument string1)
+           (canonicalize-string-equal-argument string2)))
+  (make-compatible 'string-equal
+                   "use `equal' with your own type checking, if needed")
+
+  (define-function 'string= 'string-equal)
+  (make-compatible 'string=
+                   "use `equal' with your own type checking, if needed")
+
+  ;; XEmacs
+  (defun string-equal-ignore-case (string1 string2)
+    "Return t if two strings have identical contents, ignoring case.
+
+Text properties and extents are ignored. Symbols are also allowed; their print
+names are used instead.
+
+This function is usually not appropriate, see the docstring for
+`string-equal'. See also `equalp'."
+    (equalp (canonicalize-string-equal-argument string1)
+            (canonicalize-string-equal-argument string2)))
+  (make-compatible 'string-equal-ignore-case
+                   "use `equalp' with your own type checking, if needed"))
+
+(defun fetch-bytecode (object) (check-type object compiled-function) object)
+(make-obsolete 'fetch-bytecode "Lazy loading of function instructions removed"
+               "21.5-b35")
+
 (provide 'obsolete)
 ;;; obsolete.el ends here
