@@ -88,8 +88,7 @@
   "Returns a list of package hierarchy directory names.
 These are the valid immediate directory names of package
 directories, directories with higher priority first"
-  `("site-packages" ,@(when (featurep 'mule) '("mule-packages"))
-    "xemacs-packages"))
+  (list "site-packages" "mule-packages" "xemacs-packages"))
 
 (defun package-get-key (name key)
   "Get info `key' from package `name'."
@@ -103,14 +102,6 @@ directories, directories with higher priority first"
 	  (cons (cons name info) (delete* name packages-package-list
                                           :test #'eq :key #'car)))))
 
-(defun package-suppress (package file form)
-  "Set up a package-suppress condition FORM for FILE in PACKAGE.
-When XEmacs searches for a file in the load path, it will ignore FILE
-if FORM evaluates to non-nil."
-  (setq load-suppress-alist
-	(acons (expand-file-name file load-file-name) form
-	       load-suppress-alist)))
-
 (defun package-require (name version)
   (let ((pkg (assq name packages-package-list)))
     (cond ((null pkg)
@@ -178,10 +169,10 @@ is used instead of `load-path'."
 	   (message "No library %s in search path" library)))
     result))
 
-(defun packages-add-suffix (str)
-  (if (null (string-match "\\.el\\'" str))
-      (concat str ".elc")
-    str))
+(defun packages-add-suffix (string)
+  (if (string-match-p "\\.el\\'" string)
+      string
+    (concatenate 'string string ".elc")))
 
 (defun packages-list-autoloads-path ()
   "List autoloads from precomputed load-path."
@@ -325,7 +316,7 @@ Returns list of hierarchies."
 The second component is shared with PATH."
   (let ((reverse-tail '())
 	(rest path))
-    (while (and rest (null (string-equal "" (car rest))))
+    (while (and rest (null (equal "" (car rest))))
       (setq reverse-tail (cons (car rest) reverse-tail))
       (setq rest (cdr rest)))
     (if (null rest)
