@@ -106,9 +106,7 @@ when the function is called."
       (goto-char startpt)
       (while (< (point) endpt)
 	(apply function startcol endcol args)
-	(forward-line 1)))
-    ))
-
+	(forward-line 1)))))
 
 (defun delete-rectangle-line (startcol endcol fill)
   (let ((pt (point-at-eol)))
@@ -118,8 +116,7 @@ when the function is called."
 	;; else
 	(setq pt (point))
 	(move-to-column endcol t)
-	(delete-region pt (point))))
-    ))
+	(delete-region pt (point))))))
 
 ;;;###autoload
 (defun delete-rectangle (start end &optional fill)
@@ -133,39 +130,16 @@ deleted."
   (interactive "*r\nP")
   (apply-on-rectangle 'delete-rectangle-line start end fill))
 
-
-;; I love ascii art ;-)
-(defconst spaces-strings '[""
-			   " "
-			   "  "
-			   "   "
-			   "    "
-			   "     "
-			   "      "
-			   "       "
-			   "        "])
-
-;; This function is untouched --dv
-(defun spaces-string (n)
-  (if (<= n 8) (aref spaces-strings n)
-    (let ((val ""))
-      (while (> n 8)
-	(setq val (concat "        " val)
-	      n (- n 8)))
-      (concat val (aref spaces-strings n)))))
-
-
 (defun delete-extract-rectangle-line (startcol endcol lines fill)
   (let ((pt (point-at-eol)))
     (if (< (move-to-column startcol (or fill 'coerce)) startcol)
-	(setcdr lines (cons (spaces-string (- endcol startcol))
+	(setcdr lines (cons (format "%*s" (- endcol startcol) "")
 			    (cdr lines)))
       ;; else
       (setq pt (point))
       (move-to-column endcol t)
       (setcdr lines (cons (buffer-substring pt (point)) (cdr lines)))
-      (delete-region pt (point)))
-    ))
+      (delete-region pt (point)))))
 
 ;;;###autoload
 (defun delete-extract-rectangle (start end &optional fill)
@@ -178,7 +152,6 @@ deleted."
     (apply-on-rectangle 'delete-extract-rectangle-line start end lines fill)
     (nreverse (cdr lines))))
 
-
 ;; #### NOTE: this is actually the only function that needs to do complicated
 ;; stuff like what's happening in `operate-on-rectangle', because the buffer
 ;; might be read-only. --dv
@@ -201,14 +174,12 @@ deleted."
       (let ((width (- (current-column)
 		      (save-excursion (backward-char 1)
 				      (current-column)))))
-	(setq line (concat (substring line 0 (- (point) end 1))
-			   (spaces-string width)
-			   (substring line (+ (length line)
-					      (- (point) end)))))))
+	(setq line (format "%.*s%*s%s"
+			   (- (point) end 1) line
+			   width ""
+			   (subseq line (+ (length line) (- (point) end)))))))
     (if (or (> begextra 0) (> endextra 0))
-	(setq line (concat (spaces-string begextra)
-			   line
-			   (spaces-string endextra))))
+	(setq line (format "%*s%s%*s" begextra "" line endextra "")))
     (setcdr lines (cons line (cdr lines)))))
 
 ;;;###autoload
@@ -219,7 +190,6 @@ as a list of strings, one for each line of the rectangle."
     (apply-on-rectangle 'extract-rectangle-line start end lines)
     (nreverse (cdr lines))))
 
-
 ;;;###autoload
 (defvar killed-rectangle nil
   "Rectangle for `yank-rectangle' to insert.")
@@ -245,7 +215,6 @@ deleted."
   (interactive "*")
   (insert-rectangle killed-rectangle))
 
-
 ;; This function is untouched --dv
 ;;;###autoload
 (defun insert-rectangle (rectangle)
@@ -269,7 +238,6 @@ and point is at the lower right corner."
       (insert (car lines))
       (setq lines (cdr lines)))))
 
-
 (defun open-rectangle-line (startcol endcol fill)
   (when (= (move-to-column startcol (or fill 'coerce)) startcol)
     (unless (and (not fill)
@@ -287,7 +255,6 @@ on the right side of the rectangle."
   (apply-on-rectangle 'open-rectangle-line start end fill)
   (goto-char start))
 
-
 (defun string-rectangle-line (startcol endcol string delete)
   (move-to-column startcol t)
   (if delete
@@ -315,7 +282,6 @@ as if `pending-delete-mode' were active."
   (interactive "*r\nsString rectangle: ")
   (apply-on-rectangle 'string-rectangle-line start end string t))
 
-
 (defun clear-rectangle-line (startcol endcol fill)
   (let ((pt (point-at-eol))
 	spaces)
@@ -328,8 +294,7 @@ as if `pending-delete-mode' were active."
 	(move-to-column endcol t)
 	(setq spaces (- (point) pt))
 	(delete-region pt (point))
-	(indent-to (+ (current-column) spaces))))
-    ))
+	(indent-to (+ (current-column) spaces))))))
 
 ;;;###autoload
 (defun clear-rectangle (start end &optional fill)
@@ -342,7 +307,6 @@ rectangle which were empty."
   (interactive "*r\nP")
   (apply-on-rectangle 'clear-rectangle-line start end fill))
 
-
 (provide 'rect)
 
 ;;; rect.el ends here
