@@ -148,20 +148,10 @@ before rotating to the next."
 	       nil nil nil
 	       'query-replace-history))
      (list from to current-prefix-arg)))
-  (let (replacements)
-    (if (listp to-strings)
-	(setq replacements to-strings)
-      (while (not (eql (length to-strings) 0))
-	(if (string-match " " to-strings)
-	    (setq replacements
-		  (append replacements
-			  (list (substring to-strings 0
-					   (string-match " " to-strings))))
-		  to-strings (substring to-strings
-				       (1+ (string-match " " to-strings))))
-	  (setq replacements (append replacements (list to-strings))
-		to-strings ""))))
-    (perform-replace regexp replacements t t nil arg)))
+  (perform-replace regexp (if (listp to-strings)
+                              to-strings
+                            (split-string-by-char to-strings ?\ ))
+                   t t nil arg))
 
 (defun replace-string (from-string to-string &optional delimited)
   "Replace occurrences of FROM-STRING with TO-STRING.
@@ -540,8 +530,7 @@ When searching for a match, this function uses
   (or map (setq map query-replace-map))
   (let* ((event (make-event))
 	 (nocasify (not (and case-fold-search case-replace
-			    (string-equal from-string
-					  (downcase from-string)))))
+			    (equal from-string (canoncase from-string)))))
 	 (literal (not regexp-flag))
 	 (search-function (if regexp-flag
 			      replace-re-search-function
@@ -563,7 +552,10 @@ When searching for a match, this function uses
 	 ;; XEmacs addition
 	 (qr-case-fold-search
 	  (if (and case-fold-search search-caps-disable-folding)
-	      (no-upper-case-p search-string regexp-flag)
+              (if regexp-flag
+                  (no-case-regexp-p search-string)
+                (let (case-fold-search)
+                  (not (string-match-p "[[:upper:]]" search-string))))
 	    case-fold-search))
 	 (message
 	  (if query-flag
@@ -613,7 +605,7 @@ When searching for a match, this function uses
 		      t))
 
 	  ;; Save the data associated with the real match.
-	  (setq real-match-data (match-data))
+	  (setq real-match-data (match-data-canonical))
 
 	  ;; Before we make the replacement, decide whether the search string
 	  ;; can match again just after this match.
@@ -709,7 +701,7 @@ When searching for a match, this function uses
 			 (recenter-top-bottom)))
 		      ((eq def 'edit)
 		       (store-match-data
-			(prog1 (match-data)
+			(prog1 (match-data-canonical)
 			  (save-excursion (recursive-edit))))
 		       ;; Before we make the replacement,
 		       ;; decide whether the search string
@@ -718,7 +710,7 @@ When searching for a match, this function uses
 			   (setq match-again (looking-at search-string))))
 		      ((eq def 'delete-and-edit)
 		       (delete-region (match-beginning 0) (match-end 0))
-		       (store-match-data (prog1 (match-data)
+		       (store-match-data (prog1 (match-data-canonical)
 					   (save-excursion (recursive-edit))))
 		       (setq replaced t))
 		      ;; Note: we do not need to treat `exit-prefix'
@@ -734,12 +726,10 @@ When searching for a match, this function uses
 		;; Reset recenter cycling order to initial position.
 		(setq recenter-last-op nil))
 	      ;; Record previous position for ^ when we move on.
-	      ;; Change markers to numbers in the match data
-	      ;; since lots of markers slow down editing.
 	      (setq stack
 		    (cons (cons (point)
 				(or replaced
-				    (match-data t)))
+				    (match-data-canonical)))
 			  stack))
 	      (if replaced (setq replace-count (1+ replace-count)))))
 	  (setq lastrepl (point)))
