@@ -4,7 +4,7 @@
 ;;   Free Software Foundation, Inc.
 ;; Copyright (C) 1995 Tinker Systems and INS Engineering Corp.
 ;; Copyright (C) 1995 Sun Microsystems.
-;; Copyright (C) 2000, 2001, 2002, 2003 Ben Wing.
+;; Copyright (C) 2000, 2001, 2002, 2003, 2010 Ben Wing.
 
 ;; Maintainer: XEmacs Development Team
 ;; Keywords: extensions, dumped, internal
@@ -82,7 +82,55 @@ ARGS is a list of the first N arguments to pass to FUNCTION.
 The result is a new function which does the same as FUNCTION, except that
 the first N arguments are fixed at the values with which this function
 was called."
-  `(lambda (&rest args) (apply ',function ,@(mapcar 'quote-maybe args) args)))
+  (if (and (eq 'lambda (car-safe function))
+           (<= (length args) (or (function-max-args function)
+                                most-positive-fixnum)))
+      ;; XEmacs; for our constructed function, don't just use a (&rest args)
+      ;; arglist if there is an explicit lambda supplied. This allows the
+      ;; byte-compile-arglist-signagures-congruent-p check to be useful with
+      ;; the interpreted implementation of #'lexical-let.
+      (let* ((arglist (nth 1 function)) restp bindings body header)
+        (while (and arglist args)
+          (cond ((eq (car arglist) '&optional)
+                 (if restp
+                     (error 'syntax-error "&optional found after &rest"
+                            function))
+                 (if (null (cdr arglist))
+                     (error 'syntax-error "No argument name for &optional"
+                            function)))
+                ((eq (car arglist) '&rest)
+                 (if (null (cdr arglist))
+                     (error 'syntax-error "No argument name for &rest"
+                            function))
+                 (if (cdr (cdr arglist))
+                     (error 'syntax-error "Multiple arguments after &rest"
+                            function))
+                 (setq restp t))
+                (restp
+                 (let ((new-arg-name (if (functionp 'gensym) (gensym)
+                                                             '*rest-arg*)))
+                   (setq bindings (cons (list (car arglist)
+                                              `(list*
+                                                ,@(mapcar #'quote-maybe args)
+                                                ,new-arg-name))
+                                        bindings)
+                         args nil
+                         arglist `(nil &rest ,new-arg-name))))
+                (t
+                 (setq bindings (cons (list (car arglist)
+                                            (quote-maybe (car args)))
+                                      bindings)
+                       args (cdr args))))
+          (setq arglist (cdr arglist)))
+        (setq body (cddr function))
+        (when (stringp (car body))
+          (push (pop body) header))
+        (when (eq 'interactive (car-safe (car body)))
+          (push (pop body) header))
+        `(lambda ,arglist ,@(nreverse header)
+          (let ,(nreverse bindings) ,@body)))
+    `(lambda (&rest #1=#:args)
+      (apply ',function ,@(mapcar 'quote-maybe args) #1#))))
 
 ;; FSF 21.2 has various basic macros here.  We don't because they're either
 ;; in cl*.el (which we dump and hence is always available) or built-in.
@@ -125,6 +173,118 @@ Used for compatibility among different emacs variants."
      (define-function ,@args)))
 
 
+(defun format (control-string &rest arguments)
+  "Format a string out of CONTROL-STRING and ARGUMENTS.
+
+The first argument is a control string.
+The other arguments are substituted into it to make the result, a string.
+It may contain %-sequences meaning to substitute the next argument.
+%s means print all objects as-is, using `princ'.
+%S means print all objects as s-expressions, using `prin1'.
+%d or %i means print as a signed integer in decimal (%o octal, %x lowercase
+  hex, %X uppercase hex, %b binary).
+
+%c means print as a single character.
+%f means print as a floating-point number in fixed notation (e.g. 785.200).
+%e or %E means print as a floating-point number in scientific notation
+  (e.g. 7.85200e+03).
+%g or %G means print as a floating-point number in \"pretty format\";
+  depending on the number, either %f or %e/%E format will be used, and
+  trailing zeroes are removed from the fractional part.
+The argument used for all but %s, %S, and %c must be a number.  It will be
+  converted to an integer or a floating-point number as necessary.  In
+  addition, the integer %-sequences accept character arguments as equivalent
+  to the corresponding fixnums (see `char-int'), while the floating point
+  sequences do not.
+
+%$ means reposition to read a specific numbered argument; for example,
+  %3$s would apply the `%s' to the third argument after the control string,
+  and the next format directive would use the fourth argument, the
+  following one the fifth argument, etc. (There must be a positive integer
+  between the % and the $).
+
+Zero or more of the flag characters `-', `+', ` ', `0', `!', '&', `~' and `#'
+  may be specified between the optional repositioning spec and the conversion
+  character; see below.
+
+An optional minimum field width may be specified after any flag characters
+  and before the conversion character; it specifies the minimum number of
+  characters that the converted argument will take up.  Padding will be
+  added on the left (or on the right, if the `-' flag is specified), as
+  necessary.  Padding is with zeroes if the `0' flag is specified, with the
+  character specified following the `!' flag if that is supplied (see below),
+  or by default with spaces.
+
+If the field width is specified as `*', the field width is assumed to have
+  been specified as an argument.  Any repositioning specification that
+  would normally specify the argument to be converted will now specify
+  where to find this field width argument, not where to find the argument
+  to be converted.  If there is no repositioning specification, the normal
+  next argument is used.  The argument to be converted will be the next
+  argument after the field width argument unless the precision is also
+  specified as `*' (see below).
+
+An optional period character and precision may be specified after any
+  minimum field width.  It specifies the minimum number of digits to
+  appear in %d, %i, %o, %x, and %X conversions (the number is padded
+  on the left with zeroes as necessary); the number of digits printed
+  after the decimal point for %f, %e, and %E conversions; the number
+  of significant digits printed in %g and %G conversions; and the
+  maximum number of non-padding characters printed in %s and %S
+  conversions.  The default precision for floating-point conversions
+  is six. Using a precision with %c is an error.
+If the precision is specified as `*', the precision is assumed to have been
+  specified as an argument.  The argument used will be the next argument
+  after the field width argument, if any.  If the field width was not
+  specified as an argument, any repositioning specification that would
+  normally specify the argument to be converted will now specify where to
+  find the precision argument.  If there is no repositioning specification,
+  the normal next argument is used.
+
+An optional length modifier may be specified after any minimum field width or
+  precision.  The length modifiers available are `hh', `h', `l' (ell), `ll'
+  \(ell-ell), and `z'.  These mean that a following integer conversion
+  character will print an integer truncated to eight, sixteen, thirty-two,
+  sixty-four, or (once more) thirty-two bits respectively.  Note that this
+  contrasts with ANSI C, where the bit length depends on the machine choices
+  for the bit width of various integer types.
+
+An optional unsigned modifier, the character `u', may be specified after any
+  length modifier and before an integer conversion character.  This specifies
+  that the following integer conversion is to treat its argument as unsigned.
+  If no length modifier is specified, this simply means that `format' will
+  error if the corresponding integer is negative.  With a length modifier
+  `format' will print a positive integer reflecting the twos' complement
+  representation of the argument in the given number of bits. E.g. `(format
+  \"%hux\" -1)' will return the string \"ffff\".  If no integer conversion
+  character follows `u', the specification is regarded as equivalent to `ud',
+  and the argument will be printed in decimal.
+
+The ` ' and `+' flags mean prefix non-negative numbers with a space or
+  plus sign, respectively.
+The `#' flag means print numbers in an alternate, more verbose format:
+  octal numbers begin with zero; hex numbers begin with a 0x or 0X;
+  a decimal point is printed in %f, %e, and %E conversions even if no
+  numbers are printed after it; and trailing zeroes are not omitted in
+   %g and %G conversions.
+The `&' flag is analogous to the `#' flag for rationals, but the syntax used
+  to print numbers is that of Common Lisp, rather than C, so octal numbers are
+  preceded by `#o', binary numbers by `#b' and hexadecimal numbers by `#X' or
+  `#x' depending on the particular converter character specified.
+The `!' flag is followed by a single character, to be used as a pad character
+  instead of space.  It does not override the zero flag.
+The `~' flag, when combined with `&' or `#', means to place any sign before
+  the radix specifier.
+
+Use %% to put a single % into the output.
+
+Extent information in CONTROL-STRING and in ARGS is carried over into the
+output, in the same way as `concatenate'.  Any text created by a character or
+numeric %-sequence inherits the extents of the text around it, or of the text
+abutting it if those extents' `start-open' and `end-open' properties have the
+appropriate values."
+  (apply #'format-into 'string control-string arguments))
+
 (defun delete (item sequence)
   "Delete by side effect any occurrences of ITEM as a member of SEQUENCE.
 
@@ -159,6 +319,14 @@ This is a non-destructive function; it makes a copy of SEQUENCE to avoid
 corrupting the original SEQUENCE.  See also the more general `remove*'."
   (remove* item sequence :test #'eq))
 
+(defun assoc (key alist &optional test)
+  "Return non-nil if KEY is `equal' to the car of an element of ALIST.
+The value is actually the element of ALIST whose car equals KEY.
+
+Optional third argument TEST specifies the test to use for comparison instead
+of #'equal. See also the more full-featured `assoc*'."
+  (assoc* key alist :test (or test #'equal)))
+
 (defun assoc-default (key alist &optional test default)
   "Find object KEY in a pseudo-alist ALIST.
 ALIST is a list of conses or objects.  Each element (or the element's car,
@@ -202,6 +370,222 @@ ELT must be a string.  Upper-case and lower-case letters are treated as equal."
            (the (and list (satisfies (lambda (list) (every 'stringp list))))
                 list)
            :test 'equalp))
+
+(defun concat (&rest args)
+  "Concatenate all the arguments and make the result a string.
+The result is a string whose elements are the elements of all the arguments.
+Each argument may be a list, vector, bit-vector or string.
+
+Argument elements must be characters, or integers that can be converted to
+characters; see `char-int'."
+  (apply #'concatenate 'string args))
+
+(defun vconcat (&rest args)
+  "Concatenate all the arguments and make the result a vector.
+The result is a vector whose elements are the elements of all the arguments.
+Each argument may be a list, vector, bit vector, or string."
+  (apply #'concatenate 'vector args))
+
+(defun bvconcat (&rest args)
+  "Concatenate all the arguments and make the result a bit vector.
+The result is a bit vector whose elements are the elements of all the
+arguments.  Each argument may be a list, vector, bit vector, or string.
+
+Argument elements must be the integers zero or one, or the characters ?\\x00,
+?\\x01."
+  (apply #'concatenate 'bit-vector args))
+
+;; XEmacs; move these non-basic predicates that can be easily implemented in
+;; Lisp from data.c.
+
+(defun null (object)
+  "Return t if OBJECT is nil."
+  (eq object nil))
+
+(defalias 'not #'null)
+
+(defun atom (object)
+  "Return t if OBJECT is not a cons cell.  `nil' is not a cons cell.
+
+See the documentation for `cons' or the Lisp manual for more details on what
+a cons cell is."
+  (not (consp object)))
+
+(defun listp (object)
+  "Return t if OBJECT is a list.  `nil' is a list.
+
+A list is either the Lisp object nil (a symbol), interpreted as the empty
+list in this context, or a cons cell whose CDR refers to either nil or a
+cons cell.
+
+A \"proper list\" contains no cycles.  A \"true list\" is a proper list
+which has `nil' as the CDR of its last cons cell.  See `true-list-p', an
+alternative, more expensive function, if these features are important for
+your use case."
+  (or (consp object) (eq object nil)))
+
+(defun nlistp (object)
+  "Return t if OBJECT is not a list.  `nil' is a list."
+  (not (or (consp object) (eq object nil))))
+
+(defun arrayp (object)
+  "Return t if OBJECT is an array (string, vector, or bit vector)."
+  (or (stringp object) (vectorp object) (bit-vector-p object)))
+
+(defun sequencep (object)
+  "Return t if OBJECT is a sequence (list or array)."
+  (or (stringp object) (listp object) (vectorp object) (bit-vector-p object)))
+
+(defun integerp (object)
+  "Return t if OBJECT is an integer, nil otherwise.
+
+On builds without bignum support, this function is identical to `fixnump'."
+  (or (fixnump object) (bignump object)))
+
+(defun natnump (object)
+  "Return t if OBJECT is a nonnegative integer."
+  (and (integerp object) (not (eql (signum object) -1))))
+
+(defun nonnegativep (object)
+  "Return t if OBJECT is a nonnegative rational."
+  (and (rationalp object) (not (eql (signum object) -1))))
+
+(defun bitp (object)
+  "Return t if OBJECT is a bit (0 or 1)."
+  (or (eql object 0) (eql object 1)))
+
+(defun char-int-p (object)
+  "Return t if OBJECT is an integer that can be converted into a character.
+See `char-int'."
+  (and (fixnump object) (int-to-char object) t))
+
+(defun char-or-char-int-p (object)
+  "Return t if OBJECT is a character or a fixnum that can be converted to one."
+  (or (and (fixnump object) (int-to-char object) t) (characterp object)))
+
+(defun char-or-string-p (object)
+  "Return t if OBJECT is a character (or a char-int) or a string.
+It is semi-hateful that we allow a char-int here, as it goes against
+the name of this function, but it makes the most sense considering the
+other steps we take to maintain compatibility with the old character/integer
+confoundedness in older versions of E-Lisp."
+  (or (stringp object) (and (fixnump object) (int-to-char object) t)
+      (characterp object)))
+
+(defun integer-or-marker-p (object)
+  "Return t if OBJECT is an integer or a marker (editor pointer)."
+  (or (integerp object) (markerp object)))
+
+(defun integer-or-char-p (object)
+  "Return t if OBJECT is an integer or a character."
+  (or (integerp object) (characterp object)))
+
+(defun integer-char-or-marker-p (object)
+  "Return t if OBJECT is an integer, character or a marker (editor pointer)."
+  (or (integerp object) (markerp object) (characterp object)))
+
+(defun number-or-marker-p (object)
+  "Return t if OBJECT is a number or a marker."
+  (or (numberp object) (markerp object)))
+
+(defun number-char-or-marker-p (object)
+  "Return t if OBJECT is a number, character or a marker."
+  (or (numberp object) (characterp object) (markerp object)))
+
+(defun zerop (number)
+  "Return t if NUMBER is zero."
+  (check-type number number)
+  (= number 0))
+
+(defun max (first &rest rest)
+  "Return largest of all the arguments.
+All arguments must be real numbers, characters or markers.
+
+This function does no implicit type conversion; that is, markers and
+characters supplied in the arguments will be returned as such.  This
+differs from GNU Emacs and from older XEmacs, where markers and
+characters were converted to fixnums. Floats and extended numeric
+types have never been converted to fixnums by this function."
+  (let ((max first))
+    (dolist (number rest max)
+      (when (< max number)
+	(setq max number)))))
+
+(defun min (first &rest rest)
+  "Return smallest of all the arguments.
+All arguments must be numbers, characters or markers.
+
+This function does no implicit type conversion; that is, markers and
+characters supplied in the arguments will be returned as such.  This
+differs from GNU Emacs and from older XEmacs, where markers and
+characters were converted to fixnums. Floats and extended numeric
+types have never been converted to fixnums by this function."
+  (let ((min first))
+    (dolist (number rest min)
+      (when (> min number)
+	(setq min number)))))
+
+(defun true-list-p (object)
+  "Return t if OBJECT is an acyclic, nil-terminated list."
+  (let ((hare object) (tortoise object) (length 0) flag)
+    (while (and (consp hare) (or (not (eq hare tortoise)) (eql length 0)))
+      (if flag (setq tortoise (cdr tortoise)))
+      (setq hare (cdr hare)
+            length (1+ length)
+	    flag (not flag)))
+    (eq hare nil)))
+
+(defun safe-length (list)
+  "Return the length of a list, but avoid error or infinite loop.
+This function never gets an error.  If LIST is not really a list,
+it returns 0.  If LIST is circular, it returns a finite value
+which is at least the number of distinct elements."
+  (let ((hare list) (tortoise list) (length 0) flag)
+    (while (and (consp hare) (or (not (eq hare tortoise)) (eql length 0)))
+      (if flag (setq tortoise (cdr tortoise)))
+      (setq hare (cdr hare)
+            length (1+ length)
+	    flag (not flag)))
+    length))
+
+(defun identity (argument)
+  "Return ARGUMENT unchanged."
+  argument)
+
+(defun nth (n list)
+  "Return the Nth element of LIST.
+N counts from zero.  If LIST is not that long, nil is returned."
+  (car (nthcdr n list)))
+
+(defun car-safe (object)
+  "Return the car of OBJECT if it is a cons cell, or else nil."
+  (if (consp object) (car object)))
+
+(defun cdr-safe (object)
+  "Return the cdr of OBJECT if it is a cons cell, else nil."
+  (if (consp object) (cdr object)))
+
+(defun copy-sequence (sequence)
+  "Return a copy of list, vector, bit vector or string SEQUENCE.
+The elements of a list or vector are not copied; they are shared
+with the original. SEQUENCE may be a dotted list.
+
+See also `copy-tree'."
+  (cond ((stringp sequence) (concat sequence nil))
+        ((consp sequence) (copy-list sequence))
+        ((not sequence) nil)
+        (t (concatenate (type-of sequence) sequence))))
+
+(defun replace-list (old new)
+  "Destructively replace the list OLD with NEW.
+This is like (copy-sequence NEW) except that it reuses the conses in OLD as
+much as possible.  If OLD and NEW are the same length, no consing will take
+place.
+
+See also `replace', which handles sequences generally, but does not copy the
+tail of the source sequence to the end of the destination sequence in the same
+way that `replace-list' does."
+  (nconc (replace old new) (copy-list (nthcdr (length old) new))))
 
 ;;;; Keymap support.
 ;; XEmacs: removed to keymap.el
@@ -233,9 +617,6 @@ ELT must be a string.  Upper-case and lower-case letters are treated as equal."
 (define-function 'send-string-to-terminal 'external-debugging-output)
 (define-function 'special-form-p 'special-operator-p)
 
-;; XEmacs; this is in Lisp, its bytecode now taken by subseq.
-(define-function 'substring 'subseq)
-
 (define-function 'sort 'sort*)
 (define-function 'fillarray 'fill)
   
@@ -674,22 +1055,14 @@ The original current buffer is restored afterwards."
 (defmacro save-match-data (&rest body)
   "Execute BODY forms, restoring the global value of the match data."
   (let ((original (make-symbol "match-data")))
-    (list 'let (list (list original '(match-data)))
-	  (list 'unwind-protect
-		(cons 'progn body)
-		(list 'store-match-data original)))))
-
-
-(defun match-string (num &optional string)
-  "Return string of text matched by last search.
-NUM specifies which parenthesized expression in the last regexp.
- Value is nil if NUMth pair didn't match, or there were less than NUM pairs.
-Zero means the entire text matched by the whole regexp or whole string.
-STRING should be given if the last search was by `string-match' on STRING."
-  (if (match-beginning num)
-      (if string
-          (substring string (match-beginning num) (match-end num))
-        (buffer-substring (match-beginning num) (match-end num)))))
+    `(let ((,original (match-data-canonical)))
+      (unwind-protect (progn ,@body)
+        (store-match-data ,original)
+        (while ,original
+          (and (not (listp (car ,original)))
+               (not (fixnump (car ,original)))
+               (delete-extent (car ,original)))
+          (setq ,original (cdr ,original)))))))
 
 (defun match-string-no-properties (num &optional string)
   "Return string of text matched by last search, without text properties.
@@ -697,14 +1070,9 @@ NUM specifies which parenthesized expression in the last regexp.
  Value is nil if NUMth pair didn't match, or there were less than NUM pairs.
 Zero means the entire text matched by the whole regexp or whole string.
 STRING should be given if the last search was by `string-match' on STRING."
-  (if (match-beginning num)
-      (if string
-	  (let ((result
-		 (substring string (match-beginning num) (match-end num))))
-	    (set-text-properties 0 (length result) nil result)
-	    result)
-	(buffer-substring-no-properties (match-beginning num)
-					(match-end num)))))
+  (let ((result (match-string num string)))
+    (if result (set-text-properties 0 (length result) nil result))
+    result))
 
 ;; Imported from GNU Emacs 23.3.1 -- dvl
 (defun looking-back (regexp &optional limit greedy)
@@ -824,8 +1192,8 @@ Otherwise treat `\\' in NEWTEXT as special:
   `\\U' means begin upcasing all following characters.
   `\\L' means begin downcasing all following characters.
   `\\E' means terminate the effect of any `\\U' or `\\L'."
-  (check-argument-type 'stringp str)
-  (check-argument-type 'stringp newtext)
+  (check-type str string)
+  (check-type newtext string)
   (if (> (length str) 50)
       (let ((cfs case-fold-search))
 	(with-temp-buffer
@@ -938,19 +1306,6 @@ that can be added."
 
 ;;; Basic string functions
 
-;; XEmacs
-(defun string-equal-ignore-case (str1 str2)
-  "Return t if two strings have identical contents, ignoring case differences.
-Case is not significant.  Text properties and extents are ignored.
-Symbols are also allowed; their print names are used instead.
-
-See also `equalp'."
-  (if (symbolp str1)
-      (setq str1 (symbol-name str1)))
-  (if (symbolp str2)
-      (setq str2 (symbol-name str2)))
-  (eq t (compare-strings str1 nil nil str2 nil nil t)))
-
 (defun insert-face (string face)
   "Insert STRING and highlight with FACE.  Return the extent created."
   (let ((p (point)) ext)
@@ -960,7 +1315,6 @@ See also `equalp'."
     ext))
 
 ;; not obsolete.
-(define-function 'string= 'string-equal)
 (define-function 'string< 'string-lessp)
 (define-function 'int-to-string 'number-to-string)
 (define-function 'string-to-int 'string-to-number)
@@ -998,10 +1352,6 @@ display width when using a window system.  With no international support,
 simply returns the length of the string."
   (reduce #'+ (the string string) :initial-value 0 :key #'char-width))
 
-(defun char-width (character)
-  "Return number of columns a CHARACTER occupies when displayed."
-  (charset-width (char-charset character)))
-
 ;; The following several functions are useful in GNU Emacs 20 because
 ;; of the multibyte "characters" the internal representation of which
 ;; leaks into Lisp.  In XEmacs/Mule they are trivial and unnecessary.
@@ -1335,6 +1685,15 @@ See also `cerror', `signal', and `signal-error'."
 	(t
 	 (error 'invalid-argument "datum not string or error symbol" datum))))
 
+(defun wrong-type-argument (predicate value)
+  "Signal an error until the correct type value is given by the user.
+This function loops, signalling a continuable `wrong-type-argument' error
+with PREDICATE and VALUE as the data associated with the error and then
+calling PREDICATE on the returned value, until the value gotten satisfies
+PREDICATE.  At that point, the gotten value is returned."
+  (do ((result value (signal 'wrong-type-argument (list predicate result))))
+      ((funcall predicate result) result)))
+
 (defmacro check-argument-type (predicate argument)
   "Check that ARGUMENT satisfies PREDICATE.
 This is a macro, and ARGUMENT is not evaluated.  If ARGUMENT is an lvalue,
@@ -1438,15 +1797,14 @@ error occurred (i.e. the handler itself)."
 	       ;; Try to eliminate the part of the backtrace
 	       ;; above where the error occurred.
 	       (if (string-match
-		    (concat "bind (\\(?:.* \\)?" (symbol-name handler-arg-name)
-			    "\\(?:.* \\)?)[ \t\n]*\\(?:(lambda \\|#<compiled-function \\)("
+		    (concat "bind (\\(?:.*? \\)?" (symbol-name handler-arg-name)
+			    "\\(?:.*? \\)?)[ \t\n]*\\(?:(lambda \\|#<compiled-function \\)("
 			    (symbol-name handler-arg-name)
 			    ").*\n\\(\\(?:.\\|\n\\)*\\)$")
 		    bt) (match-string 1 bt) bt))))
     bt))
 
-(put 'with-trapping-errors 'lisp-indent-function 0)
-(defmacro with-trapping-errors (&rest keys-body)
+(defmacro with-trapping-errors (&rest rest)
   "Trap errors in BODY, outputting a warning and a backtrace.
 Usage looks like
 
@@ -1457,6 +1815,7 @@ Usage looks like
     [:class CLASS]
     [:level LEVEL]
     [:resignal RESIGNAL]
+    [:debug DEBUG]
     BODY)
 
 Return value without error is whatever BODY returns.  With error, return
@@ -1465,47 +1824,77 @@ occurs), which defaults to nil.  OPERATION is given in the warning message.
 CLASS and LEVEL are the warning class and level (default to class
 `general', level `warning').  If NO-BACKTRACE is given, no backtrace is
 displayed.  If RESIGNAL is given, the error is resignaled after the warning
-is displayed and the ERROR-FORM is executed."
-  (let ((operation "unknown")
-	(error-form nil)
-	(no-backtrace nil)
-	(class ''general)
-	(level ''warning)
-	(resignal nil)
-	(cte-cc-var '#:cte-cc-var)
-	(call-trapping-errors-arg '#:call-trapping-errors-Ldc9FC5Hr))
-    (let* ((keys '(operation error-form no-backtrace class level resignal))
-	   (keys-with-colon
-	    (mapcar #'(lambda (sym)
-			(intern (concat ":" (symbol-name sym)))) keys)))
-      (while (memq (car keys-body) keys-with-colon)
-	(let* ((key-with-colon (pop keys-body))
-	       (key (intern (substring (symbol-name key-with-colon) 1))))
-	  (set key (pop keys-body)))))
-    `(condition-case ,(if resignal cte-cc-var nil)
-	 (call-with-condition-handler
-	     #'(lambda (,call-trapping-errors-arg)
-		 (let ((errstr (error-message-string
-				,call-trapping-errors-arg)))
-		   ,(if no-backtrace
-			`(lwarn ,class ,level
-			   (if (warning-level-<
-				,level
-				display-warning-minimum-level)
-			       "Error in %s: %s"
-			     "Error in %s:\n%s\n")
-			   ,operation errstr)
-		      `(lwarn ,class ,level
-			 "Error in %s: %s\n\nBacktrace follows:\n\n%s"
-			 ,operation errstr
-			 (backtrace-in-condition-handler-eliminating-handler
-			  ',call-trapping-errors-arg)))))
-	     #'(lambda ()
-		 (progn ,@keys-body)))
-       (error
-	,error-form
-	,@(if resignal `((signal (car ,cte-cc-var) (cdr ,cte-cc-var)))))
-       )))
+is displayed and the ERROR-FORM is executed.  If DEBUG is given, the
+debugger is entered.
+
+Common Lisp conventions are accepted for the keywords, with the constraint
+that any keywords need to be constant at macro-expansion time (so, in
+practice, you need to type the keywords explicitly in your call to
+`with-trapping-errors').  Otherwise it becomes impractical to figure out where
+BODY starts.
+
+arguments: (&key (OPERATION \"unknown\") (CLASS ''GENERAL) (LEVEL ''WARNING) ERROR-FORM NO-BACKTRACE RESIGNAL DEBUG &body BODY)"
+  (declare (indent 0))
+  (let ((body rest))
+    (while (keywordp (car body)) (setf body (cddr body)))
+    (funcall
+     (function*
+      (lambda ((&key (operation "unknown") (class ''general)
+                    (level ''warning) error-form no-backtrace resignal
+                    debug)
+               body
+               &aux (cte-cc-var '#:cte-cc-var) 
+               ;; This ideally should be lexically unique within the call
+               ;; stack, can't use the same uninterned symbol name for every
+               ;; macro expansion:
+               (call-trapping-errors-arg (gensym "AymGisUF")))
+       (labels
+           ((handler (call-trapping-errors-arg no-backtrace class level
+                                               operation debug &optional
+                                               return-block)
+              `(lambda (,call-trapping-errors-arg)
+                ,@(if debug
+                      `((funcall debugger 'error ,call-trapping-errors-arg)))
+                ,(if no-backtrace
+                     `(lwarn ,class ,level
+                       (if (warning-level-< ,level
+                                            display-warning-minimum-level)
+                           "Error in %s: %s"
+                         "Error in %s:\n%s\n")
+                       ,operation (error-message-string
+                                   ,call-trapping-errors-arg))
+                     `(lwarn ,class ,level
+                       "Error in %s: %s\n\nBacktrace follows:\n\n%s"
+                       ,operation (error-message-string
+                                   ,call-trapping-errors-arg)
+                       (backtrace-in-condition-handler-eliminating-handler
+                        ',call-trapping-errors-arg)))
+                ,@(if return-block `((return-from ,return-block))))))
+         (declare (inline handler))
+         (cond
+           (error-form
+            `(condition-case ,(if resignal cte-cc-var nil)
+              (call-with-condition-handler
+                  ,(handler call-trapping-errors-arg no-backtrace class
+                           level operation debug)
+                  #'(lambda () (progn ,@body)))
+              (error ,error-form
+               ,@(if resignal
+                     `((signal (car ,cte-cc-var) (cdr ,cte-cc-var)))))))
+           (resignal
+            `(call-with-condition-handler
+              ,(handler call-trapping-errors-arg no-backtrace class
+                        level operation debug)
+              #'(lambda () (progn ,@body))))
+           (t
+            `(block trapping-errors
+              (call-with-condition-handler
+                  ,(handler call-trapping-errors-arg no-backtrace class
+                            level operation debug 'trapping-errors)
+                  #'(lambda () (progn ,@body)))))))))
+     ;; Get CL-conformant keyword parsing for free:
+     (ldiff rest body)
+     body)))
 
 ;;;; Miscellanea.
 
@@ -1788,4 +2177,26 @@ If not supplied, SPECIFIED-TIME defaults to the result of `current-time'."
 	      (/ (car specified-time) 1000000.0))
 	 0.0)))
 
+(defun char-to-string (character)
+  "Convert CHARACTER to a one-character string containing that character."
+  (when (eventp character)
+    (let ((event-to-character (event-to-character character t)))
+      (setq character
+	    (or event-to-character (error 'no-character-typed character)))))
+  (string character))
+
+(defun string-to-char (string)
+  "Convert arg STRING to a character, the first character of that string.
+An empty string will return the constant `nil'."
+  (check-type string string)
+  (unless (eql (length string) 0) (aref string 0)))
+
+(defun char-equal (character1 character2 &optional buffer)
+  "Return t if two characters match, optionally ignoring case.
+Both arguments must be characters (i.e. NOT integers).
+Case is ignored if `case-fold-search' is non-nil in BUFFER.
+If BUFFER is nil, the current buffer is assumed."
+  (or (eql character1 character2)
+      (eql (canoncase character1 buffer) (canoncase character2 buffer))))
+
 ;;; subr.el ends here
