@@ -35,29 +35,26 @@
 ;; Currently this only does ISO 8859-1 (Latin-1).  Other charsets defined
 ;; in C either have no translation map or are special-cased in the code,
 ;; and internal charsets defined in Lisp specify their own maps.
-(when (featurep 'mule)
-  (let ((parse-args
-	 '(;; Due to the braindamaged way Mule treats the ASCII and Control-1
-	   ;; charsets' types, trying to load them results in out-of-range
-	   ;; warnings at unicode.c:1439.  They're no-ops anyway, they're
-	   ;; hardwired in unicode.c (unicode_to_ichar, ichar_to_unicode).
-	   ;; (ascii "unicode/unicode-consortium/ISO8859/ISO-8859-1.TXT"
-	   ;;        #x00 #x7F #x0)
-	   ;; (control-1 "unicode/unicode-consortium/ISO8859/ISO-8859-1.TXT"
-	   ;;   	 #x80 #x9F #x-80)
-	   ;; The 8859-1.TXT G1 assignments are half no-ops, hardwired in
-	   ;; unicode.c ichar_to_unicode, but not in unicode_to_ichar.
-	   (latin-iso8859-1 "unicode/unicode-consortium/ISO8859/ISO-8859-1.TXT"
-			    #xA0)
-	   )))
-    (mapc #'(lambda (args)
-	      (apply 'load-unicode-mapping-table
-		     (expand-file-name (cadr args)
-				       (expand-file-name "etc"
-							 source-directory))
-		     (car args) (cddr args)))
-	  parse-args)
-    ))
+(let ((parse-args
+       '(;; Due to the braindamaged way Mule treats the ASCII and Control-1
+	 ;; charsets' types, trying to load them results in out-of-range
+	 ;; warnings at unicode.c:1439.  They're no-ops anyway, they're
+	 ;; hardwired in unicode.c (unicode_to_ichar, ichar_to_unicode).
+	 ;; (ascii "unicode/unicode-consortium/ISO8859/ISO-8859-1.TXT"
+	 ;;        #x00 #x7F #x0)
+	 ;; (control-1 "unicode/unicode-consortium/ISO8859/ISO-8859-1.TXT"
+	 ;;   	 #x80 #x9F #x-80)
+	 ;; The 8859-1.TXT G1 assignments are half no-ops, hardwired in
+	 ;; unicode.c ichar_to_unicode, but not in unicode_to_ichar.
+	 (latin-iso8859-1 "unicode/unicode-consortium/ISO8859/ISO-8859-1.TXT"
+	  #xA0))))
+  (mapc #'(lambda (args)
+	    (apply 'load-unicode-mapping-table
+		   (expand-file-name (cadr args)
+				     (expand-file-name "etc"
+						       source-directory))
+		   (car args) (cddr args)))
+	parse-args))
 
 (defun initialize-default-unicode-precedence-list
   ()
@@ -226,150 +223,144 @@ Standard encoding for representing UTF-8 under MS Windows."
   ;; called from mule-charset.el, after defining functions necessary to be
   ;; able to create charsets (e.g. charset-tag-to-charset-list), but before
   ;; creating the Asian charsets and loading their Unicode tables.
-  (when (featurep 'mule)
-    (make-charset 'windows-glyph-list-4
-		  "Windows Glyph List 4 etc."
-		  '(dimension 1
-		    offset 128
-		    chars 128))
-
-    ;; We don't want the characters in Windows Glyph List 4 (and a few
-    ;; others) to map to East Asian character sets when they are displayed.
-    ;; WGL4 is a character repertoire from Microsoft that gives a guideline
-    ;; for font implementors as to what characters are sufficient for
-    ;; pan-European support.  East Asian fonts generally clash strongly with
-    ;; European characters both in font choice and character width, and since
-    ;; the WGL 4 characters are basically European characters, we want them
-    ;; displayed with European fonts.  So create a special charset
-    ;; `windows-glyph-list-4' that we put higher in the default Unicode
-    ;; precedence list than any Asian charsets. (Under old-Mule, these
-    ;; characters will also end up in jit-ucs-charset-0, which likewise we
-    ;; put higher than the Asian charsets.)
-    ;; 
-    (loop for (ucs ascii-or-latin-1)
-      in '((#x2013 ?-) ;; U+2013 EN DASH
-	   (#x2014 ?-) ;; U+2014 EM DASH
-	   (#x2105 ?%) ;; U+2105 CARE OF
-	   (#x203e ?-) ;; U+203E OVERLINE
-	   (#x221f ?|) ;; U+221F RIGHT ANGLE
-	   (#x2584 ?|) ;; U+2584 LOWER HALF BLOCK
-	   (#x2588 ?|) ;; U+2588 FULL BLOCK
-	   (#x258c ?|) ;; U+258C LEFT HALF BLOCK
-	   (#x2550 ?|) ;; U+2550 BOX DRAWINGS DOUBLE HORIZONTAL
-	   (#x255e ?|) ;; U+255E BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE
-	   (#x256a ?|) ;; U+256A BOX DRAWINGS VERTICAL SINGLE & HORIZONTAL DOUBLE
-	   (#x2561 ?|) ;; U+2561 BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE
-	   (#x2215 ?/) ;; U+2215 DIVISION SLASH
-	   (#x02c9 ?`) ;; U+02C9 MODIFIER LETTER MACRON
-	   (#x2211 ?s) ;; U+2211 N-ARY SUMMATION
-	   (#x220f ?s) ;; U+220F N-ARY PRODUCT
-	   (#x2248 ?=) ;; U+2248 ALMOST EQUAL TO
-	   (#x2264 ?=) ;; U+2264 LESS-THAN OR EQUAL TO
-	   (#x2265 ?=) ;; U+2265 GREATER-THAN OR EQUAL TO
-	   (#x201c ?') ;; U+201C LEFT DOUBLE QUOTATION MARK
-	   (#x2026 ?.) ;; U+2026 HORIZONTAL ELLIPSIS
-	   (#x2212 ?-) ;; U+2212 MINUS SIGN
-	   (#x2260 ?=) ;; U+2260 NOT EQUAL TO
-	   (#x221e ?=) ;; U+221E INFINITY
-	   (#x2642 ?=) ;; U+2642 MALE SIGN
-	   (#x2640 ?=) ;; U+2640 FEMALE SIGN
-	   (#x2032 ?=) ;; U+2032 PRIME
-	   (#x2033 ?=) ;; U+2033 DOUBLE PRIME
-	   (#x25cb ?=) ;; U+25CB WHITE CIRCLE
-	   (#x25cf ?=) ;; U+25CF BLACK CIRCLE
-	   (#x25a1 ?=) ;; U+25A1 WHITE SQUARE
-	   (#x25a0 ?=) ;; U+25A0 BLACK SQUARE
-	   (#x25b2 ?=) ;; U+25B2 BLACK UP-POINTING TRIANGLE
-	   (#x25bc ?=) ;; U+25BC BLACK DOWN-POINTING TRIANGLE
-	   (#x2192 ?=) ;; U+2192 RIGHTWARDS ARROW
-	   (#x2190 ?=) ;; U+2190 LEFTWARDS ARROW
-	   (#x2191 ?=) ;; U+2191 UPWARDS ARROW
-	   (#x2193 ?=) ;; U+2193 DOWNWARDS ARROW
-	   (#x2229 ?=) ;; U+2229 INTERSECTION
-	   (#x2202 ?=) ;; U+2202 PARTIAL DIFFERENTIAL
-	   (#x2261 ?=) ;; U+2261 IDENTICAL TO
-	   (#x221a ?=) ;; U+221A SQUARE ROOT
-	   (#x222b ?=) ;; U+222B INTEGRAL
-	   (#x2030 ?=) ;; U+2030 PER MILLE SIGN
-	   (#x266a ?=) ;; U+266A EIGHTH NOTE
-	   (#x2020 ?*) ;; U+2020 DAGGER
-	   (#x2021 ?*) ;; U+2021 DOUBLE DAGGER
-	   (#x2500 ?|) ;; U+2500 BOX DRAWINGS LIGHT HORIZONTAL
-	   (#x2502 ?|) ;; U+2502 BOX DRAWINGS LIGHT VERTICAL
-	   (#x250c ?|) ;; U+250C BOX DRAWINGS LIGHT DOWN AND RIGHT
-	   (#x2510 ?|) ;; U+2510 BOX DRAWINGS LIGHT DOWN AND LEFT
-	   (#x2518 ?|) ;; U+2518 BOX DRAWINGS LIGHT UP AND LEFT
-	   (#x2514 ?|) ;; U+2514 BOX DRAWINGS LIGHT UP AND RIGHT
-	   (#x251c ?|) ;; U+251C BOX DRAWINGS LIGHT VERTICAL AND RIGHT
-	   (#x252c ?|) ;; U+252C BOX DRAWINGS LIGHT DOWN AND HORIZONTAL
-	   (#x2524 ?|) ;; U+2524 BOX DRAWINGS LIGHT VERTICAL AND LEFT
-	   (#x2534 ?|) ;; U+2534 BOX DRAWINGS LIGHT UP AND HORIZONTAL
-	   (#x253c ?|) ;; U+253C BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL
-	   (#x02da ?^) ;; U+02DA RING ABOVE
-	   (#x2122 ?\xa9) ;; U+2122 TRADE MARK SIGN, ?©
-
-	   (#x0132 ?\xe6) ;; U+0132 LATIN CAPITAL LIGATURE IJ, ?æ
-	   (#x013f ?\xe6) ;; U+013F LATIN CAPITAL LETTER L WITH MIDDLE DOT, ?æ
-
-	   (#x0133 ?\xe6) ;; U+0133 LATIN SMALL LIGATURE IJ, ?æ
-	   (#x0140 ?\xe6) ;; U+0140 LATIN SMALL LETTER L WITH MIDDLE DOT, ?æ
-	   (#x0149 ?\xe6) ;; U+0149 LATIN SMALL LETTER N PRECEDED BY APOSTROPH,?æ
-
-	   (#x2194 ?|) ;; U+2194 LEFT RIGHT ARROW
-	   (#x2660 ?*) ;; U+2660 BLACK SPADE SUIT
-	   (#x2665 ?*) ;; U+2665 BLACK HEART SUIT
-	   (#x2663 ?*) ;; U+2663 BLACK CLUB SUIT
-	   (#x2592 ?|) ;; U+2592 MEDIUM SHADE
-	   (#x2195 ?|) ;; U+2195 UP DOWN ARROW
-
-	   (#x2113 ?\xb9) ;; U+2113 SCRIPT SMALL L, ?¹
-	   (#x215b ?\xbe) ;; U+215B VULGAR FRACTION ONE EIGHTH, ?¾
-	   (#x215c ?\xbe) ;; U+215C VULGAR FRACTION THREE EIGHTHS, ?¾
-	   (#x215d ?\xbe) ;; U+215D VULGAR FRACTION FIVE EIGHTHS, ?¾
-	   (#x215e ?\xbe) ;; U+215E VULGAR FRACTION SEVEN EIGHTHS, ?¾
-	   (#x207f ?\xbe) ;; U+207F SUPERSCRIPT LATIN SMALL LETTER N, ?¾
+  (make-charset 'windows-glyph-list-4
+		"Windows Glyph List 4 etc."
+		'(dimension 1
+		  offset 128
+		  chars 128))
   
-	   ;; These are not in WGL 4, but are IPA characters that should not
-	   ;; be double width. They are the only IPA characters that both
-	   ;; occur in packages/mule-packages/leim/ipa.el and end up in East
-	   ;; Asian character sets when that file is loaded in an XEmacs
-	   ;; without packages.
-	   (#x2197 ?|) ;; U+2197 NORTH EAST ARROW
-	   (#x2199 ?|) ;; U+2199 SOUTH WEST ARROW
-	   (#x2191 ?|) ;; U+2191 UPWARDS ARROW
-	   (#x207f ?\xb9)) ;; U+207F SUPERSCRIPT LATIN SMALL LETTER N, ?¹
-      with decoded = nil
-      with codepoint = #x80
-      with syntax-table = (standard-syntax-table)
-      do
-      ;; Under old-Mule, the call to `decode-char' creates
-      ;; jit-ucs-charset-0 entries because it is run at dump time, before
-      ;; we create any charsets containing Asian characters, or more
-      ;; specifically before we load the Unicode mapping tables for the
-      ;; Asian-character charsets, which currently happens at charset
-      ;; creation time. (That happens in mule-charset.el.) It doesn't
-      ;; matter whether we move the call to (set-unicode-conversion code
-      ;; 'windows-glyph-list-4 ...) before the call to `decode-char',
-      ;; because windows-glyph-list-4, being size 128, is a non-encodable
-      ;; charset.  By putting jit-ucs-charset-0 above the Asian charsets,
-      ;; we ensure that the properties of jit-ucs-charset-0 determine how
-      ;; the characters are represented.
-      ;;
-      ;; Under Unicode-internal, we get the same behavior by putting
-      ;; windows-glyph-list-4 before the Asian charsets.
-      ;;
-      (set-unicode-conversion ucs 'windows-glyph-list-4 codepoint)
-      (setq decoded (decode-char 'ucs ucs))
-      (assert (memq (declare-fboundp (char-charset decoded))
-		    '(jit-ucs-charset-0 windows-glyph-list-4))
-	      nil "Unexpected Unicode decoding behavior, actual charset=%s.  "
-	      (char-charset decoded))
-      (incf codepoint)
-      (modify-syntax-entry decoded
-			   (string 
-			    (char-syntax ascii-or-latin-1))
-			   syntax-table))
-    ))
+  ;; We don't want the characters in Windows Glyph List 4 (and a few others) to
+  ;; map to East Asian character sets when they are displayed.  WGL4 is a
+  ;; character repertoire from Microsoft that gives a guideline for font
+  ;; implementors as to what characters are sufficient for pan-European
+  ;; support.  East Asian fonts generally clash strongly with European
+  ;; characters both in font choice and character width, and since the WGL 4
+  ;; characters are basically European characters, we want them displayed with
+  ;; European fonts.  So create a special charset `windows-glyph-list-4' that
+  ;; we put higher in the default Unicode precedence list than any Asian
+  ;; charsets. (Under old-Mule, these characters will also end up in
+  ;; jit-ucs-charset-0, which likewise we put higher than the Asian charsets.)
+  ;; 
+  (loop for (ucs ascii-or-latin-1)
+    in '((#x2013 ?-) ;; U+2013 EN DASH
+	 (#x2014 ?-) ;; U+2014 EM DASH
+	 (#x2105 ?%) ;; U+2105 CARE OF
+	 (#x203e ?-) ;; U+203E OVERLINE
+	 (#x221f ?|) ;; U+221F RIGHT ANGLE
+	 (#x2584 ?|) ;; U+2584 LOWER HALF BLOCK
+	 (#x2588 ?|) ;; U+2588 FULL BLOCK
+	 (#x258c ?|) ;; U+258C LEFT HALF BLOCK
+	 (#x2550 ?|) ;; U+2550 BOX DRAWINGS DOUBLE HORIZONTAL
+	 (#x255e ?|) ;; U+255E BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE
+	 (#x256a ?|) ;; U+256A BOX DRAWINGS VERTICAL SINGLE & HORIZONTAL DOUBLE
+	 (#x2561 ?|) ;; U+2561 BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE
+	 (#x2215 ?/) ;; U+2215 DIVISION SLASH
+	 (#x02c9 ?`) ;; U+02C9 MODIFIER LETTER MACRON
+	 (#x2211 ?s) ;; U+2211 N-ARY SUMMATION
+	 (#x220f ?s) ;; U+220F N-ARY PRODUCT
+	 (#x2248 ?=) ;; U+2248 ALMOST EQUAL TO
+	 (#x2264 ?=) ;; U+2264 LESS-THAN OR EQUAL TO
+	 (#x2265 ?=) ;; U+2265 GREATER-THAN OR EQUAL TO
+	 (#x201c ?') ;; U+201C LEFT DOUBLE QUOTATION MARK
+	 (#x2026 ?.) ;; U+2026 HORIZONTAL ELLIPSIS
+	 (#x2212 ?-) ;; U+2212 MINUS SIGN
+	 (#x2260 ?=) ;; U+2260 NOT EQUAL TO
+	 (#x221e ?=) ;; U+221E INFINITY
+	 (#x2642 ?=) ;; U+2642 MALE SIGN
+	 (#x2640 ?=) ;; U+2640 FEMALE SIGN
+	 (#x2032 ?=) ;; U+2032 PRIME
+	 (#x2033 ?=) ;; U+2033 DOUBLE PRIME
+	 (#x25cb ?=) ;; U+25CB WHITE CIRCLE
+	 (#x25cf ?=) ;; U+25CF BLACK CIRCLE
+	 (#x25a1 ?=) ;; U+25A1 WHITE SQUARE
+	 (#x25a0 ?=) ;; U+25A0 BLACK SQUARE
+	 (#x25b2 ?=) ;; U+25B2 BLACK UP-POINTING TRIANGLE
+	 (#x25bc ?=) ;; U+25BC BLACK DOWN-POINTING TRIANGLE
+	 (#x2192 ?=) ;; U+2192 RIGHTWARDS ARROW
+	 (#x2190 ?=) ;; U+2190 LEFTWARDS ARROW
+	 (#x2191 ?=) ;; U+2191 UPWARDS ARROW
+	 (#x2193 ?=) ;; U+2193 DOWNWARDS ARROW
+	 (#x2229 ?=) ;; U+2229 INTERSECTION
+	 (#x2202 ?=) ;; U+2202 PARTIAL DIFFERENTIAL
+	 (#x2261 ?=) ;; U+2261 IDENTICAL TO
+	 (#x221a ?=) ;; U+221A SQUARE ROOT
+	 (#x222b ?=) ;; U+222B INTEGRAL
+	 (#x2030 ?=) ;; U+2030 PER MILLE SIGN
+	 (#x266a ?=) ;; U+266A EIGHTH NOTE
+	 (#x2020 ?*) ;; U+2020 DAGGER
+	 (#x2021 ?*) ;; U+2021 DOUBLE DAGGER
+	 (#x2500 ?|) ;; U+2500 BOX DRAWINGS LIGHT HORIZONTAL
+	 (#x2502 ?|) ;; U+2502 BOX DRAWINGS LIGHT VERTICAL
+	 (#x250c ?|) ;; U+250C BOX DRAWINGS LIGHT DOWN AND RIGHT
+	 (#x2510 ?|) ;; U+2510 BOX DRAWINGS LIGHT DOWN AND LEFT
+	 (#x2518 ?|) ;; U+2518 BOX DRAWINGS LIGHT UP AND LEFT
+	 (#x2514 ?|) ;; U+2514 BOX DRAWINGS LIGHT UP AND RIGHT
+	 (#x251c ?|) ;; U+251C BOX DRAWINGS LIGHT VERTICAL AND RIGHT
+	 (#x252c ?|) ;; U+252C BOX DRAWINGS LIGHT DOWN AND HORIZONTAL
+	 (#x2524 ?|) ;; U+2524 BOX DRAWINGS LIGHT VERTICAL AND LEFT
+	 (#x2534 ?|) ;; U+2534 BOX DRAWINGS LIGHT UP AND HORIZONTAL
+	 (#x253c ?|) ;; U+253C BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL
+	 (#x02da ?^) ;; U+02DA RING ABOVE
+	 (#x2122 ?\xa9) ;; U+2122 TRADE MARK SIGN, ?©
+
+	 (#x0132 ?\xe6) ;; U+0132 LATIN CAPITAL LIGATURE IJ, ?æ
+	 (#x013f ?\xe6) ;; U+013F LATIN CAPITAL LETTER L WITH MIDDLE DOT, ?æ
+
+	 (#x0133 ?\xe6) ;; U+0133 LATIN SMALL LIGATURE IJ, ?æ
+	 (#x0140 ?\xe6) ;; U+0140 LATIN SMALL LETTER L WITH MIDDLE DOT, ?æ
+	 (#x0149 ?\xe6) ;; U+0149 LATIN SMALL LETTER N PRECEDED BY APOSTROPH,?æ
+
+	 (#x2194 ?|) ;; U+2194 LEFT RIGHT ARROW
+	 (#x2660 ?*) ;; U+2660 BLACK SPADE SUIT
+	 (#x2665 ?*) ;; U+2665 BLACK HEART SUIT
+	 (#x2663 ?*) ;; U+2663 BLACK CLUB SUIT
+	 (#x2592 ?|) ;; U+2592 MEDIUM SHADE
+	 (#x2195 ?|) ;; U+2195 UP DOWN ARROW
+
+	 (#x2113 ?\xb9) ;; U+2113 SCRIPT SMALL L, ?¹
+	 (#x215b ?\xbe) ;; U+215B VULGAR FRACTION ONE EIGHTH, ?¾
+	 (#x215c ?\xbe) ;; U+215C VULGAR FRACTION THREE EIGHTHS, ?¾
+	 (#x215d ?\xbe) ;; U+215D VULGAR FRACTION FIVE EIGHTHS, ?¾
+	 (#x215e ?\xbe) ;; U+215E VULGAR FRACTION SEVEN EIGHTHS, ?¾
+	 (#x207f ?\xbe) ;; U+207F SUPERSCRIPT LATIN SMALL LETTER N, ?¾
+  
+	 ;; These are not in WGL 4, but are IPA characters that should not be
+	 ;; double width. They are the only IPA characters that both occur in
+	 ;; packages/mule-packages/leim/ipa.el and end up in East Asian
+	 ;; character sets when that file is loaded in an XEmacs without
+	 ;; packages.
+	 (#x2197 ?|) ;; U+2197 NORTH EAST ARROW
+	 (#x2199 ?|) ;; U+2199 SOUTH WEST ARROW
+	 (#x2191 ?|) ;; U+2191 UPWARDS ARROW
+	 (#x207f ?\xb9)) ;; U+207F SUPERSCRIPT LATIN SMALL LETTER N, ?¹
+    with decoded = nil
+    with codepoint = #x80
+    with syntax-table = (standard-syntax-table)
+    do
+    ;; Under old-Mule, the call to `decode-char' creates jit-ucs-charset-0
+    ;; entries because it is run at dump time, before we create any charsets
+    ;; containing Asian characters, or more specifically before we load the
+    ;; Unicode mapping tables for the Asian-character charsets, which currently
+    ;; happens at charset creation time. (That happens in mule-charset.el.) It
+    ;; doesn't matter whether we move the call to (set-unicode-conversion code
+    ;; 'windows-glyph-list-4 ...) before the call to `decode-char', because
+    ;; windows-glyph-list-4, being size 128, is a non-encodable charset.  By
+    ;; putting jit-ucs-charset-0 above the Asian charsets, we ensure that the
+    ;; properties of jit-ucs-charset-0 determine how the characters are
+    ;; represented.
+    ;;
+    ;; Under Unicode-internal, we get the same behavior by putting
+    ;; windows-glyph-list-4 before the Asian charsets.
+    ;;
+    (set-unicode-conversion ucs 'windows-glyph-list-4 codepoint)
+    (setq decoded (decode-char 'ucs ucs))
+    (assert (memq (declare-fboundp (char-charset decoded))
+		  '(jit-ucs-charset-0 windows-glyph-list-4))
+	    nil "Unexpected Unicode decoding behavior, actual charset=%s.  "
+	    (char-charset decoded))
+    (incf codepoint)
+    (modify-syntax-entry decoded (string (char-syntax ascii-or-latin-1))
+			 syntax-table)))
 
 ;; *Sigh*, declarations needs to be at the start of the line to be picked up
 ;; by make-docfile.
@@ -381,7 +372,6 @@ Standard encoding for representing UTF-8 under MS Windows."
   (loop 
     with char-table = (make-char-table 'generic)
     for i from ?\x00 to ?\xFF
-    initially (unless (featurep 'mule) (return))
     do
     (put-char-table (aref
                      ;; #xd800 is the first leading surrogate;
@@ -400,11 +390,10 @@ To transform XEmacs Unicode error sequences to the Latin-1 characters that
 correspond to the octets on disk, you can use this variable.  ")
 
 (defvar unicode-invalid-sequence-regexp-range
-  (and (featurep 'mule)
-       (format "%c%c-%c"
-               (aref (decode-coding-string "\xd8\x00\x00\x00" 'utf-16-be) 0)
-               (aref (decode-coding-string "\xd8\x00\x00\x00" 'utf-16-be) 3)
-               (aref (decode-coding-string "\xd8\x00\x00\xFF" 'utf-16-be) 3)))
+  (format "%c%c-%c"
+	  (aref (decode-coding-string "\xd8\x00\x00\x00" 'utf-16-be) 0)
+	  (aref (decode-coding-string "\xd8\x00\x00\x00" 'utf-16-be) 3)
+	  (aref (decode-coding-string "\xd8\x00\x00\xFF" 'utf-16-be) 3))
   "Regular expression range to match Unicode error sequences in XEmacs.
 
 Invalid Unicode sequences on input are represented as XEmacs
@@ -414,34 +403,6 @@ invalid octet.  You can use this variable (with `re-search-forward' or
 `skip-chars-forward') to search for such characters; see also
 `unicode-error-translate-region'.  ")
 
-;; Check that the lookup table is correct, and that all the actual error
-;; sequences are caught by the regexp.
-(with-temp-buffer
-  (loop
-    for i from ?\x00 to ?\xFF
-    with to-check = (make-string 20 ?\x20) 
-    initially (unless (featurep 'mule) (return))
-    do 
-    (delete-region (point-min) (point-max))
-    (insert to-check)
-    (goto-char 10)
-    (insert (decode-coding-string (format "\xd8\x00\x00%c" i)
-                                  'utf-16-be))
-    (backward-char)
-    (assert (= i (get-char-table (char-after (point)) 
-                                 unicode-error-default-translation-table))
-            (format "Char ?\\x%x not the expected error sequence!"
-                    i))
-
-    (goto-char (point-min))
-    ;; Comment out until the issue in
-    ;; 18179.49815.622843.336527@parhasard.net is fixed.
-    (assert t ; (re-search-forward (concat "[" 
-              ;                        unicode-invalid-sequence-regexp-range
-              ;                        "]"))
-            nil
-            (format "Could not find char ?\\x%x in buffer" i))))
-
 (defun frob-unicode-errors-region (frob-function begin end &optional buffer)
   "Call FROB-FUNCTION on the Unicode error sequences between BEGIN and END.
 
@@ -487,18 +448,6 @@ mapping from the error sequences to the desired characters.  "
 ;; Sure would be nice to be able to use defface here. 
 (copy-face 'highlight 'unicode-invalid-sequence-warning-face)
 
-(unless (featurep 'mule)
-  ;; We do this in such a roundabout way--instead of having the above defun
-  ;; and defvar calls inside a (when (featurep 'mule) ...) form--to have
-  ;; make-docfile.c pick up symbol and function documentation correctly. An
-  ;; alternative approach would be to fix make-docfile.c to be able to read
-  ;; Lisp.
-  (mapc #'unintern
-        '(unicode-error-default-translation-table
-          unicode-invalid-regexp-range frob-unicode-errors-region
-          unicode-error-translate-region unicode-query-coding-region
-          unicode-query-coding-skip-chars-arg)))
-
 (make-obsolete 'char-octet 'char-to-charset-codepoint)
 
 ;; Ben made this obsolete with Unicode-internal, but it's sufficiently widely
