@@ -58,7 +58,7 @@ static widget_info *all_widget_info = NULL;
 int lw_menu_active = 0;
 
 /* X11 menubar widget */
-Widget lw_menubar_widget = NULL;
+Widget lw_menubar_widget;
 
 /* whether the last menu operation was a keyboard accelerator */
 int lw_menu_accelerate = False;
@@ -1249,7 +1249,7 @@ lw_get_all_values (LWLIB_ID id)
 widget_value*
 lw_get_widget_value_for_widget (widget_instance *instance, Widget w)
 {
-  char *name = XtName (w);
+  String name = XtName (w);
   widget_value *cur;
   for (cur = instance->info->val; cur; cur = cur->next)
     if (!strcmp (cur->name, name))
@@ -1270,7 +1270,7 @@ lw_internal_update_other_instances (Widget widget, XtPointer closure,
   static Boolean updating;
 
   widget_instance *instance = (widget_instance*)closure;
-  char *name = XtName (widget);
+  String name = XtName (widget);
   widget_info *info;
   widget_instance *cur;
   widget_value *val;
