@@ -21,8 +21,13 @@ along with the Lucid Widget Library.  If not, see
 #ifndef INCLUDED_lwlib_h_
 #define INCLUDED_lwlib_h_
 
+#include <compiler.h>
+
+#ifndef _CONST_X_STRING
+#define _CONST_X_STRING
+#endif
+
 #include <X11/Intrinsic.h>
-#include "xt-wrappers.h"
 
 #if defined (LWLIB_MENUBARS_LUCID) || defined (LWLIB_MENUBARS_MOTIF) || defined (LWLIB_MENUBARS_ATHENA)
 #define NEED_MENUBARS
@@ -61,6 +66,8 @@ along with the Lucid Widget Library.  If not, see
 ** by the new xlwmenu code.
 */
 
+BEGIN_C_DECLS
+
 typedef unsigned long LWLIB_ID;
 
 typedef enum _change_type
@@ -254,4 +261,9 @@ void lw_show_busy (Widget w, Boolean busy);
 
 void lw_remove_accelerator_spec (char *val);
 
+#define MINL(x,y) ((((unsigned long) (x)) < ((unsigned long) (y))) \
+		   ? ((unsigned long) (x)) : ((unsigned long) (y)))
+
+END_C_DECLS
+
 #endif /* INCLUDED_lwlib_h_ */
