@@ -3,25 +3,24 @@
 
 This file is part of the Lucid Widget Library.
 
-The Lucid Widget Library is free software; you can redistribute it and/or 
-modify it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+The Lucid Widget Library is free software: you can redistribute it
+and/or modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
 
-The Lucid Widget Library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of 
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+The Lucid Widget Library is distributed in the hope that it will be
+useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with the Lucid Widget Library.  If not, see
+<http://www.gnu.org/licenses/>. */
 
 /* Created by Douglas Keller <dkeller@vnet.ibm.com> */
 
-#ifndef _XlwScrollbar_h
-#define _XlwScrollbar_h
+#ifndef INCLUDED_xlwscrollbar_h_
+#define INCLUDED_xlwscrollbar_h_
 
 #include <X11/Core.h>
 
@@ -88,9 +87,14 @@ Boston, MA 02111-1307, USA.  */
 
 /* New resources that Motif does not have.
    Maybe we should use a different prefix. */
+
+/* "knob" is obsolete; use "slider" instead. */
 #define XmNknobStyle		"knobStyle"
 #define XmCKnobStyle		"KnobStyle"
 
+#define XmNsliderStyle		"sliderStyle"
+#define XmCSliderStyle		"SliderStyle"
+
 #define XmNarrowPosition	"arrowPosition"
 #define XmCArrowPosition	"ArrowPosition"
 
@@ -114,7 +118,7 @@ enum {
     XmHORIZONTAL
 };
 
-#endif /* _Xm_h */
+#endif /* ! _Xm_h */
 
 extern WidgetClass xlwScrollBarWidgetClass;
 
@@ -135,4 +139,4 @@ void XlwScrollBarGetValues(Widget widget, int *value, int *sliderSize,
 void XlwScrollBarSetValues(Widget widget, int value, int sliderSize,
 			   int increment, int pageIncrement, Boolean notify);
 
-#endif
+#endif /* INCLUDED_xlwscrollbar_h_ */
