@@ -144,7 +144,7 @@ static	XtAccelerators	defaultAccelerators ; /* #### Never used */
 static XtResource resources[] = {
 #define	offset(field)	XtOffsetOf(TabsRec, tabs.field)
 #define res(name,_class,intrepr,type,member,extrepr,value) \
-  Xt_RESOURCE (name, _class, intrepr, type, offset(member), extrepr, value)
+  { name, _class, intrepr, sizeof (type), offset(member), extrepr, (XtPointer) value }
 
   res (XtNselectInsensitive, XtCSelectInsensitive, XtRBoolean, Boolean, 
        selectInsensitive,  XtRImmediate, True),
@@ -164,8 +164,9 @@ static XtResource resources[] = {
        internalWidth,  XtRImmediate, 4),
   res (XtNinternalHeight, XtCHeight, XtRDimension, Dimension, 
        internalHeight,  XtRImmediate, 4),
-  Xt_RESOURCE (XtNborderWidth, XtCBorderWidth, XtRDimension, Dimension, 
-	       XtOffsetOf(RectObjRec,rectangle.border_width), XtRImmediate, 0),
+  { XtNborderWidth, XtCBorderWidth, XtRDimension, sizeof (Dimension), 
+    XtOffsetOf(RectObjRec,rectangle.border_width), XtRImmediate,
+    (XtPointer) 0 },
   res (XtNtopWidget, XtCTopWidget, XtRWidget, Widget, 
        topWidget,  XtRImmediate, NULL),
   res (XtNcallback, XtCCallback, XtRCallback, XtPointer, 
@@ -180,9 +181,9 @@ static XtResource resources[] = {
        bot_shadow_contrast,  XtRImmediate, 40),
   res (XtNinsensitiveContrast, XtCInsensitiveContrast, XtRInt, int, 
        insensitive_contrast,  XtRImmediate, 33),
-  Xt_RESOURCE (XtNaccelerators, XtCAccelerators, XtRAcceleratorTable,
-	       XtTranslations, XtOffsetOf(TabsRec,core.accelerators),
-	       XtRString, accelTable),
+  { XtNaccelerators, XtCAccelerators, XtRAcceleratorTable,
+    sizeof (XtTranslations), XtOffsetOf(TabsRec, core.accelerators), 
+    XtRString, (XtPointer) accelTable },
 #undef	offset
 #undef  res
 };
@@ -193,7 +194,7 @@ static XtResource resources[] = {
 static XtResource tabsConstraintResources[] = {
 #define	offset(field)	XtOffsetOf(TabsConstraintsRec, tabs.field)
 #define res(name,_class,intrepr,type,member,extrepr,value) \
-  Xt_RESOURCE (name, _class, intrepr, type, offset(member), extrepr, value)
+  { name, _class, intrepr, sizeof (type), offset(member), extrepr, (XtPointer) value }
   res (XtNtabLabel, XtCLabel, XtRString, String, label,  XtRString, NULL),
   res (XtNtabLeftBitmap, XtCLeftBitmap, XtRBitmap, Pixmap, left_bitmap,
        XtRImmediate, None),
@@ -428,24 +429,6 @@ WidgetClass tabsWidgetClass = (WidgetClass)&tabsClassRec;
 
 static int debug_tabs = 0;	/* increase for more verbosity */
 
-#ifdef HAVE_XFT_TABS
-/* #### duplicated from xlwmenu.c -- CLEAN THIS SHIT UP!
-   Undeclared so define at top. */
-#define MINL(x,y) ((((unsigned long) (x)) < ((unsigned long) (y))) \
-		   ? ((unsigned long) (x)) : ((unsigned long) (y)))
-
-static int
-x_xft_text_width (Display *dpy, XftFont *xft_font, FcChar8 *run, int len)
-{
-  static XGlyphInfo glyphinfo;	/* #### static? */
-
-  XftTextExtents8 (dpy,
-		   xft_font,
-		   run, len, &glyphinfo);
-  return glyphinfo.xOff;
-}
-#endif
-
 /****************************************************************
  *
  * Member Procedures
@@ -750,8 +733,8 @@ TabsSetValues(Widget current, Widget UNUSED (request), Widget new_,
 
 	    XRaiseWindow(XtDisplay(w), XtWindow(w)) ;
 #ifdef	NEED_MOTIF
-	    XtVaSetValues(curtw->tabs.topWidget, XmNtraversalOn, False, 0) ;
-	    XtVaSetValues(w, XmNtraversalOn, True, 0) ;
+	    XtVaSetValues(curtw->tabs.topWidget, XmNtraversalOn, False, NULL) ;
+	    XtVaSetValues(w, XmNtraversalOn, True, NULL) ;
 #endif
 
 	    if( tab->tabs.row != (int) tw->tabs.numRows-1 )
@@ -1219,11 +1202,11 @@ TabsChangeManaged(Widget w)
         --i >= 0;
 	++childP)
     {
-      XtVaSetValues(*childP, XmNtraversalOn, False, 0) ;
+      XtVaSetValues(*childP, XmNtraversalOn, False, NULL) ;
     }
 
     if( tw->tabs.topWidget != NULL )
-      XtVaSetValues(tw->tabs.topWidget, XmNtraversalOn, True, 0) ;
+      XtVaSetValues(tw->tabs.topWidget, XmNtraversalOn, True, NULL) ;
 #endif
 }
 
@@ -1474,8 +1457,8 @@ XawTabsSetTop(Widget w, Bool callCallbacks)
 
 	XRaiseWindow(XtDisplay(w), XtWindow(w)) ;
 #ifdef	NEED_MOTIF
-	XtVaSetValues(oldtop, XmNtraversalOn, False, 0) ;
-	XtVaSetValues(w, XmNtraversalOn, True, 0) ;
+	XtVaSetValues(oldtop, XmNtraversalOn, False, NULL) ;
+	XtVaSetValues(w, XmNtraversalOn, True, NULL) ;
 #endif
 
 	tab = (TabsConstraints) w->core.constraints ;
@@ -1718,8 +1701,6 @@ DrawTab(TabsWidget tw, Widget child, Bool labels)
 	      XftDraw *xftDraw = XftDrawCreate (dpy, win, visual, cmap);
 	      XftFont *renderFont = tw->tabs.renderFont;
 	      XGlyphInfo glyphinfo;
-	      XftColor colorDBG;
-	      XftColorAllocName (dpy, visual, cmap, "wheat", &colorDBG);
 	      XftTextExtents8 (dpy, renderFont, (FcChar8 *) lbl,
 			       (int) strlen (lbl), &glyphinfo);
 	      /* #### unnecessary? for the moment, give visual extent */
@@ -1728,16 +1709,16 @@ DrawTab(TabsWidget tw, Widget child, Bool labels)
 		{
 		  fprintf (stderr, "background color:  pixel=%08lx, r=%04x,"
 			   " g=%04x, b=%04x, alpha=%04x.\n",
-			   colorDBG.pixel, colorDBG.color.red,
-			   colorDBG.color.green, colorDBG.color.blue,
-			   colorDBG.color.alpha);
+			   colorBG.pixel, colorBG.color.red,
+			   colorBG.color.green, colorBG.color.blue,
+			   colorBG.color.alpha);
  		  fprintf (stderr, "label geometry: x=%d, y=%d, xOff=%d,"
  			   " yOff=%d, width=%d, height=%d\n",
  			   glyphinfo.x, glyphinfo.y, glyphinfo.xOff,
  			   glyphinfo.yOff, glyphinfo.width, glyphinfo.height);
  		}
 	      if (debug_tabs > 2)
-		XftDrawRect (xftDraw, &colorDBG,
+		XftDrawRect (xftDraw, &colorBG,
 			     /* left, top, width, height */
 			     x+tab->tabs.l_x-glyphinfo.x,
 			     y+tab->tabs.l_y-glyphinfo.y,
@@ -2003,8 +1984,7 @@ TabWidth(Widget w)
 	{
 #ifdef HAVE_XFT_TABS
 	  tab->tabs.width += x_xft_text_width (XtDisplay(tw), font,
-					       (FcChar8 *) lbl,
-					       (int)strlen(lbl)) + iw;
+					       lbl, (int) strlen (lbl)) + iw;
 	  tab->tabs.l_y = (tw->tabs.tab_height
 			   + tw->tabs.renderFont->ascent
 			   /* #### how can this subtraction be correct? */
