@@ -173,9 +173,9 @@ extern int (*jrBeepFunc) (void);
 #define KEYTOSTRSIZE 2048
 static Extbyte key_buffer[KEYTOSTRSIZE];
 
-static int canna_empty_info, canna_through_info;
-static int canna_underline;
-static int canna_inhibit_hankakukana;
+static Boolint canna_empty_info, canna_through_info;
+static Boolint canna_underline;
+static Boolint canna_inhibit_hankakukana;
 
 static Lisp_Object Vcanna_kakutei_string;
 static Lisp_Object Vcanna_kakutei_yomi;
@@ -349,6 +349,14 @@ call_Fding (void)
   return 0;
 }
 
+#ifdef KC_SETAPPNAME
+#ifndef CANNA_MULE
+static char appname[] = "nemacs";
+#else /* CANNA_MULE */
+static char appname[] = "mule";
+#endif /* CANNA_MULE */
+#endif /* KC_SETAPPNAME */
+
 DEFUN ("canna-initialize", Fcanna_initialize, 0, 3, 0, /*
 Initialize ``canna'', which is a kana-to-kanji converter for XEmacs.
 The first arg specifies if inserting space character between BUNSETSU when
@@ -430,11 +438,7 @@ If nil is specified for each arg, the default value will be used.
       jrBeepFunc = call_Fding;
 
 #ifdef KC_SETAPPNAME
-#ifndef CANNA_MULE
-      jrKanjiControl (0, KC_SETAPPNAME, "nemacs");
-#else /* CANNA_MULE */
-      jrKanjiControl (0, KC_SETAPPNAME, "mule");
-#endif /* CANNA_MULE */
+      jrKanjiControl (0, KC_SETAPPNAME, appname);
 #endif /* KC_SETAPPNAME */
 
       jrKanjiControl (0, KC_SETBUNSETSUKUGIRI, kugiri);
