@@ -412,8 +412,11 @@ Dynarr_delete_many (void *d, Elemcount pos, Elemcount len)
   /* Not checking on overflow for BCOUNT because we're reducing the size
      used, we had to have overflowed already. */
 
-  memmove ((Rawbyte *) dy->base + pos*Dynarr_elsize (dy),
-	   (Rawbyte *) dy->base + (pos + len)*Dynarr_elsize (dy), bcount);
+  if (bcount)
+    {
+      memmove ((Rawbyte *) dy->base + pos*Dynarr_elsize (dy),
+               (Rawbyte *) dy->base + (pos + len)*Dynarr_elsize (dy), bcount);
+    }
 
   Dynarr_set_length_1 (dy, Dynarr_length (dy) - len);
 }
@@ -478,14 +481,6 @@ Dynarr_memory_usage (void *d, struct usage_stats *stats)
 
 #endif /* MEMORY_USAGE_STATS */
 
-void
-mark_Lisp_Object_dynarr (Lisp_Object_dynarr *dyn)
-{
-  int i;
-  for (i = 0; i < Dynarr_length (dyn); i++)
-    mark_object (Dynarr_at (dyn, i));
-}
-
 
 /*****************************************************************************/
 /*                              static dynarrs                               */
