@@ -501,10 +501,11 @@ Dynarr_add_many (void *d, const void *base, Elemcount len)
      from that function and to make it inline */
   Dynarr *dy = Dynarr_verify_mod (d);
   Elemcount pos = Dynarr_length (dy);
+  size_t bcount = (size_t) len * (size_t) Dynarr_elsize (dy);
+
   Dynarr_increase_length (dy, Dynarr_length (dy) + len);
-  if (base)
+  if (base && bcount)
     {
-      size_t bcount = (size_t) len * (size_t) Dynarr_elsize (dy);
       memcpy ((Rawbyte *) dy->base + pos*Dynarr_elsize (dy), base, bcount);
     }
 }
@@ -629,7 +630,16 @@ Stynarr_verify_pos (void *st, int pos, const Ascbyte *file, int line)
   Stynarr *sty = (Stynarr *) st;
   /* #### See comment above in Dynarr_verify_pos() about accessing just
      past end of the real used memory block using Stynarr_atp(). */
-  assert_at_line (pos >= 0 && pos < sty->nels, file, line);
+  assert_at_line (pos < sty->nels, file, line);
+#if (GCC_VERSION >= NEED_GCC (15, 0, 0))
+  /* GCC 15 with -O6 warns within
+     faces.c:ensure_face_cachel_contains_charset() that POS may be negative
+     if we do not rephrase the assertion in this fashion. */
+  assert_at_line ((unsigned int) pos < (unsigned int) INT_MAX || pos == 0,
+                  file, line);
+#else
+  assert_at_line (pos >= 0, file, line);
+#endif
   return pos;
 }
 #else
@@ -834,4 +844,6 @@ void free_gap_array (Gap_Array *ga);
 Bytecount gap_array_memory_usage (Gap_Array *ga, struct usage_stats *stats,
 				  Bytecount *marker_ancillary);
 
+END_C_DECLS
+
 #endif /* INCLUDED_array_h_ */
