@@ -2,7 +2,7 @@
    Copyright (C) 1985, 1986, 1992, 1993, 1994, 1995
    Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 2001, 2002, 2004 Ben Wing.
+   Copyright (C) 2001, 2002, 2004, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -31,8 +31,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #ifndef INCLUDED_buffer_h_
 #define INCLUDED_buffer_h_
 
-#include "casetab.h"
-#include "chartab.h"
+#include "charset.h"
 
 /************************************************************************/
 /*                                                                      */
@@ -77,9 +76,6 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 struct buffer_text
 {
-#ifdef NEW_GC
-  NORMAL_LISP_OBJECT_HEADER header;
-#endif /* NEW_GC */
   Ibyte *beg;		/* Actual address of buffer contents. */
   Bytebpos gpt;		/* Index of gap in buffer. */
   Charbpos bufgpt;	/* Equivalent as a Charbpos. */
@@ -87,14 +83,13 @@ struct buffer_text
   Charbpos bufz;	/* Equivalent as a Charbpos. */
   Bytecount gap_size;/* Size of buffer's gap */
   Bytecount end_gap_size;/* Size of buffer's end gap */
-  long modiff;		/* This counts buffer-modification events
+  EMACS_INT modiff;	/* This counts buffer-modification events
 			   for this buffer.  It is incremented for
 			   each such event, and never otherwise
 			   changed.  */
-  long save_modiff;	/* Previous value of modiff, as of last
-			   time buffer visited or saved a file.  */
+  EMACS_INT save_modiff; /* Previous value of modiff, as of last
+			    time buffer visited or saved a file.  */
 
-#ifdef MULE
 
 #ifdef OLD_BYTE_CHAR
   /* We keep track of a "known" region for very fast access.  This
@@ -129,7 +124,6 @@ struct buffer_text
   
   /* Currently we only handle 8 bit fixed and default */
   Internal_Format format;
-#endif /* MULE */
 
   /* Similar to the above, we keep track of positions for which line
      number has last been calculated.  See line-number.c. */
@@ -139,18 +133,6 @@ struct buffer_text
   struct buffer_text_change_data *changes;
 };
 
-#ifdef NEW_GC
-typedef struct buffer_text Lisp_Buffer_Text;
-
-DECLARE_LISP_OBJECT (buffer_text, Lisp_Buffer_Text);
-
-#define XBUFFER_TEXT(x) \
-  XRECORD (x, buffer_text, Lisp_Buffer_Text)
-#define wrap_buffer_text(p) wrap_record (p, buffer_text)
-#define BUFFER_TEXT_P(x) RECORDP (x, buffer_text)
-#define CHECK_BUFFER_TEXT(x) CHECK_RECORD (x, buffer_text)
-#define CONCHECK_BUFFER_TEXT(x) CONCHECK_RECORD (x, buffer_text)
-#endif /* NEW_GC */
 
 
 struct buffer
@@ -173,8 +155,8 @@ struct buffer
   Bytebpos zv;		/* Index of end of accessible range. */
   Charbpos bufzv;		/* Equivalent as a Charbpos. */
 
-  int face_change;	/* This is set when a change in how the text should
-			   be displayed (e.g., font, color) is made. */
+  EMACS_INT face_change; /* This is set when a change in how the text should
+			    be displayed (e.g., font, color) is made. */
 
   /* Whether buffer specific face is specified. */
   int buffer_local_face_property;
@@ -240,24 +222,20 @@ struct buffer
 
   /* Flags saying which DEFVAR_PER_BUFFER variables
      are local to this buffer.  */
-  int local_var_flags;
+  EMACS_INT local_var_flags;
 
   /* Set to the modtime of the visited file when read or written.
      -1 means visited file was nonexistent.
      0  means visited file modtime unknown; in no case complain
      about any mismatch on next save attempt.  */
-  int modtime;
+  time_t modtime;
 
   /* the value of text->modiff at the last auto-save.  */
-  long auto_save_modified;
+  EMACS_INT auto_save_modified;
 
   /* The time at which we detected a failure to auto-save,
      Or -1 if we didn't have a failure.  */
-  int auto_save_failure_time;
-
-  /* Position in buffer at which display started
-     the last time this buffer was displayed.  */
-  int last_window_start;
+  time_t auto_save_failure_time;
 
   /* Everything from here down must be a Lisp_Object */
 
@@ -346,12 +324,8 @@ for (mps_bufcons = Qunbound,					\
 #define BYTE_BUF_PT(buf) ((buf)->pt + 0)
 #define BUF_PT(buf) ((buf)->bufpt + 0)
 
-/* Internal format of buffer.  */
-#ifdef MULE
-#define BUF_FORMAT(buf) ((buf)->text->format)
-#else
-#define BUF_FORMAT(buf) FORMAT_DEFAULT
-#endif
+/* Internal format of buffer. Always FORMAT_DEFAULT currently. */
+#define BUF_FORMAT(buf) FORMAT_DEFAULT /* ((buf)->text->format) */
 
 /*----------------------------------------------------------------------*/
 /*		         Validating byte positions         		*/
@@ -370,7 +344,6 @@ BYTE_BUF_BYTE_ADDRESS_NO_VERIFY (struct buffer *buf, Bytebpos pos)
 
 /* Given a byte position, does it point to the beginning of a character?
 */
-#ifdef MULE
 DECLARE_INLINE_HEADER (
 int
 VALID_BYTEBPOS_P (struct buffer *buf, Bytebpos x)
@@ -388,9 +361,6 @@ VALID_BYTEBPOS_P (struct buffer *buf, Bytebpos x)
       return 1;
     }
 }
-#else
-# define VALID_BYTEBPOS_P(buf, x) 1
-#endif
 
 /* If error-checking is enabled, assert that the given char position is
    within range.  Otherwise, do nothing.
@@ -432,7 +402,6 @@ VALID_BYTEBPOS_P (struct buffer *buf, Bytebpos x)
   text_checking_assert (VALID_BYTEBPOS_P (buf, x));			    \
 } while (0)
 
-#ifdef MULE
 /* Make sure that the given byte position is pointing to the beginning of a
    character.  If not, back up until this is the case.  Note that there are
    not too many places where it is legitimate to do this sort of thing.
@@ -444,7 +413,7 @@ VALID_BYTEBPOS_P (struct buffer *buf, Bytebpos x)
     case FORMAT_DEFAULT:						\
       {									\
 	Ibyte *VBB_ptr = BYTE_BUF_BYTE_ADDRESS_NO_VERIFY (buf, x);	\
-	while (!ibyte_first_byte_p (*VBB_ptr))			\
+	while (!ibyte_first_byte_p (*VBB_ptr))				\
 	  VBB_ptr--, (x)--;						\
       }									\
       break;								\
@@ -460,11 +429,7 @@ VALID_BYTEBPOS_P (struct buffer *buf, Bytebpos x)
       break;								\
     }									\
 } while (0)
-#else
-# define VALIDATE_BYTEBPOS_BACKWARD(buf, x)
-#endif
 
-#ifdef MULE
 /* Make sure that the given byte position is pointing to the beginning of a
    character.  If not, move forward until this is the case.  Note that
    there are not too many places where it is legitimate to do this sort of
@@ -476,7 +441,7 @@ VALID_BYTEBPOS_P (struct buffer *buf, Bytebpos x)
     case FORMAT_DEFAULT:						\
       {									\
 	Ibyte *VBF_ptr = BYTE_BUF_BYTE_ADDRESS_NO_VERIFY (buf, x);	\
-	while (!ibyte_first_byte_p (*VBF_ptr))			\
+	while (!ibyte_first_byte_p (*VBF_ptr))				\
 	  VBF_ptr++, (x)++;						\
       }									\
       break;								\
@@ -492,9 +457,6 @@ VALID_BYTEBPOS_P (struct buffer *buf, Bytebpos x)
       break;								\
     }									\
 } while (0)
-#else
-# define VALIDATE_BYTEBPOS_FORWARD(buf, x)
-#endif
 
 /*----------------------------------------------------------------------*/
 /*			 Working with byte positions			*/
@@ -503,10 +465,7 @@ VALID_BYTEBPOS_P (struct buffer *buf, Bytebpos x)
 
 /*  Given a byte position (assumed to point at the beginning of a
     character), modify that value so it points to the beginning of the next
-    character.
-    
-    Note that in the simplest case (no MULE, no ERROR_CHECK_TEXT),
-    this crap reduces down to simply (x)++. */
+    character. */
 
 #define INC_BYTEBPOS(buf, x) do				\
 {							\
@@ -523,10 +482,7 @@ VALID_BYTEBPOS_P (struct buffer *buf, Bytebpos x)
     character), modify that value so it points to the beginning of the
     previous character.  Unlike for DEC_IBYTEPTR(), we can do all the
     assert()s because there are sentinels at the beginning of the gap and
-    the end of the buffer.
-
-    Note that in the simplest case (no MULE, no ERROR_CHECK_TEXT), this
-    crap reduces down to simply (x)--. */
+    the end of the buffer. */
 
 #define DEC_BYTEBPOS(buf, x) do				\
 {							\
@@ -573,13 +529,11 @@ Info on Byte-Char conversion:
   (Info-goto-node "(internals)Byte-Char Position Conversion")
 */
 
-#ifdef MULE
 
 Bytebpos charbpos_to_bytebpos_func (struct buffer *buf, Charbpos x);
 Charbpos bytebpos_to_charbpos_func (struct buffer *buf, Bytebpos x);
 extern short three_to_one_table[];
 
-#endif /* MULE */
 
 /* Given a Charbpos, return the equivalent Bytebpos. */
 
@@ -590,7 +544,6 @@ charbpos_to_bytebpos (struct buffer *buf, Charbpos x)
 {
   Bytebpos retval;
   ASSERT_VALID_CHARBPOS_UNSAFE (buf, x);
-#ifdef MULE
   if (buf->text->entirely_one_byte_p)
     retval = (Bytebpos) x;
   else if (BUF_FORMAT (buf) == FORMAT_16_BIT_FIXED)
@@ -606,9 +559,6 @@ charbpos_to_bytebpos (struct buffer *buf, Charbpos x)
 #endif /* OLD_BYTE_CHAR */
   else
     retval = charbpos_to_bytebpos_func (buf, x);
-#else
-  retval = (Bytebpos) x;
-#endif
   ASSERT_VALID_BYTEBPOS_UNSAFE (buf, retval);
   return retval;
 }
@@ -622,7 +572,6 @@ bytebpos_to_charbpos (struct buffer *buf, Bytebpos x)
 {
   Charbpos retval;
   ASSERT_VALID_BYTEBPOS_UNSAFE (buf, x);
-#ifdef MULE
   if (buf->text->entirely_one_byte_p)
     retval = (Charbpos) x;
   else if (BUF_FORMAT (buf) == FORMAT_16_BIT_FIXED)
@@ -638,16 +587,13 @@ bytebpos_to_charbpos (struct buffer *buf, Bytebpos x)
 #endif /* OLD_BYTE_CHAR */
   else
     retval = bytebpos_to_charbpos_func (buf, x);
-#else
-  retval = (Charbpos) x;
-#endif
   ASSERT_VALID_CHARBPOS_UNSAFE (buf, retval);
   return retval;
 }
 
 /* Given a Bytebpos, return the equivalent Charbpos as a Lisp Object. */
 
-#define make_charbpos(buf, ind) make_int (bytebpos_to_charbpos (buf, ind))
+#define make_charbpos(buf, ind) make_fixnum (bytebpos_to_charbpos (buf, ind))
 
 /*----------------------------------------------------------------------*/
 /*	    Converting between byte and memory positions		*/
@@ -704,18 +650,18 @@ charbpos_to_membpos (struct buffer *buf, Charbpos x)
 }
 
 /*----------------------------------------------------------------------*/
-/*             Generalized buffer/string position convertors            */
+/*       Generalized buffer/other object position convertors            */
 /*----------------------------------------------------------------------*/
 
 /* These macros generalize many standard buffer-position functions to
-   either a buffer or a string. */
+   either a buffer or another object that can have an associated extent. */
 
-/* Converting between Memxpos's and Bytexpos's, for a buffer-or-string.
-   For strings, this is a no-op.  For buffers, this resolves
-   to the standard membpos<->bytebpos converters. */
+/* Converting between Memxpos's and Bytexpos's, for some object that has
+   extents.  For buffers, this resolves to the standard membpos<->bytebpos
+   converters.  For other objects, this is a no-op.  */
 
 DECLARE_INLINE_HEADER (
-Memxpos buffer_or_string_bytexpos_to_memxpos (Lisp_Object obj, Bytexpos pos)
+Memxpos object_bytexpos_to_memxpos (Lisp_Object obj, Bytexpos pos)
 )
 {
   return (BUFFERP (obj) ? bytebpos_to_membpos (XBUFFER (obj), pos) :
@@ -723,7 +669,7 @@ Memxpos buffer_or_string_bytexpos_to_memxpos (Lisp_Object obj, Bytexpos pos)
 }
 
 DECLARE_INLINE_HEADER (
-Bytexpos buffer_or_string_memxpos_to_bytexpos (Lisp_Object obj, Memxpos pos)
+Bytexpos object_memxpos_to_bytexpos (Lisp_Object obj, Memxpos pos)
 )
 {
   return (BUFFERP (obj) ? membpos_to_bytebpos (XBUFFER (obj), pos) :
@@ -763,6 +709,14 @@ DECLARE_INLINE_HEADER (
 Charxpos buffer_or_string_memxpos_to_charxpos (Lisp_Object obj, Memxpos pos)
 )
 {	
+  /* This is intentionally unimplemented for lstreams, nothing that is
+     interested in char positions (in particular, no Lisp code) should
+     be looking at the contents of a resizing_buffer_lstream without
+     calling #'get-string-output-stream.
+
+     When ERROR_CHECK_TYPES is turned on, treating OBJECT as a string inside
+     string_index_byte_to_char() will trip an assertion failure, so we'll get
+     appropriate warning that this isn't working. */
   return (BUFFERP (obj) ? membpos_to_charbpos (XBUFFER (obj), pos) :
    (Charxpos) string_index_byte_to_char (obj, pos));
 }
@@ -852,7 +806,7 @@ BYTE_BUF_BYTE_ADDRESS_BEFORE (struct buffer *buf, Bytebpos pos)
 
 /* The character at position POS in buffer. */
 
-#define BYTE_BUF_FETCH_CHAR(buf, pos)					   \
+#define BYTE_BUF_FETCH_CHAR(buf, pos)					\
    itext_ichar_fmt (BYTE_BUF_BYTE_ADDRESS (buf, pos), BUF_FORMAT (buf), \
 		       wrap_buffer (buf))
 #define BUF_FETCH_CHAR(buf, pos) \
@@ -877,6 +831,181 @@ BYTE_BUF_BYTE_ADDRESS_BEFORE (struct buffer *buf, Bytebpos pos)
 #define BUF_ITEXT_COPY_ICHAR(buf, pos, str) \
   BYTE_BUF_ITEXT_COPY_ICHAR (buf, charbpos_to_bytebpos (buf, pos), str)
 
+
+/*--------------------------------------------------------------------------*/
+/* Converting between characters, charset codepoints and Unicode codepoints */
+/*--------------------------------------------------------------------------*/
+
+DECLARE_INLINE_HEADER (
+Ichar
+buffer_filtered_unicode_to_ichar (int code, struct buffer *buf,
+				  charset_pred predicate,
+				  enum converr fail)
+)
+{
+  ASSERT_VALID_UNICODE_CODEPOINT (code);
+  text_checking_assert (buf);
+
+#ifndef UNICODE_INTERNAL
+  {
+    Ichar ch;
+    ch = filtered_unicode_to_ichar (code, buf->unicode_precedence_array,
+				    predicate, CONVERR_FAIL);
+    if (ch < 0)
+      ch = filtered_unicode_to_ichar (code, Vdefault_unicode_precedence_array,
+				      predicate, fail);
+    return ch;
+  }
+#else
+  return filtered_unicode_to_ichar (code, Qnil, predicate, fail);
+#endif /* !defined (UNICODE_INTERNAL) */
+}
+
+DECLARE_INLINE_HEADER (
+Ichar
+buffer_unicode_to_ichar (int code, struct buffer *buf, enum converr fail)
+)
+{
+  return buffer_filtered_unicode_to_ichar (code, buf, NULL, fail);
+}
+
+DECLARE_INLINE_HEADER (
+void
+buffer_filtered_unicode_to_charset_codepoint (int code, struct buffer *buf,
+					      charset_pred predicate,
+					      Lisp_Object *charset,
+					      int *c1, int *c2,
+					      enum converr fail)
+)
+{
+  text_checking_assert (buf);
+  filtered_unicode_to_charset_codepoint (code, buf->unicode_precedence_array,
+					 predicate, charset, c1, c2,
+					 CONVERR_FAIL);
+  if (NILP (*charset))
+    filtered_unicode_to_charset_codepoint (code,
+					   Vdefault_unicode_precedence_array,
+					   predicate, charset, c1, c2, fail);
+}
+
+DECLARE_INLINE_HEADER (
+void
+buffer_unicode_to_charset_codepoint (int code, struct buffer *buf,
+				     Lisp_Object *charset, int *c1, int *c2,
+				     enum converr fail)
+)
+{
+  buffer_filtered_unicode_to_charset_codepoint (code, buf, NULL, charset,
+						c1, c2, fail);
+}
+
+DECLARE_INLINE_HEADER (
+void
+buffer_filtered_ichar_to_charset_codepoint (Ichar ch, struct buffer *buf,
+					    charset_pred predicate,
+					    Lisp_Object *charset,
+					    int *c1, int *c2,
+					    enum converr fail)
+)
+{
+  text_checking_assert (buf);
+#ifdef UNICODE_INTERNAL
+  filtered_ichar_to_charset_codepoint (ch, buf->unicode_precedence_array,
+				       predicate, charset, c1, c2,
+				       CONVERR_FAIL);
+  if (NILP (*charset))
+    filtered_ichar_to_charset_codepoint (ch, Vdefault_unicode_precedence_array,
+					 predicate, charset, c1, c2, fail);
+#else
+  filtered_ichar_to_charset_codepoint (ch, Qnil, predicate, charset, c1, c2,
+				       fail);
+#endif /* (not) UNICODE_INTERNAL */
+}
+
+DECLARE_INLINE_HEADER (
+void
+buffer_ichar_to_charset_codepoint (Ichar ch, struct buffer *buf,
+				   Lisp_Object *charset, int *c1, int *c2,
+				   enum converr fail)
+)
+{
+  buffer_filtered_ichar_to_charset_codepoint (ch, buf, NULL, charset,
+					      c1, c2, fail);
+}
+
+DECLARE_INLINE_HEADER (
+void
+buffer_filtered_itext_to_charset_codepoint (const Ibyte *ptr,
+					    struct buffer *buf,
+					    charset_pred predicate,
+					    Lisp_Object *charset,
+					    int *c1, int *c2,
+					    enum converr fail)
+)
+{
+  text_checking_assert (buf);
+#ifdef UNICODE_INTERNAL
+  filtered_itext_to_charset_codepoint (ptr, buf->unicode_precedence_array,
+				       predicate, charset, c1, c2,
+				       CONVERR_FAIL);
+  if (NILP (*charset))
+    filtered_itext_to_charset_codepoint (ptr,
+					 Vdefault_unicode_precedence_array,
+					 predicate, charset, c1, c2, fail);
+#else
+  filtered_itext_to_charset_codepoint (ptr, Qnil, predicate, charset, c1, c2,
+				       fail);
+#endif /* (not) UNICODE_INTERNAL */
+}
+
+DECLARE_INLINE_HEADER (
+void
+buffer_itext_to_charset_codepoint (const Ibyte *ptr, struct buffer *buf,
+				     Lisp_Object *charset, int *c1, int *c2,
+				     enum converr fail)
+)
+{
+  buffer_filtered_itext_to_charset_codepoint (ptr, buf, NULL, charset,
+					      c1, c2, fail);
+}
+
+/* @@####
+   Get rid of this crap now!!!!!!!!!!!!!!
+
+   This will simply not fly in a Unicode world, where there may not be any
+   national charset for a particular character.  Almost everywhere that this
+   is used, it's used for font handling.  We need to replace device methods
+   like find_charset_font() and font_spec_matches_charset() with similar
+   methods that operate on a character, not a charset.  We might still need
+   to do some charset lookup if we want to implement the idea that we use
+   the appropriate Chinese, Japanese or Korean specific font depending
+   on the language that a particular character is tagged as (as determined
+   by the string extent surrounding the character in a buffer, or a
+   buffer-local value indicating the language) -- but we absolutely do not
+   want to be *dependent* on finding some national charset. (And in any
+   case it probably makes more sense to do such conditionalizing on the
+   Unicode range of the character, and just check whether a font
+   contains the appropriate character -- or maybe not even conditionalize
+   at all on any character-specific property.) */
+DECLARE_INLINE_HEADER (
+Lisp_Object
+buffer_ichar_charset_obsolete_me_baby (struct buffer *
+				       USED_IF_UNICODE_INTERNAL (buf),
+				       Ichar ch)
+)
+{
+#ifdef UNICODE_INTERNAL
+  int byte1, byte2;
+  Lisp_Object charset;
+  buffer_ichar_to_charset_codepoint (ch, buf, &charset, &byte1, &byte2,
+				     CONVERR_FAIL);
+  return charset;
+#else
+  return old_mule_ichar_charset (ch);
+#endif
+
+}
+
 
 /************************************************************************/
 /*                                                                      */
@@ -941,13 +1070,28 @@ do									   \
 #define BUF_SIZE(buf) (BUF_Z (buf) - BUF_BEG (buf))
 
 /* Is this buffer narrowed? */
-#define BUF_NARROWED(buf) \
-   ((BYTE_BUF_BEGV (buf) != BYTE_BUF_BEG (buf)) || \
+#define BUF_NARROWED(buf)				\
+   ((BYTE_BUF_BEGV (buf) != BYTE_BUF_BEG (buf)) ||	\
     (BYTE_BUF_ZV   (buf) != BYTE_BUF_Z   (buf)))
 
 /* Modification count */
 #define BUF_MODIFF(buf) ((buf)->text->modiff)
 
+/* The modification count wraps around, use the following to compare, which
+   uses the same approach as Fevent_timestamp_lessp(). */
+DECLARE_INLINE_HEADER (
+int
+buf_tick_arithcompare (EMACS_INT val1, EMACS_INT val2)
+)
+{
+  if (val1 < val2 && ((val2 - val1) < (EMACS_INT) ((~(EMACS_UINT)(0) >> 2))))
+    {
+      return -1;
+    }
+
+  return val1 != val2;
+}
+
 /* Saved modification count */
 #define BUF_SAVE_MODIFF(buf) ((buf)->text->save_modiff)
 
@@ -965,35 +1109,6 @@ POINT_MARKER_P (Lisp_Object marker)
 
 #define BUF_MARKERS(buf) ((buf)->markers)
 
-#ifdef MULE
-
-DECLARE_INLINE_HEADER (
-Lisp_Object
-BUFFER_CATEGORY_TABLE (struct buffer *buf)
-)
-{
-  return buf ? buf->category_table : Vstandard_category_table;
-}
-
-#endif /* MULE */
-
-DECLARE_INLINE_HEADER (
-Lisp_Object
-BUFFER_SYNTAX_TABLE (struct buffer *buf)
-)
-{
-  return buf ? buf->syntax_table : Vstandard_syntax_table;
-}
-
-DECLARE_INLINE_HEADER (
-Lisp_Object
-BUFFER_MIRROR_SYNTAX_TABLE (struct buffer *buf)
-)
-{
-  return buf ? buf->mirror_syntax_table :
-    XCHAR_TABLE (Vstandard_syntax_table)->mirror_table;
-}
-
 /* WARNING:
 
    The new definitions of CEILING_OF() and FLOOR_OF() differ semantically
@@ -1152,106 +1267,21 @@ void r_alloc_free (unsigned char **);
 
 #endif /* !REL_ALLOC */
 
-
-/************************************************************************/
-/*                         Case conversion                              */
-/************************************************************************/
-
-/* A "trt" table is a mapping from characters to other characters,
-   typically used to convert between uppercase and lowercase.
-   */
-
-/* The _1 macros are named as such because they assume that you have
-   already guaranteed that the character values are all in the range
-   0 - 255.  Bad lossage will happen otherwise. */
-
-#define MAKE_TRT_TABLE() Fmake_char_table (Qgeneric)
+/* This gets to go here because it's used enough in redisplay that it's worth
+   making it inline, and because lisp.h doesn't have access to the
+   buffer.h-internal macros. */
 DECLARE_INLINE_HEADER (
-Ichar
-TRT_TABLE_OF (Lisp_Object table, Ichar ch)
+Boolint
+byte_beginning_of_line_p (struct buffer *b, Bytebpos bpt)
 )
 {
-  Lisp_Object TRT_char;
-  TRT_char = get_char_table (ch, table);
-  if (NILP (TRT_char))
-    return ch;
-  else
-    return XCHAR (TRT_char);
+  if (bpt <= BYTE_BUF_BEGV (b))
+    {
+      return 1;
+    }
+
+  DEC_BYTEBPOS (b, bpt);
+  return BYTE_BUF_FETCH_CHAR (b, bpt) == '\n';
 }
-#define SET_TRT_TABLE_OF(table, ch1, ch2)	\
-  Fput_char_table (make_char (ch1), make_char (ch2), table)
-
-DECLARE_INLINE_HEADER (
-Lisp_Object
-BUFFER_CASE_TABLE (struct buffer *buf)
-)
-{
-  return buf ? buf->case_table : current_buffer->case_table;
-  /* When buf=0, was Vstandard_case_table, but this sucks.  If I set a
-     different case table in this buffer, operations that use a case table
-     by default should use the current one. */
-}
-
-/* Macros used below. */
-#define DOWNCASE_TABLE_OF(buf, c)	\
-  TRT_TABLE_OF (XCASE_TABLE_DOWNCASE (BUFFER_CASE_TABLE (buf)), c)
-#define UPCASE_TABLE_OF(buf, c)		\
-  TRT_TABLE_OF (XCASE_TABLE_UPCASE (BUFFER_CASE_TABLE (buf)), c)
-#define CANON_TABLE_OF(buf, c)	\
-  TRT_TABLE_OF (XCASE_TABLE_CANON (BUFFER_CASE_TABLE (buf)), c)
-
-/* 1 if CH is upper case.  */
-
-DECLARE_INLINE_HEADER (
-int
-UPPERCASEP (struct buffer *buf, Ichar ch)
-)
-{
-  return DOWNCASE_TABLE_OF (buf, ch) != ch;
-}
-
-/* 1 if CH is lower case.  */
-
-DECLARE_INLINE_HEADER (
-int
-LOWERCASEP (struct buffer *buf, Ichar ch)
-)
-{
-  return (UPCASE_TABLE_OF   (buf, ch) != ch &&
-	  DOWNCASE_TABLE_OF (buf, ch) == ch);
-}
-
-/* 1 if CH is neither upper nor lower case.  */
-
-DECLARE_INLINE_HEADER (
-int
-NOCASEP (struct buffer *buf, Ichar ch)
-)
-{
-  return UPCASE_TABLE_OF (buf, ch) == ch;
-}
-
-/* Upcase a character, or make no change if that cannot be done.  */
-
-DECLARE_INLINE_HEADER (
-Ichar
-UPCASE (struct buffer *buf, Ichar ch)
-)
-{
-  return (DOWNCASE_TABLE_OF (buf, ch) == ch) ? UPCASE_TABLE_OF (buf, ch) : ch;
-}
-
-/* Upcase a character known to be not upper case.  Unused. */
-
-#define UPCASE1(buf, ch) UPCASE_TABLE_OF (buf, ch)
-
-/* Downcase a character, or make no change if that cannot be done. */
-
-#define DOWNCASE(buf, ch) DOWNCASE_TABLE_OF (buf, ch)
-
-/* Convert a character to a canonical representation, so that case-independent
-   comparisons will work. */
-
-#define CANONCASE(buf, ch) CANON_TABLE_OF (buf, ch)
 
 #endif /* INCLUDED_buffer_h_ */
