@@ -66,8 +66,6 @@ static void set_compiled_function_constants (Lisp_Compiled_Function *,
 static void set_compiled_function_interactive (Lisp_Compiled_Function *,
                                                Lisp_Object);
 
-EXFUN (Ffetch_bytecode, 1);
-
 Lisp_Object Qbyte_code, Qcompiled_functionp, Qinvalid_byte_code;
 
 
@@ -181,7 +179,7 @@ assert_failed_with_remembered_ops (const Ascbyte *file, int line,
 #ifdef BYTE_CODE_METER
 
 Lisp_Object Vbyte_code_meter, Qbyte_code_meter;
-int byte_metering_on;
+Boolint byte_metering_on;
 
 static void
 meter_code (Opcode prev_opcode, Opcode this_opcode)
@@ -514,25 +512,9 @@ static Lisp_Object
 bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 {
 #ifdef WITH_NUMBER_TYPES
-  switch (promote_args_lazy (&obj1, &obj2))
+  switch (promote_args (&obj1, &obj2))
     {
-    case LAZY_MARKER_T:
-      {
-	switch (opcode)
-	  {
-	  case Bmax:
-	    return marker_byte_position (obj1) < marker_byte_position (obj2)
-	      ? obj2 : obj1;
-	  case Bmin:
-	    return marker_byte_position (obj1) > marker_byte_position (obj2)
-	      ? obj2 : obj1;
-	  default:
-	    obj1 = make_fixnum (marker_position (obj1));
-	    obj2 = make_fixnum (marker_position (obj2));
-	    /* FALLTHROUGH */
-	  }
-      }
-    case LAZY_FIXNUM_T:
+    case FIXNUM_T:
       {
 	EMACS_INT ival1 = XREALFIXNUM (obj1), ival2 = XREALFIXNUM (obj2);
 	switch (opcode)
@@ -554,13 +536,11 @@ bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 	      signal_error_2 (Qarith_error, "division by zero", obj1, obj2);
 	    ival1 /= ival2;
 	    break;
-	  case Bmax:  if (ival1 < ival2) ival1 = ival2; break;
-	  case Bmin:  if (ival1 > ival2) ival1 = ival2; break;
 	  }
 	return make_integer (ival1);
       }
 #ifdef HAVE_BIGNUM
-    case LAZY_BIGNUM_T:
+    case BIGNUM_T:
       switch (opcode)
 	{
 	case Bplus:
@@ -581,17 +561,11 @@ bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 	  bignum_div (scratch_bignum, XBIGNUM_DATA (obj1),
 		      XBIGNUM_DATA (obj2));
 	  break;
-	case Bmax:
-	  return bignum_gt (XBIGNUM_DATA (obj1), XBIGNUM_DATA (obj2))
-	    ? obj1 : obj2;
-	case Bmin:
-	  return bignum_lt (XBIGNUM_DATA (obj1), XBIGNUM_DATA (obj2))
-	    ? obj1 : obj2;
 	}
       return Fcanonicalize_number (make_bignum_bg (scratch_bignum));
 #endif
 #ifdef HAVE_RATIO
-    case LAZY_RATIO_T:
+    case RATIO_T:
       switch (opcode)
 	{
 	case Bplus:
@@ -608,17 +582,11 @@ bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 	    signal_error_2 (Qarith_error, "division by zero", obj1, obj2);
 	  ratio_div (scratch_ratio, XRATIO_DATA (obj1), XRATIO_DATA (obj2));
 	  break;
-	case Bmax:
-	  return ratio_gt (XRATIO_DATA (obj1), XRATIO_DATA (obj2))
-	    ? obj1 : obj2;
-	case Bmin:
-	  return ratio_lt (XRATIO_DATA (obj1), XRATIO_DATA (obj2))
-	    ? obj1 : obj2;
 	}
       return make_ratio_rt (scratch_ratio);
 #endif
 #ifdef HAVE_BIGFLOAT
-    case LAZY_BIGFLOAT_T:
+    case BIGFLOAT_T:
       bigfloat_set_prec (scratch_bigfloat, max (XBIGFLOAT_GET_PREC (obj1),
 						XBIGFLOAT_GET_PREC (obj2)));
       switch (opcode)
@@ -641,12 +609,6 @@ bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 	  bigfloat_div (scratch_bigfloat, XBIGFLOAT_DATA (obj1),
 			XBIGFLOAT_DATA (obj2));
 	  break;
-	case Bmax:
-	  return bigfloat_gt (XBIGFLOAT_DATA (obj1), XBIGFLOAT_DATA (obj2))
-	    ? obj1 : obj2;
-	case Bmin:
-	  return bigfloat_lt (XBIGFLOAT_DATA (obj1), XBIGFLOAT_DATA (obj2))
-	    ? obj1 : obj2;
 	}
       return make_bigfloat_bf (scratch_bigfloat);
 #endif
@@ -663,8 +625,6 @@ bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 	      signal_error_2 (Qarith_error, "division by zero", obj1, obj2);
 	    dval1 /= dval2;
 	    break;
-	  case Bmax:  if (dval1 < dval2) dval1 = dval2; break;
-	  case Bmin:  if (dval1 > dval2) dval1 = dval2; break;
 	  }
 	return make_float (dval1);
       }
@@ -681,21 +641,6 @@ bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
   else if (CHARP   (obj1)) ival1 = XCHAR (obj1);
   else if (MARKERP (obj1))
     {
-      if (MARKERP (obj2)
-	  && (XMARKER (obj1)->buffer == XMARKER (obj2)->buffer))
-	{
-	  if (opcode == Bmax)
-	    {
-	      return marker_byte_position (obj1) < marker_byte_position (obj2)
-		? obj2 : obj1;
-	    }
-	  else if (opcode == Bmin)
-	    {
-	      return marker_byte_position (obj1) > marker_byte_position (obj2)
-		? obj2 : obj1;
-	    }
-	  /* Otherwise, convert to a fixnum in the normal way. */
-	}
       ival1 = marker_position (obj1);
     }
 
@@ -728,8 +673,6 @@ bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 	    signal_error_2 (Qarith_error, "division by zero", obj1, obj2);
 	  ival1 /= ival2;
 	  break;
-	case Bmax:  if (ival1 < ival2) ival1 = ival2; break;
-	case Bmin:  if (ival1 > ival2) ival1 = ival2; break;
 	}
       return make_fixnum (ival1);
     }
@@ -747,8 +690,6 @@ bytecode_arithop (Lisp_Object obj1, Lisp_Object obj2, Opcode opcode)
 	    signal_error_2 (Qarith_error, "division by zero", obj1, obj2);
 	  dval1 /= dval2;
 	  break;
-	case Bmax:  if (dval1 < dval2) dval1 = dval2; break;
-	case Bmin:  if (dval1 > dval2) dval1 = dval2; break;
 	}
       return make_float (dval1);
     }
@@ -1396,6 +1337,13 @@ execute_optimized_program (const Opbyte *program,
 	    break;
 	  }
 
+	case Bmin:
+	  {
+	    Lisp_Object arg = POP;
+            TOP_LVALUE = bytecode_gtr (TOP, arg) ? arg : TOP;
+	    break;
+	  }
+
 	case Blss:
 	  {
 	    Lisp_Object arg = POP;
@@ -1403,6 +1351,13 @@ execute_optimized_program (const Opbyte *program,
 	    break;
 	  }
 
+	case Bmax:
+          {
+	    Lisp_Object arg = POP;
+            TOP_LVALUE = bytecode_lss (TOP, arg) ? arg : TOP;
+            break;
+          }
+
 	case Bleq:
 	  {
 	    Lisp_Object arg = POP;
@@ -1458,8 +1413,6 @@ execute_optimized_program (const Opbyte *program,
 
 	case Bmult:
 	case Bquo:
-	case Bmax:
-	case Bmin:
 	  {
 	    Lisp_Object arg = POP;
 	    TOP_LVALUE = bytecode_arithop (TOP, arg, opcode);
@@ -1921,7 +1874,7 @@ execute_rare_opcode (Lisp_Object *stack_ptr,
         Lisp_Object upper = POP, first = TOP, speccount;
 
         check_integer_range (upper, Qzero,
-                             make_integer (Vmultiple_values_limit));
+                             make_integer (multiple_values_limit));
         check_integer_range (first, Qzero, upper);
 
         speccount = make_fixnum (bind_multiple_value_limits (XFIXNUM (first),
@@ -2400,11 +2353,6 @@ optimize_compiled_function (Lisp_Object compiled_function)
     f->args_in_array = totalargs;
   }
   
-  /* If we have not actually read the bytecode string
-     and constants vector yet, fetch them from the file.  */
-  if (CONSP (f->instructions))
-    Ffetch_bytecode (compiled_function);
-
   if (STRINGP (f->instructions))
     {
       /* XSTRING_LENGTH() is more efficient than string_char_length(),
@@ -2610,13 +2558,6 @@ static const struct memory_description compiled_function_description[] = {
   { XD_END }
 };
 
-DEFINE_DUMPABLE_FROB_BLOCK_LISP_OBJECT ("compiled-function", compiled_function,
-					print_compiled_function, 0,
-					compiled_function_equal,
-					compiled_function_hash,
-					compiled_function_description,
-					Lisp_Compiled_Function);
-
 
 DEFUN ("compiled-function-p", Fcompiled_function_p, 1, 1, 0, /*
 Return t if OBJECT is a byte-compiled function object.
@@ -2920,6 +2861,11 @@ DEFUN ("compiled-function-interactive", Fcompiled_function_interactive, 1, 1, 0,
 Return the interactive spec of the compiled-function object FUNCTION, or nil.
 If non-nil, the return value will be a list whose first element is
 `interactive' and whose second element is the interactive spec.
+
+See the documentation of `interactive'. As well as a string and an S-expression
+to be evaluated, a compiled function's interactive spec can be itself a
+compiled function, reflecting the compiled form of the S-expression and
+metadata to aid `call-interactively' in constructing the command history.
 */
        (function))
 {
@@ -2962,36 +2908,6 @@ This is only meaningful if I18N3 was enabled when emacs was compiled.
 }
 
 
-
-DEFUN ("fetch-bytecode", Ffetch_bytecode, 1, 1, 0, /*
-If the byte code for compiled function FUNCTION is lazy-loaded, fetch it now.
-*/
-       (function))
-{
-  Lisp_Compiled_Function *f;
-  CHECK_COMPILED_FUNCTION (function);
-  f = XCOMPILED_FUNCTION (function);
-
-  if (OPAQUEP (f->instructions) || STRINGP (f->instructions))
-    return function;
-
-  if (CONSP (f->instructions))
-    {
-      Lisp_Object tem = read_doc_string (f->instructions);
-      if (!CONSP (tem))
-	signal_error (Qinvalid_byte_code,
-			   "Invalid lazy-loaded byte code", tem);
-      /* v18 or v19 bytecode file.  Need to Ebolify. */
-      if (f->flags.ebolified && VECTORP (XCDR (tem)))
-	ebolify_bytecode_constants (XCDR (tem));
-      f->instructions = XCAR (tem);
-      f->constants    = XCDR (tem);
-      return function;
-    }
-  ABORT ();
-  return Qnil; /* not (usually) reached */
-}
-
 DEFUN ("optimize-compiled-function", Foptimize_compiled_function, 1, 1, 0, /*
 Convert compiled function FUNCTION into an optimized internal form.
 */
@@ -3042,24 +2958,24 @@ If STACK-DEPTH is incorrect, Emacs may crash.
 }
 
 
-void
-bytecode_objects_create (void)
-{
-  OBJECT_HAS_METHOD (compiled_function, print_preprocess);
-  OBJECT_HAS_METHOD (compiled_function, nsubst_structures_descend);
-}
-
 void
 syms_of_bytecode (void)
 {
-  INIT_LISP_OBJECT (compiled_function);
+  DEFINE_DUMPABLE_FROB_BLOCK_LISP_OBJECT ("compiled-function",
+                                          compiled_function,
+                                          print_compiled_function, 0,
+                                          compiled_function_equal,
+                                          compiled_function_hash,
+                                          compiled_function_description,
+                                          Lisp_Compiled_Function);
+  OBJECT_HAS_METHOD (compiled_function, print_preprocess);
+  OBJECT_HAS_METHOD (compiled_function, nsubst_structures_descend);
 
   DEFERROR_STANDARD (Qinvalid_byte_code, Qinvalid_state);
   DEFSYMBOL (Qbyte_code);
   DEFSYMBOL_MULTIWORD_PREDICATE (Qcompiled_functionp);
 
   DEFSUBR (Fbyte_code);
-  DEFSUBR (Ffetch_bytecode);
   DEFSUBR (Foptimize_compiled_function);
 
   DEFSUBR (Fcompiled_function_p);
