@@ -34,6 +34,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #include "events.h"
 #include "insdel.h"
 #include "window-impl.h" /* WINDOW_MINI_P */
+#include "extents.h"
 
 extern Charcount num_input_chars;
 
@@ -57,8 +58,7 @@ Lisp_Object Vmark_even_if_inactive;
 Lisp_Object Vmouse_leave_buffer_hook, Qmouse_leave_buffer_hook;
 #endif
 
-Lisp_Object QletX, Qsave_excursion;
-
+Lisp_Object Qbyte_compile_annotate_interactive;
 Lisp_Object Qprefix_numeric_value;
 Lisp_Object Qread_from_minibuffer;
 Lisp_Object Qread_file_name;
@@ -95,9 +95,17 @@ The argument of `interactive' is usually a string containing a code letter
  the argument and do not need prompts.)  To prompt for multiple arguments,
  give a code letter, its prompt, a newline, and another code letter, etc.
  Prompts are passed to format, and may use % escapes to print the
- arguments that have already been read.
-If the argument is not a string, it is evaluated to get a list of
- arguments to pass to the function.
+ arguments that have already been read. Note that if you wish to be compatible
+ with GNU Emacs only %s is accepted for these escapes, and any buffer position
+ arguments will not have useful values in the prompt passed to the user in that
+ program. Consider use of a list argument to `interactive' in that context.
+
+If the argument is not a string, it is a Lisp form which is evaluated to get a
+ list of arguments to pass to the function.  When the function is byte compiled,
+ this Lisp form becomes a function with no parameters; the list of arguments
+ is the `cdr' of the result of calling it, and the `car' is used by
+ `call-interactively' to aid construction of `command-history'.
+
 Just `(interactive)' means pass no args when calling interactively.
 
 Code letters available are:
@@ -150,11 +158,7 @@ b -- (let ((def (current-buffer)))
            (setq def (other-buffer def))
        (read-buffer "PROMPT" def t)))
 B -- (read-buffer "PROMPT" (other-buffer (current-buffer)))
-c -- (prog1
-         (let ((cursor-in-echo-area t))
-           (message "%s" "PROMPT")
-           (read-char))
-       (message nil))
+c -- (read-char "PROMPT")
 C -- (read-command "PROMPT")
 d -- (point)
 D -- (read-directory-name "PROMPT" nil default-directory t)
@@ -212,15 +216,6 @@ to a separate line causes docstring lossage! */
   return Qnil;
 }
 
-/* Modify EXPR by quotifying each element (except the first).  */
-static Lisp_Object
-quotify_args (Lisp_Object expr)
-{
-  EXTERNAL_LIST_LOOP_3 (elt, expr, tail)
-    XSETCAR (tail, Fquote_maybe (elt));
-  return expr;
-}
-
 static Bytebpos
 check_mark (void)
 {
@@ -237,19 +232,50 @@ check_mark (void)
 }
 
 static Lisp_Object
-callint_prompt (const Ibyte *prompt_start, Bytecount prompt_length,
+callint_prompt (Lisp_Object reloc,
+		const Ibyte *prompt_start, Bytecount prompt_length,
                 const Lisp_Object *args, int nargs)
 {
-  if (memchr (prompt_start, '%', prompt_length))
+  Lisp_Object control_string = make_string (prompt_start, prompt_length);
+
+  if (STRINGP (reloc) && prompt_length > 0)
     {
-      Lisp_Object *args0 = alloca_array (Lisp_Object, nargs + 1);
-      args0[0] = make_string (prompt_start, prompt_length);
-      memcpy (args0 + 1, args, sizeof (Lisp_Object) * nargs);
-      /* format_into() will GCPRO S, no need for us to. */
-      return Fformat_into (nargs + 1, args0);
+      text_checking_assert (prompt_start >= XSTRING_DATA (reloc));
+      text_checking_assert (prompt_start + prompt_length
+			    <= (XSTRING_DATA (reloc)
+                                + XSTRING_LENGTH (reloc)));
+      copy_string_extents (control_string, reloc, 0,
+			   prompt_start - XSTRING_DATA (reloc),
+			   prompt_length);
+    }
+
+  if (memchr (prompt_start, '%', prompt_length) && nargs)
+    {
+      Lisp_Object *argz = alloca_array (Lisp_Object, nargs + 2);
+      int ii;
+
+      argz[0] = Qstring;
+      argz[1] = control_string;
+
+      for (ii = 0; ii < nargs; ii++)
+	{
+	  if (MARKERP (args[ii]))
+	    {
+	      /* This facility is used very rarely, the byte-char conversion
+		 hit is not an issue. */
+	      argz[ii + 2] = Fmarker_position (args[ii]);
+	    }
+	  else
+	    {
+	      argz[ii + 2] = args[ii];
+	    }
+	}
+
+      /* format_into() will GCPRO argz, no need for us to. */
+      return Fformat_into (nargs + 2, argz);
     }
   
-  return make_string (prompt_start, prompt_length);
+  return control_string;
 }
 
 /* `lambda' for RECORD-FLAG is an XEmacs addition. */
@@ -317,22 +343,17 @@ when reading the arguments.
 
   /* Decode the kind of function.  Either handle it and return,
      or go to `lose' if not interactive, or go to `retry'
-     to specify a different function, or set either PROMPT_DATA or SPECS. */
+     to specify a different function, or set SPECS. */
 
   if (SUBRP (fun))
     {
-      prompt_data = (const Ibyte *) XSUBR (fun)->prompt;
-      if (!prompt_data)
+      specs = XSUBR (fun)->prompt;
+      if (!STRINGP (specs))
 	{
 	lose:
 	  function = wrong_type_argument (Qcommandp, function);
 	  goto retry;
 	}
-#if 0 /* FSFmacs */ /* Huh? Where is this used? */
-      if ((EMACS_INT) prompt_data == 1)
-	/* Let SPECS (which is nil) be used as the args.  */
-	prompt_data = 0;
-#endif
     }
   else if (COMPILED_FUNCTIONP (fun))
     {
@@ -370,16 +391,39 @@ when reading the arguments.
   /* FSFmacs makes an ALLOCA() copy of prompt_data here.
      We're more intelligent about this and just reset prompt_data
      as necessary. */
-  /* If either specs or prompt_data is set to a string, use it.  */
-  if (!STRINGP (specs) && prompt_data == 0)
+  /* If specs is set to a string, use it.  */
+  if (!STRINGP (specs))
     {
       struct gcpro gcpro1, gcpro2, gcpro3;
       Charcount i = num_input_chars;
-      Lisp_Object input = specs;
+      Lisp_Object structure = Qnil, values = Qnil;
+
+      GCPRO3 (function, specs, structure);
 
-      GCPRO3 (function, specs, input);
       /* Compute the arg values using the user's expression.  */
-      specs = IGNORE_MULTIPLE_VALUES (Feval (specs));
+      if (COMPILED_FUNCTIONP (specs))
+	{
+	  /* If SPECS is a compiled function object, the byte-compiler has
+	     called #'byte-compile-annotate-interactive at compile time, and
+	     the result of calling the compiled function is a cons of the
+	     annotation and the result of the S-expression supplied to
+	     (interactive).  */
+	  specs = IGNORE_MULTIPLE_VALUES (Ffuncall (1, &specs));
+	  if (CONSP (specs))
+	    {
+	      structure = XCAR (specs);
+	      specs = XCDR (specs);
+	    }
+	}
+      else if (!NILP (specs))
+	{
+	  /* SPECS is not a compiled function, either this is an interpreted
+	     call or we are dealing with compiled code from 21.4. Generate
+	     STRUCTURE at runtime. */
+	  structure = call1 (Qbyte_compile_annotate_interactive, specs);
+	  specs = IGNORE_MULTIPLE_VALUES (Feval (specs));
+	}
+
       if (EQ (record_flag, Qlambda)) /* XEmacs addition */
 	{
 	  UNGCPRO;
@@ -387,54 +431,26 @@ when reading the arguments.
 	}
       if (!NILP (record_flag) || i != num_input_chars)
 	{
-	  /* We should record this command on the command history.  */
-	  /* #### The following is too specific; should have general
-	     mechanism for doing this. */
-	  Lisp_Object values, car;
-	  /* Make a copy of the list of values, for the command history,
-	     and turn them into things we can eval.  */
-	  values = quotify_args (Fcopy_list (specs));
 	  /* If the list of args was produced with an explicit call to `list',
 	     look for elements that were computed with (region-beginning)
 	     or (region-end), and put those expressions into VALUES
 	     instead of the present values.  */
-	  if (CONSP (input))
+	  EXTERNAL_LIST_LOOP_2 (elt, specs)
 	    {
-	      car = XCAR (input);
-	      /* Skip through certain special forms.  */
-	      while (EQ (car, Qlet) || EQ (car, QletX)
-		     || EQ (car, Qsave_excursion))
+	      if (CONSP (structure) && CONSP (XCAR (structure)))
 		{
-		  while (CONSP (XCDR (input)))
-		    input = XCDR (input);
-		  input = XCAR (input);
-		  if (!CONSP (input))
-		    break;
-		  car = XCAR (input);
+		  values = Fcons (XCAR (structure), values);
 		}
-	      if (EQ (car, Qlist))
+	      else
 		{
-		  Lisp_Object intail, valtail;
-		  for (intail = Fcdr (input), valtail = values;
-		       CONSP (valtail);
-		       intail = Fcdr (intail), valtail = Fcdr (valtail))
-		    {
-		      Lisp_Object elt;
-		      elt = Fcar (intail);
-		      if (CONSP (elt))
-			{
-			  Lisp_Object eltcar = Fcar (elt);
-			  if (EQ (eltcar, Qpoint) ||
-			      EQ (eltcar, Qmark)  ||
-			      EQ (eltcar, Qregion_beginning) ||
-			      EQ (eltcar, Qregion_end))
-			    Fsetcar (valtail, Fcar (intail));
-			}
-		    }
+		  values = Fcons (Fquote_maybe (elt), values);
 		}
+
+	      structure = Fcdr (structure);
 	    }
+
 	  Vcommand_history
-	    = Fcons (Fcons (function, values), Vcommand_history);
+	    = Fcons (Fcons (function, Fnreverse (values)), Vcommand_history);
 	}
       single_console_state ();
       RETURN_UNGCPRO (apply1 (fun, specs));
@@ -454,11 +470,6 @@ when reading the arguments.
       else
 	specs = Fdgettext (domain, specs);
     }
-  else if (prompt_data)
-    /* We do not have to worry about domains in this case because
-       prompt_data is non-nil only for built-in functions, which
-       always use the default domain. */
-    prompt_data = gettext (prompt_data);
 #endif
 
   /* Handle special starting chars `*' and `@' and `_'.  */
@@ -502,7 +513,7 @@ when reading the arguments.
 		  {
 		    if (MINI_WINDOW_P (XWINDOW (window))
 			&& ! (minibuf_level > 0 && EQ (window,
-						       minibuf_window)))
+						       Vminibuf_window)))
 		      invalid_operation ("Attempt to select inactive minibuffer window", Qunbound);
 
 #if 0 /* unclean! see event-stream.c */
@@ -622,24 +633,22 @@ when reading the arguments.
 	    prompt_limit = 0;	/* "sfoo:\n" -- strip tailing return */
 	    prompt_length -= 1;
 	  }
-	/* This uses `visargs' instead of `args' so that global-set-key
-	   prompts with "Set key C-x C-f to command: "instead of printing
-	   event objects in there.
-	 */
-#define PROMPT() callint_prompt ((const Ibyte *) prompt_start, prompt_length, visargs, argnum)
+	/* This uses `visargs' instead of `args' so that global-set-key prompts
+	   with "Set key C-x C-f to command: " instead of printing event
+	   objects in there. */
+#define PROMPT() callint_prompt (specs, prompt_start, prompt_length, visargs, argnum)
 	switch (prompt_data[prompt_index])
 	  {
 	  case 'a':		/* Symbol defined as a function */
 	    {
-	      Lisp_Object tem = call1 (Qread_function, PROMPT ());
-	      args[argnum] = tem;
+	      args[argnum] = call1 (Qread_function, PROMPT ());
 	      arg_from_tty = 1;
 	      break;
 	    }
 	  case 'b':   		/* Name of existing buffer */
 	    {
 	      Lisp_Object def = Fcurrent_buffer ();
-	      if (EQ (Fselected_window (Qnil), minibuf_window))
+	      if (EQ (Fselected_window (Qnil), Vminibuf_window))
 		def = Fother_buffer (def, Qnil, Qnil);
               /* read-buffer returns a buffer name, not a buffer! */
 	      args[argnum] = call3 (Qread_buffer, PROMPT (), def,
@@ -658,30 +667,13 @@ when reading the arguments.
 	    }
 	  case 'c':		/* Character */
 	    {
-              Lisp_Object tem;
-	      int shadowing_speccount = specpdl_depth ();
-
-	      specbind (Qcursor_in_echo_area, Qt);
-	      message ("%s", XSTRING_DATA (PROMPT ()));
-	      tem = (call0 (Qread_char));
-              args[argnum] = tem;
-              /* visargs[argnum] = Fsingle_key_description (tem); */
-	      /* FSF has visargs[argnum] = Fchar_to_string (tem); */
-
-	      unbind_to (shadowing_speccount);
-
-	      /* #### `C-x / a' should not leave the prompt in the minibuffer.
-		 This isn't the right fix, because (message ...) (read-char)
-		 shouldn't leave the message there either... */
-	      clear_message ();
-
+	      args[argnum] = call1 (Qread_char, PROMPT ());
 	      arg_from_tty = 1;
 	      break;
 	    }
 	  case 'C':		/* Command: symbol with interactive function */
 	    {
-	      Lisp_Object tem = call1 (Qread_command, PROMPT ());
-	      args[argnum] = tem;
+	      args[argnum] = call1 (Qread_command, PROMPT ());
 	      arg_from_tty = 1;
 	      break;
 	    }
@@ -731,22 +723,19 @@ when reading the arguments.
 	    }
 	  case 'f':		/* Existing file name. */
 	    {
-	      Lisp_Object tem = call4 (Qread_file_name, PROMPT (),
-				       Qnil, /* dir */
-				       Qnil, /* default */
-				       Qzero /* must-match */
-				       );
-              args[argnum] = tem;
+              args[argnum] = call4 (Qread_file_name, PROMPT (),
+                                    Qnil,   /* dir */
+                                    Qnil,   /* default */
+                                    Qzero); /* must-match */
 	      arg_from_tty = 1;
 	      break;
 	    }
 	  case 'F':		/* Possibly nonexistent file name. */
 	    {
 	      args[argnum] = call4 (Qread_file_name, PROMPT (),
-				    Qnil, /* dir */
-				    Qnil, /* default */
-				    Qnil /* must-match */
-				    );
+				    Qnil,  /* dir */
+				    Qnil,  /* default */
+				    Qnil); /* must-match */
 	      arg_from_tty = 1;
 	      break;
 	    }
@@ -879,8 +868,7 @@ when reading the arguments.
 	    }
 	  case 'v':		/* Variable name: user-variable-p symbol */
 	    {
-	      Lisp_Object tem = call1 (Qread_variable, PROMPT ());
-	      args[argnum] = tem;
+	      args[argnum] = call1 (Qread_variable, PROMPT ());
 	      arg_from_tty = 1;
 	      break;
 	    }
@@ -984,6 +972,7 @@ void
 syms_of_callint (void)
 {
   DEFSYMBOL (Qcall_interactively);
+  DEFSYMBOL (Qbyte_compile_annotate_interactive);
   DEFSYMBOL (Qread_from_minibuffer);
   DEFSYMBOL (Qcompleting_read);
   DEFSYMBOL (Qprefix_numeric_value);
@@ -1002,8 +991,6 @@ syms_of_callint (void)
   DEFSYMBOL (Qcommand_debug_status);
   DEFSYMBOL (Qenable_recursive_minibuffers);
 
-  defsymbol (&QletX, "let*");
-  DEFSYMBOL (Qsave_excursion);
 #if 0 /* ill-conceived */
   DEFSYMBOL (Qmouse_leave_buffer_hook);
 #endif
