@@ -23,8 +23,10 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #include "lisp.h"
 
 #include "buffer.h"
+#include "casetab.h"
 #include "insdel.h"
 #include "syntax.h"
+#include "extents.h"
 
 enum case_action {CASE_UP, CASE_DOWN, CASE_CAPITALIZE, CASE_CAPITALIZE_UP,
                   CASE_CANONICALIZE};
@@ -60,13 +62,13 @@ casify_object (enum case_action flag, Lisp_Object string_or_char,
 
   if (STRINGP (string_or_char))
     {
-      Lisp_Object syntax_table = buf->mirror_syntax_table;
+      Lisp_Object syntax_table = BUFFER_MIRROR_SYNTAX_TABLE (buf), result;
       Ibyte *storage =
 	alloca_ibytes (XSTRING_LENGTH (string_or_char) * MAX_ICHAR_LEN);
       Ibyte *newp = storage;
       Ibyte *oldp = XSTRING_DATA (string_or_char);
       Ibyte *endp = oldp + XSTRING_LENGTH (string_or_char);
-      int wordp = 0, wordp_prev;
+      Boolint wordp = 0, wordp_prev;
 
       while (oldp < endp)
 	{
@@ -101,7 +103,18 @@ casify_object (enum case_action flag, Lisp_Object string_or_char,
 	  INC_IBYTEPTR (oldp);
 	}
 
-      return make_string (storage, newp - storage);
+      result = make_string (storage, newp - storage);
+
+      if (string_extent_info (string_or_char) != NULL)
+        {
+          /* Usually copy_string_extents() would be fine, but it is possible
+             for the transformed object to have a different byte length. */
+          stretch_string_extents (result, string_or_char, 0, 0,
+                                  XSTRING_LENGTH (string_or_char),
+                                  XSTRING_LENGTH (result));
+        }
+
+      return result;
     }
 
   string_or_char = wrong_type_argument (Qchar_or_string_p, string_or_char);
@@ -188,9 +201,9 @@ casify_region_internal (enum case_action flag, Lisp_Object start,
 {
   /* This function can GC */
   Charbpos pos, s, e;
-  Lisp_Object syntax_table = buf->mirror_syntax_table;
+  Lisp_Object syntax_table = BUFFER_MIRROR_SYNTAX_TABLE (buf);
   int mccount;
-  int wordp = 0, wordp_prev;
+  Boolint wordp = 0, wordp_prev;
 
   if (EQ (start, end))
     /* Not modifying because nothing marked */
