@@ -107,10 +107,6 @@ static const struct memory_description case_table_description [] = {
   { XD_END }
 };
 
-
-DEFINE_DUMPABLE_LISP_OBJECT ("case-table", case_table, print_case_table, 0,
-			     0, 0, case_table_description, Lisp_Case_Table);
-
 static Lisp_Object
 allocate_case_table (int init_tables)
 {
@@ -527,17 +523,13 @@ case_table_memory_usage (Lisp_Object casetab,
 
 
 void
-casetab_objects_create (void)
+syms_of_casetab (void)
 {
+  DEFINE_DUMPABLE_LISP_OBJECT ("case-table", case_table, print_case_table, 0,
+                               0, 0, case_table_description, Lisp_Case_Table);
 #ifdef MEMORY_USAGE_STATS
   OBJECT_HAS_METHOD (case_table, memory_usage);
 #endif
-}
-
-void
-syms_of_casetab (void)
-{
-  INIT_LISP_OBJECT (case_table);
 
   DEFSYMBOL_MULTIWORD_PREDICATE (Qcase_tablep);
   DEFSYMBOL (Qdowncase);
@@ -559,12 +551,9 @@ void
 vars_of_casetab (void)
 {
 #ifdef MEMORY_USAGE_STATS
-  OBJECT_HAS_PROPERTY (case_table, memusage_stats_list,
-		       list5 (Qt,
-			      intern ("downcase"),
-			      intern ("upcase"),
-			      intern ("case-canon"),
-			      intern ("case-eqv")));
+  INIT_MEMORY_USAGE_STATS (case_table,
+                           list5 (Qt, intern ("downcase"), intern ("upcase"),
+                                  intern ("case-canon"), intern ("case-eqv")));
 #endif /* MEMORY_USAGE_STATS */
 }
 
