@@ -61,7 +61,6 @@ Lisp_Object Qchar_tablep, Qchar_table;
 
 Lisp_Object Vall_syntax_tables;
 
-#ifdef MULE
 Lisp_Object Qcategory_tablep;
 Lisp_Object Qcategory_designator_p;
 
@@ -69,7 +68,6 @@ Lisp_Object Vstandard_category_table;
 
 /* Variables to determine word boundary.  */
 Lisp_Object Vword_combining_categories, Vword_separating_categories;
-#endif /* MULE */
 
 #ifdef MEMORY_USAGE_STATS
 Lisp_Object Qpage_tables;
@@ -215,11 +213,6 @@ static const struct memory_description char_subtable_description[] = {
   { XD_LISP_OBJECT_ARRAY, offsetof (Lisp_Char_Subtable, ptr), 256 }, 
   { XD_END }
 };
-
-DEFINE_DUMPABLE_INTERNAL_LISP_OBJECT ("char-subtable", char_subtable,
-				      char_subtable_description,
-				      Lisp_Char_Subtable);
-
 
 /************************************************************************/
 /*                      Char table implementation                       */
@@ -935,7 +928,7 @@ char_table_hash (Lisp_Object obj, int depth, Boolint equalp)
       for (ch = 0xAC00; ch <= 0xD7AF; ch += 791)
 	hashval = HASH2 (hashval, hash_raw_chartab_val (ch, obj, depth,
 							equalp));
-#elif defined (MULE)
+#else
 /* 0xA1 is usually the first alphabetic character and differs across
    charsets, whereas 0xA0 is no-break-space across many of them.
    charset_codepoint_to_ichar_raw() can't fail because we are in non-
@@ -1009,9 +1002,7 @@ char_table_type_to_symbol (enum char_table_type type)
   case CHAR_TABLE_TYPE_SYNTAX:   return Qsyntax;
   case CHAR_TABLE_TYPE_DISPLAY:  return Qdisplay;
   case CHAR_TABLE_TYPE_CHAR:     return Qchar;
-#ifdef MULE
   case CHAR_TABLE_TYPE_CATEGORY: return Qcategory;
-#endif
   }
 }
 
@@ -1024,9 +1015,7 @@ symbol_to_char_table_type (Lisp_Object symbol)
   if (EQ (symbol, Qsyntax))   return CHAR_TABLE_TYPE_SYNTAX;
   if (EQ (symbol, Qdisplay))  return CHAR_TABLE_TYPE_DISPLAY;
   if (EQ (symbol, Qchar))     return CHAR_TABLE_TYPE_CHAR;
-#ifdef MULE
   if (EQ (symbol, Qcategory)) return CHAR_TABLE_TYPE_CATEGORY;
-#endif
 
   invalid_constant ("Unrecognized char table type", symbol);
   RETURN_NOT_REACHED (CHAR_TABLE_TYPE_GENERIC);
@@ -1042,9 +1031,7 @@ char_table_default_for_type (enum char_table_type type)
       break;
     case CHAR_TABLE_TYPE_DISPLAY:
     case CHAR_TABLE_TYPE_GENERIC:
-#ifdef MULE
     case CHAR_TABLE_TYPE_CATEGORY:
-#endif /* MULE */
       return Qnil;
       break;
 
@@ -1128,12 +1115,6 @@ static const struct memory_description char_table_description[] = {
   { XD_END }
 };
 
-DEFINE_DUMPABLE_LISP_OBJECT ("char-table", char_table,
-			     print_char_table, 0,
-			     char_table_equal, char_table_hash,
-			     char_table_description,
-			     Lisp_Char_Table);
-
 static void
 decode_char_table_range (Lisp_Object range, struct chartab_range *outrange)
 {
@@ -1153,10 +1134,6 @@ decode_char_table_range (Lisp_Object range, struct chartab_range *outrange)
       outrange->ch = XCHAR_OR_CHAR_INT (XCAR (range));
       outrange->chtop = XCHAR_OR_CHAR_INT (XCDR (range));
     }
-#ifndef MULE
-  else
-    sferror ("Range must be t, character or cons of char range", range);
-#else /* MULE */
   else if (VECTORP (range))
     {
       Lisp_Vector *vec = XVECTOR (range);
@@ -1186,7 +1163,6 @@ decode_char_table_range (Lisp_Object range, struct chartab_range *outrange)
       outrange->type = CHARTAB_RANGE_CHARSET;
       outrange->charset = Fget_charset (range);
     }
-#endif /* MULE */
 }
 
 static void
@@ -1212,11 +1188,7 @@ See `make-char-table'.
 */
        ())
 {
-#ifdef MULE
   return list5 (Qchar, Qcategory, Qdisplay, Qgeneric, Qsyntax);
-#else
-  return list4 (Qchar, Qdisplay, Qgeneric, Qsyntax);
-#endif
 }
 
 DEFUN ("valid-char-table-type-p", Fvalid_char_table_type_p, 1, 1, 0, /*
@@ -1226,9 +1198,7 @@ See `make-char-table'.
        (type))
 {
   return (EQ (type, Qchar)     ||
-#ifdef MULE
 	  EQ (type, Qcategory) ||
-#endif
 	  EQ (type, Qdisplay)  ||
 	  EQ (type, Qgeneric)  ||
 	  EQ (type, Qsyntax)) ? Qt : Qnil;
@@ -1499,13 +1469,11 @@ check_valid_char_table_value (Lisp_Object value, enum char_table_type type,
         CHECK_FIXNUM (value);
       break;
 
-#ifdef MULE
     case CHAR_TABLE_TYPE_CATEGORY:
       maybe_signal_error (Qinvalid_operation,
 			  "Can't set category char tables in this fashion",
 			  value, Qchar_table, errb);
       return 0;
-#endif /* MULE */
 
     case CHAR_TABLE_TYPE_GENERIC:
       return 1;
@@ -1581,9 +1549,7 @@ put_char_table_range (Lisp_Object table, struct chartab_range *range,
 		      Lisp_Object val)
 {
   Lisp_Char_Table *ct = XCHAR_TABLE (table);
-#ifdef MULE
   int l1, l2, h1, h2;
-#endif
 
   switch (range->type)
     {
@@ -1600,7 +1566,6 @@ put_char_table_range (Lisp_Object table, struct chartab_range *range,
       break;
 
 
-#ifdef MULE
     case CHARTAB_RANGE_ROW:
       {
 	get_charset_limits (range->charset, &l1, &l2, &h1, &h2);
@@ -1638,7 +1603,6 @@ put_char_table_range (Lisp_Object table, struct chartab_range *range,
 	put_char_table (table, from, to, val);
       }
 #endif /* (not) UNICODE_INTERNAL */
-#endif /* MULE */
     }
 
   if (ct->type == CHAR_TABLE_TYPE_SYNTAX)
@@ -1721,9 +1685,7 @@ map_char_table (Lisp_Object table,
 			   Lisp_Object val, void *arg),
 		void *arg)
 {
-#ifdef MULE
   int l1, h1, l2, h2;
-#endif
   int levels = XCHAR_TABLE_LEVELS (table);
   /* Compute maximum allowed value for this table, which may be less than
      the range we have been requested to map over. */
@@ -1759,7 +1721,6 @@ map_char_table (Lisp_Object table,
 	else
 	  return 0;
       }
-#ifdef MULE
     case CHARTAB_RANGE_ROW:
       {
 	get_charset_limits (range->charset, &l1, &l2, &h1, &h2);
@@ -1810,7 +1771,6 @@ map_char_table (Lisp_Object table,
 				    table, fn, arg, &rainj);
       }
 #endif /* (not) UNICODE_INTERNAL */
-#endif /* MULE */
 
     default:
       ABORT ();
@@ -2021,7 +1981,6 @@ chartab_instantiate (Lisp_Object plist)
   return chartab;
 }
 
-#ifdef MULE
 
 
 /************************************************************************/
@@ -2076,12 +2035,6 @@ static const struct memory_description category_table_description[] = {
   { XD_END }
 };
 
-DEFINE_DUMPABLE_LISP_OBJECT ("category-table", category_table,
-			     print_category_table, 0,
-			     category_table_equal, category_table_hash,
-			     category_table_description,
-			     Lisp_Category_Table);
-
 DEFUN ("make-category-table", Fmake_category_table, 0, 0, 0, /*
 Construct a new and empty category table and return it.
 A category table is a table used for keeping track of categories.
@@ -2346,8 +2299,7 @@ range, a charset or a vector giving a charset and a row in that charset.
    characters C1 and C2 if they appear in this order, else return 0.
    Use the macro WORD_BOUNDARY_P instead of calling this function
    directly.  */
-
-int
+Boolint
 word_boundary_p (struct buffer *buf, Ichar c1, Ichar c2)
 {
   Lisp_Object tail;
@@ -2374,44 +2326,46 @@ word_boundary_p (struct buffer *buf, Ichar c1, Ichar c2)
       default_result = 1;
     }
 
-  for (; CONSP (tail); tail = XCDR (tail))
-    {
-      Lisp_Object elt = XCAR (tail);
+  {
+    EXTERNAL_LIST_LOOP_2 (elt, tail)
+      {
+        if (CONSP (elt)
+            && CATEGORY_DESIGNATORP (XCAR (elt))
+            && CATEGORY_DESIGNATORP (XCDR (elt))
+            && check_char_in_category (c1, table, XCHAR (XCAR (elt)), 0)
+            && check_char_in_category (c2, table, XCHAR (XCDR (elt)), 0))
+          return !default_result;
+      }
+  }
 
-      if (CONSP (elt)
-	  && CATEGORY_DESIGNATORP (XCAR (elt))
-	  && CATEGORY_DESIGNATORP (XCDR (elt))
-	  && check_char_in_category (c1, table, XCHAR (XCAR (elt)), 0)
-	  && check_char_in_category (c2, table, XCHAR (XCDR (elt)), 0))
-	return !default_result;
-    }
   return default_result;
 }
 
-#endif /* MULE */
 
 
 void
-chartab_objects_create (void)
+syms_of_chartab (void)
 {
+  DEFINE_DUMPABLE_LISP_OBJECT ("char-table", char_table, print_char_table, 0,
+                               char_table_equal, char_table_hash,
+                               char_table_description, Lisp_Char_Table);
   OBJECT_HAS_METHOD (char_table, print_preprocess);
   OBJECT_HAS_METHOD (char_table, nsubst_structures_descend);
 #ifdef MEMORY_USAGE_STATS
   OBJECT_HAS_METHOD (char_table, memory_usage);
 #endif
-}
 
-void
-syms_of_chartab (void)
-{
-  INIT_LISP_OBJECT (char_table);
-  INIT_LISP_OBJECT (char_subtable);
+  DEFINE_DUMPABLE_INTERNAL_LISP_OBJECT ("char-subtable", char_subtable,
+                                        char_subtable_description,
+                                        Lisp_Char_Subtable);
 
-#ifdef MULE
-  INIT_LISP_OBJECT (category_table);
+  DEFINE_DUMPABLE_LISP_OBJECT ("category-table", category_table,
+                               print_category_table, 0, category_table_equal,
+                               category_table_hash,
+                               category_table_description,
+                               Lisp_Category_Table);
   DEFSYMBOL_MULTIWORD_PREDICATE (Qcategory_tablep);
   DEFSYMBOL (Qcategory_designator_p);
-#endif /* MULE */
 
   DEFSYMBOL (Qchar_table);
   DEFSYMBOL_MULTIWORD_PREDICATE (Qchar_tablep);
@@ -2436,7 +2390,6 @@ syms_of_chartab (void)
   DEFSUBR (Fremove_char_table);
   DEFSUBR (Fmap_char_table);
 
-#ifdef MULE
   DEFSUBR (Fmake_category_table);
   DEFSUBR (Fcategory_table_p);
   DEFSUBR (Fcategory_table);
@@ -2448,7 +2401,6 @@ syms_of_chartab (void)
   DEFSUBR (Fcategory_designator_p);
   DEFSUBR (Fmodify_category_entry_internal);
   DEFSUBR (Fmap_category_table);
-#endif /* MULE */
 
 }
 
@@ -2456,8 +2408,7 @@ void
 vars_of_chartab (void)
 {
 #ifdef MEMORY_USAGE_STATS
-  OBJECT_HAS_PROPERTY
-    (char_table, memusage_stats_list, list2 (Qt, Qpage_tables));
+  INIT_MEMORY_USAGE_STATS (char_table, list2 (Qt, Qpage_tables));
 #endif /* MEMORY_USAGE_STATS */
 
   staticpro (&Vall_syntax_tables);
@@ -2493,7 +2444,6 @@ structure_type_create_chartab (void)
 void
 complex_vars_of_chartab (void)
 {
-#ifdef MULE
   /* Set this now, so first buffer creation can refer to it. */
   Vstandard_category_table = Fmake_category_table ();
   staticpro (&Vstandard_category_table);
@@ -2542,5 +2492,4 @@ See the documentation of the variable `word-combining-categories'.
 */ );
 
   Vword_separating_categories = Qnil;
-#endif /* MULE */
 }
