@@ -51,7 +51,8 @@ evalcost (int UNUSED (c))
 struct console *cmputc_console;
 
 void
-send_string_to_tty_console (struct console *c, unsigned char *str, int len)
+send_string_to_tty_console (struct console *c, const unsigned char *str,
+                            Bytecount len)
 {
   /* #### Ben sez: don't some terminals need nulls outputted
      for proper timing? */
@@ -66,7 +67,7 @@ send_string_to_tty_console (struct console *c, unsigned char *str, int len)
     }
 
   if (len == 1)
-    Lstream_putc (lstr, *str);
+    (void) Lstream_putc (lstr, *str);
   else if (len > 0)
     Lstream_write (lstr, str, len);
 }
