@@ -27,7 +27,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #include "syntax.h"
 #include "insdel.h"
 
-Lisp_Object Qkill_forward_chars;
+Lisp_Object Qkill_region;
 Lisp_Object Qself_insert_command;
 Lisp_Object Qno_self_insert;
 
@@ -266,7 +266,7 @@ COUNT was explicitly specified.
     }
   else
     {
-      call1 (Qkill_forward_chars, count);
+      call2 (Qkill_region, make_fixnum (BUF_PT (buf)), make_fixnum (pos));
     }
   return Qnil;
 }
@@ -443,26 +443,13 @@ internal_self_insert (Ichar c1, int noautofill)
 
   /* return hairy; */
 }
-
-/* (this comes from Mule but is a generally good idea) */
-
-DEFUN ("self-insert-internal", Fself_insert_internal, 1, 1, 0, /*
-Invoke `self-insert-command' as if CHARACTER is entered from keyboard.
-*/
-       (character))
-{
-  /* This function can GC */
-  CHECK_CHAR_COERCE_INT (character);
-  internal_self_insert (XCHAR (character), 0);
-  return Qnil;
-}
 
 /* module initialization */
 
 void
 syms_of_cmds (void)
 {
-  DEFSYMBOL (Qkill_forward_chars);
+  DEFSYMBOL (Qkill_region);
   DEFSYMBOL (Qself_insert_command);
   DEFSYMBOL (Qoverwrite_mode_binary);
   DEFSYMBOL (Qno_self_insert);
@@ -476,7 +463,6 @@ syms_of_cmds (void)
   DEFSUBR (Fdelete_char);
 
   DEFSUBR (Fself_insert_command);
-  DEFSUBR (Fself_insert_internal);
 }
 
 void
