@@ -1,7 +1,7 @@
 /* Compiler-specific definitions for XEmacs.
    Copyright (C) 1998-1999, 2003 Free Software Foundation, Inc.
    Copyright (C) 1994 Richard Mlynarik.
-   Copyright (C) 1995, 1996, 2000-2004, 2010 Ben Wing.
+   Copyright (C) 1995, 1996, 2000-2004, 2005, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -116,7 +116,14 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #endif
 
 #ifndef DOESNT_RETURN_TYPE
-# if (GCC_VERSION > NEED_GCC (0, 0, 0))
+#ifdef __clang__
+# if __has_builtin(__builtin_unreachable)
+#  define RETURN_NOT_REACHED(value) __builtin_unreachable()
+# endif
+# define DOESNT_RETURN_TYPE(rettype) rettype
+# define DECLARE_DOESNT_RETURN_TYPE(rettype,decl) rettype decl \
+  __attribute__ ((noreturn))
+#elif (GCC_VERSION > NEED_GCC (0, 0, 0))
 #  if (GCC_VERSION >= NEED_GCC (2, 5, 0))
 #   ifndef __INTEL_COMPILER
 #    define RETURN_NOT_REACHED(value) DO_NOTHING
@@ -148,6 +155,24 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 # define DECLARE_DOESNT_RETURN(decl) DECLARE_DOESNT_RETURN_TYPE (void, decl)
 #endif /* DOESNT_RETURN */
 
+#if defined (__clang__) || GCC_VERSION > NEED_GCC (4, 6, 0)
+#define PRAGMA_PUSH_DIAGNOSTICS _Pragma ("GCC diagnostic push")
+#define PRAGMA_POP_DIAGNOSTICS _Pragma ("GCC diagnostic pop")
+#define PRAGMA_IGNORE_DEPRECATED \
+  _Pragma ("GCC diagnostic ignored \"-Wdeprecated-declarations\"")
+#else
+#define PRAGMA_PUSH_DIAGNOSTICS DO_NOTHING
+#define PRAGMA_POP_DIAGNOSTICS DO_NOTHING
+#define PRAGMA_IGNORE_DEPRECATED DO_NOTHING
+#endif /* defined (__clang__) || defined (__GNUC__) */
+
+#if GCC_VERSION > NEED_GCC (12, 0, 0)
+#define PRAGMA_IGNORE_DANGLING_POINTER \
+  _Pragma ("GCC diagnostic ignored \"-Wdangling-pointer\"")
+#else
+#define PRAGMA_IGNORE_DANGLING_POINTER DO_NOTHING
+#endif
+
 /* Another try to fix SunPro C compiler warnings */
 /* "end-of-loop code not reached" */
 /* "statement not reached */
@@ -236,10 +261,12 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 /* Various macros for params/variables used or unused depending on
    config flags. */
 
-#ifdef MULE
-# define USED_IF_MULE(decl) decl
+#ifdef UNICODE_INTERNAL
+# define USED_IF_UNICODE_INTERNAL(decl) decl
+# define USED_IF_OLD_MULE(decl) UNUSED (decl)
 #else
-# define USED_IF_MULE(decl) UNUSED (decl)
+# define USED_IF_UNICODE_INTERNAL(decl) UNUSED (decl)
+# define USED_IF_OLD_MULE(decl) decl
 #endif
 #ifdef HAVE_XFT
 # define USED_IF_XFT(decl) decl
@@ -251,13 +278,6 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #else
 # define USED_IF_SCROLLBARS(decl) UNUSED (decl)
 #endif
-#ifdef NEW_GC
-# define USED_IF_NEW_GC(decl) decl
-# define UNUSED_IF_NEW_GC(decl) UNUSED (decl)
-#else
-# define USED_IF_NEW_GC(decl) UNUSED (decl)
-# define UNUSED_IF_NEW_GC(decl) decl
-#endif
 #ifdef HAVE_TTY
 #define USED_IF_TTY(decl) decl
 #else
@@ -276,9 +296,8 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
    just be deleted. */
 #define USED(x) ((void) (x))
 
-#ifdef DEBUG_XEMACS
+#if defined (DEBUG_XEMACS) || defined (__cplusplus)
 # define REGISTER
-# define register
 #else
 # define REGISTER register
 #endif
