@@ -192,15 +192,11 @@ console_type_create_gtk (void)
   /* CONSOLE_HAS_METHOD (gtk, delete_console); */
 }
 
-void
-reinit_console_type_create_gtk (void)
-{
-  REINITIALIZE_CONSOLE_TYPE (gtk);
-}
-
 void
 vars_of_console_gtk (void)
 {
   staticpro (&Vgtk_seen_characters);
   Vgtk_seen_characters = Qnil;
 }
+
+/* console-gtk.c ends here. */
