@@ -1,14 +1,14 @@
 /* Define mswindows-specific console, device, and frame object for XEmacs.
    Copyright (C) 1989, 1992, 1993, 1994, 1995 Free Software Foundation, Inc.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 2001, 2002 Ben Wing.
+   Copyright (C) 2001, 2002, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -57,7 +55,7 @@ DECLARE_CONSOLE_TYPE (msprinter);
 
 struct Lisp_Devmode
 {
-  struct LCRECORD_HEADER header;
+  NORMAL_LISP_OBJECT_HEADER header;
 
   /* Pointer to the DEVMODE structure */
   DEVMODEW *devmode;
@@ -81,9 +79,6 @@ struct Lisp_Devmode
 
 struct mswindows_device
 {
-#ifdef NEW_GC
-  struct lrecord_header header;
-#endif /* NEW_GC */
   Lisp_Object fontlist;		/* List of (STRING . FIXED-P), device fonts */
   HDC hcdc;			/* Compatible DC */
   DWORD update_tick;		/* Used when device is modified through
@@ -91,16 +86,6 @@ struct mswindows_device
 				   in event-msw.c */
 };
 
-#ifdef NEW_GC
-typedef struct mswindows_device Lisp_Mswindows_Device;
-
-DECLARE_LRECORD (mswindows_device, Lisp_Mswindows_Device);
-
-#define XMSWINDOWS_DEVICE(x) \
-  XRECORD (x, mswindows_device, Lisp_Mswindows_Device)
-#define wrap_mswindows_device(p) wrap_record (p, mswindows_device)
-#define MSWINDOWS_DEVICE_P(x) RECORDP (x, mswindows_device)
-#endif /* NEW_GC */
 
 #define DEVICE_MSWINDOWS_DATA(d) DEVICE_TYPE_DATA (d, mswindows)
 #define DEVICE_MSWINDOWS_FONTLIST(d)    (DEVICE_MSWINDOWS_DATA (d)->fontlist)
@@ -109,9 +94,6 @@ DECLARE_LRECORD (mswindows_device, Lisp_Mswindows_Device);
 
 struct msprinter_device
 {
-#ifdef NEW_GC
-  struct lrecord_header header;
-#endif /* NEW_GC */
   HDC hdc, hcdc;		/* Printer and the comp. DCs */
   HANDLE hprinter;
   Lisp_Object name;
@@ -119,16 +101,6 @@ struct msprinter_device
   Lisp_Object fontlist;
 };
 
-#ifdef NEW_GC
-typedef struct msprinter_device Lisp_Msprinter_Device;
-
-DECLARE_LRECORD (msprinter_device, Lisp_Msprinter_Device);
-
-#define XMSPRINTER_DEVICE(x) \
-  XRECORD (x, msprinter_device, Lisp_Msprinter_Device)
-#define wrap_msprinter_device(p) wrap_record (p, msprinter_device)
-#define MSPRINTER_DEVICE_P(x) RECORDP (x, msprinter_device)
-#endif /* NEW_GC */
 
 #define DEVICE_MSPRINTER_DATA(d) DEVICE_TYPE_DATA (d, msprinter)
 #define DEVICE_MSPRINTER_HDC(d) 	(DEVICE_MSPRINTER_DATA (d)->hdc)
@@ -167,9 +139,6 @@ DECLARE_LRECORD (msprinter_device, Lisp_Msprinter_Device);
 
 struct mswindows_frame
 {
-#ifdef NEW_GC
-  struct lrecord_header header;
-#endif /* NEW_GC */
 
   /* win32 window handle */
   HWND hwnd;
@@ -206,15 +175,9 @@ struct mswindows_frame
   /* Frame title hash value. See frame-msw.c */
   unsigned int title_checksum;
 
-  /* Real character width and height of the frame.
-     FRAME_{HEIGHT,WIDTH} do not work for pixel geometry! */
-  int charheight, charwidth;
-
-#ifdef MULE
   int cursor_x;
   int cursor_y;
   face_index cursor_findex;
-#endif
 
   /* Misc flags */
   int button2_need_lbutton : 1;
@@ -231,16 +194,6 @@ struct mswindows_frame
   XEMACS_RECT_WH *target_rect;
 };
 
-#ifdef NEW_GC
-typedef struct mswindows_frame Lisp_Mswindows_Frame;
-
-DECLARE_LRECORD (mswindows_frame, Lisp_Mswindows_Frame);
-
-#define XMSWINDOWS_FRAME(x) \
-  XRECORD (x, mswindows_frame, Lisp_Mswindows_Frame)
-#define wrap_mswindows_frame(p) wrap_record (p, mswindows_frame)
-#define MSWINDOWS_FRAME_P(x) RECORDP (x, mswindows_frame)
-#endif /* NEW_GC */
 
 #define FRAME_MSWINDOWS_DATA(f) FRAME_TYPE_DATA (f, mswindows)
 
@@ -259,27 +212,23 @@ DECLARE_LRECORD (mswindows_frame, Lisp_Mswindows_Frame);
  (FRAME_MSWINDOWS_DATA (f)->toolbar_checksum[pos])
 #define FRAME_MSWINDOWS_MENU_CHECKSUM(f)  (FRAME_MSWINDOWS_DATA (f)->menu_checksum)
 #define FRAME_MSWINDOWS_TITLE_CHECKSUM(f) (FRAME_MSWINDOWS_DATA (f)->title_checksum)
-#define FRAME_MSWINDOWS_CHARWIDTH(f)	  (FRAME_MSWINDOWS_DATA (f)->charwidth)
-#define FRAME_MSWINDOWS_CHARHEIGHT(f)	  (FRAME_MSWINDOWS_DATA (f)->charheight)
 #define FRAME_MSWINDOWS_TARGET_RECT(f)	  (FRAME_MSWINDOWS_DATA (f)->target_rect)
 
 #define FRAME_MSWINDOWS_POPUP(f)	  (FRAME_MSWINDOWS_DATA (f)->popup)
 
-#ifdef MULE
 # define FRAME_MSWINDOWS_CURSOR_X(f) (FRAME_MSWINDOWS_DATA (f)->cursor_x)
 # define FRAME_MSWINDOWS_CURSOR_Y(f) (FRAME_MSWINDOWS_DATA (f)->cursor_y)
 # define FRAME_MSWINDOWS_CURSOR_FINDEX(f) (FRAME_MSWINDOWS_DATA (f)->cursor_findex)
-#endif
 
 /* Frame check and validation macros */
 #define FRAME_MSWINDOWS_P(frm) CONSOLE_TYPESYM_MSWINDOWS_P (FRAME_TYPE (frm))
 #define CHECK_MSWINDOWS_FRAME(z) CHECK_FRAME_TYPE (z, mswindows)
 #define CONCHECK_MSWINDOWS_FRAME(z) CONCHECK_FRAME_TYPE (z, mswindows)
 
-/* win32 window LONG indices */
-#define XWL_FRAMEOBJ	0
-#define XWL_COUNT	1	/* Number of LONGs that we use */
-#define MSWINDOWS_WINDOW_EXTRA_BYTES	(XWL_COUNT * 4)
+/* win32 window LONG_PTR indices */
+#define XWLP_FRAMEOBJ	0
+#define XWLP_COUNT	1
+#define MSWINDOWS_WINDOW_EXTRA_BYTES	(XWLP_COUNT * sizeof (LONG_PTR))
 
 /*
  * Printer frame, aka printer job
@@ -318,7 +267,7 @@ struct msprinter_frame
 
 struct mswindows_dialog_id
 {
-  struct LCRECORD_HEADER header;
+  NORMAL_LISP_OBJECT_HEADER header;
 
   Lisp_Object frame;
   Lisp_Object callbacks;
