@@ -54,6 +54,7 @@ static const struct memory_description tty_console_data_description_1 [] = {
   { XD_LISP_OBJECT, offsetof (struct tty_console, terminal_type) },
   { XD_LISP_OBJECT, offsetof (struct tty_console, instream) },
   { XD_LISP_OBJECT, offsetof (struct tty_console, outstream) },
+  { XD_LISP_OBJECT, offsetof (struct tty_console, color_alist) },
   { XD_END }
 };
 
@@ -70,6 +71,7 @@ allocate_tty_console_struct (struct console *con)
   CONSOLE_TTY_DATA (con)->terminal_type = Qnil;
   CONSOLE_TTY_DATA (con)->instream = Qnil;
   CONSOLE_TTY_DATA (con)->outstream = Qnil;
+  CONSOLE_TTY_DATA (con)->color_alist = Qnil;
 }
 
 static void
@@ -110,6 +112,7 @@ tty_init_console (struct console *con, Lisp_Object props)
 
   allocate_tty_console_struct (con);
   tty_con = CONSOLE_TTY_DATA (con);
+  tty_con->color_alist = Qnil;
 
   if (internal_equal (tty, Vstdio_str, 0))
     {
@@ -180,15 +183,6 @@ tty_init_console (struct console *con, Lisp_Object props)
   UNGCPRO;
 }
 
-static void
-tty_mark_console (struct console *con)
-{
-  struct tty_console *tty_con = CONSOLE_TTY_DATA (con);
-  mark_object (tty_con->terminal_type);
-  mark_object (tty_con->instream);
-  mark_object (tty_con->outstream);
-}
-
 static int
 tty_initially_selected_for_input (struct console *UNUSED (con))
 {
@@ -474,7 +468,6 @@ console_type_create_tty (void)
 
   /* console methods */
   CONSOLE_HAS_METHOD (tty, init_console);
-  CONSOLE_HAS_METHOD (tty, mark_console);
   CONSOLE_HAS_METHOD (tty, initially_selected_for_input);
   CONSOLE_HAS_METHOD (tty, delete_console);
   CONSOLE_HAS_METHOD (tty, canonicalize_console_connection);
@@ -484,12 +477,6 @@ console_type_create_tty (void)
   CONSOLE_HAS_METHOD (tty, perhaps_init_unseen_key_defaults);
 }
 
-void
-reinit_console_type_create_tty (void)
-{
-  REINITIALIZE_CONSOLE_TYPE (tty);
-}
-
 void
 image_instantiator_format_create_glyphs_tty (void)
 {
