@@ -102,7 +102,7 @@ void x_wm_set_shell_iconic_p (Widget shell, int iconic_p);
 void x_wm_set_cell_size (Widget wmshell, int cw, int ch);
 void x_wm_set_variable_size (Widget wmshell, int width, int height);
 
-const Ascbyte *x_event_name (int event_type);
+const Ibyte *x_event_name (int event_type);
 int check_if_pending_expose_event (struct device *d);
 int x_error_handler (Display *disp, XErrorEvent *event);
 void expect_x_error (Display *dpy);
@@ -131,7 +131,7 @@ Extbyte *menu_separator_style_and_to_external (const Ibyte *s);
 Lisp_Object widget_value_unwind (Lisp_Object closure);
 
 void x_output_string (struct window *w, struct display_line *dl,
-		      Ichar_dynarr *buf, int xpos, int xoffset,
+		      const Ibyte *buf, Bytecount len, int xpos, int xoffset,
 		      int start_pixpos, int width, face_index findex,
 		      int cursor, int cursor_start, int cursor_width,
 		      int cursor_height);
