@@ -145,10 +145,6 @@ print_console (Lisp_Object obj, Lisp_Object printcharfun,
     write_fmt_string_lisp (printcharfun, " on %S", CONSOLE_CONNECTION (con));
   write_fmt_string (printcharfun, " 0x%x>", LISP_OBJECT_UID (obj));
 }
-
-DEFINE_NODUMP_LISP_OBJECT ("console", console, print_console, 0, 0, 0, 
-			   console_description,
-			   struct console);
 
 
 static void
@@ -1159,7 +1155,9 @@ The elements of this list correspond to the arguments of
 void
 syms_of_console (void)
 {
-  INIT_LISP_OBJECT (console);
+  DEFINE_NODUMP_LISP_OBJECT ("console", console, print_console, 0, 0, 0, 
+                             console_description,
+                             struct console);
 
   DEFSUBR (Fvalid_console_type_p);
   DEFSUBR (Fconsole_type_list);
@@ -1219,9 +1217,301 @@ const struct sized_memory_description cted_description = {
 };
 
 static const struct memory_description console_methods_description_1[] = {
+  { XD_ASCII_STRING, offsetof (struct console_methods, name) },
   { XD_LISP_OBJECT, offsetof (struct console_methods, symbol) },
   { XD_LISP_OBJECT, offsetof (struct console_methods, predicate_symbol) },
+
+  /* console methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+                                   init_console_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+                                   initially_selected_for_input_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   delete_console_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   semi_canonicalize_device_connection_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   semi_canonicalize_console_connection_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   canonicalize_console_connection_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   canonicalize_device_connection_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   device_to_console_connection_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   perhaps_init_unseen_key_defaults_method) },
+
+  /* device methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   init_device_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   finish_init_device_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   delete_device_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   asynch_device_change_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   device_system_metrics_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   own_selection_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   disown_selection_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   get_foreign_selection_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   selection_exists_p_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   available_selection_types_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   register_selection_data_type_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   selection_data_type_name_method) },
+  { XD_BLOCK_DATA_PTR, offsetof (struct console_methods,
+				 device_specific_frame_props), 1,
+    { &lisp_object_description } },
+
+  /* frame methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   init_frame_1_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   init_frame_2_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   init_frame_3_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   after_init_frame_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   delete_frame_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   focus_on_frame_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   raise_frame_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   lower_frame_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   enable_frame_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   disable_frame_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   get_mouse_position_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   set_mouse_position_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   make_frame_visible_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   make_frame_invisible_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   iconify_frame_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   frame_property_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   internal_frame_property_p_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   frame_properties_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   set_frame_properties_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   set_frame_size_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   set_frame_position_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   frame_visible_p_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   frame_totally_visible_p_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   frame_iconified_p_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   set_title_from_ibyte_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   set_icon_name_from_ibyte_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   set_frame_pointer_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   set_frame_icon_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   popup_menu_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   get_frame_parent_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   update_frame_external_traits_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   frame_size_fixed_p_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   eject_page_method) },
+
+  /* redisplay methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   left_margin_width_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   right_margin_width_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   text_width_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   output_display_block_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   divider_height_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   eol_cursor_width_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   output_vertical_divider_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   clear_to_window_end_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   clear_region_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   clear_frame_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   window_output_begin_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   frame_output_begin_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   window_output_end_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   frame_output_end_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   flash_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   ring_bell_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   frame_redraw_cursor_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   set_final_cursor_coords_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   bevel_area_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   output_pixmap_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   output_string_method) },
+
+  /* color methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   initialize_color_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   print_color_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   finalize_color_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   color_instance_equal_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   color_instance_hash_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   color_instance_rgb_components_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   valid_color_name_p_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   color_list_method) },
+
+  /* font methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   initialize_font_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   print_font_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   finalize_font_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   font_instance_truename_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   font_instance_properties_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   font_list_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   find_charset_font_method) }, 
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   font_spec_matches_charset_method) },
+
+  /* image methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   print_image_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   finalize_image_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   unmap_subwindow_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   map_subwindow_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   resize_subwindow_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   redisplay_subwindow_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   redisplay_widget_method) },
+  /* Maybe this should be a specifier. Unfortunately specifiers don't
+     allow us to represent things at the toolkit level, which is what
+     is required here. */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   widget_border_width_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   widget_spacing_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   image_instance_equal_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   image_instance_hash_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   init_image_instance_from_eimage_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   locate_pixmap_file_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   colorize_image_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   widget_query_string_geometry_method) },
+
   { XD_LISP_OBJECT, offsetof (struct console_methods, image_conversion_list) },
+
+#ifdef HAVE_TOOLBARS
+  /* toolbar methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   output_frame_toolbars_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   clear_frame_toolbars_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   initialize_frame_toolbars_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   free_frame_toolbars_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   output_toolbar_button_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   redraw_frame_toolbars_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   redraw_exposed_toolbars_method) },
+#endif
+
+#ifdef HAVE_SCROLLBARS
+  /* scrollbar methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   inhibit_scrollbar_slider_size_change_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   free_scrollbar_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   release_scrollbar_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   create_scrollbar_instance_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   update_scrollbar_instance_values_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   update_scrollbar_instance_status_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   scrollbar_pointer_changed_in_window_method) },
+#ifdef MEMORY_USAGE_STATS
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   compute_scrollbar_instance_usage_method) },
+#endif
+  /* Paint the window's deadbox, a rectangle between window
+     borders and two short edges of both scrollbars. */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   redisplay_deadbox_method) },
+#endif /* HAVE_SCROLLBARS */
+
+#ifdef HAVE_MENUBARS
+  /* menubar methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   update_frame_menubars_method) },
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   free_frame_menubars_method) },
+#endif
+
+#ifdef HAVE_DIALOGS
+  /* dialog methods */
+  { XD_FUNCTION_POINTER, offsetof (struct console_methods,
+				   make_dialog_box_internal_method) },
+#endif
   { XD_END }
 };
 
@@ -1245,19 +1535,10 @@ console_type_create (void)
 
   /* then reset the console-type lists, because `dead' is not really
      a valid console type */
-  Dynarr_reset (the_console_type_entry_dynarr);
+  Dynarr_delete (the_console_type_entry_dynarr, 0);
   Vconsole_type_list = Qnil;
 }
 
-void
-reinit_vars_of_console (void)
-{
-  staticpro_nodump (&Vconsole_list);
-  Vconsole_list = Qnil;
-  staticpro_nodump (&Vselected_console);
-  Vselected_console = Qnil;
-}
-
 void
 vars_of_console (void)
 {
@@ -1288,37 +1569,32 @@ for bindings in `function-key-map'.
 #ifdef HAVE_WINDOW_SYSTEM
   Fprovide (intern ("window-system"));
 #endif
+
+  Vconsole_list = Qnil;
+  staticpro_dump_nil (&Vconsole_list);
+
+  Vselected_console = Qnil;
+  staticpro_dump_nil (&Vselected_console);
+
+  Vconsole_defaults = Qnil;
+  staticpro_dump_nil (&Vconsole_defaults);
+
+  Vconsole_local_symbols = Qnil;
+  staticpro_dump_nil (&Vconsole_local_symbols);
 }
 
 /* The docstrings for DEFVAR_* are recorded externally by make-docfile.  */
 #define DEFVAR_CONSOLE_LOCAL_1(lname, field_name, forward_type, magicfun)   \
 do {									    \
-  static const struct symbol_value_forward I_hate_C =			    \
-  { /* struct symbol_value_forward */					    \
-    { /* struct symbol_value_magic */					    \
-      { /* struct old_lcrecord_header */				    \
-	{ /* struct lrecord_header */					    \
-	  lrecord_type_symbol_value_forward, /* lrecord_type_index */	    \
-	  1, /* mark bit */						    \
-	  1, /* c_readonly bit */					    \
-	  1  /* lisp_readonly bit */					    \
-	},								    \
-	0, /* next */							    \
-      },								    \
-      &(console_local_flags.field_name),				    \
-      forward_type							    \
-    },									    \
-    magicfun								    \
-  };									    \
-									    \
+  DEFVAR_SYMVAL_FWD_OBJECT (lname, &(console_local_flags.field_name),	    \
+			   forward_type, magicfun);			    \
   {									    \
-    int offset = ((char *)symbol_value_forward_forward (&I_hate_C)	    \
-		  - (char *)&console_local_flags);			    \
-									    \
-    defvar_magic (lname, &I_hate_C);					    \
-									    \
-    *((Lisp_Object *)(offset + (char *)XCONSOLE (Vconsole_local_symbols)))  \
-      = intern (lname);							    \
+    Bytecount offset = ((Rawbyte *) XSYMBOL_VALUE_FORWARD_OBJECT_FORWARD    \
+			(XSYMBOL_VALUE (intern (lname)))		    \
+			- (Rawbyte *) &console_local_flags);		    \
+    *((Lisp_Object *)(offset +						    \
+		      (Rawbyte *) XCONSOLE (Vconsole_local_symbols)))       \
+	    = intern (lname);						    \
   }									    \
 } while (0)
 
@@ -1333,9 +1609,10 @@ do {									    \
 #define DEFVAR_CONST_CONSOLE_LOCAL(lname, field_name)			\
 	DEFVAR_CONST_CONSOLE_LOCAL_MAGIC (lname, field_name, 0)
 
-#define DEFVAR_CONSOLE_DEFAULTS_MAGIC(lname, field_name, magicfun)	\
-	DEFVAR_SYMVAL_FWD(lname, &(console_local_flags.field_name),	\
-			  SYMVAL_DEFAULT_CONSOLE_FORWARD, magicfun)
+#define DEFVAR_CONSOLE_DEFAULTS_MAGIC(lname, field_name, magicfun)	    \
+	DEFVAR_SYMVAL_FWD_OBJECT(lname, &(console_local_flags.field_name),  \
+				 SYMVAL_DEFAULT_CONSOLE_FORWARD,	    \
+				 magicfun)
 #define DEFVAR_CONSOLE_DEFAULTS(lname, field_name)			\
 	DEFVAR_CONSOLE_DEFAULTS_MAGIC (lname, field_name, 0)
 
@@ -1343,20 +1620,12 @@ static void
 common_init_complex_vars_of_console (void)
 {
   /* Make sure all markable slots in console_defaults
-     are initialized reasonably, so mark_console won't choke.
-   */
-  Lisp_Object defobj = ALLOC_NORMAL_LISP_OBJECT (console);
-  struct console *defs = XCONSOLE (defobj);
-  Lisp_Object symobj = ALLOC_NORMAL_LISP_OBJECT (console);
-  struct console *syms = XCONSOLE (symobj);
+     are initialized reasonably, so KKCC won't choke. */
+  Vconsole_defaults = ALLOC_NORMAL_LISP_OBJECT (console);
+  Vconsole_local_symbols = ALLOC_NORMAL_LISP_OBJECT (console);
 
-  staticpro_nodump (&Vconsole_defaults);
-  staticpro_nodump (&Vconsole_local_symbols);
-  Vconsole_defaults = defobj;
-  Vconsole_local_symbols = symobj;
-
-  nuke_all_console_slots (syms, Qnil);
-  nuke_all_console_slots (defs, Qnil);
+  nuke_all_console_slots (XCONSOLE (Vconsole_local_symbols), Qnil);
+  nuke_all_console_slots (XCONSOLE (Vconsole_defaults), Qnil);
 
   /* Set up the non-nil default values of various console slots.
      Must do these before making the first console.
@@ -1392,7 +1661,8 @@ common_init_complex_vars_of_console (void)
        console.  */
 
     set_lheader_implementation ((struct lrecord_header *)
-				&console_local_flags, &lrecord_console);
+				&console_local_flags,
+                                LRECORD_IMPLEMENTATION (console));
     nuke_all_console_slots (&console_local_flags, make_fixnum (-2));
     console_local_flags.defining_kbd_macro = always_local_resettable;
     console_local_flags.last_kbd_macro = always_local_resettable;
