@@ -253,6 +253,8 @@ gtk_init_device (struct device *d, Lisp_Object UNUSED (props))
 
   if (NULL == visual)
     {
+      /* This is deprecated, the alternative is gdk_screen_get_rgba_visual(),
+         which we have already attempted to call.. */
       visual = gdk_visual_get_best();
     }
 
@@ -375,13 +377,6 @@ gtk_finish_init_device (struct device *d, Lisp_Object UNUSED (props))
   call1 (Qmake_device_late_gtk_entry_point, wrap_device(d));
 }
 
-static void
-gtk_mark_device (struct device *d)
-{
-  mark_object (DEVICE_GTK_WM_COMMAND_FRAME (d));
-  mark_object (DEVICE_GTK_DATA (d)->x_keysym_map_hashtable);
-}
-
 
 /************************************************************************/
 /*                       closing an X connection	                */
@@ -396,28 +391,8 @@ free_gtk_device_struct (struct device *d)
 }
 
 static void
-gtk_delete_device (struct device *d)
+gtk_delete_device (struct device *d, Boolint UNUSED (from_io_error))
 {
-#ifdef FREE_CHECKING
-  extern void (*__free_hook)();
-  int checking_free;
-#endif
-
-  if (1)
-    {
-#ifdef FREE_CHECKING
-      checking_free = (__free_hook != 0);
-
-      /* Disable strict free checking, to avoid bug in X library */
-      if (checking_free)
-	disable_strict_free_check ();
-#endif
-
-#ifdef FREE_CHECKING
-      if (checking_free)
-	enable_strict_free_check ();
-#endif
-    }
   /* g_free(DEVICE_GTK_CONTEXT (d)); */
   free_gtk_device_struct (d);
 }
@@ -601,10 +576,7 @@ The two names differ in capitalization and underscoring.
 */
        (keysym, device))
 {
-  struct device *d = decode_device (device);
-
-  if (!DEVICE_GTK_P (d))
-    gui_error ("Not a GTK device", device);
+  struct device *d = decode_gtk_device (device);
 
   return (NILP (Fgethash (keysym, DEVICE_GTK_DATA (d)->x_keysym_map_hashtable, Qnil)) ?
 	  Qnil : Qt);
@@ -787,9 +759,9 @@ Get the style information for a Gtk device.
 */
        (device))
 {
-  struct device *d = decode_device (device);
   Lisp_Object result = Qnil;
 #ifdef HAVE_GTK2
+  struct device *d = decode_gtk_device (device);
   GtkStyle *style = NULL;
   GtkWidget *app_shell = GTK_WIDGET (DEVICE_GTK_APP_SHELL (d));
   GdkWindow *w = gtk_widget_get_window (app_shell);
@@ -908,7 +880,6 @@ console_type_create_device_gtk (void)
 {
   CONSOLE_HAS_METHOD (gtk, init_device);
   CONSOLE_HAS_METHOD (gtk, finish_init_device);
-  CONSOLE_HAS_METHOD (gtk, mark_device);
   CONSOLE_HAS_METHOD (gtk, delete_device);
   CONSOLE_HAS_METHOD (gtk, device_system_metrics);
   /* CONSOLE_IMPLEMENTATION_FLAGS (gtk, XDEVIMPF_PIXEL_GEOMETRY); */
