@@ -263,7 +263,7 @@ mswindows_finish_init_device (struct device *d,
 }
 
 static void
-mswindows_delete_device (struct device *d)
+mswindows_delete_device (struct device *d, Boolint UNUSED (from_io_error))
 {
 #ifdef HAVE_DRAGNDROP
   DdeNameService (mswindows_dde_mlid, 0L, 0L, DNS_UNREGISTER);
@@ -289,12 +289,6 @@ mswindows_get_workspace_coords (RECT *rc)
   qxeSystemParametersInfo (SPI_GETWORKAREA, 0, rc, 0);
 }
 
-static void
-mswindows_mark_device (struct device *d)
-{
-  mark_object (DEVICE_MSWINDOWS_FONTLIST (d));
-}
-
 static Lisp_Object
 mswindows_device_system_metrics (struct device *d,
 				 enum device_metrics m)
@@ -322,10 +316,21 @@ mswindows_device_system_metrics (struct device *d,
       return make_fixnum (GetDeviceCaps (hdc, BITSPIXEL));
       break;
     case DM_num_color_cells:
-      /* #### SIZEPALETTE only valid if RC_PALETTE bit set in RASTERCAPS,
-         what should we return for a non-palette-based device? */
-      return make_fixnum (GetDeviceCaps (hdc, SIZEPALETTE));
-      break;
+      {
+	if (GetDeviceCaps(hdc, RASTERCAPS) & RC_PALETTE)
+	  {
+	    return make_fixnum (GetDeviceCaps (hdc, SIZEPALETTE));
+	  }
+	else
+	  {
+	    /* SIZEPALETTE is only valid if the RC_PALETTE bit set in
+	       RASTERCAPS. Emulate this if it is zero. */
+	    int bits = min (24, GetDeviceCaps (hdc, BITSPIXEL));
+	    int colors = 1 << bits;
+	    return make_fixnum (colors);
+	  }
+	break;
+      }
 
       /*** Colors ***/
 #define FROB(met, fore, back)				\
@@ -444,7 +449,8 @@ msprinter_init_device_internal (struct device *d, Lisp_Object printer_name)
 }
 
 static void
-msprinter_delete_device_internal (struct device *d)
+msprinter_delete_device_internal (struct device *d,
+				  Boolint UNUSED (from_io_error))
 {
   if (DEVICE_MSPRINTER_HPRINTER (d))
     ClosePrinter (DEVICE_MSPRINTER_HPRINTER (d));
@@ -459,7 +465,7 @@ msprinter_delete_device_internal (struct device *d)
 static int
 msprinter_reinit_device (struct device *d, Lisp_Object devname)
 {
-  msprinter_delete_device_internal (d);
+  msprinter_delete_device_internal (d, 0);
   return msprinter_init_device_internal (d, devname);
 }
 
@@ -534,11 +540,11 @@ msprinter_init_device (struct device *d, Lisp_Object UNUSED (props))
 }
 
 static void
-msprinter_delete_device (struct device *d)
+msprinter_delete_device (struct device *d, Boolint from_io_error)
 {
   if (d->device_data)
     {
-      msprinter_delete_device_internal (d);
+      msprinter_delete_device_internal (d, from_io_error);
 
       /* Disassociate the selected devmode with the device */
       if (!NILP (DEVICE_MSPRINTER_DEVMODE (d)))
@@ -587,14 +593,6 @@ msprinter_device_system_metrics (struct device *d,
   return Qunbound;
 }
 
-static void
-msprinter_mark_device (struct device *d)
-{
-  mark_object (DEVICE_MSPRINTER_FONTLIST (d));
-  mark_object (DEVICE_MSPRINTER_DEVMODE (d));
-  mark_object (DEVICE_MSPRINTER_NAME (d));
-}
-
 
 /************************************************************************/
 /*                      printer Lisp subroutines                        */
@@ -1161,12 +1159,6 @@ hash_devmode (Lisp_Object obj, int depth, Boolint UNUSED (equalp))
 		internal_hash (dm->printer_name, depth + 1, 0));
 }
 
-DEFINE_NODUMP_LISP_OBJECT ("msprinter-settings", devmode, print_devmode,
-			   finalize_devmode,
-			   equal_devmode, hash_devmode, 
-			   devmode_description,
-			   Lisp_Devmode);
-
 static Lisp_Object
 allocate_devmode (DEVMODEW* src_devmode, int do_copy,
 		  Lisp_Object src_name, struct device *d)
@@ -1321,8 +1313,9 @@ values.  Return value is nil if there are no printers installed.
 void
 syms_of_device_mswindows (void)
 {
-  INIT_LISP_OBJECT (devmode);
-
+  DEFINE_NODUMP_LISP_OBJECT ("msprinter-settings", devmode, print_devmode,
+                             finalize_devmode, equal_devmode, hash_devmode, 
+                             devmode_description, Lisp_Devmode);
 
   DEFSUBR (Fmsprinter_get_settings);
   DEFSUBR (Fmsprinter_select_settings);
@@ -1346,13 +1339,11 @@ console_type_create_device_mswindows (void)
 {
   CONSOLE_HAS_METHOD (mswindows, init_device);
   CONSOLE_HAS_METHOD (mswindows, finish_init_device);
-  CONSOLE_HAS_METHOD (mswindows, mark_device);
   CONSOLE_HAS_METHOD (mswindows, delete_device);
   CONSOLE_HAS_METHOD (mswindows, device_system_metrics);
   CONSOLE_IMPLEMENTATION_FLAGS (mswindows, XDEVIMPF_PIXEL_GEOMETRY);
 
   CONSOLE_HAS_METHOD (msprinter, init_device);
-  CONSOLE_HAS_METHOD (msprinter, mark_device);
   CONSOLE_HAS_METHOD (msprinter, delete_device);
   CONSOLE_HAS_METHOD (msprinter, device_system_metrics);
   CONSOLE_IMPLEMENTATION_FLAGS (msprinter, (XDEVIMPF_PIXEL_GEOMETRY
