@@ -60,9 +60,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #endif /* HAVE_SHLIB and LWLIB_USES_ATHENA and not HAVE_ATHENA_3D */
 
 Lisp_Object Vx_app_defaults_directory;
-#ifdef MULE
 Lisp_Object Qget_coding_system_from_locale;
-#endif
 
 /* Qdisplay in general.c */
 Lisp_Object Qx_error;
@@ -75,21 +73,22 @@ Lisp_Object Vx_initial_argv_list; /* #### ugh! */
 
 Lisp_Object Vgc_cache_hash_table_test;
 
-/* Shut up G++ 4.3. */
+/* Shut up G++ 4.3. OPTION and RESOURCE are declared as char *, not
+   const char *, not String, in XrmOptionDescRec. */
 #define Xrm_ODR(option,resource,type,default) \
-  { (String) option, (String) resource, type, default }
+  { (char *) option, (char *) resource, type, (XPointer) default }
 
 static XrmOptionDescRec emacs_options[] =
 {
   Xrm_ODR ("-geometry", ".geometry", XrmoptionSepArg, NULL),
-  Xrm_ODR ("-iconic", ".iconic", XrmoptionNoArg, (String) "yes"),
+  Xrm_ODR ("-iconic", ".iconic", XrmoptionNoArg, "yes"),
 
   Xrm_ODR ("-internal-border-width", "*EmacsFrame.internalBorderWidth", XrmoptionSepArg, NULL),
   Xrm_ODR ("-ib",                    "*EmacsFrame.internalBorderWidth", XrmoptionSepArg, NULL),
   Xrm_ODR ("-scrollbar-width",       "*EmacsFrame.scrollBarWidth",      XrmoptionSepArg, NULL),
   Xrm_ODR ("-scrollbar-height",      "*EmacsFrame.scrollBarHeight",     XrmoptionSepArg, NULL),
 
-  Xrm_ODR ("-privatecolormap", ".privateColormap", XrmoptionNoArg,  (String) "yes"),
+  Xrm_ODR ("-privatecolormap", ".privateColormap", XrmoptionNoArg,  "yes"),
   Xrm_ODR ("-visual",   ".EmacsVisual",	    XrmoptionSepArg, NULL),
 
   /* #### Beware!  If the type of the shell changes, update this. */
@@ -191,9 +190,8 @@ get_x_display (Lisp_Object device)
 }
 
 static Lisp_Object
-coding_system_of_xrm_database (XrmDatabase USED_IF_MULE (db))
+coding_system_of_xrm_database (XrmDatabase db)
 {
-#ifdef MULE
   const Extbyte *locale;
   static XrmDatabase last_xrm_db; 
 
@@ -215,9 +213,6 @@ coding_system_of_xrm_database (XrmDatabase USED_IF_MULE (db))
 			      build_extstring (locale, Qx_hpc_encoding));
 
   return last_coding_system;
-#else
-  return Qbinary;
-#endif
 }
 
 
@@ -248,16 +243,23 @@ Xatoms_of_device_x (struct device *d)
 static void
 sanity_check_geometry_resource (Display *dpy)
 {
-  Extbyte *app_name, *app_class, *s;
-  Extbyte buf1 [255], buf2 [255];
+  String app_name, app_class;
+  Extbyte *buf1, *buf2, *s;
   Extbyte *type;
   XrmValue value;
   XtGetApplicationNameAndClass (dpy, &app_name, &app_class);
+
+  buf1 = alloca_extbytes (strlen (app_name)
+                          + sizeof ("._no_._such_._resource_.geometry"));
+  buf2 = alloca_extbytes (strlen (app_class)
+                          + sizeof ("._no_._such_._resource_.geometry"));
+
   strcpy (buf1, app_name);
   strcpy (buf2, app_class);
   for (s = buf1; *s; s++) if (*s == '.') *s = '_';
   strcat (buf1, "._no_._such_._resource_.geometry");
   strcat (buf2, "._no_._such_._resource_.Geometry");
+
   if (XrmGetResource (XtDatabase (dpy), buf1, buf2, &type, &value) == True)
     {
       Ibyte *app_name_int, *app_class_int, *value_addr_int;
@@ -525,8 +527,8 @@ x_init_device (struct device *d, Lisp_Object UNUSED (props))
   Widget app_shell;
   int argc;
   Extbyte **argv;
-  char *app_class;
-  char *app_name;
+  String app_class;
+  String app_name;
   const char *disp_name;
   Visual *visual = NULL;
   int depth = 8;		/* shut up the compiler */
@@ -670,7 +672,6 @@ x_init_device (struct device *d, Lisp_Object UNUSED (props))
 
   screen = DefaultScreen (dpy);
 
-#ifdef MULE
   {
     /* Read in locale-specific resources from
        data-directory/app-defaults/$LANG/Emacs.
@@ -747,7 +748,6 @@ x_init_device (struct device *d, Lisp_Object UNUSED (props))
  }
 
  no_data_directory:
-#endif /* MULE */
 
   if (NILP (DEVICE_NAME (d)))
     DEVICE_NAME (d) = display;
@@ -885,9 +885,9 @@ x_init_device (struct device *d, Lisp_Object UNUSED (props))
      be the place.  Make sure it doesn't conflict with GNOME. */
   {
     Arg al[3];
-    Xt_SET_ARG (al[0], XtNvisual,   visual);
-    Xt_SET_ARG (al[1], XtNdepth,    depth);
-    Xt_SET_ARG (al[2], XtNcolormap, cmap);
+    XtSetArg (al[0], XtNvisual,   visual);
+    XtSetArg (al[1], XtNdepth,    depth);
+    XtSetArg (al[2], XtNcolormap, cmap);
 
     app_shell = XtAppCreateShell (NULL, app_class,
 				  applicationShellWidgetClass,
@@ -904,12 +904,12 @@ x_init_device (struct device *d, Lisp_Object UNUSED (props))
      and set it to the size of the root window for child placement purposes */
   {
     Arg al[5];
-    Xt_SET_ARG (al[0], XtNmappedWhenManaged, False);
-    Xt_SET_ARG (al[1], XtNx, 0);
-    Xt_SET_ARG (al[2], XtNy, 0);
-    Xt_SET_ARG (al[3], XtNwidth,
+    XtSetArg (al[0], XtNmappedWhenManaged, False);
+    XtSetArg (al[1], XtNx, 0);
+    XtSetArg (al[2], XtNy, 0);
+    XtSetArg (al[3], XtNwidth,
 		WidthOfScreen  (ScreenOfDisplay (dpy, screen)));
-    Xt_SET_ARG (al[4], XtNheight,
+    XtSetArg (al[4], XtNheight,
 		HeightOfScreen (ScreenOfDisplay (dpy, screen)));
     XtSetValues (app_shell, al, countof (al));
     XtRealizeWidget (app_shell);
@@ -955,14 +955,6 @@ x_finish_init_device (struct device *d, Lisp_Object UNUSED (props))
   call1 (Qmake_device_late_x_entry_point, wrap_device (d));
 }
 
-static void
-x_mark_device (struct device *d)
-{
-  mark_object (DEVICE_X_WM_COMMAND_FRAME (d));
-  mark_object (DEVICE_X_DATA (d)->x_keysym_map_hash_table);
-  mark_object (DEVICE_X_GC_CACHE (d)->table);
-}
-
 
 /************************************************************************/
 /*                       closing an X connection	                */
@@ -975,25 +967,14 @@ free_x_device_struct (struct device *d)
 }
 
 static void
-x_delete_device (struct device *d)
+x_delete_device (struct device *d, Boolint from_io_error)
 {
-  Display *display;
-#ifdef FREE_CHECKING
-  extern void (*__free_hook) (void *);
-  int checking_free;
-#endif
+  Display *display = DEVICE_X_DISPLAY (d);
 
-  display = DEVICE_X_DISPLAY (d);
-
-  if (display)
+  if (display && !from_io_error)
     {
-#ifdef FREE_CHECKING
-      checking_free = (__free_hook != 0);
-
-      /* Disable strict free checking, to avoid bug in X library */
-      if (checking_free)
-	disable_strict_free_check ();
-#endif
+      /* This leaks if we are called from an IO error, which is better than
+	 crashing. */
 
       free_x_gc_cache_entries (d);
       if (DEVICE_X_DATA (d)->x_modifier_keymap)
@@ -1009,10 +990,6 @@ x_delete_device (struct device *d)
 
       XtCloseDisplay (display);
       DEVICE_X_DISPLAY (d) = 0;
-#ifdef FREE_CHECKING
-      if (checking_free)
-	enable_strict_free_check ();
-#endif
     }
 
   free_x_device_struct (d);
@@ -1457,7 +1434,7 @@ x_get_resource_prefix (Lisp_Object locale, Lisp_Object device,
   *display_out = DEVICE_X_DISPLAY (XDEVICE (device));
 
   {
-    Extbyte *appname, *appclass;
+    String appname, appclass;
     int name_len, class_len;
     XtGetApplicationNameAndClass (*display_out, &appname, &appclass);
     name_len  = strlen (appname);
@@ -2122,9 +2099,7 @@ syms_of_device_x (void)
   DEFSYMBOL (Qmake_device_early_x_entry_point);
   DEFSYMBOL (Qmake_device_late_x_entry_point);
 
-#ifdef MULE
   DEFSYMBOL (Qget_coding_system_from_locale);
-#endif
 
   Vgc_cache_hash_table_test = define_gc_cache_hash_table_test ();
   staticpro (&Vgc_cache_hash_table_test);
@@ -2149,20 +2124,10 @@ console_type_create_device_x (void)
   reinit_console_type_create_device_x ();
   CONSOLE_HAS_METHOD (x, init_device);
   CONSOLE_HAS_METHOD (x, finish_init_device);
-  CONSOLE_HAS_METHOD (x, mark_device);
   CONSOLE_HAS_METHOD (x, delete_device);
   CONSOLE_HAS_METHOD (x, device_system_metrics);
 }
 
-void
-reinit_vars_of_device_x (void)
-{
-  error_expected = 0;
-  error_occurred = 0;
-
-  in_resource_setting = 0;
-}
-
 void
 vars_of_device_x (void)
 {
