@@ -135,10 +135,6 @@ print_device (Lisp_Object obj, Lisp_Object printcharfun,
     write_fmt_string_lisp (printcharfun, " on %S", DEVICE_CONNECTION (d));
   write_fmt_string (printcharfun, " 0x%x>", LISP_OBJECT_UID (obj));
 }
-
-DEFINE_NODUMP_LISP_OBJECT ("device", device, print_device, 0, 0, 0, 
-			   device_description,
-			   struct device);
 
 int
 valid_device_class_p (Lisp_Object class_)
@@ -191,9 +187,7 @@ allocate_device (Lisp_Object console)
     make_lisp_hash_table (20, HASH_TABLE_KEY_WEAK, Qequal);
   d->font_instance_cache =
     make_lisp_hash_table (20, HASH_TABLE_KEY_WEAK, Qequal);
-#ifdef MULE
   initialize_charset_font_caches (d);
-#endif
   /*
      Note that the image instance cache is actually bi-level.
      See device.h.  We use a low number here because most of the
@@ -452,7 +446,6 @@ clear_all_console_local_caches (void)
 void
 clear_default_devices (void)
 {
-  Vdefault_device_plist = Qnil;
   clear_all_console_local_caches ();
 }
 
@@ -875,7 +868,7 @@ delete_device_internal (struct device *d, Boolint force,
   if (EQ (device, Vdefault_device))
     Vdefault_device = find_other_device (device, 0);
 
-  MAYBE_DEVMETH (d, delete_device, (d));
+  MAYBE_DEVMETH (d, delete_device, (d, from_io_error));
 
   /* Now see if we're the default device, and thus need to be changed. */
   {
@@ -1375,7 +1368,8 @@ call_critical_lisp_code (struct device *d, Lisp_Object function,
 void
 syms_of_device (void)
 {
-  INIT_LISP_OBJECT (device);
+  DEFINE_NODUMP_LISP_OBJECT ("device", device, print_device, 0, 0, 0, 
+                             device_description, struct device);
 
   DEFSUBR (Fvalid_device_class_p);
   DEFSUBR (Fdevice_class_list);
@@ -1456,14 +1450,6 @@ syms_of_device (void)
   DEFSYMBOL (Qsecurity);
 }
 
-void
-reinit_vars_of_device (void)
-{
-  staticpro_nodump (&Vdefault_device);
-  Vdefault_device = Qnil;
-  asynch_device_change_pending = 0;
-}
-
 void
 vars_of_device (void)
 {
@@ -1484,11 +1470,14 @@ One argument, the to-be-deleted device.
 
   /* Plist of device types and their default devices. */
   Vdefault_device_plist = Qnil;
-  staticpro (&Vdefault_device_plist);
+  staticpro_dump_nil (&Vdefault_device_plist);
 
   Vdevice_class_list = list3 (Qcolor, Qgrayscale, Qmono);
   staticpro (&Vdevice_class_list);
 
+  Vdefault_device = Qnil;
+  staticpro_dump_nil (&Vdefault_device);
+
   /* Death to devices.el !!! */
   Fprovide (intern ("devices"));
 }
