@@ -1,7 +1,7 @@
 /* Implementation of the hash table lisp object type.
    Copyright (C) 1992, 1993, 1994 Free Software Foundation, Inc.
    Copyright (C) 1995, 1996, 2002, 2004, 2010 Ben Wing.
-   Copyright (C) 1997 Free Software Foundation, Inc.
+   Copyright (C) 1997, 2025 Free Software Foundation, Inc.
 
 This file is part of XEmacs.
 
@@ -123,14 +123,13 @@ struct Hash_Table_Test
 static const struct memory_description hash_table_test_description[] =
   {
     { XD_LISP_OBJECT, offsetof (struct Hash_Table_Test, name) },
+    { XD_FUNCTION_POINTER, offsetof (struct Hash_Table_Test, equal_function) },
+    { XD_FUNCTION_POINTER, offsetof (struct Hash_Table_Test, hash_function) },
     { XD_LISP_OBJECT, offsetof (struct Hash_Table_Test, lisp_equal_function) },
     { XD_LISP_OBJECT, offsetof (struct Hash_Table_Test, lisp_hash_function) },
     { XD_END }
   };
 
-DEFINE_DUMPABLE_INTERNAL_LISP_OBJECT ("hash-table-test", hash_table_test,
-                                      hash_table_test_description,
-                                      Hash_Table_Test);
 /* A hash table. */
 
 struct Lisp_Hash_Table
@@ -158,14 +157,10 @@ struct Lisp_Hash_Table
   (((size) > 4096 && EQ (Veq_hash_table_test, test)) ? 0.7 : 0.6)
 
 #define HASHCODE(key, ht, http)						\
-  ((((!EQ (Veq_hash_table_test, ht->test)) ?                            \
+  ((((!EQ (Veq_hash_table_test, wrap_hash_table_test (http))) ?		\
      (http)->hash_function (http, key) :                                \
      LISP_HASH (key)) * (ht)->golden_ratio) % (ht)->size)
 
-#define KEYS_EQUAL_P(key1, key2, test, http)                      \
-  (EQ (key1, key2) || ((!EQ (Veq_hash_table_test, test) &&        \
-                        (http->equal_function) (http, key1, key2))))
-
 #define LINEAR_PROBING_LOOP(probe, entries, size)		\
   for (;							\
        !HTENTRY_CLEAR_P (probe) ||				\
@@ -173,7 +168,7 @@ struct Lisp_Hash_Table
 	  (probe = entries, !HTENTRY_CLEAR_P (probe)) : 0);	\
        probe++)
 
-#ifdef ERROR_CHECK_STRUCTURES
+#ifdef SLEDGEHAMMER_CHECK_HASH_TABLES /* Not usually defined. */
 static void
 check_hash_table_invariants (Lisp_Hash_Table *ht)
 {
@@ -221,8 +216,7 @@ hash_table_size (Elemcount requested_size)
 }
 
 
-
-static int
+static Boolint
 lisp_object_eql_equal (const Hash_Table_Test *UNUSED (http), Lisp_Object obj1,
                        Lisp_Object obj2)
 {
@@ -237,7 +231,7 @@ lisp_object_eql_hash (const Hash_Table_Test *UNUSED (http), Lisp_Object obj)
     internal_hash (obj, 0, 0) : LISP_HASH (obj);
 }
 
-static int
+static Boolint
 lisp_object_equal_equal (const Hash_Table_Test *UNUSED (http),
                          Lisp_Object obj1, Lisp_Object obj2)
 {
@@ -256,7 +250,7 @@ lisp_object_equalp_hash (const Hash_Table_Test *UNUSED (http), Lisp_Object obj)
   return internal_hash (obj, 0, 1);
 }
 
-static int
+static Boolint
 lisp_object_equalp_equal (const Hash_Table_Test *UNUSED (http),
                           Lisp_Object obj1, Lisp_Object obj2)
 {
@@ -296,7 +290,7 @@ lisp_object_general_hash (const Hash_Table_Test *http, Lisp_Object obj)
   dead_wrong_type_argument (Qintegerp, res);
 }
 
-static int
+static Boolint
 lisp_object_general_equal (const Hash_Table_Test *http, Lisp_Object obj1,
                            Lisp_Object obj2)
 {
@@ -645,21 +639,18 @@ const struct memory_description hash_table_description[] = {
   { XD_END }
 };
 
-DEFINE_DUMPABLE_LISP_OBJECT ("hash-table", hash_table, print_hash_table,
-			     finalize_hash_table,
-			     hash_table_equal, hash_table_hash,
-			     hash_table_description,
-			     Lisp_Hash_Table);
-
+#ifdef SLEDGEHAMMER_CHECK_HASH_TABLES
 static Lisp_Hash_Table *
-xhash_table (Lisp_Object hash_table)
+xhash_table (Lisp_Object obj)
 {
-  /* #### What's going on here?  Why the gc_in_progress check? */
-  if (!gc_in_progress)
-    CHECK_HASH_TABLE (hash_table);
-  check_hash_table_invariants (XHASH_TABLE (hash_table));
-  return XHASH_TABLE (hash_table);
+  CHECK_HASH_TABLE (obj);
+  check_hash_table_invariants (XRECORD (obj, hash_table,
+					Lisp_Hash_Table));
+  return XRECORD (obj, hash_table, Lisp_Hash_Table);
 }
+#undef XHASH_TABLE
+#define XHASH_TABLE xhash_table
+#endif
 
 
 /************************************************************************/
@@ -839,7 +830,8 @@ hash_table_test_validate (Lisp_Object UNUSED (keyword), Lisp_Object value,
       return 1;
     }
 
-  lookup = Fassq (value, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
+  lookup
+    = assq_no_quit (value, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
   if (NILP (lookup))
     {
       maybe_invalid_constant ("Invalid hash table test",
@@ -859,7 +851,8 @@ decode_hash_table_test (Lisp_Object obj)
       obj = Qeql;
     }
 
-  result = Fassq (obj, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
+  result
+    = assq_no_quit (obj, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
   if (NILP (result))
     {
       invalid_constant ("Invalid hash table test", obj);
@@ -1089,7 +1082,7 @@ Return t if OBJECT is a hash table, else nil.
   return HASH_TABLEP (object) ? Qt : Qnil;
 }
 
-DEFUN ("make-hash-table", Fmake_hash_table, 0, MANY, 0, /*
+DEFUN ("make-hash-table", Fmake_hash_table, 0, KEYWORDS, 0, /*
 Return a new empty hash table object.
 Use Common Lisp style keywords to specify hash table properties.
 
@@ -1144,11 +1137,11 @@ arguments: (&key TEST SIZE REHASH-SIZE REHASH-THRESHOLD WEAKNESS)
        (int nargs, Lisp_Object *args))
 {
 #ifndef NEED_TO_HANDLE_21_4_CODE
-  PARSE_KEYWORDS (Fmake_hash_table, nargs, args, 5,
+  PARSE_KEYWORDS (Fmake_hash_table,
                   (test, size, rehash_size, rehash_threshold, weakness),
                   NULL);
 #else
-  PARSE_KEYWORDS (Fmake_hash_table, nargs, args, 6,
+  PARSE_KEYWORDS (Fmake_hash_table,
                   (test, size, rehash_size, rehash_threshold, weakness,
 		   type), (type = Qunbound, weakness = Qunbound));
 
@@ -1188,11 +1181,15 @@ The keys and values will not themselves be copied.
 */
        (hash_table))
 {
-  const Lisp_Hash_Table *ht_old = xhash_table (hash_table);
   Lisp_Object obj = ALLOC_NORMAL_LISP_OBJECT (hash_table);
   Lisp_Hash_Table *ht = XHASH_TABLE (obj);
-  copy_lisp_object (obj, hash_table);
+  const Lisp_Hash_Table *ht_old;
 
+  CHECK_HASH_TABLE (hash_table);
+  ht_old = XHASH_TABLE (hash_table);
+
+  copy_lisp_object (obj, hash_table);
+  
   /* We leave room for one never-occupied sentinel htentry at the end.  */
   ht->hentries = allocate_hash_table_entries (ht_old->size + 1);
   memcpy (ht->hentries, ht_old->hentries, (ht_old->size + 1) * sizeof (htentry));
@@ -1241,7 +1238,7 @@ resize_hash_table (Lisp_Hash_Table *ht, Elemcount new_size)
 void
 pdump_reorganize_hash_table (Lisp_Object hash_table)
 {
-  const Lisp_Hash_Table *ht = xhash_table (hash_table);
+  const Lisp_Hash_Table *ht = XHASH_TABLE (hash_table);
   /* We leave room for one never-occupied sentinel htentry at the end.  */
   htentry *new_entries = allocate_hash_table_entries (ht->size + 1);
   htentry *e, *sentinel;
@@ -1269,18 +1266,63 @@ enlarge_hash_table (Lisp_Hash_Table *ht)
   resize_hash_table (ht, new_size);
 }
 
+static inline Lisp_Object
+choose_hash_table_test_for_lookup (const Lisp_Hash_Table *ht, Lisp_Object key)
+{
+  Lisp_Object test = ht->test;
+
+  if (EQ (test, Veq_hash_table_test))
+    {
+      return test;
+    }
+
+  if (EQ (test, Veql_hash_table_test))
+    {
+      if (!NON_FIXNUM_NUMBER_P (key))
+	{
+	  return Veq_hash_table_test;
+	}
+      return test;
+    }
+  
+  if (EQ (test, Vequal_hash_table_test))
+    {
+      if (FIXNUMP (key) || CHARP (key) || SYMBOLP (key))
+	{
+	  return Veq_hash_table_test;
+	}
+      return test;
+    }
+
+  if (EQ (test, Vequalp_hash_table_test) && SYMBOLP (key))
+    {
+      return Veq_hash_table_test;
+    }
+
+  return test;
+}
+
 htentry *
 find_htentry (Lisp_Object key, const Lisp_Hash_Table *ht)
 {
-  Lisp_Object test = ht->test;
+  Lisp_Object test = choose_hash_table_test_for_lookup (ht, key);
   Hash_Table_Test *http = XHASH_TABLE_TEST (test);
 
   htentry *entries = ht->hentries;
   htentry *probe = entries + HASHCODE (key, ht, http);
 
-  LINEAR_PROBING_LOOP (probe, entries, ht->size)
-    if (KEYS_EQUAL_P (probe->key, key, test, http))
-      break;
+  if (EQ (test, Veq_hash_table_test))
+    {
+      LINEAR_PROBING_LOOP (probe, entries, ht->size)
+	if (EQ (probe->key, key))
+	  break;
+    }
+  else
+    {
+      LINEAR_PROBING_LOOP (probe, entries, ht->size)
+	if ((http->equal_function) (http, probe->key, key))
+	  break;
+    }
 
   return probe;
 }
@@ -1325,8 +1367,10 @@ If there is no corresponding value, return DEFAULT (which defaults to nil).
 */
        (key, hash_table, default_))
 {
-  const Lisp_Hash_Table *ht = xhash_table (hash_table);
-  htentry *e = find_htentry (key, ht);
+  htentry *e;
+
+  CHECK_HASH_TABLE (hash_table);
+  e = find_htentry (key, XHASH_TABLE (hash_table));
 
   return HTENTRY_CLEAR_P (e) ? default_ : e->value;
 }
@@ -1336,8 +1380,12 @@ Hash KEY to VALUE in HASH-TABLE, and return VALUE.
 */
        (key, value, hash_table))
 {
-  Lisp_Hash_Table *ht = xhash_table (hash_table);
-  htentry *e = find_htentry (key, ht);
+  htentry *e;
+  Lisp_Hash_Table *ht;
+
+  CHECK_HASH_TABLE (hash_table);
+  ht = XHASH_TABLE (hash_table);
+  e = find_htentry (key, ht);
 
   if (!HTENTRY_CLEAR_P (e))
     return e->value = value;
@@ -1385,8 +1433,12 @@ Return non-nil if an entry was removed.
 */
        (key, hash_table))
 {
-  Lisp_Hash_Table *ht = xhash_table (hash_table);
-  htentry *e = find_htentry (key, ht);
+  Lisp_Hash_Table *ht;
+  htentry *e;
+
+  CHECK_HASH_TABLE (hash_table);
+  ht = XHASH_TABLE (hash_table);
+  e = find_htentry (key, ht);
 
   if (HTENTRY_CLEAR_P (e))
     return Qnil;
@@ -1401,9 +1453,12 @@ Return HASH-TABLE.
 */
        (hash_table))
 {
-  Lisp_Hash_Table *ht = xhash_table (hash_table);
+  Lisp_Hash_Table *ht;
   htentry *e, *sentinel;
 
+  CHECK_HASH_TABLE (hash_table);
+  ht = XHASH_TABLE (hash_table);
+
   for (e = ht->hentries, sentinel = e + ht->size; e < sentinel; e++)
     CLEAR_HTENTRY (e);
   ht->count = 0;
@@ -1420,7 +1475,8 @@ Return the number of entries in HASH-TABLE.
 */
        (hash_table))
 {
-  return make_fixnum (xhash_table (hash_table)->count);
+  CHECK_HASH_TABLE (hash_table);
+  return make_fixnum (XHASH_TABLE (hash_table)->count);
 }
 
 DEFUN ("hash-table-test", Fhash_table_test, 1, 1, 0, /*
@@ -1441,7 +1497,8 @@ This is the current number of slots in HASH-TABLE, whether occupied or not.
 */
        (hash_table))
 {
-  return make_fixnum (xhash_table (hash_table)->size);
+  CHECK_HASH_TABLE (hash_table);
+  return make_fixnum (XHASH_TABLE (hash_table)->size);
 }
 
 DEFUN ("hash-table-rehash-size", Fhash_table_rehash_size, 1, 1, 0, /*
@@ -1451,7 +1508,8 @@ is enlarged when the rehash threshold is exceeded.
 */
        (hash_table))
 {
-  return make_float (xhash_table (hash_table)->rehash_size);
+  CHECK_HASH_TABLE (hash_table);
+  return make_float (XHASH_TABLE (hash_table)->rehash_size);
 }
 
 DEFUN ("hash-table-rehash-threshold", Fhash_table_rehash_threshold, 1, 1, 0, /*
@@ -1461,7 +1519,8 @@ beyond which the HASH-TABLE is enlarged by rehashing.
 */
        (hash_table))
 {
-  return make_float (xhash_table (hash_table)->rehash_threshold);
+  CHECK_HASH_TABLE (hash_table);
+  return make_float (XHASH_TABLE (hash_table)->rehash_threshold);
 }
 
 DEFUN ("hash-table-weakness", Fhash_table_weakness, 1, 1, 0, /*
@@ -1470,7 +1529,8 @@ This can be one of `nil', `key-and-value', `key-or-value', `key' or `value'.
 */
        (hash_table))
 {
-  switch (xhash_table (hash_table)->weakness)
+  CHECK_HASH_TABLE (hash_table);
+  switch (XHASH_TABLE (hash_table)->weakness)
     {
     case HASH_TABLE_WEAK:		return Qkey_and_value;
     case HASH_TABLE_KEY_WEAK:		return Qkey;
@@ -1487,7 +1547,8 @@ This can be one of `non-weak', `weak', `key-weak' or `value-weak'.
 */
        (hash_table))
 {
-  switch (xhash_table (hash_table)->weakness)
+  CHECK_HASH_TABLE (hash_table);
+  switch (XHASH_TABLE (hash_table)->weakness)
     {
     case HASH_TABLE_WEAK:		return Qweak;
     case HASH_TABLE_KEY_WEAK:		return Qkey_weak;
@@ -1590,13 +1651,18 @@ may remhash or puthash the entry currently being processed by FUNCTION.
 */
        (function, hash_table))
 {
-  const Lisp_Hash_Table * const ht = xhash_table (hash_table);
-  Lisp_Object * const objs = copy_compress_hentries (ht);
+  const Lisp_Hash_Table *ht;
+  Lisp_Object *objs;
   Lisp_Object args[3];
   const Lisp_Object *pobj, *end;
   int speccount = specpdl_depth ();
   struct gcpro gcpro1;
 
+  CHECK_HASH_TABLE (hash_table);
+  ht = XHASH_TABLE (hash_table);
+
+  objs = copy_compress_hentries (ht);
+
   record_unwind_protect (maphash_unwind, make_opaque_ptr ((void *)objs));
   GCPRO1 (objs[0]);
   gcpro1.nvars = 2 * ht->count;
@@ -1622,7 +1688,7 @@ may remhash or puthash the entry currently being processed by FUNCTION.
    Mapping terminates if FUNCTION returns something other than 0. */
 void
 elisp_maphash_unsafe (maphash_function_t function,
-	       Lisp_Object hash_table, void *extra_arg)
+	              Lisp_Object hash_table, void *extra_arg)
 {
   const Lisp_Hash_Table *ht = XHASH_TABLE (hash_table);
   const htentry *e, *sentinel;
@@ -1640,11 +1706,15 @@ void
 elisp_maphash (maphash_function_t function,
 	       Lisp_Object hash_table, void *extra_arg)
 {
-  const Lisp_Hash_Table * const ht = xhash_table (hash_table);
-  Lisp_Object * const objs = copy_compress_hentries (ht);
+  const Lisp_Hash_Table *ht;
+  Lisp_Object *objs;
   const Lisp_Object *pobj, *end;
   int speccount = specpdl_depth ();
   struct gcpro gcpro1;
+  
+  CHECK_HASH_TABLE (hash_table);
+  ht = XHASH_TABLE (hash_table);
+  objs = copy_compress_hentries (ht);
 
   record_unwind_protect (maphash_unwind, make_opaque_ptr ((void *)objs));
   GCPRO1 (objs[0]);
@@ -1664,12 +1734,16 @@ void
 elisp_map_remhash (maphash_function_t predicate,
 		   Lisp_Object hash_table, void *extra_arg)
 {
-  const Lisp_Hash_Table * const ht = xhash_table (hash_table);
-  Lisp_Object * const objs = copy_compress_hentries (ht);
+  const Lisp_Hash_Table *ht;
+  Lisp_Object *objs;
   const Lisp_Object *pobj, *end;
   int speccount = specpdl_depth ();
   struct gcpro gcpro1;
 
+  CHECK_HASH_TABLE (hash_table);
+  ht = XHASH_TABLE (hash_table);
+  objs = copy_compress_hentries (ht);
+
   record_unwind_protect (maphash_unwind, make_opaque_ptr ((void *)objs));
   GCPRO1 (objs[0]);
   gcpro1.nvars = 2 * ht->count;
@@ -1771,11 +1845,17 @@ oblookup (Lisp_Object package, const Ibyte *ptr, Bytecount size)
 /*                              Intern				      */
 /**********************************************************************/
 
-/* As of So 20 Nov 2016 13:16:04 GMT obarray has about 16000 entries after
-   starting up with -vanilla with all packages in the load path, but none
-   installed. OBARRAY_INITIAL_SIZE reflects the :size argument to
-   #'make-hash-table, and not the number of buckets, as previously. */
-#define OBARRAY_INITIAL_SIZE 16000
+/* Since obarray is dumped, and our hash table type does not use buckets, if
+   it is resized post pdump_load() its htentries are just leaked. 40000 will
+   make this less likely in normal use, and should have the side benefit of
+   fewer collisions in loadup and non-intense use.
+
+   This is the :size argument to #'make-hash-table, so the table ends up being
+   71429 entries in size, taking up 1.1 megabytes on a 64-bit machine. In
+   comparison, if this were not explicitly specified, it would be 500 k,
+   reaching the 1.1 megabytes threshold (including the leaked data) when
+   19,000 symbols are interned. */
+#define OBARRAY_INITIAL_SIZE 40000
 
 Lisp_Object Vobarray;
 static Lisp_Object initial_obarray;
@@ -2492,7 +2572,7 @@ a hash table test.
 
   CHECK_SYMBOL (name);
 
-  lookup = Fassq (name, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
+  lookup = assq_no_quit (name, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
 
   if (!NILP (lookup))
     {
@@ -2538,7 +2618,8 @@ equivalent to `eql'.
       return Qt;
     }
 
-  lookup = Fassq (object, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
+  lookup
+    = assq_no_quit (object, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
 
   if (!NILP (lookup))
     {
@@ -2583,7 +2664,8 @@ See `define-hash-table-test' and `make-hash-table'.
       test = Qeql;
     }
 
-  lookup = Fassq (test, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
+  lookup
+    = assq_no_quit (test, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
   if (NILP (lookup))
     {
       invalid_argument ("Not a defined hash table test", test);
@@ -2606,7 +2688,8 @@ See `define-hash-table-test' and `make-hash-table'.
       test = Qeql;
     }
 
-  lookup = Fassq (test, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
+  lookup
+    = assq_no_quit (test, XWEAK_LIST_LIST (Vhash_table_test_weak_list));
   if (NILP (lookup))
     {
       invalid_argument ("Not a defined hash table test", test);
@@ -2740,16 +2823,6 @@ See `with-hash-table-iterator'.  This function is not to be called directly.
 /*                            initialization                            */
 /************************************************************************/
 
-void
-hash_table_objects_create (void)
-{
-#ifdef MEMORY_USAGE_STATS
-  OBJECT_HAS_METHOD (hash_table, memory_usage);
-#endif
-  OBJECT_HAS_METHOD (hash_table, print_preprocess);
-  OBJECT_HAS_METHOD (hash_table, nsubst_structures_descend);
-}
-
 extern Lisp_Object Vall_weak_lists;
 
 void
@@ -2874,10 +2947,10 @@ vars_of_elhash (void)
      that was done before vars_of_data() was called. Create a sane
      weak list object now, set its list appropriately, assert that our
      data haven't been garbage collected. */
-  assert (!NILP (Fassq (Qeq, weak_list_list)));
-  assert (!NILP (Fassq (Qeql, weak_list_list)));
-  assert (!NILP (Fassq (Qequal, weak_list_list)));
-  assert (!NILP (Fassq (Qequalp, weak_list_list)));
+  assert (!NILP (assq_no_quit (Qeq, weak_list_list)));
+  assert (!NILP (assq_no_quit (Qeql, weak_list_list)));
+  assert (!NILP (assq_no_quit (Qequal, weak_list_list)));
+  assert (!NILP (assq_no_quit (Qequalp, weak_list_list)));
 
   Vhash_table_test_weak_list = make_weak_list (WEAK_LIST_KEY_ASSOC);
   XWEAK_LIST_LIST (Vhash_table_test_weak_list) = weak_list_list;
@@ -2887,8 +2960,7 @@ vars_of_elhash (void)
   staticpro (&Vall_weak_hash_tables);
 
 #ifdef MEMORY_USAGE_STATS
-  OBJECT_HAS_PROPERTY
-    (hash_table, memusage_stats_list, list1 (intern ("hash-entries")));
+  INIT_MEMORY_USAGE_STATS (hash_table, list1 (intern ("hash-entries")));
 #endif /* MEMORY_USAGE_STATS */
 
   DEFVAR_LISP ("obarray", &Vobarray /*
@@ -2903,9 +2975,19 @@ use `mapatoms'.
 void
 init_elhash_once_early (void)
 {
-  INIT_LISP_OBJECT (hash_table);
-  INIT_LISP_OBJECT (hash_table_test);
+  DEFINE_DUMPABLE_LISP_OBJECT ("hash-table", hash_table, print_hash_table,
+                               finalize_hash_table, hash_table_equal,
+                               hash_table_hash, hash_table_description,
+                               Lisp_Hash_Table);
+#ifdef MEMORY_USAGE_STATS
+  OBJECT_HAS_METHOD (hash_table, memory_usage);
+#endif
+  OBJECT_HAS_METHOD (hash_table, print_preprocess);
+  OBJECT_HAS_METHOD (hash_table, nsubst_structures_descend);
 
+  DEFINE_DUMPABLE_INTERNAL_LISP_OBJECT ("hash-table-test", hash_table_test,
+                                        hash_table_test_description,
+                                        Hash_Table_Test);
 
   /* Create Vequal_hash_table_test so we can create Vobarray. */
   staticpro (&Vequal_hash_table_test);
