@@ -70,7 +70,7 @@ extern int mswindows_is_blocking;
 #endif
 
 /* Do we accept events sent by other clients? */
-int gtk_allow_sendevents;
+Boolint gtk_allow_sendevents;
 
 static int process_events_occurred;
 static int tty_events_occurred;
@@ -679,32 +679,14 @@ struct what_is_ready_closure
   gint id;
 };
 
-static Lisp_Object *filedesc_with_input;
-static struct what_is_ready_closure **filedesc_to_what_closure;
+static Lisp_Object Vfiledesc_with_input;
 
-static void
-init_what_input_once (void)
-{
-  int i;
-
-  filedesc_with_input = xnew_array (Lisp_Object, MAXDESC);
-  filedesc_to_what_closure =
-    xnew_array (struct what_is_ready_closure *, MAXDESC);
-
-  for (i = 0; i < MAXDESC; i++)
-    {
-      filedesc_to_what_closure[i] = 0;
-      filedesc_with_input[i] = Qnil;
-    }
-
-  process_events_occurred = 0;
-  tty_events_occurred = 0;
-}
+static struct what_is_ready_closure *filedesc_to_what_closure[MAXDESC];
 
 static void
 mark_what_as_being_ready (struct what_is_ready_closure *closure)
 {
-  if (NILP (filedesc_with_input[closure->fd]))
+  if (NILP (XVECTOR_DATA (Vfiledesc_with_input)[closure->fd]))
     {
       SELECT_TYPE temp_mask;
       FD_ZERO (&temp_mask);
@@ -729,7 +711,7 @@ mark_what_as_being_ready (struct what_is_ready_closure *closure)
 #endif
 	  return;
 	}
-      filedesc_with_input[closure->fd] = closure->what;
+      XVECTOR_DATA (Vfiledesc_with_input)[closure->fd] = closure->what;
       if (PROCESSP (closure->what))
 	/* Don't increment this if the current process is already marked
 	 *  as having input. */
@@ -787,7 +769,7 @@ unselect_filedesc (int fd)
   struct what_is_ready_closure *closure = filedesc_to_what_closure[fd];
 
   assert (closure);
-  if (!NILP (filedesc_with_input[fd]))
+  if (!NILP (XVECTOR_DATA (Vfiledesc_with_input)[fd]))
     {
       /* We are unselecting this process before we have drained the rest of
 	 the input from it, probably from status_notify() in the command loop.
@@ -811,7 +793,7 @@ unselect_filedesc (int fd)
 	 throwing away the last block of output - status_notify() has already
 	 taken care of running the proc filter or whatever.
        */
-      filedesc_with_input[fd] = Qnil;
+      XVECTOR_DATA (Vfiledesc_with_input)[fd] = Qnil;
       if (PROCESSP (closure->what))
 	{
 	  assert (process_events_occurred > 0);
@@ -920,10 +902,10 @@ gtk_process_to_emacs_event (struct Lisp_Event *emacs_event)
 
   for (i = 0; i < MAXDESC; i++)
     {
-      Lisp_Object process = filedesc_with_input[i];
+      Lisp_Object process = XVECTOR_DATA (Vfiledesc_with_input)[i];
       if (PROCESSP (process))
 	{
-	  filedesc_with_input[i] = Qnil;
+	  XVECTOR_DATA (Vfiledesc_with_input)[i] = Qnil;
 	  process_events_occurred--;
 	  /* process events have nil as channel */
 	  set_event_type (emacs_event, process_event);
@@ -975,12 +957,12 @@ gtk_tty_to_emacs_event (struct Lisp_Event *emacs_event)
   assert (tty_events_occurred > 0);
   for (i = 0; i < MAXDESC; i++)
     {
-      Lisp_Object console = filedesc_with_input[i];
+      Lisp_Object console = XVECTOR_DATA (Vfiledesc_with_input)[i];
       if (CONSOLEP (console))
 	{
 	  assert (tty_events_occurred > 0);
 	  tty_events_occurred--;
-	  filedesc_with_input[i] = Qnil;
+	  XVECTOR_DATA (Vfiledesc_with_input)[i] = Qnil;
 	  if (read_event_from_tty_or_stream_desc (emacs_event,
 						  XCONSOLE (console)))
 	    return 1;
@@ -1690,31 +1672,6 @@ syms_of_event_gtk (void)
 {
 }
 
-void
-reinit_vars_of_event_gtk (void)
-{
-  gtk_event_stream = xnew_and_zero (struct event_stream);
-  gtk_event_stream->event_pending_p 	= emacs_gtk_event_pending_p;
-  gtk_event_stream->next_event_cb	= emacs_gtk_next_event;
-  gtk_event_stream->handle_magic_event_cb= emacs_gtk_handle_magic_event;
-  gtk_event_stream->format_magic_event_cb= emacs_gtk_format_magic_event;
-  gtk_event_stream->compare_magic_event_cb= emacs_gtk_compare_magic_event;
-  gtk_event_stream->hash_magic_event_cb  = emacs_gtk_hash_magic_event;
-  gtk_event_stream->add_timeout_cb 	= emacs_gtk_add_timeout;
-  gtk_event_stream->remove_timeout_cb 	= emacs_gtk_remove_timeout;
-  gtk_event_stream->select_console_cb 	= emacs_gtk_select_console;
-  gtk_event_stream->unselect_console_cb = emacs_gtk_unselect_console;
-  gtk_event_stream->select_process_cb 	= emacs_gtk_select_process;
-  gtk_event_stream->unselect_process_cb = emacs_gtk_unselect_process;
-  gtk_event_stream->drain_queue_cb	= emacs_gtk_drain_queue;
-  gtk_event_stream->create_io_streams_cb= emacs_gtk_create_io_streams;
-  gtk_event_stream->delete_io_streams_cb= emacs_gtk_delete_io_streams;
-  gtk_event_stream->force_event_pending_cb= emacs_gtk_force_event_pending;
-
-  /* this function only makes safe calls */
-  init_what_input_once ();
-}
-
 void
 init_event_gtk_late (void) /* called when already initialized */
 {
@@ -2140,5 +2097,34 @@ Do NOT modify.
 
   Vgtk_completed_timeouts = Qnil;
   staticpro (&Vgtk_completed_timeouts);
+
+  Vfiledesc_with_input = make_vector (MAXDESC, Qnil);
+  /* Dump this vector. */
+  dump_add_root_lisp_object (&Vfiledesc_with_input);
+  /* Make it reachable while we're dumping. Don't make it reachable for GC
+     after pdump_load(), it's usually 1024 entries long, full of Qnil, and the
+     process objects are reachable anyway. */
+  staticpro_nodump (&Vfiledesc_with_input);
+
+  /* filedesc_to_what_closure[] is in BSS and set to all zeroes already,
+     which is what we want. */
+
+  DEFINE_EVENT_STREAM (gtk);
+  EVENT_STREAM_HAS_METHOD (gtk, event_pending_p);
+  EVENT_STREAM_HAS_METHOD (gtk, next_event);
+  EVENT_STREAM_HAS_METHOD (gtk, handle_magic_event);
+  EVENT_STREAM_HAS_METHOD (gtk, format_magic_event);
+  EVENT_STREAM_HAS_METHOD (gtk, compare_magic_event);
+  EVENT_STREAM_HAS_METHOD (gtk, hash_magic_event);
+  EVENT_STREAM_HAS_METHOD (gtk, add_timeout);
+  EVENT_STREAM_HAS_METHOD (gtk, remove_timeout);
+  EVENT_STREAM_HAS_METHOD (gtk, select_console);
+  EVENT_STREAM_HAS_METHOD (gtk, unselect_console);
+  EVENT_STREAM_HAS_METHOD (gtk, select_process);
+  EVENT_STREAM_HAS_METHOD (gtk, unselect_process);
+  EVENT_STREAM_HAS_METHOD (gtk, drain_queue);
+  EVENT_STREAM_HAS_METHOD (gtk, create_io_streams);
+  EVENT_STREAM_HAS_METHOD (gtk, delete_io_streams);
+  EVENT_STREAM_HAS_METHOD (gtk, force_event_pending);
 }
 
