@@ -36,6 +36,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #include "redisplay.h"
 #include "toolbar.h"
 #include "window.h"
+#include "casetab.h"
 
 #include "console-tty-impl.h" /* for stuff in character_to_event */
 
@@ -156,6 +157,8 @@ static const struct sized_memory_description misc_user_data_description = {
 };
 
 static const struct memory_description magic_eval_data_description_1 [] = {
+  { XD_FUNCTION_POINTER, offsetof (struct Lisp_Magic_Eval_Data,
+				 internal_function) },
   { XD_LISP_OBJECT, offsetof (struct Lisp_Magic_Eval_Data, object) },
   { XD_END }
 };
@@ -199,54 +202,6 @@ static const struct memory_description event_description [] = {
   { XD_END }
 };
 
-#ifdef EVENT_DATA_AS_OBJECTS
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("key-data", key_data,
-				      internal_object_printer, 0, 0, 0,
-				      key_data_description, 
-				      Lisp_Key_Data);
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("button-data", button_data,
-				      internal_object_printer, 0, 0, 0,
-				      button_data_description, 
-				      Lisp_Button_Data);
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("motion-data", motion_data,
-				      internal_object_printer, 0, 0, 0,
-				      motion_data_description,
-				      Lisp_Motion_Data);
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("process-data", process_data,
-				      internal_object_printer, 0, 0, 0,
-				      process_data_description,
-				      Lisp_Process_Data);
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("timeout-data", timeout_data,
-				      internal_object_printer, 0, 0, 0,
-				      timeout_data_description,
-				      Lisp_Timeout_Data);
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("eval-data", eval_data,
-				      internal_object_printer, 0, 0, 0,
-				      eval_data_description,
-				      Lisp_Eval_Data);
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("misc-user-data", misc_user_data,
-				      internal_object_printer, 0, 0, 0,
-				      misc_user_data_description, 
-				      Lisp_Misc_User_Data);
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("magic-eval-data", magic_eval_data,
-				      internal_object_printer, 0, 0, 0,
-				      magic_eval_data_description, 
-				      Lisp_Magic_Eval_Data);
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("magic-data", magic_data,
-				      internal_object_printer, 0, 0, 0,
-				      magic_data_description,
-				      Lisp_Magic_Data);
-
-#endif /* EVENT_DATA_AS_OBJECTS */
 
 static void
 print_event (Lisp_Object obj, Lisp_Object printcharfun,
@@ -283,6 +238,21 @@ print_event (Lisp_Object obj, Lisp_Object printcharfun,
 	assert (FIXNUMP (Vy));
 	write_fmt_string (printcharfun, "#<motion-event %zd, %zd",
 			  XFIXNUM (Vx), XFIXNUM (Vy));
+        if (XEVENT_MOTION_MODIFIERS (obj) != 0)
+          {
+            int modifiers = XEVENT_MOTION_MODIFIERS (obj);
+            write_ascstring (printcharfun, " (");
+
+#define FROB_MODIFIER(upcase_mod, downcase_mod, num)			\
+	    if (modifiers & XEMACS_MOD_##upcase_mod)			\
+	      {								\
+		modifiers &= ~XEMACS_MOD_##upcase_mod;			\
+		write_ascstring (printcharfun, #downcase_mod);		\
+		if (modifiers) write_ascstring (printcharfun, " ");	\
+	      }
+#include "modifiers.h"
+	    write_ascstring (printcharfun, ")");
+	  }
 	break;
       }
     case process_event:
@@ -300,6 +270,21 @@ print_event (Lisp_Object obj, Lisp_Object printcharfun,
       write_fmt_string_lisp (printcharfun, "#<misc-user-event (%S %S)",
 			     XEVENT_MISC_USER_FUNCTION (obj),
 			     XEVENT_MISC_USER_OBJECT (obj));
+      if (XEVENT_MISC_USER_MODIFIERS (obj) != 0)
+	{
+	  int modifiers = XEVENT_MISC_USER_MODIFIERS (obj);
+	  write_ascstring (printcharfun, " (");
+
+#define FROB_MODIFIER(upcase_mod, downcase_mod, num)			\
+	  if (modifiers & XEMACS_MOD_##upcase_mod)			\
+	    {								\
+	      modifiers &= ~XEMACS_MOD_##upcase_mod;			\
+	      write_ascstring (printcharfun, #downcase_mod);		\
+	      if (modifiers) write_ascstring (printcharfun, " ");	\
+	    }
+#include "modifiers.h"
+	  write_ascstring (printcharfun, ")");
+	}
       break;
     case eval_event:
       write_fmt_string_lisp (printcharfun, "#<eval-event (%S %S)",
@@ -440,11 +425,6 @@ event_hash (Lisp_Object obj, int depth, Boolint UNUSED (equalp))
   return 0; /* unreached */
 }
 
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("event", event, print_event, 0,
-				      event_equal, event_hash,
-				      event_description,
-				      Lisp_Event);
-
 DEFUN ("make-event", Fmake_event, 0, 2, 0, /*
 Return a new event of type TYPE, with properties described by PLIST.
 
@@ -613,18 +593,13 @@ WARNING: the event object returned may be a reused one; see the function
 
 	    EXTERNAL_LIST_LOOP_2 (sym, value)
 	      {
-		if      (EQ (sym, Qcontrol)) modifiers |= XEMACS_MOD_CONTROL;
-		else if (EQ (sym, Qmeta))    modifiers |= XEMACS_MOD_META;
-		else if (EQ (sym, Qsuper))   modifiers |= XEMACS_MOD_SUPER;
-		else if (EQ (sym, Qhyper))   modifiers |= XEMACS_MOD_HYPER;
-		else if (EQ (sym, Qalt))     modifiers |= XEMACS_MOD_ALT;
-		else if (EQ (sym, Qsymbol))  modifiers |= XEMACS_MOD_ALT;
-		else if (EQ (sym, Qshift))   modifiers |= XEMACS_MOD_SHIFT;
-		else if (EQ (sym, Qbutton1))   modifiers |= XEMACS_MOD_BUTTON1;
-		else if (EQ (sym, Qbutton2))   modifiers |= XEMACS_MOD_BUTTON2;
-		else if (EQ (sym, Qbutton3))   modifiers |= XEMACS_MOD_BUTTON3;
-		else if (EQ (sym, Qbutton4))   modifiers |= XEMACS_MOD_BUTTON4;
-		else if (EQ (sym, Qbutton5))   modifiers |= XEMACS_MOD_BUTTON5;
+		if (0) { DO_NOTHING; }
+#define FROB_MODIFIER(upcasemod, downcasemod, num)		\
+		else if (EQ (sym, Q##downcasemod))		\
+		  {						\
+		    modifiers |= XEMACS_MOD_##upcasemod;	\
+		  }
+#include "modifiers.h"		
 		else
 		  invalid_constant ("Invalid key modifier", sym);
 	      }
@@ -881,11 +856,7 @@ function `deallocate-event'.
   XSET_EVENT_CHANNEL (event2, XEVENT_CHANNEL (event1));
   XSET_EVENT_TIMESTAMP (event2, XEVENT_TIMESTAMP (event1));
   
-#ifdef EVENT_DATA_AS_OBJECTS
-  copy_lisp_object (XEVENT_DATA (event2), XEVENT_DATA (event1));
-#else
   XEVENT (event2)->event = XEVENT (event1)->event;
-#endif
   return event2;
 }
 
@@ -1195,9 +1166,6 @@ character_to_event (Ichar c, Lisp_Event *event, struct console *con,
   if (EVENT_TYPE (event) == dead_event)
     invalid_argument ("character-to-event called with a deallocated event!", Qunbound);
 
-#ifndef MULE
-  c &= 255;
-#endif
   if (meta_behavior != latin_1_maps_to_itself && c > 127 && c <= 255)
     {
       int meta_flag = 1;
@@ -1273,22 +1241,22 @@ event_to_character (Lisp_Object event,
       XEVENT_KEY_MODIFIERS (event) & 
       (XEMACS_MOD_SUPER|XEMACS_MOD_HYPER|XEMACS_MOD_ALT))
     return -1;
-  if (CHAR_OR_CHAR_INTP (XEVENT_KEY_KEYSYM (event)))
-    c = XCHAR_OR_CHAR_INT (XEVENT_KEY_KEYSYM (event));
+  if (CHARP (XEVENT_KEY_KEYSYM (event)))
+    c = XCHAR (XEVENT_KEY_KEYSYM (event));
   else if (!SYMBOLP (XEVENT_KEY_KEYSYM (event)))
     ABORT ();
-  else if (CHAR_OR_CHAR_INTP (code = Fget (XEVENT_KEY_KEYSYM (event),
-					   Qcharacter_of_keysym, Qnil)))
-    c = XCHAR_OR_CHAR_INT (code);
+  else if (CHARP ((code = Fget (XEVENT_KEY_KEYSYM (event),
+				Qcharacter_of_keysym, Qnil))))
+    c = XCHAR (code);
   else
     {
       Lisp_Object thekeysym = XEVENT_KEY_KEYSYM (event);
 
-      if (CHAR_OR_CHAR_INTP (code = Fget (thekeysym, Qascii_character, Qnil)))
+      if (CHARP ((code = Fget (thekeysym, Qascii_character, Qnil))))
 	{
-	  c = XCHAR_OR_CHAR_INT (code);
-	  warn_when_safe(Qkey_mapping, Qwarning, 
-			 "Obsolete key binding technique.\n"
+	  c = XCHAR (code);
+	  warn_when_safe (Qkey_mapping, Qwarning, 
+			  "Obsolete key binding technique.\n"
 
 "Some code you're using bound %s to `self-insert-command' and messed around\n"
 "with its `ascii-character' property.  Doing this is deprecated, and the code\n"
@@ -1297,7 +1265,7 @@ event_to_character (Lisp_Object event,
 "for it; we support many more X11 keysyms out of the box now than we did\n"
 "in the past. ", XSTRING_DATA(XSYMBOL_NAME(thekeysym)));
 	  /* Only show the warning once for each keysym. */
-	  Fput(thekeysym, Qcharacter_of_keysym, code);
+	  Fput (thekeysym, Qcharacter_of_keysym, code);
 	}
       else
 	{
@@ -1519,28 +1487,53 @@ format_event_object (Lisp_Object printcharfun, Lisp_Object event,
       return 0;
     }
 
-  if (mod && !brief)
+  if (mod)
     {
-      result += write_ascstring (printcharfun, "(");
+      if (brief)
+	{
+#define FROB_MODIFIER(upcasemod, downcasemod, num)			\
+	  if (mod & XEMACS_MOD_##upcasemod)				\
+	    {								\
+	      if (XEMACS_MOD_##upcasemod > XEMACS_MOD_SHIFT)		\
+		{							\
+		  result += write_ascstring (printcharfun, #upcasemod);	\
+		  result += write_ascstring (printcharfun, "-");	\
+		}							\
+	      else if (XEMACS_MOD_##upcasemod == XEMACS_MOD_SHIFT)	\
+		{							\
+		  /* Can't generate the short form for shift		\
+		     programmatically. */				\
+		  result += write_ascstring (printcharfun, "Sh-");	\
+		}							\
+	      else							\
+		{							\
+		  result += write_fmt_string (printcharfun, "%c-",	\
+					      #upcasemod [0]);		\
+		}							\
+	    }
+#include "modifiers.h"
+	}
+      else
+	{
+	  int modifiers = mod;
+	  result += write_ascstring (printcharfun, "(");
+
+#define FROB_MODIFIER(upcasemod, downcasemod, num)			\
+	  if (modifiers & XEMACS_MOD_##upcasemod)			\
+	    {								\
+	      result += write_ascstring (printcharfun, #downcasemod);	\
+	      result += write_ascstring (printcharfun, " ");		\
+	    }
+#include "modifiers.h"
+	}
     }
 
-#define modprint(x,y)							\
-  do { if (brief) result += write_ascstring (printcharfun, (y));	\
-    else result += write_ascstring (printcharfun, (x)); } while (0)
-  if (mod & XEMACS_MOD_CONTROL) modprint ("control ", "C-");
-  if (mod & XEMACS_MOD_META)    modprint ("meta ",    "M-");
-  if (mod & XEMACS_MOD_SUPER)   modprint ("super ",   "S-");
-  if (mod & XEMACS_MOD_HYPER)   modprint ("hyper ",   "H-");
-  if (mod & XEMACS_MOD_ALT)	modprint ("alt ",     "A-");
-  if (mod & XEMACS_MOD_SHIFT)   modprint ("shift ",   "Sh-");
   if (mouse_p)
     {
       result += write_ascstring (printcharfun, "button");
       --mouse_p;
     }
 
-#undef modprint
-
   if (CHARP (key))
     {
       Ibyte str[MAX_ICHAR_LEN];
@@ -1592,46 +1585,51 @@ void
 upshift_event (Lisp_Object event)
 {
   Lisp_Object keysym = XEVENT_KEY_KEYSYM (event);
-  Ichar c = 0;
 
-  if (CHAR_OR_CHAR_INTP (keysym)
-      && ((c = XCHAR_OR_CHAR_INT (keysym)),
-	  c >= 'a' && c <= 'z'))
-    XSET_EVENT_KEY_KEYSYM (event, make_char (c + 'A' - 'a'));
-  else
-    if (!(XEVENT_KEY_MODIFIERS (event) & XEMACS_MOD_SHIFT))
+  if (CHARP (keysym))
+    {
+      Ichar upcase = UPCASE (0, XCHAR (keysym));
+
+      if (upcase != XCHAR (keysym))
+	{
+	  XSET_EVENT_KEY_KEYSYM (event, make_char (upcase));
+	  return;
+	}
+    }
+
+  if (!(XEVENT_KEY_MODIFIERS (event) & XEMACS_MOD_SHIFT))
+    {
       XSET_EVENT_KEY_MODIFIERS
 	(event, XEVENT_KEY_MODIFIERS (event) | XEMACS_MOD_SHIFT);
+    }
 }
 
 void
 downshift_event (Lisp_Object event)
 {
   Lisp_Object keysym = XEVENT_KEY_KEYSYM (event);
-  Ichar c = 0;
 
   if (XEVENT_KEY_MODIFIERS (event) & XEMACS_MOD_SHIFT)
     XSET_EVENT_KEY_MODIFIERS
       (event, XEVENT_KEY_MODIFIERS (event) & ~XEMACS_MOD_SHIFT);
-  else if (CHAR_OR_CHAR_INTP (keysym)
-	   && ((c = XCHAR_OR_CHAR_INT (keysym)),
-	       c >= 'A' && c <= 'Z'))
-    XSET_EVENT_KEY_KEYSYM (event, make_char (c + 'a' - 'A'));
+  else if (CHARP (keysym))
+    {
+      Ichar downcase = DOWNCASE (0, XCHAR (keysym));
+
+      if (downcase != XCHAR (keysym))
+	{
+	  XSET_EVENT_KEY_KEYSYM (event, make_char (downcase));
+	}
+    }
 }
 
-int
+Boolint
 event_upshifted_p (Lisp_Object event)
 {
   Lisp_Object keysym = XEVENT_KEY_KEYSYM (event);
-  Ichar c = 0;
 
-  if ((XEVENT_KEY_MODIFIERS (event) & XEMACS_MOD_SHIFT)
-      || (CHAR_OR_CHAR_INTP (keysym)
-	  && ((c = XCHAR_OR_CHAR_INT (keysym)),
-	      c >= 'A' && c <= 'Z')))
-    return 1;
-  else
-    return 0;
+  return (XEVENT_KEY_MODIFIERS (event) & XEMACS_MOD_SHIFT)
+    || (CHARP (keysym) && UPPERCASEP (0, XCHAR (keysym)));
 }
 
 DEFUN ("eventp", Feventp, 1, 1, 0, /*
@@ -1806,8 +1804,14 @@ See also `event-timestamp' and `current-event-timestamp'.
 } while (0)
 
 DEFUN ("event-key", Fevent_key, 1, 1, 0, /*
-Return the Keysym of the key-press event EVENT.
+Return the key of the key-press event EVENT.
+
 This will be a character if the event is associated with one, else a symbol.
+See also `event-modifiers' and `canonicalize-keysym'.
+
+`event-key' always returns a non-list result without modifiers; for an event
+which does not have modifiers `(eq (canonicalize-keysym (event-key EVENT))
+(event-key EVENT))' will always hold.
 */
        (event))
 {
@@ -1905,21 +1909,12 @@ clever things.
 {
   int mod = XFIXNUM (Fevent_modifier_bits (event));
   Lisp_Object result = Qnil;
-  struct gcpro gcpro1;
 
-  GCPRO1 (result);
-  if (mod & XEMACS_MOD_SHIFT)   result = Fcons (Qshift, result);
-  if (mod & XEMACS_MOD_ALT)	result = Fcons (Qalt, result);
-  if (mod & XEMACS_MOD_HYPER)   result = Fcons (Qhyper, result);
-  if (mod & XEMACS_MOD_SUPER)   result = Fcons (Qsuper, result);
-  if (mod & XEMACS_MOD_META)    result = Fcons (Qmeta, result);
-  if (mod & XEMACS_MOD_CONTROL) result = Fcons (Qcontrol, result);
-  if (mod & XEMACS_MOD_BUTTON1) result = Fcons (Qbutton1, result);
-  if (mod & XEMACS_MOD_BUTTON2) result = Fcons (Qbutton2, result);
-  if (mod & XEMACS_MOD_BUTTON3) result = Fcons (Qbutton3, result);
-  if (mod & XEMACS_MOD_BUTTON4) result = Fcons (Qbutton4, result);
-  if (mod & XEMACS_MOD_BUTTON5) result = Fcons (Qbutton5, result);
-  RETURN_UNGCPRO (Fnreverse (result));
+#define FROB_MODIFIER(upcase_mod, downcase_mod, val)                        \
+  if (mod & XEMACS_MOD_##upcase_mod) result = Fcons (Q##downcase_mod, result);
+#include "modifiers.h"
+
+  return Fnreverse (result);
 }
 
 static int
@@ -2569,18 +2564,10 @@ This is in the form of a property list (alternating keyword/value pairs).
 void
 syms_of_events (void)
 {
-  INIT_LISP_OBJECT (event);
-#ifdef EVENT_DATA_AS_OBJECTS
-  INIT_LISP_OBJECT (key_data);
-  INIT_LISP_OBJECT (button_data);
-  INIT_LISP_OBJECT (motion_data);
-  INIT_LISP_OBJECT (process_data);
-  INIT_LISP_OBJECT (timeout_data);
-  INIT_LISP_OBJECT (eval_data);
-  INIT_LISP_OBJECT (misc_user_data);
-  INIT_LISP_OBJECT (magic_eval_data);
-  INIT_LISP_OBJECT (magic_data);
-#endif /* EVENT_DATA_AS_OBJECTS */  
+  DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("event", event, print_event, 0,
+                                        event_equal, event_hash,
+                                        event_description,
+                                        Lisp_Event);
 
   DEFSUBR (Fcharacter_to_event);
   DEFSUBR (Fevent_to_character);
@@ -2646,14 +2633,9 @@ syms_of_events (void)
   defsymbol (&QKdelete, "delete");
 }
 
-
-void
-reinit_vars_of_events (void)
-{
-  Vevent_resource = Qnil;
-}
-
 void
 vars_of_events (void)
 {
+  Vevent_resource = Qnil;
+  dump_add_nil_lisp_object (&Vevent_resource);
 }
