@@ -172,10 +172,10 @@ gtk_color_instance_rgb_components (struct Lisp_Color_Instance *c)
 		 make_fixnum (color->blue)));
 #else
   GdkRGBA *color = COLOR_INSTANCE_GTK_COLOR (c);
-  return (list4 (make_float (color->red),
-		 make_float (color->green),
-		 make_float (color->blue),
-		 make_float (color->alpha)));
+  return list3 (make_fixnum ((UINT_16_BIT) (color->red * 65535)),
+                make_fixnum ((UINT_16_BIT) (color->green * 65535)),
+                make_fixnum ((UINT_16_BIT) (color->blue * 65535)));
+  /* Don't return the alpha for now, Lisp can't handle it. */
 #endif
 }
 
@@ -195,7 +195,7 @@ gtk_valid_color_name_p (struct device *UNUSED (d), Lisp_Object color)
 }
 
 static Lisp_Object
-gtk_color_list (void)
+gtk_color_list (Lisp_Object UNUSED (device))
 {
   /* #### BILL!!!
      Is this correct? */
@@ -715,10 +715,8 @@ console_type_create_fontcolor_gtk (void)
   CONSOLE_HAS_METHOD (gtk, font_instance_truename);
   CONSOLE_HAS_METHOD (gtk, font_instance_properties);
   CONSOLE_HAS_METHOD (gtk, font_list);
-#ifdef MULE
   CONSOLE_HAS_METHOD (gtk, find_charset_font);
   CONSOLE_HAS_METHOD (gtk, font_spec_matches_charset);
-#endif
 }
 
 void
