@@ -43,7 +43,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #include "font-mgr.h"
 #endif
 
-int x_handle_non_fully_specified_fonts;
+Boolint x_handle_non_fully_specified_fonts;
 
 #ifdef DEBUG_XEMACS 
 Fixnum debug_x_fonts;
@@ -200,7 +200,7 @@ x_valid_color_name_p (struct device *d, Lisp_Object color)
 }
 
 static Lisp_Object
-x_color_list (void)
+x_color_list (Lisp_Object UNUSED (device))
 {
   return call0 (intern ("x-color-list-internal"));
 }
@@ -414,7 +414,7 @@ x_print_font_instance (Lisp_Font_Instance *f,
 #ifdef HAVE_XFT
   /* #### What should we do here?  For now, print the address. */
   if (FONT_INSTANCE_X_XFTFONT (f))
-    write_fmt_string (printcharfun, " xft font: 0x%zx",
+    write_fmt_string (printcharfun, " xft font: 0x%p",
                       FONT_INSTANCE_X_XFTFONT (f));
 #endif
 }
@@ -585,19 +585,25 @@ truename_via_random_props (Display *dpy, XFontStruct *font)
   int ok = 0;
   Extbyte *result;
 
-#define get_string(atom,var)				\
-  if (XGetFontProperty (font, (atom), &value))		\
-    {                                                   \
-      var = EXTERNAL_TO_ITEXT (XGetAtomName (dpy, value),       \
-                               Qx_hpc_encoding);        \
-    }                                                   \
-  else	{						\
-    var = 0;						\
-    goto FAIL; }
-#define get_number(atom,var)				\
-  if (!XGetFontProperty (font, (atom), &var) ||		\
-      var > 999)					\
-    goto FAIL;
+#define get_string(atom,var) do {                                       \
+    if (XGetFontProperty (font, (atom), &value))                        \
+      {                                                                 \
+        Extbyte *staging = XGetAtomName (dpy, value);                   \
+        var = EXTERNAL_TO_ITEXT (staging, Qx_hpc_encoding);             \
+        XFree (staging);                                                \
+      }                                                                 \
+    else                                                                \
+      {                                                                 \
+        var = 0;                                                        \
+        goto FAIL;                                                      \
+      }                                                                 \
+    } while (0)
+#define get_number(atom,var) do {                                       \
+    if (!XGetFontProperty (font, (atom), &var) || var > 999)            \
+      {                                                                 \
+        goto FAIL;                                                      \
+      }                                                                 \
+    } while (0)
 
   foundry = family = weight = slant = setwidth = 0;
   add_style = spacing = registry = encoding = 0;
@@ -632,16 +638,6 @@ truename_via_random_props (Display *dpy, XFontStruct *font)
   else
     result = 0;
 
-  if (foundry) XFree (foundry);
-  if (family) XFree (family);
-  if (weight) XFree (weight);
-  if (slant) XFree (slant);
-  if (setwidth) XFree (setwidth);
-  if (add_style) XFree (add_style);
-  if (spacing) XFree (spacing);
-  if (registry) XFree (registry);
-  if (encoding) XFree (encoding);
-
   return result;
 }
 
@@ -938,10 +934,8 @@ console_type_create_fontcolor_x (void)
   CONSOLE_HAS_METHOD (x, font_instance_truename);
   CONSOLE_HAS_METHOD (x, font_instance_properties);
   CONSOLE_HAS_METHOD (x, font_list);
-#ifdef MULE
   CONSOLE_HAS_METHOD (x, find_charset_font);
   CONSOLE_HAS_METHOD (x, font_spec_matches_charset);
-#endif
 }
 
 void
