@@ -91,7 +91,6 @@ do {								\
 # define DEBUG_FONTS_LISP4(format, arg1, arg2, arg3, arg4)
 #endif
 
-#ifdef MULE
 
 /* For some code it's reasonable to have only one copy and conditionalize
    at run-time.  For other code it isn't. */
@@ -140,18 +139,11 @@ XFUN (font_spec_matches_charset) (struct device * USED_IF_XFT (d),
   fixup_internal_substring (nonreloc, reloc, offset, &the_length);
   the_nonreloc += offset;
 
-#ifdef USE_XFT
+#ifdef XEMACS_USE_XFT
   if (stage == STAGE_FINAL)
     {
-      Display *dpy = DEVICE_X_DISPLAY (d);
-      Extbyte *extname;
-      XftFont *rf;
-
       if (!NILP (reloc))
 	{
-	  the_nonreloc = XSTRING_DATA (reloc);
-	  extname = LISP_STRING_TO_EXTERNAL (reloc, Qx_font_name_encoding);
-	  rf = xft_open_font_by_name (dpy, extname);
 	  return 0;	 /* #### maybe this will compile and run ;) */
 			 /* Jesus, Stephen, what the fuck? */
 	}
@@ -267,7 +259,7 @@ xlistfonts_checking_charset (Lisp_Object device, const Ibyte *xlfd,
 }
 #endif
 
-#ifdef USE_XFT
+#ifdef XEMACS_USE_XFT
 /* #### debug functions: find a better place for us */
 const char *FcResultToString (FcResult r);
 const char *
@@ -690,7 +682,7 @@ xft_find_charset_font (Lisp_Object font, Lisp_Object charset,
 }
 #undef DECLARE_DEBUG_FONTNAME
 
-#endif /* USE_XFT */
+#endif /* XEMACS_USE_XFT */
 
 #if !defined (HAVE_GTK)
 /* find a font spec that matches font spec FONT and also matches
@@ -708,7 +700,7 @@ XFUN (find_charset_font) (Lisp_Object device, Lisp_Object font,
   DECLARE_EISTRING (ei_xlfd_without_registry);
   DECLARE_EISTRING (ei_xlfd);
 
-#ifdef USE_XFT 
+#ifdef XEMACS_USE_XFT 
   result = xft_find_charset_font (font, charset, stage);
   if (!NILP (result)) 
     {
@@ -904,4 +896,3 @@ XFUN (find_charset_font) (Lisp_Object device, Lisp_Object font,
 }
 #endif /* HAVE_GTK */
 
-#endif /* MULE */
