@@ -31,17 +31,13 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
    (Info-goto-node "(internals)Discussion -- Incremental Collector")
    
-   We currently have two mark algorithms:
-
-   - The "recursive mark algorithm" marks live objects by recursively
-     calling mark_* functions on live objects.  It is the default mark 
-     algorithm.
-
-   - The "KKCC mark algorithm" uses an explicit stack that to keep track of
-     the current progress of traversal and uses memory layout descriptions
-     (that are also used by the portable dumper) instead of the mark_*
-     functions.  This is used if you configure `--with-kkcc'.
-*/
+   The mark algorthim uses an explicit stack to keep track of the current
+   progress of traversal and uses memory layout descriptions (that are also
+   used by the portable dumper) instead of the previous mark_* functions.  This
+   work is called the "KKCC" mark algorithm from the initials of its
+   implementors, Marcus Crestani and Markus Kaltenbach. It is significantly
+   faster than the old mark algorithm, since it has much more locality of code
+   reference. */
 
 #include <config.h>
 #include "lisp.h"
@@ -87,13 +83,15 @@ EMACS_INT total_gc_usage;
 int total_gc_usage_set;
 
 /* Number of bytes of consing since gc before another gc should be done. */
-EMACS_INT gc_cons_threshold;
+Fixnum gc_cons_threshold = GC_CONS_THRESHOLD;
 
 /* Nonzero during gc */
 int gc_in_progress;
 
+#define GC_CONS_PERCENTAGE              40 /* #### what is optimal? */
+
 /* Percentage of consing of total data size before another GC. */
-EMACS_INT gc_cons_percentage;
+Fixnum gc_cons_percentage = GC_CONS_PERCENTAGE;
 
 
 /************************************************************************/
@@ -102,32 +100,22 @@ EMACS_INT gc_cons_percentage;
 
 int need_to_garbage_collect;
 
-#ifdef ERROR_CHECK_GC
-int always_gc = 0;    		/* Debugging hack; equivalent to
-				   (setq gc-cons-thresold -1) */
-#else
-#define always_gc 0
-#endif
-
 /* True if it's time to garbage collect now. */
 void
 recompute_need_to_garbage_collect (void)
 {
-  if (always_gc)
-    need_to_garbage_collect = 1;
-  else
-    need_to_garbage_collect = 
-      (consing_since_gc > gc_cons_threshold
-       &&
+  need_to_garbage_collect = 
+    (consing_since_gc > gc_cons_threshold
+     &&
 #if 0 /* #### implement this better */
-       ((double)consing_since_gc) / total_data_usage()) >=
-      ((double)gc_cons_percentage / 100)
+     ((double)consing_since_gc) / total_data_usage()) >=
+    ((double)gc_cons_percentage / 100)
 #else
-       (!total_gc_usage_set ||
-	((double)consing_since_gc / total_gc_usage) >=
-	((double)gc_cons_percentage / 100))
+    (!total_gc_usage_set ||
+     ((double)consing_since_gc / total_gc_usage) >=
+     ((double)gc_cons_percentage / 100))
 #endif
-       );
+    );
   recompute_funcall_allocation_flag ();
 }
 
@@ -263,7 +251,15 @@ lispdesc_one_description_line_size (void *rdata,
       return sizeof (Lisp_Object);
     case XD_OPAQUE_PTR:
       return sizeof (void *);
+    case XD_FUNCTION_POINTER:
+      return sizeof (lisp_fn_t);
+    case XD_MEMORY_DESCRIPTION:
+      return sizeof (struct memory_description *);
+    case XD_SIZED_MEMORY_DESCRIPTION:
+      return sizeof (struct sized_memory_description *);
     case XD_BLOCK_PTR:
+    case XD_BLOCK_DATA_PTR:
+    case XD_BLOCK_OFFSET_PTR:
       {
 	EMACS_INT val = lispdesc_indirect_count (desc1->data1, desc, obj);
 	return val * sizeof (void *);
@@ -325,8 +321,6 @@ lispdesc_one_description_line_size (void *rdata,
       }
     case XD_ASCII_STRING:
       return sizeof (void *);
-    case XD_DOC_STRING:
-      return sizeof (void *);
     case XD_INT_RESET:
       return sizeof (int);
     case XD_BYTECOUNT:
@@ -478,7 +472,9 @@ kkcc_backtrace_1 (int size, int detailed)
 	stderr_out ("WARNING! Bad Lisp Object type %d",
 		    XRECORD_LHEADER (obj)->type);
       else
-	stderr_out ("%s", XRECORD_LHEADER_IMPLEMENTATION (obj)->name);
+	stderr_out ("%s",
+                    LRECORD_IMPLEMENTATION_IBYTE_NAME
+                    (XRECORD_LHEADER_IMPLEMENTATION (obj)));
       if (detailed && kkcc_bt[i].is_lisp)
 	{
 	  stderr_out (" ");
@@ -613,10 +609,6 @@ kkcc_bt_push (void *obj, const struct memory_description *desc,
 #define kkcc_bt_push(obj, desc)
 #endif /* not DEBUG_XEMACS */
 
-/* Object memory descriptions are in the lrecord_implementation structure.
-   But copying them to a parallel array is much more cache-friendly. */
-const struct memory_description *lrecord_memory_descriptions[countof (lrecord_implementations_table)];
-
 /* the initial stack size in kkcc_gc_stack_entries */
 #define KKCC_INIT_GC_STACK_SIZE 16384
 
@@ -891,8 +883,10 @@ kkcc_marking (void)
 	    case XD_LO_LINK:
 	    case XD_OPAQUE_PTR:
 	    case XD_OPAQUE_DATA_PTR:
+	    case XD_FUNCTION_POINTER:
+	    case XD_MEMORY_DESCRIPTION:
+	    case XD_SIZED_MEMORY_DESCRIPTION:
 	    case XD_ASCII_STRING:
-	    case XD_DOC_STRING:
 	      break;
 	    case XD_LISP_OBJECT: 
 	      {
@@ -929,6 +923,8 @@ kkcc_marking (void)
 		break;
 	      }
 	    case XD_BLOCK_PTR:
+	    case XD_BLOCK_DATA_PTR:
+            case XD_BLOCK_OFFSET_PTR:
 	      {
 		EMACS_INT count = lispdesc_indirect_count (desc1->data1, desc,
 							   data);
@@ -1091,7 +1087,7 @@ run_post_gc_actions (void)
 static Lisp_Object QSin_garbage_collection;
 
 /* Nonzero means display messages at beginning and end of GC.  */
-int garbage_collection_messages;
+Boolint garbage_collection_messages;
 
 /* "Garbage collecting" */
 Lisp_Object Vgc_message;
@@ -1103,7 +1099,7 @@ Lisp_Object Qgarbage_collecting;
 struct frame *f;
 int speccount;
 int cursor_changed;
-Lisp_Object pre_gc_cursor;
+Lisp_Object Vpre_gc_cursor;
 
 /* PROFILE_DECLARE */
 int do_backtrace;
@@ -1118,7 +1114,7 @@ static void
 show_gc_cursor_and_message (void) 
 {
   /* Now show the GC cursor/message. */
-  pre_gc_cursor = Qnil;
+  Vpre_gc_cursor = Qnil;
   cursor_changed = 0;
 
   /* We used to call selected_frame() here.
@@ -1128,7 +1124,8 @@ show_gc_cursor_and_message (void)
   {
     Lisp_Object frame;
     Lisp_Object device = Fselected_device (Qnil);
-    if (NILP (device)) /* Could happen during startup, eg. if always_gc */
+    if (NILP (device)) /* Could happen during startup, eg. if
+                          gc-cons-threshold negative. */
       return;
     frame = Fselected_frame (device);
     if (NILP (frame))
@@ -1144,10 +1141,10 @@ show_gc_cursor_and_message (void)
 	  Lisp_Object cursor = glyph_image_instance (Vgc_pointer_glyph,
 						     FRAME_SELECTED_WINDOW (f),
 						     ERROR_ME_NOT, 1);
-	  pre_gc_cursor = f->pointer;
+	  Vpre_gc_cursor = f->pointer;
 	  if (POINTER_IMAGE_INSTANCEP (cursor)
 	      /* don't change if we don't know how to change back. */
-	      && POINTER_IMAGE_INSTANCEP (pre_gc_cursor))
+	      && POINTER_IMAGE_INSTANCEP (Vpre_gc_cursor))
 	    {
 	      cursor_changed = 1;
 	      Fset_frame_pointer (frame, cursor);
@@ -1178,7 +1175,7 @@ remove_gc_cursor_and_message (void)
   if (!noninteractive)
     {
       if (cursor_changed)
-	Fset_frame_pointer (wrap_frame (f), pre_gc_cursor);
+	Fset_frame_pointer (wrap_frame (f), Vpre_gc_cursor);
       else if (!FRAME_STREAM_P (f))
 	{
 	  /* Show "...done" only if the echo area would otherwise be empty. */
@@ -1457,38 +1454,6 @@ void garbage_collect_1 (void)
 /************************************************************************/
 
 /* Initialization */
-static void
-common_init_gc_early (void)
-{
-  Vgc_message = Qzero;
-
-  gc_currently_forbidden = 0;
-  gc_hooks_inhibited = 0;
-
-  need_to_garbage_collect = always_gc;
-
-  gc_cons_threshold = GC_CONS_THRESHOLD;
-  gc_cons_percentage = 40; /* #### what is optimal? */
-  total_gc_usage_set = 0;
-}
-
-void
-init_gc_early (void)
-{
-}
-
-void
-reinit_gc_early (void)
-{
-  common_init_gc_early ();
-}
-
-void
-init_gc_once_early (void)
-{
-  common_init_gc_early ();
-}
-
 void
 syms_of_gc (void)
 {
@@ -1499,7 +1464,8 @@ syms_of_gc (void)
 void
 vars_of_gc (void)
 {
-  staticpro_nodump (&pre_gc_cursor);
+  Vpre_gc_cursor = Qnil;
+  staticpro_dump_nil (&Vpre_gc_cursor);
 
   QSin_garbage_collection = build_defer_string ("(in garbage collection)");
   staticpro (&QSin_garbage_collection);
@@ -1528,6 +1494,7 @@ long enough to be able to set it back.
  
 See also `consing-since-gc' and `gc-cons-percentage'.
 */ );
+  gc_cons_threshold = GC_CONS_THRESHOLD;
 
   DEFVAR_INT ("gc-cons-percentage", &gc_cons_percentage /*
 *Percentage of memory allocated between garbage collections.
@@ -1544,12 +1511,6 @@ usage.
 See also `consing-since-gc' and `gc-cons-threshold'.
 */ );
 
-
-  DEFVAR_BOOL ("purify-flag", &purify_flag /*
-Non-nil means loading Lisp code in order to dump an executable.
-This means that certain objects should be allocated in readonly space.
-*/ );
-
   DEFVAR_BOOL ("garbage-collection-messages", &garbage_collection_messages /*
 *Non-nil means display messages at start and end of garbage collection.
 */ );
@@ -1578,7 +1539,7 @@ window system and `gc-pointer-glyph' specifies a value (i.e. a pointer
 image instance) in the domain of the selected frame, the mouse pointer
 will change instead of this message being printed.
 */ );
-  Vgc_message = build_defer_string (gc_default_message);
+  Vgc_message = Qzero;
 
   DEFVAR_LISP ("gc-pointer-glyph", &Vgc_pointer_glyph /*
 Pointer glyph used to indicate that a garbage collection is in progress.
