@@ -58,7 +58,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #include "console-x-impl.h"
 #include "elhash.h"
 
-static int
+static Boolint
 gc_cache_eql (const Hash_Table_Test * UNUSED (http),
               Lisp_Object arg1, Lisp_Object arg2)
 {
@@ -155,7 +155,7 @@ gc_cache_hash (const Hash_Table_Test * UNUSED (http), Lisp_Object arg)
 }
 
 Lisp_Object
-define_gc_cache_hash_table_test ()
+define_gc_cache_hash_table_test (void)
 {
   return define_hash_table_test (Qunbound, gc_cache_eql, gc_cache_hash,
                                  Qnil, Qnil);
