@@ -58,8 +58,11 @@ SYMBOL (Qautodetect);
 SYMBOL (Qbad_variable);
 SYMBOL (Qbefore);
 SYMBOL (Qbigfloat);
+SYMBOL (Qbigfloatp);
+SYMBOL (Qbignump);
 SYMBOL (Qbinary);
 SYMBOL (Qbitmap);
+SYMBOL (Qbit_vector);
 SYMBOL (Qboolean);
 SYMBOL_KEYWORD (Q_border);
 SYMBOL (Qbottom);
@@ -80,8 +83,10 @@ SYMBOL (Qchain);
 SYMBOL (Qchange);
 SYMBOL (Qchannel);
 SYMBOL (Qchar);
+SYMBOL (Qchar_or_char_int_p);
 SYMBOL (Qcharacter);
 SYMBOL (Qchars);
+SYMBOL (Qclicked);
 SYMBOL (Qcode_page);
 SYMBOL (Qcoding_system);
 SYMBOL (Qcoerce);
@@ -90,6 +95,7 @@ SYMBOL (Qcolumns);
 SYMBOL (Qcommand);
 SYMBOL_KEYWORD (Q_config);
 SYMBOL (Qconsole);
+SYMBOL (Qcontext);
 SYMBOL (Qcontrol_1);
 SYMBOL (Qcopies);
 SYMBOL (Qcount);
@@ -111,25 +117,27 @@ SYMBOL (Qdelq);
 SYMBOL (Qdescription);
 SYMBOL_KEYWORD (Q_descriptor);
 SYMBOL (Qdevice);
-SYMBOL (Qdevices);
 SYMBOL_KEYWORD (Q_device);
+SYMBOL (Qdevices);
 SYMBOL (Qdialog);
-SYMBOL (Qdirectory);
 SYMBOL (Qdimension);
+SYMBOL (Qdirectory);
+SYMBOL (Qdiscard);
 SYMBOL (Qdisplay);
 SYMBOL (Qdoc_string);
 SYMBOL (Qdocumentation);
 SYMBOL (Qduplex);
 SYMBOL (Qemergency);
 SYMBOL (Qempty);
-SYMBOL_KEYWORD (Q_end);
 SYMBOL (Qencode_as_utf_8);
+SYMBOL_KEYWORD (Q_end);
 SYMBOL (Qeval);
 SYMBOL (Qevent);
 SYMBOL (Qextents);
 SYMBOL (Qexternal);
 SYMBOL (Qface);
 SYMBOL (Qfaces);
+SYMBOL (Qfail);
 SYMBOL (Qfallback);
 SYMBOL (Qfile);
 SYMBOL_MODULE_API (Qfile_name);
@@ -138,6 +146,7 @@ SYMBOL (Qfinal);
 SYMBOL (Qfixnum);
 SYMBOL_MODULE_API (Qfixnump);
 SYMBOL (Qfloat);
+SYMBOL (Qfloatingp);
 SYMBOL (Qfont);
 SYMBOL (Qframe);
 SYMBOL (Qframes);
@@ -146,6 +155,8 @@ SYMBOL (Qfrom_unicode);
 SYMBOL (Qfull_assoc);
 SYMBOL (Qfuncall);
 SYMBOL (Qfunction);
+SYMBOL (Qfunction_min_args);
+SYMBOL (Qfunction_max_args);
 SYMBOL (Qgarbage_collection);
 SYMBOL (Qgeneric);
 SYMBOL (Qgeometry);
@@ -160,23 +171,23 @@ SYMBOL (Qhelp);
 SYMBOL (Qhigh);
 SYMBOL (Qhighlight);
 SYMBOL (Qhorizontal);
+SYMBOL_KEYWORD (Q_horizontally_justify);
 SYMBOL (Qicon);
 SYMBOL (Qid);
 SYMBOL (Qignore);
-SYMBOL (Qinitial);
 SYMBOL (Qimage);
 SYMBOL_KEYWORD (Q_image);
 SYMBOL_KEYWORD (Q_included);
 SYMBOL (Qinfo);
 SYMBOL (Qinherit);
+SYMBOL (Qinitial);
 SYMBOL_KEYWORD (Q_initial_focus);
 SYMBOL (Qinteger);
 SYMBOL (Qinternal);
+SYMBOL (Qinvalid_sequence);
 SYMBOL (Qiso2022);
 SYMBOL_KEYWORD (Q_items);
 SYMBOL_KEYWORD (Q_justify);
-SYMBOL_KEYWORD (Q_vertically_justify);
-SYMBOL_KEYWORD (Q_horizontally_justify);
 SYMBOL_KEYWORD (Q_key);
 SYMBOL (Qkey);
 SYMBOL (Qkey_assoc);
@@ -213,11 +224,14 @@ SYMBOL (Qname);
 SYMBOL_MODULE_API (Qnative);
 SYMBOL (Qnatnum);
 SYMBOL (Qno);
+SYMBOL (Qno_character_typed);
 SYMBOL (Qnone);
 SYMBOL (Qnot);
 SYMBOL (Qnothing);
 SYMBOL_MODULE_API (Qnotice);
+SYMBOL (Qnull);
 SYMBOL (Qobject);
+SYMBOL (Qoffset);
 SYMBOL (Qok);
 SYMBOL (Qold_assoc);
 SYMBOL (Qold_delete);
@@ -251,6 +265,9 @@ SYMBOL (Qradio);
 SYMBOL (Qrassoc);
 SYMBOL (Qrassq);
 SYMBOL (Qratio);
+SYMBOL (Qrationalp);
+SYMBOL (Qratiop);
+SYMBOL (Qrealp);
 SYMBOL (Qredisplay);
 SYMBOL (Qremove_all);
 SYMBOL (Qrequire);
@@ -269,21 +286,27 @@ SYMBOL (Qseven);
 SYMBOL (Qsignal);
 SYMBOL_MODULE_API (Qsimple);
 SYMBOL (Qsize);
+SYMBOL (Qsize_device);
 SYMBOL (Qsound);
 SYMBOL (Qspace);
 SYMBOL (Qspecifier);
 SYMBOL (Qstandard);
 SYMBOL_KEYWORD (Q_start);
 SYMBOL (Qstream);
+SYMBOL (Qstreamp);
 SYMBOL (Qstring);
 SYMBOL (Qstring_match);
 SYMBOL_KEYWORD (Q_style);
-SYMBOL_KEYWORD (Q_suffix);
+SYMBOL (Qsubstitute);
 SYMBOL (Qsubtype);
+SYMBOL (Qsucceed);
 SYMBOL (Qsucceeded);
+SYMBOL_KEYWORD (Q_suffix);
 SYMBOL (Qsymbol);
+SYMBOL (Qsymbol_file);
 SYMBOL (Qsyntax);
 SYMBOL (Qsystem_default);
+SYMBOL (Qtags);
 SYMBOL (Qterminal);
 SYMBOL (Qtest);
 SYMBOL_KEYWORD (Q_test);
@@ -295,6 +318,7 @@ SYMBOL (Qtimestamp);
 SYMBOL_KEYWORD (Q_title);
 SYMBOL (Qto_page);
 SYMBOL (Qtoggle);
+SYMBOL (Qtoggled);
 SYMBOL (Qtoolbar);
 SYMBOL (Qtop);
 SYMBOL (Qtop_margin);
@@ -304,22 +328,25 @@ SYMBOL_KEYWORD (Q_type);
 SYMBOL (Qundecided);
 SYMBOL (Qundefined);
 SYMBOL (Qunencodable);
-SYMBOL (Qinvalid_sequence);
-SYMBOL (Qunimplemented);
 SYMBOL (Qunicode_registries);
 SYMBOL (Qunicode_type);
+SYMBOL (Qunsupported_type);
+SYMBOL (Qunimplemented);
 SYMBOL (Quser_default);
 SYMBOL_KEYWORD (Q_value);
 SYMBOL (Qvalue_assoc);
+SYMBOL (Qvector);
 SYMBOL (Qvertical);
+SYMBOL_KEYWORD (Q_vertically_justify);
+SYMBOL_KEYWORD (Q_visible);
 SYMBOL (Qwarning);
 SYMBOL (Qwidget);
 SYMBOL (Qwidth);
 SYMBOL_KEYWORD (Q_width);
 SYMBOL (Qwindow);
-SYMBOL (Qwindows);
 SYMBOL (Qwindow_id);
 SYMBOL (Qwindow_system);
+SYMBOL (Qwindows);
 SYMBOL (Qx);
 SYMBOL (Qy);
 SYMBOL (Qyes);
