@@ -387,15 +387,18 @@ jpeg_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
        */
 
       {
-	Lisp_Object errstring;
 	Extbyte buffer[JMSG_LENGTH_MAX];
+        Ibyte *ibuffer;
 
 	/* Create the message */
 	(*cinfo.err->format_message) ((j_common_ptr) &cinfo, buffer);
-	errstring = build_extstring (buffer, Qjpeg_error_message_encoding);
 
-	signal_image_error_2 ("JPEG decoding error",
-			      errstring, instantiator);
+        TO_INTERNAL_FORMAT (C_STRING, buffer, C_STRING_ALLOCA, ibuffer,
+                            Qjpeg_error_message_encoding);
+        error_or_quit_failed_instantiator_in_domain ((const Ascbyte *) ibuffer,
+                                                     instantiator,
+                                                     IMAGE_INSTANCE_DOMAIN
+                                                     (ii));
       }
     }
 
@@ -453,12 +456,23 @@ jpeg_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
     pixels_sq =
       (UINT_64_BIT) cinfo.output_width * (UINT_64_BIT) cinfo.output_height;
     if (pixels_sq > ((size_t) -1) / 3)
-      signal_image_error ("JPEG image too large to instantiate", instantiator);
+      {
+        error_or_quit_failed_instantiator_in_domain ("JPEG image too large to"
+                                                     " instantiate",
+                                                     instantiator,
+                                                     IMAGE_INSTANCE_DOMAIN
+                                                     (ii));
+      }
     unwind.eimage =
       xnew_binbytes (cinfo.output_width * cinfo.output_height * 3);
     if (!unwind.eimage)
-      signal_image_error("Unable to allocate enough memory for image", instantiator);
-
+      {
+        error_or_quit_failed_instantiator_in_domain ("Unable to allocate "
+                                                     "enough memory for image",
+                                                     instantiator,
+                                                     IMAGE_INSTANCE_DOMAIN
+                                                     (ii));
+      }
     {
       JSAMPARRAY row_buffer;	/* Output row buffer */
       JSAMPLE *jp;
@@ -586,8 +600,14 @@ gif_instantiate_unwind (Lisp_Object unwind_obj)
   free_opaque_ptr (unwind_obj);
   if (data->giffile)
     {
+#if defined(GIFLIB_MAJOR) && ((GIFLIB_MAJOR > 5) || (GIFLIB_MAJOR == 5 && GIFLIB_MINOR > 0))
+      int error;
+      DGifCloseFile (data->giffile, &error);
+      GifFreeSavedImages (data->giffile);
+#else
       DGifCloseFile (data->giffile);
       FreeSavedImages (data->giffile);
+#endif
       data->giffile = 0;
     }
   if (data->eimage)
@@ -619,9 +639,9 @@ gif_read_from_memory (GifFileType *gif, GifByteType *buf, int size)
 }
 
 static const char *
-gif_decode_error_string (void)
+gif_decode_error_string (int error)
 {
-  switch (GifLastError ())
+  switch (error)
     {
     case D_GIF_ERR_OPEN_FAILED:
       return "GIF error: unable to open";
@@ -685,27 +705,52 @@ gif_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
     mem_struct.bytes = bytes;
     mem_struct.len = len;
     mem_struct.index = 0;
+#if defined(GIFLIB_MAJOR) && ((GIFLIB_MAJOR > 5) || (GIFLIB_MAJOR == 5 && GIFLIB_MINOR > 0))
+    {
+      int error = 0;
+      unwind.giffile = DGifOpen (&mem_struct, gif_read_from_memory, &error);
+      if (unwind.giffile == NULL)
+	signal_image_error (gif_decode_error_string (error), instantiator);
+      /* Then slurp the image into memory, decoding along the way.
+	 The result is the image in a simple one-byte-per-pixel
+	 format. */
+      if (DGifSlurp (unwind.giffile) == GIF_ERROR)
+	signal_image_error (gif_decode_error_string (unwind.giffile->Error),
+			    instantiator);
+    }
+#else
     unwind.giffile = DGifOpen (&mem_struct, gif_read_from_memory);
     if (unwind.giffile == NULL)
-      signal_image_error (gif_decode_error_string (), instantiator);
-
+      signal_image_error (gif_decode_error_string (GifLastError ()),
+                          instantiator);
     /* Then slurp the image into memory, decoding along the way.
        The result is the image in a simple one-byte-per-pixel
        format. */
     if (DGifSlurp (unwind.giffile) == GIF_ERROR)
-      signal_image_error (gif_decode_error_string (), instantiator);
+      signal_image_error (gif_decode_error_string (GifLastError ()),
+                          instantiator);
+#endif
+    /* dgif_lib.c doesn't do any validation on these. */
+    check_integer_range (make_fixnum (1),
+                         make_fixnum (unwind.giffile->SHeight),
+                         make_fixnum (MOST_POSITIVE_FIXNUM));
+    check_integer_range (make_fixnum (1),
+                         make_fixnum (unwind.giffile->SWidth),
+                         make_fixnum (MOST_POSITIVE_FIXNUM));
   }
 
   /* 3. Now create the EImage(s) */
   {
-    ColorMapObject *cmo = (unwind.giffile->Image.ColorMap ? unwind.giffile->Image.ColorMap : unwind.giffile->SColorMap);
-    int i, j, row, pass, interlace, slice;
+    ColorMapObject *cmo
+      = (unwind.giffile->Image.ColorMap ? unwind.giffile->Image.ColorMap
+         : unwind.giffile->SColorMap);
+    int i, j, row, interlace, slice;
     UINT_64_BIT pixels_sq;
     Binbyte *eip;
     /* interlaced gifs have rows in this order:
        0, 8, 16, ..., 4, 12, 20, ..., 2, 6, 10, ..., 1, 3, 5, ...  */
-    static int InterlacedOffset[] = { 0, 4, 2, 1 };
-    static int InterlacedJumps[] = { 8, 8, 4, 2 };
+    static const int InterlacedOffset[] = { 0, 4, 2, 1 };
+    static const int InterlacedJumps[] = { 8, 8, 4, 2 };
 
     if (cmo == NULL)
       signal_image_error ("GIF image has no color map", instantiator);
@@ -718,13 +763,16 @@ gif_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
     unwind.eimage =
       xnew_binbytes (width * height * 3 * unwind.giffile->ImageCount);
     if (!unwind.eimage)
-      signal_image_error("Unable to allocate enough memory for image", instantiator);
+      signal_image_error("Unable to allocate enough memory for image",
+                         instantiator);
 
     /* write the data in EImage format (8bit RGB triples) */
 
     for (slice = 0; slice < unwind.giffile->ImageCount; slice++)
       {
-	/* We check here that the current image covers the full "screen" size. */
+	unsigned int pass;
+	/* We check here that the current image covers the full "screen"
+           size. */
 	if (unwind.giffile->SavedImages[slice].ImageDesc.Height != height
 	    || unwind.giffile->SavedImages[slice].ImageDesc.Width != width
 	    || unwind.giffile->SavedImages[slice].ImageDesc.Left != 0
@@ -733,27 +781,41 @@ gif_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 			      instantiator);
 
 	interlace = unwind.giffile->SavedImages[slice].ImageDesc.Interlace;
-	pass = 0;
-	row = interlace ? InterlacedOffset[pass] : 0;
+
+        pass = 0;
+	row = 0; /* interlace ? InterlacedOffset[pass] : 0; */
 	eip = unwind.eimage + (width * height * 3 * slice);
 	for (i = 0; i < height; i++)
 	  {
 	    if (interlace)
 	      if (row >= height) {
-		row = InterlacedOffset[++pass];
+		row = InterlacedOffset[(++pass) & 3];
+                glyph_checking_assert (pass < countof (InterlacedOffset));
 		while (row >= height)
-		  row = InterlacedOffset[++pass];
+                  {
+                    /* We only run into trouble here if height is less than 1
+                       (the last value in InterlacedOffset), and we've checked
+                       for that above.
+
+                       The & 3 is not necessary--see the assertions--but it
+                       quiets compiler warnings about reading beyond the end
+                       of InterlacedOffset. */
+                    row = InterlacedOffset[(++pass) & 3];
+                    glyph_checking_assert (pass < countof (InterlacedOffset));
+                  }
 	      }
-	    eip = unwind.eimage + (width * height * 3 * slice) + (row * width * 3);
+	    eip = unwind.eimage + (width * height * 3 * slice)
+              + (row * width * 3);
 	    for (j = 0; j < width; j++)
 	      {
 		Binbyte pixel =
-		  unwind.giffile->SavedImages[slice].RasterBits[(i * width) + j];
+		  unwind.giffile->SavedImages[slice].RasterBits[(i * width)
+                                                                + j];
 		*eip++ = cmo->Colors[pixel].Red;
 		*eip++ = cmo->Colors[pixel].Green;
 		*eip++ = cmo->Colors[pixel].Blue;
 	      }
-	    row += interlace ? InterlacedJumps[pass] : 1;
+	    row += interlace ? InterlacedJumps[pass & 3] : 1;
 	  }
       }
 
@@ -775,6 +837,18 @@ gif_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
       unsigned short timeout = 0;
       Lisp_Object tid;
 
+#if defined(GIFLIB_MAJOR) && ((GIFLIB_MAJOR > 5) || (GIFLIB_MAJOR == 5 && GIFLIB_MINOR > 0))
+      if (unwind.giffile->SavedImages[0].ExtensionBlockCount
+	  &&
+	  unwind.giffile->SavedImages[0].ExtensionBlocks[0].Function
+	  == GRAPHICS_EXT_FUNC_CODE
+	  )
+	{
+	  timeout = (unsigned short)
+	    ((unwind.giffile->SavedImages[0].ExtensionBlocks[0].Bytes[2] << 8) +
+	     unwind.giffile-> SavedImages[0].ExtensionBlocks[0].Bytes[1]) * 10;
+	}
+#else
       if (unwind.giffile->SavedImages[0].Function == GRAPHICS_EXT_FUNC_CODE
 	  &&
 	  unwind.giffile->SavedImages[0].ExtensionBlockCount)
@@ -783,6 +857,7 @@ gif_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 	    ((unwind.giffile->SavedImages[0].ExtensionBlocks[0].Bytes[2] << 8) +
 	     unwind.giffile-> SavedImages[0].ExtensionBlocks[0].Bytes[1]) * 10;
 	}
+#endif
 
       /* Too short a timeout will crucify us performance-wise. */
       tid = add_glyph_animated_timeout (timeout > 10 ? timeout : 10, image_instance);
@@ -1031,10 +1106,18 @@ png_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 	  Lisp_Object rgb = MAYBE_LISP_DEVMETH (XDEVICE (c->device),
 						color_instance_rgb_components,
 						(c));
+#ifdef HAVE_GTK3
+#define GETCOLOR(col) my_background.col = (unsigned short) (XFLOATFIXNUM (XCAR (rgb)) * 65535.0)
+	  GETCOLOR(red); rgb = XCDR (rgb);
+	  GETCOLOR(green); rgb = XCDR (rgb);
+	  GETCOLOR(blue);
+	  /* What about alpha? - jsparkes */
+#else
 #define GETCOLOR(col) my_background.col = (unsigned short) XFIXNUM (XCAR (rgb))
 	  GETCOLOR(red); rgb = XCDR (rgb);
 	  GETCOLOR(green); rgb = XCDR (rgb);
 	  GETCOLOR(blue);
+#endif
 #undef GETCOLOR
 	}
 
@@ -1260,65 +1343,54 @@ tiff_memory_size (thandle_t data)
   return mem->len;
 }
 
-struct tiff_error_struct
-{
-#ifdef HAVE_VSNPRINTF
-  char err_str[256];
-#else
-  char err_str[1024];		/* return the error string */
-#endif
-  jmp_buf setjmp_buffer;	/* for return to caller */
-};
-
-/* jh 98/03/12 - ###This struct for passing data to the error functions
-   is an ugly hack caused by the fact that libtiff (as of v3.4) doesn't
-   have any place to store error func data.  This should be rectified
-   before XEmacs gets threads! */
-static struct tiff_error_struct tiff_err_data;
-
 static void
-tiff_error_func (const char *UNUSED (module), const char *fmt, ...)
+tiff_error_func (const char *module_external, const char *fmt, ...)
 {
+  Lisp_Object stream = make_resizing_buffer_output_stream ();
+  Ibyte *module = EXTERNAL_TO_ITEXT (module_external,
+                                     Qerror_message_encoding);
   va_list vargs;
+  struct gcpro gcpro1;
+
+  GCPRO1 (stream); /* Apotropaic, we won't GC in this. */
+
+  write_istring (stream, module);
+  write_ascstring (stream, " - ");
 
   va_start (vargs, fmt);
-#ifdef HAVE_VSNPRINTF
-  vsnprintf (tiff_err_data.err_str, 255, fmt, vargs);
-#else
-  /* pray this doesn't overflow... */
-  vsprintf (tiff_err_data.err_str, fmt, vargs);
-#endif
+  write_external_fmt_string_va (stream, Qerror_message_encoding, fmt, vargs);
   va_end (vargs);
-  /* return to setjmp point */
-  longjmp (tiff_err_data.setjmp_buffer, 1);
+
+  UNGCPRO;
+
+  /* An error was signaled. No clean up is needed, as unwind handles that
+     for us.  Just pass the error along. */
+  signal_image_error ("TIFF decoding error",
+                      resizing_buffer_to_lisp_string (XLSTREAM (stream)));
 }
 
 static void
-tiff_warning_func (const char *module, const char *fmt, ...)
+tiff_warning_func (const char *module_external, const char *fmt, ...)
 {
+  Lisp_Object stream = make_resizing_buffer_output_stream ();
+  Ibyte *module = EXTERNAL_TO_ITEXT (module_external, Qerror_message_encoding);
   va_list vargs;
-#ifdef HAVE_VSNPRINTF
-  char warn_str[256];
-#else
-  char warn_str[1024];
-#endif
-  DECLARE_EISTRING (eimodule);
-  DECLARE_EISTRING (eiwarnstr);
+  struct gcpro gcpro1;
+
+  GCPRO1 (stream); /* Apotropaic, we won't GC in this. */
+
+  write_istring (stream, module);
+  write_ascstring (stream, " - ");
 
   va_start (vargs, fmt);
-#ifdef HAVE_VSNPRINTF
-  vsnprintf (warn_str, 255, fmt, vargs);
-#else
-  vsprintf (warn_str, fmt, vargs);
-#endif
+  write_external_fmt_string_va (stream, Qerror_message_encoding, fmt, vargs);
   va_end (vargs);
 
-  eicpy_ext(eimodule, module, Qbinary);  
-  eicpy_ext(eiwarnstr, warn_str, Qbinary);  
+  warn_when_safe_lispobj (Qtiff, Qinfo,
+                          resizing_buffer_to_lisp_string (XLSTREAM (stream)));
+  UNGCPRO;
 
-  warn_when_safe (Qtiff, Qinfo, "%s - %s",
-                  eidata(eimodule), 
-                  eidata(eiwarnstr));
+  Lstream_delete (XLSTREAM (stream));
 }
 
 static void
@@ -1333,21 +1405,12 @@ tiff_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
      stack frame is still valid. */
   struct tiff_unwind_data unwind;
   int speccount = specpdl_depth ();
-  uint32 width, height;
+  UINT_32_BIT width, height;
 
   xzero (unwind);
   record_unwind_protect (tiff_instantiate_unwind, make_opaque_ptr (&unwind));
 
   /* set up error facilities */
-  if (setjmp (tiff_err_data.setjmp_buffer))
-    {
-      /* An error was signaled. No clean up is needed, as unwind handles that
-	 for us.  Just pass the error along. */
-      signal_image_error_2 ("TIFF decoding error",
-			    build_extstring (tiff_err_data.err_str,
-					      Qerror_message_encoding),
-			    instantiator);
-    }
   TIFFSetErrorHandler ((TIFFErrorHandler)tiff_error_func);
   TIFFSetWarningHandler ((TIFFErrorHandler)tiff_warning_func);
   {
@@ -1355,7 +1418,7 @@ tiff_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
     Binbyte *bytes;
     Bytecount len;
 
-    uint32 *raster;
+    UINT_32_BIT *raster;
     Binbyte *ep;
     UINT_64_BIT pixels_sq;
 
@@ -1386,11 +1449,13 @@ tiff_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
     /* #### This is little more than proof-of-concept/function testing.
        It needs to be reimplemented via scanline reads for both memory
        compactness. */
-    raster = (uint32*) _TIFFmalloc ((tsize_t) (pixels_sq * sizeof (uint32)));
+    raster
+      = (UINT_32_BIT*) _TIFFmalloc ((tsize_t) (pixels_sq *
+					       sizeof (UINT_32_BIT)));
     if (raster != NULL)
       {
 	int i, j;
-	uint32 *rp;
+	UINT_32_BIT *rp;
 	ep = unwind.eimage;
 	rp = raster;
 	if (TIFFReadRGBAImage (unwind.tiff, width, height, raster, 0))
