@@ -57,6 +57,8 @@ static void (*orig_next_event_cb) (Lisp_Event *);
 static Lisp_Object gpm_event_queue;
 static Lisp_Object gpm_event_queue_tail;
 
+static Lisp_Object Qreceive_gpm_event;
+
 struct __gpm_state
 {
   int gpm_tried;
@@ -114,7 +116,6 @@ This function is the process handler for the GPM connection.
   Lisp_Object fake_event = Qnil;
   Lisp_Event *event = NULL;
   struct gcpro gcpro1;
-  static int num_events;
 
   CHECK_PROCESS (process);
 
@@ -129,8 +130,6 @@ This function is the process handler for the GPM connection.
 
   GCPRO1 (fake_event);
 
-  num_events++;
-
   fake_event = Fmake_event (Qnil, Qnil);
   event = XEVENT (fake_event);
 
@@ -451,9 +450,9 @@ static void hook_event_callbacks_once (void)
 
   if (!hooker)
     {
-      orig_event_pending_p = event_stream->event_pending_p;
+      orig_event_pending_p = event_stream->event_pending_p_cb;
       orig_next_event_cb = event_stream->next_event_cb;
-      event_stream->event_pending_p = gpm_event_pending_p;
+      event_stream->event_pending_p_cb = gpm_event_pending_p;
       event_stream->next_event_cb = gpm_next_event_cb;
       hooker = 1;
     }
@@ -505,7 +504,6 @@ Toggle accepting of GPM mouse events.
   Gpm_Connect conn;
   int rval;
   Lisp_Object gpm_process;
-  Lisp_Object gpm_filter;
   Lisp_Object console_count;
   struct device *d = decode_device (device);
   int fd = DEVICE_INFD (d);
@@ -602,8 +600,7 @@ Toggle accepting of GPM mouse events.
 	{
 	  rval = 0;
 	  Fprocess_kill_without_query (gpm_process, Qnil);
-	  gpm_filter = GET_DEFUN_LISP_OBJECT (Freceive_gpm_event);
-	  set_process_filter (gpm_process, gpm_filter, 1, 0);
+	  set_process_filter (gpm_process, Qreceive_gpm_event, 1, 0);
 
 	  /* Keep track of the device for later */
 	  /* Fput (gpm_process, intern ("gpm-device"), device); */
@@ -618,18 +615,20 @@ Toggle accepting of GPM mouse events.
   return (rval ? Qnil : Qt);
 }
 
-void vars_of_gpmevent (void)
+void
+vars_of_gpmevent (void)
 {
   gpm_event_queue = Qnil;
-  gpm_event_queue_tail = Qnil;
   staticpro (&gpm_event_queue);
-  staticpro (&gpm_event_queue_tail);
-  dump_add_root_lisp_object (&gpm_event_queue);
+
+  gpm_event_queue_tail = Qnil;
   dump_add_root_lisp_object (&gpm_event_queue_tail);
 }
 
-void syms_of_gpmevent (void)
+void
+syms_of_gpmevent (void)
 {
+  DEFSYMBOL (Qreceive_gpm_event);
   DEFSUBR (Freceive_gpm_event);
   DEFSUBR (Fgpm_enable);
   DEFSUBR (Fgpm_enabled_p);
