@@ -42,37 +42,42 @@ extern Lisp_Object Vscrollbar_on_left_p;
 EXFUN (Fmake_image_instance, 4);
 
 static void gtk_xemacs_class_init	(GtkXEmacsClass *klass);
-static void gtk_xemacs_init		(GtkXEmacs *xemacs);
+static void gtk_xemacs_init		(GtkXEmacs *xemacs, GtkXEmacsClass *klass);
 static void gtk_xemacs_size_allocate	(GtkWidget *widget, GtkAllocation *allocaction);
-static void gtk_xemacs_draw		(GtkWidget *widget, GdkRectangle *area);
-static void gtk_xemacs_paint		(GtkWidget *widget, GdkRectangle *area);
 static void gtk_xemacs_size_request	(GtkWidget *widget, GtkRequisition *requisition);
 static void gtk_xemacs_realize		(GtkWidget *widget);
 static void gtk_xemacs_style_set        (GtkWidget *widget, GtkStyle *previous_style);
+#ifdef HAVE_GTK3
+static void gtk_xemacs_paint (GtkWidget *widget, cairo_rectangle_int_t *cr);
+static void gtk_xemacs_get_preferred_height (GtkWidget *, gint *minimal_height,
+					     gint *natural_height);
+static void gtk_xemacs_get_preferred_width (GtkWidget *, gint *minimal_width,
+						gint *natural_width);
+static gboolean gtk_xemacs_draw		(GtkWidget *widget, cairo_t *cr);
+#endif
+#ifdef HAVE_GTK2
+static void gtk_xemacs_paint		(GtkWidget *widget, GdkRectangle *area);
 static gint gtk_xemacs_expose		(GtkWidget *widget, GdkEventExpose *event);
+#endif
 
-guint
+GType
 gtk_xemacs_get_type (void)
 {
-  static guint xemacs_type = 0;
+  static GType xemacs_type = G_TYPE_INVALID;
 
-  if (!xemacs_type)
+  if (xemacs_type == G_TYPE_INVALID)
     {
-      static const GtkTypeInfo xemacs_info =
-      {
-	"GtkXEmacs",
-	sizeof (GtkXEmacs),
-	sizeof (GtkXEmacsClass),
-	(GtkClassInitFunc) gtk_xemacs_class_init,
-	(GtkObjectInitFunc) gtk_xemacs_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
-      };
-
-      xemacs_type = gtk_type_unique (gtk_fixed_get_type (), &xemacs_info);
+      xemacs_type =
+        g_type_register_static_simple (GTK_TYPE_FIXED,
+                                       "GtkXEmacs",
+                                       sizeof (GtkXEmacsClass),
+                                       (GClassInitFunc) gtk_xemacs_class_init,
+                                       sizeof (GtkXEmacs),
+                                       (GInstanceInitFunc) gtk_xemacs_init,
+                                       (GTypeFlags) 0);
     }
 
+  assert (xemacs_type != G_TYPE_INVALID);
   return xemacs_type;
 }
 
@@ -83,26 +88,32 @@ gtk_xemacs_class_init (GtkXEmacsClass *class_)
 {
   GtkWidgetClass *widget_class;
 
-  widget_class = (GtkWidgetClass*) class_;
-  parent_class = (GtkWidgetClass *) gtk_type_class (gtk_fixed_get_type ());
+  widget_class = (GtkWidgetClass *) class_;
+  parent_class = (GtkWidgetClass *) g_type_class_peek (GTK_TYPE_FIXED);
 
   widget_class->size_allocate = gtk_xemacs_size_allocate;
+#ifdef HAVE_GTK2
   widget_class->size_request = gtk_xemacs_size_request;
-  widget_class->draw = gtk_xemacs_draw;
   widget_class->expose_event = gtk_xemacs_expose;
+  widget_class->style_set = gtk_xemacs_style_set;
+#endif
+#ifdef HAVE_GTK3
+  widget_class->get_preferred_height = gtk_xemacs_get_preferred_height;
+  widget_class->get_preferred_width  = gtk_xemacs_get_preferred_width;
+  widget_class->draw = gtk_xemacs_draw;
+#endif
   widget_class->realize = gtk_xemacs_realize;
   widget_class->button_press_event = emacs_gtk_button_event_handler;
   widget_class->button_release_event = emacs_gtk_button_event_handler;
   widget_class->key_press_event = emacs_gtk_key_event_handler;
   widget_class->key_release_event = emacs_gtk_key_event_handler;
   widget_class->motion_notify_event = emacs_gtk_motion_event_handler;
-  widget_class->style_set = gtk_xemacs_style_set;
 }
 
 static void
-gtk_xemacs_init (GtkXEmacs *xemacs)
+gtk_xemacs_init (GtkXEmacs *xemacs, GtkXEmacsClass * UNUSED (klass))
 {
-    GTK_WIDGET_SET_FLAGS (xemacs, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (GTK_WIDGET (xemacs), TRUE);
 }
 
 GtkWidget*
@@ -110,14 +121,24 @@ gtk_xemacs_new (struct frame *f)
 {
   GtkXEmacs *xemacs;
 
-  xemacs = (GtkXEmacs*) gtk_type_new (gtk_xemacs_get_type ());
+  xemacs = GTK_XEMACS (g_object_new (GTK_TYPE_XEMACS, NULL));
+  gtk_widget_set_name (GTK_WIDGET (xemacs), "xemacs");
+  gtk_widget_set_has_window (GTK_WIDGET (xemacs), TRUE);
   xemacs->f = f;
+  gtk_widget_add_events (GTK_WIDGET (xemacs), GDK_BUTTON_RELEASE);
 
   return GTK_WIDGET (xemacs);
 }
 
 static void
-__nuke_background_items (GtkWidget *widget)
+__nuke_background_items (GtkWidget
+#ifdef HAVE_GTK2
+			 *widget
+#endif
+#ifdef HAVE_GTK3
+			 * UNUSED (widget)
+#endif
+			 )
 {
   /* This bit of voodoo is here to get around the annoying flicker
      when GDK tries to futz with our background pixmap as well as
@@ -131,8 +152,9 @@ __nuke_background_items (GtkWidget *widget)
      Well, wait, we do... otherwise there sre weird 'seethru' areas
      even when XEmacs does a full redisplay.  Most noticeable in some
      areas of the modeline, or in the right-hand-side of the window
-     between the scrollbar ad n the edge of the window.
+     between the scrollbar and the edge of the window.
   */
+#ifdef HAVE_GTK2
   if (widget->window)
     {
       gdk_window_set_back_pixmap (widget->window, NULL, 0);
@@ -142,12 +164,13 @@ __nuke_background_items (GtkWidget *widget)
       gdk_window_set_background (widget->window,
 				 &widget->style->bg[GTK_STATE_NORMAL]);
     }
+#endif
 }
 
 extern Lisp_Object xemacs_gtk_convert_color(GdkColor *c, GtkWidget *w);
 
 /* From fontcolor-gtk.c */
-extern Lisp_Object __get_gtk_font_truename (GdkFont *gdk_font, int expandp);
+extern Lisp_Object __get_gtk_font_truename (PangoFont *gdk_font, int expandp);
 
 #define convert_font(f) __get_gtk_font_truename (f, 0)
 
@@ -200,19 +223,25 @@ static void
 smash_scrollbar_specifiers (struct frame *f, GtkStyle *style)
 {
   Lisp_Object frame;
-  int slider_size = 0;
+  int slider_size = 10;
   int hsize, vsize;
-  GtkRangeClass *klass;
 
   frame = wrap_frame (f);
 
-  klass = (GtkRangeClass *) gtk_type_class (GTK_TYPE_SCROLLBAR);
-  slider_size = klass->slider_width;
-  hsize = slider_size + (style->klass->ythickness * 2);
-  vsize = slider_size + (style->klass->xthickness * 2);
+#ifdef HAVE_GTK2
+  // Note: where do I get this in Gtk 2.X?  -- jsparkes
+  //slider_size = style->slider_width;
+  slider_size = 10;
+  hsize = slider_size + (style->ythickness * 2);
+  vsize = slider_size + (style->xthickness * 2);
 
   style = gtk_style_attach (style,
 			    GTK_WIDGET (DEVICE_GTK_APP_SHELL (XDEVICE (FRAME_DEVICE (f))))->window);
+#endif
+#ifdef HAVE_GTK3
+  /* No properties for this in Gtk 3.X? */
+  hsize = vsize = slider_size;
+#endif
 
   Fadd_spec_to_specifier (Vscrollbar_width, make_fixnum (vsize), frame, Qnil, Qnil);
   Fadd_spec_to_specifier (Vscrollbar_height, make_fixnum (hsize), frame, Qnil, Qnil);
@@ -226,12 +255,12 @@ static void
 smash_toolbar_specifiers(struct frame *f, GtkStyle *style)
 {
   Lisp_Object frame;
-  GtkStyleClass *klass = (GtkStyleClass *) style->klass;
 
   frame = wrap_frame (f);
 
-  Fadd_spec_to_specifier (Vtoolbar_shadow_thickness, make_fixnum (klass->xthickness),
-			  Qnil, list2 (Qgtk, Qdefault), Qprepend);
+  Fadd_spec_to_specifier (Vtoolbar_shadow_thickness,
+			  make_fixnum (style->xthickness), Qnil, 
+			  list2 (Qgtk, Qdefault), Qprepend);
 }
 #endif /* HAVE_TOOLBARS */
 
@@ -270,73 +299,67 @@ gtk_xemacs_size_request (GtkWidget *widget, GtkRequisition *requisition)
 {
     GtkXEmacs *x = GTK_XEMACS (widget);
     struct frame *f = GTK_XEMACS_FRAME (x);
-    int width, height;
+    int width, height, unused;
 
     if (f)
       {
 	frame_unit_to_pixel_size (f, FRAME_WIDTH (f), FRAME_HEIGHT (f),
-			    &width, &height);
+				  &width, &height);
 	requisition->width = width;
 	requisition->height = height;
       }
     else
       {
+#ifdef HAVE_GTK2
 	parent_class->size_request (widget, requisition);
+#endif
+#ifdef HAVE_GTK3
+	GTK_WIDGET_GET_CLASS (widget)->get_preferred_height (widget, &unused,
+							     &height);
+	requisition->height = height;
+	GTK_WIDGET_GET_CLASS (widget)->get_preferred_width  (widget, &unused,
+							     &width);
+	requisition->width  = width;
+#endif
       }
 }
 
-/* Assign a size and position to the child widgets.  This differs from the
-   super class method in that for all widgets except the scrollbars the size
-   and position are not caclulated here.  This is because these widgets have
-   this function performed for them by the redisplay code (see
-   gtk_map_subwindow()). If the superclass method is called then the widgets
-   can change size and position as the two pieces of code move the widgets at
-   random.
-*/
+#ifdef HAVE_GTK3
+static void
+gtk_xemacs_get_preferred_height (GtkWidget *widget,
+				 gint *minimal_height,
+				 gint *natural_height)
+{
+  GtkRequisition req;
+
+  gtk_xemacs_size_request (widget, &req);
+  *minimal_height = *natural_height = req.height;
+}
+
+static void
+gtk_xemacs_get_preferred_width (GtkWidget *widget,
+				gint *minimal_width,
+				gint *natural_width)
+{
+  GtkRequisition req;
+
+  gtk_xemacs_size_request (widget, &req);
+  *minimal_width = *natural_width = req.width;
+}
+#endif
+
 static void
 gtk_xemacs_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
     GtkXEmacs *x = GTK_XEMACS (widget);
-    GtkFixed *fixed = GTK_FIXED (widget);
     struct frame *f = GTK_XEMACS_FRAME (x);
     int columns, rows;
-    GList *children;
-    guint16 border_width;
 
-    widget->allocation = *allocation;
-    if (GTK_WIDGET_REALIZED (widget))
-      gdk_window_move_resize (widget->window,
-			      allocation->x, 
-			      allocation->y,
-			      allocation->width, 
-			      allocation->height);
-
-    border_width = GTK_CONTAINER (fixed)->border_width;
-  
-    children = fixed->children;
-    while (children)
-      {
-	GtkFixedChild* child = (GtkFixedChild*) children->data;
-	children = children->next;
-      
-	/*
-	  Scrollbars are the only widget that is managed by GTK.  See
-	  comments in gtk_create_scrollbar_instance().
-	*/
-	if (GTK_WIDGET_VISIBLE (child->widget) &&
-	    gtk_type_is_a(GTK_OBJECT_TYPE(child->widget), GTK_TYPE_SCROLLBAR))
-	  {
-	    GtkAllocation child_allocation;
-	    GtkRequisition child_requisition;
-
-	    gtk_widget_get_child_requisition (child->widget, &child_requisition);
-	    child_allocation.x = child->x + border_width;
-	    child_allocation.y = child->y + border_width;
-	    child_allocation.width = child_requisition.width;
-	    child_allocation.height = child_requisition.height;
-	    gtk_widget_size_allocate (child->widget, &child_allocation);
-	  }
-      }
+    /* Now that the toolbar is a widget, all of the children are
+       widgets.  This means that the standard size allocation function
+       is appropriate, no need for custom.  This fixes visibility of
+       both toolbar and scrollbars. */
+    GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
 
     if (f)
       {
@@ -351,18 +374,34 @@ gtk_xemacs_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
       }
 }
 
+#ifdef HAVE_GTK2
 static void
 gtk_xemacs_paint (GtkWidget *widget, GdkRectangle *area)
 {
     GtkXEmacs *x = GTK_XEMACS (widget);
     struct frame *f = GTK_XEMACS_FRAME (x);
 
-    if (GTK_WIDGET_DRAWABLE (widget))
+    if (gtk_widget_is_drawable (widget))
       redisplay_redraw_exposed_area (f, area->x, area->y, area->width,
 				     area->height);
 }
+#endif
 
+#ifdef HAVE_GTK3
 static void
+gtk_xemacs_paint (GtkWidget *widget, cairo_rectangle_int_t *area)
+{
+  GtkXEmacs *x = GTK_XEMACS (widget);
+  struct frame *f = GTK_XEMACS_FRAME (x);
+
+  if (gtk_widget_is_drawable (widget))
+    redisplay_redraw_exposed_area (f, area->x, area->y, area->width,
+				   area->height);
+}
+#endif
+
+#ifdef HAVE_GTK2
+static gboolean
 gtk_xemacs_draw (GtkWidget *widget, GdkRectangle *area)
 {
     GtkFixed *fixed = GTK_FIXED (widget);
@@ -375,11 +414,11 @@ gtk_xemacs_draw (GtkWidget *widget, GdkRectangle *area)
        gtk_fixed_paint() directly, which clears the background window,
        which causes A LOT of flashing. */
 
-  if (GTK_WIDGET_DRAWABLE (widget))
+  if (gtk_widget_is_drawable (widget))
     {
       gtk_xemacs_paint (widget, area);
 
-      children = fixed->children;
+      children = gtk_container_get_children (GTK_CONTAINER (fixed));
 
       while (children)
 	{
@@ -404,8 +443,86 @@ gtk_xemacs_draw (GtkWidget *widget, GdkRectangle *area)
 	    }
 	}
     }
+  return TRUE;
 }
+#endif
 
+#ifdef HAVE_GTK3
+static gboolean
+gtk_xemacs_draw (GtkWidget *widget, cairo_t *cr)
+{
+  GtkFixed *fixed = GTK_FIXED (widget);
+  GtkAllocation allocation = { 0 };
+  GtkStyleContext *context;
+  GtkStateFlags state;
+  GtkFixedChild *child;
+  GdkRectangle child_area;
+  GList *children;
+
+  gtk_widget_get_allocation (widget, &allocation);
+  context = gtk_widget_get_style_context (widget);
+  state = gtk_widget_get_state_flags (widget);
+
+  gtk_render_background (context, cr, 0, 0,
+			 allocation.width, allocation.height);
+
+  /* I need to manually iterate over the children instead of just
+     chaining to parent_class->draw() because it calls
+     gtk_fixed_paint() directly, which clears the background window,
+     which causes A LOT of flashing. */
+
+  if (gtk_widget_is_drawable (widget))
+    {
+      cairo_rectangle_list_t *rects = cairo_copy_clip_rectangle_list (cr);
+      if (rects->status == CAIRO_STATUS_SUCCESS)
+	{
+	  int i;
+	  for (i=0; i < rects->num_rectangles; i++)
+	    {
+	      cairo_rectangle_int_t r;
+
+	      r.x = floor (rects->rectangles[i].x);
+	      r.y = floor (rects->rectangles[i].y);
+	      r.width = ceil (rects->rectangles[i].width);
+	      r.height = ceil (rects->rectangles[i].height);
+	      gtk_xemacs_paint (widget, &r);
+	    }
+	}
+      cairo_rectangle_list_destroy (rects);
+      rects = 0;
+
+      children = gtk_container_get_children (GTK_CONTAINER (fixed));
+#if 0
+      while (children)
+	{
+	  child = (GtkFixedChild*) children->data;
+	  children = children->next;
+	  /* #### This is what causes the scrollbar flickering!
+	     Evidently the scrollbars pretty much take care of drawing
+	     themselves in most cases.  Then we come along and tell them
+	     to redraw again!
+
+	     But if we just leave it out, then they do not get drawn
+	     correctly the first time!
+
+	     Scrollbar flickering has been greatly helped by the
+	     optimizations in scrollbar-gtk.c /
+	     gtk_update_scrollbar_instance_status (), so this is not that
+	     big a deal anymore.
+	  */
+	  if (gtk_widget_intersect (child->widget, area, &child_area))
+	    {
+	      gtk_widget_draw (child->widget, &child_area);
+	    }
+	}
+#endif
+    }
+  GTK_WIDGET_CLASS (parent_class)->draw (widget, cr);
+  return TRUE;
+}
+#endif
+
+#ifdef HAVE_GTK2
 static gint
 gtk_xemacs_expose (GtkWidget *widget, GdkEventExpose *event)
 {
@@ -413,7 +530,7 @@ gtk_xemacs_expose (GtkWidget *widget, GdkEventExpose *event)
     struct frame *f = GTK_XEMACS_FRAME (x);
     GdkRectangle *a = &event->area;
 
-  if (GTK_WIDGET_DRAWABLE (widget))
+  if (gtk_widget_is_drawable (widget))
     {
       /* This takes care of drawing the scrollbars, etc */
       parent_class->expose_event (widget, event);
@@ -427,13 +544,10 @@ gtk_xemacs_expose (GtkWidget *widget, GdkEventExpose *event)
 
   return FALSE;
 }
+#endif
 
 Lisp_Object
 xemacs_gtk_convert_color(GdkColor *c, GtkWidget *UNUSED (w))
 {
-  char color_buf[255];
-
-  sprintf (color_buf, "#%04x%04x%04x", c->red, c->green, c->blue);
-
-  return (build_cistring (color_buf));
+  return emacs_sprintf_string ("#%04x%04x%04x", c->red, c->green, c->blue);
 }
