@@ -24,14 +24,15 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 #include "lisp.h"
+
+#include "casetab.h"
 #include "console-msw-impl.h"
-#include "redisplay.h"
-#include "gui.h"
-#include "glyphs.h"
-#include "frame-impl.h"
 #include "elhash.h"
 #include "events.h"
-#include "buffer.h"
+#include "frame-impl.h"
+#include "glyphs.h"
+#include "gui.h"
+#include "redisplay.h"
 
 /*
  * Return value is Qt if we have dispatched the command,
