@@ -494,30 +494,6 @@ clear_gutter (struct frame *f, enum edge_pos pos)
   redisplay_clear_region (window, findex, x, y, width, height);
 }
 
-/* [[#### I don't currently believe that redisplay needs to mark the
-   glyphs in its structures since these will always be referenced from
-   somewhere else. However, I'm not sure enough to stake my life on it
-   at this point, so we do the safe thing.]]
-
-   ALWAYS mark everything. --ben */
-
-/* See the comment in image_instantiate_cache_result as to why marking
-   the glyph will also mark the image_instance. */
-void
-mark_gutters (struct frame *f)
-{
-  enum edge_pos pos;
-  EDGE_POS_LOOP (pos)
-    {
-      if (f->current_display_lines[pos])
-	mark_redisplay_structs (f->current_display_lines[pos]);
-      /* [[#### Do we really need to mark the desired lines?]]
-	 ALWAYS mark everything. --ben */
-      if (f->desired_display_lines[pos])
-	mark_redisplay_structs (f->desired_display_lines[pos]);
-    }
-}
-
 /* This is called by extent_changed_for_redisplay, so that redisplay
    knows exactly what extents have changed. */
 void
@@ -1190,14 +1166,6 @@ specifier_type_create_gutter (void)
   SPECIFIER_HAS_METHOD (gutter_visible, validate);
 }
 
-void
-reinit_specifier_type_create_gutter (void)
-{
-  REINITIALIZE_SPECIFIER_TYPE (gutter);
-  REINITIALIZE_SPECIFIER_TYPE (gutter_size);
-  REINITIALIZE_SPECIFIER_TYPE (gutter_visible);
-}
-
 void
 specifier_vars_of_gutter (void)
 {
