@@ -2,10 +2,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -13,9 +13,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -55,13 +53,8 @@ Boston, MA 02111-1307, USA.  */
 
 #include "sound.h"
 
-#ifdef HPUX10
 #include <Alib.h>
 #include <CUlib.h>
-#else /* !HPUX 10 */
-#include <audio/Alib.h>
-#include <audio/CUlib.h>
-#endif /* !HPUX 10 */
 
 
 Lisp_Object Vhp_play_server;
@@ -78,9 +71,9 @@ player_error_internal (Audio * audio, Ascbyte * text, long errorCode)
   Ibyte *interr;
 
   AGetErrorText (audio, errorCode, errorbuff, 131);
-  EXTERNAL_TO_C_STRING (errorbuf, interr, Qnative);
+  interr = EXTERNAL_TO_ITEXT (errorbuf, Qerror_message_encoding);
   
-  signal_error (Qsound_error, text, build_string (interr));
+  signal_error (Qsound_error, text, build_istring (interr));
 }
 
 long
@@ -153,7 +146,8 @@ play_sound_file (Extbyte * sound_file, int volume)
   Extbyte *server;
 
   if (STRINGP (Vhp_play_server))
-    LISP_STRING_TO_EXTERNAL (Vhp_play_server, server, Qnative);
+    server = LISP_STRING_TO_EXTERNAL (Vhp_play_server,
+				      Qunix_host_name_encoding);
   else
     server = "";
 
@@ -206,7 +200,8 @@ play_sound_data (Binbyte * data, int UNUSED (length), int volume)
        
     
   if (STRINGP (Vhp_play_server))
-    LISP_STRING_TO_EXTERNAL (Vhp_play_server, server, Qnative);
+    server = LISP_STRING_TO_EXTERNAL (Vhp_play_server,
+				      Qunix_host_name_encoding);
   else
     server = "";
 
