@@ -1,6 +1,6 @@
 /* Fundamental definitions for XEmacs Lisp interpreter -- non-union objects.
    Copyright (C) 1985, 1986, 1987, 1992, 1993 Free Software Foundation, Inc.
-   Copyright (C) 2001, 2002 Ben Wing.
+   Copyright (C) 2001, 2002, 2005, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -41,7 +41,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
  The object is obtained by masking off the type bits.
      Bit 1 is used as a value bit by splitting the Lisp integer type
- into two subtypes, Lisp_Type_Int_Even and Lisp_Type_Int_Odd.
+ into two subtypes, Lisp_Type_Fixnum_Even and Lisp_Type_Fixnum_Odd.
  By this trickery we get 31 bits for integers instead of 30.
 
  For non-integral types, the value bits of a Lisp_Object contain
@@ -67,46 +67,53 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
  XTYPE     The type bits of a Lisp_Object
  XPNTRVAL  The value bits of a Lisp_Object storing a pointer
  XCHARVAL  The value bits of a Lisp_Object storing a Ichar
- XREALINT  The value bits of a Lisp_Object storing an integer, signed
+ XREALFIXNUM  The value bits of a Lisp_Object storing an integer, signed
  XUINT     The value bits of a Lisp_Object storing an integer, unsigned
- INTP      Non-zero if this Lisp_Object is an integer
+ FIXNUMP      Non-zero if this Lisp_Object is an integer
  Qzero     Lisp Integer 0
+ Qone      Lisp Integer 1
  EQ        Non-zero if two Lisp_Objects are identical, not merely equal. */
 
 
 typedef EMACS_INT Lisp_Object;
 
-#define Lisp_Type_Int_Bit (Lisp_Type_Int_Even & Lisp_Type_Int_Odd)
-#define VALMASK (((1UL << VALBITS) - 1UL) << GCTYPEBITS)
+#define Lisp_Type_Fixnum_Bit (Lisp_Type_Fixnum_Even & Lisp_Type_Fixnum_Odd)
+#define VALMASK ((((EMACS_UINT)1 << VALBITS) - 1UL) << GCTYPEBITS)
 #define XTYPE(x) ((enum Lisp_Type) (((EMACS_UINT)(x)) & ~VALMASK))
 #define XPNTRVAL(x) (x) /* This depends on Lisp_Type_Record == 0 */
-#define XCHARVAL(x) ((x) >> GCBITS)
-#define XREALINT(x) ((x) >> INT_GCBITS)
-#define XUINT(x) ((EMACS_UINT)(x) >> INT_GCBITS)
+/* A character is always >= 0, so get 30 bits out of it by treating it as
+   unsigned */
+#define XCHARVAL(x) ((EMACS_UINT)(x) >> (EMACS_UINT) GCBITS)
+#define XREALFIXNUM(x) ((x) >> FIXNUM_GCBITS)
+#define XUINT(x) ((EMACS_UINT)(x) >> FIXNUM_GCBITS)
 
 #define wrap_pointer_1(ptr) ((Lisp_Object) (ptr))
 
 DECLARE_INLINE_HEADER (
 Lisp_Object
-make_int_verify (EMACS_INT val)
+make_fixnum_verify (EMACS_INT val)
 )
 {
-  Lisp_Object obj = (Lisp_Object) ((val << INT_GCBITS) | Lisp_Type_Int_Bit);
-  type_checking_assert (XREALINT (obj) == val);
+  Lisp_Object obj = (Lisp_Object) (((EMACS_UINT) val << FIXNUM_GCBITS)
+				   | Lisp_Type_Fixnum_Bit);
+  type_checking_assert (XREALFIXNUM (obj) == val);
   return obj;
 }
 
-#define make_int(x) ((Lisp_Object) (((x) << INT_GCBITS) | Lisp_Type_Int_Bit))
+/* Yes, EMACS_UINT is intentional here, left shift is technically an
+   undefined operation in C. */
+#define make_fixnum(x) ((Lisp_Object) ((((EMACS_UINT)(x)) << FIXNUM_GCBITS) | Lisp_Type_Fixnum_Bit))
 
-#define make_char_1(x) ((Lisp_Object) (((x) << GCBITS) | Lisp_Type_Char))
+#define make_char_1(x) ((Lisp_Object) ((((EMACS_UINT)(x)) << GCBITS) | Lisp_Type_Char))
 
-#define INTP(x) ((EMACS_UINT)(x) & Lisp_Type_Int_Bit)
-#define INT_PLUS(x,y)  ((x)+(y)-Lisp_Type_Int_Bit)
-#define INT_MINUS(x,y) ((x)-(y)+Lisp_Type_Int_Bit)
-#define INT_PLUS1(x)   INT_PLUS  (x, make_int (1))
-#define INT_MINUS1(x)  INT_MINUS (x, make_int (1))
+#define FIXNUMP(x) ((EMACS_UINT)(x) & Lisp_Type_Fixnum_Bit)
+#define FIXNUM_PLUS(x,y)  ((x)+(y)-Lisp_Type_Fixnum_Bit)
+#define FIXNUM_MINUS(x,y) ((x)-(y)+Lisp_Type_Fixnum_Bit)
+#define FIXNUM_PLUS1(x)   FIXNUM_PLUS  (x, Qone)
+#define FIXNUM_MINUS1(x)  FIXNUM_MINUS (x, Qone)
 
-#define Qzero make_int (0)
+#define Qzero make_fixnum (0)
+#define Qone make_fixnum (1)
 #define Qnull_pointer ((Lisp_Object) 0)
 #define EQ(x,y) ((x) == (y))
 
