@@ -107,6 +107,37 @@ sizeof_lstream (Lisp_Object obj)
 
 static const struct memory_description lstream_implementation_description_1[]
 = {
+  { XD_ASCII_STRING, offsetof (struct lstream_implementation, name) },
+  { XD_SIZED_MEMORY_DESCRIPTION, offsetof (struct lstream_implementation,
+					   extra_description) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   reader) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   writer) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   write_with_extents) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   was_blocked_p) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   rewinder) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   seekable_p) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+                                   seek_from_end) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   character_tell) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   flusher) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   closer) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   finalizer) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   tls_p) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+				   tls_negotiater) },
+  { XD_FUNCTION_POINTER, offsetof (struct lstream_implementation,
+                                   extent_info) },
   { XD_END }
 };
 
@@ -140,13 +171,6 @@ const struct sized_memory_description lstream_empty_extra_description = {
   0, lstream_empty_extra_description_1
 };
 
-DEFINE_NODUMP_SIZABLE_LISP_OBJECT ("stream", lstream, print_lstream,
-				   finalize_lstream,
-				   0, 0, /* no equal or hash */
-				   lstream_description,
-				   sizeof_lstream, Lstream);
-
-
 /* Change the buffering of a stream.  See lstream.h.  By default the
    buffering is STREAM_BLOCK_BUFFERED. */
 
@@ -170,7 +194,8 @@ Lstream_set_buffering (Lstream *lstr, Lstream_buffering buffering,
 }
 
 static const Lstream_implementation *lstream_types[32];
-static Lisp_Object Vlstream_free_list[32];
+static Lisp_Object *lstream_free_list;
+static Lisp_Object Vlstream_free_list;
 static int lstream_type_count;
 
 /* Allocate and return a new Lstream.  This function is not really
@@ -195,14 +220,13 @@ Lstream_new (const Lstream_implementation *imp, int flags)
     {
       assert (lstream_type_count < countof (lstream_types));
       lstream_types[lstream_type_count] = imp;
-      Vlstream_free_list[lstream_type_count] =
+      lstream_free_list[lstream_type_count] =
 	make_lcrecord_list (aligned_sizeof_lstream (imp->size),
-			    &lrecord_lstream);
-      staticpro_nodump (&Vlstream_free_list[i]);
+                            LRECORD_IMPLEMENTATION (lstream));
       lstream_type_count++;
     }
 
-  p = XLSTREAM (alloc_managed_lcrecord (Vlstream_free_list[i]));
+  p = XLSTREAM (alloc_managed_lcrecord (lstream_free_list[i]));
   /* Formerly, we zeroed out the object minus its header, but it's now
      handled automatically.  ALLOC_SIZED_LISP_OBJECT() always zeroes out
      the whole object other than its header, and alloc_managed_lcrecord()
@@ -280,7 +304,7 @@ Lstream_delete (Lstream *lstr)
     {
       if (lstream_types[i] == lstr->imp)
 	{
-	  free_managed_lcrecord (Vlstream_free_list[i], val);
+	  free_managed_lcrecord (lstream_free_list[i], val);
 	  return;
 	}
     }
@@ -873,6 +897,37 @@ Lstream_rewind (Lstream *lstr)
   return (lstr->imp->rewinder) (lstr);
 }
 
+/* Seek to OFFSET bytes (which must be negative) from the end of the
+   underlying stream. */
+Boolint
+Lstream_seek_from_end (Lstream *lstr, OFF_T offset)
+{
+  Boolint result;
+  if (!lstr->imp->seek_from_end)
+    {
+      return 0;
+    }
+
+  if (offset >= 0)
+    {
+      return 0;
+    }
+
+  if (Lstream_flush (lstr) < 0)
+    {
+      return 0;
+    }
+
+  result = (lstr->imp->seek_from_end) (lstr, offset);
+  if (result)
+    {
+      lstr->byte_count = 0;
+      lstr->unget_character_count = 0;
+    }
+
+  return result;
+}
+
 int
 Lstream_seekable_p (Lstream *lstr)
 {
@@ -1075,7 +1130,7 @@ struct stdio_stream
 
 #define STDIO_STREAM_DATA(stream) LSTREAM_TYPE_DATA (stream, stdio)
 
-DEFINE_LSTREAM_IMPLEMENTATION ("stdio", stdio);
+DECLARE_LSTREAM_IMPLEMENTATION (stdio);
 
 static Lisp_Object
 make_stdio_stream_1 (FILE *stream, int flags)
@@ -1165,6 +1220,24 @@ stdio_seekable_p (Lstream *stream)
   return S_ISREG (lestat.st_mode);
 }
 
+static Boolint
+stdio_seek_from_end (Lstream *stream, OFF_T offset)
+{
+  FILE *file = STDIO_STREAM_DATA (stream)->file;
+  int fsought;
+
+  assert (offset < 0);
+
+  fsought = FSEEK (file, offset, SEEK_END);
+
+  if (fsought == 0)
+    {
+      return 1;
+    }
+
+  return 0;
+}
+
 static int
 stdio_flusher (Lstream *stream)
 {
@@ -1192,20 +1265,20 @@ stdio_closer (Lstream *stream)
 
 struct filedesc_stream
 {
+  OFF_T starting_pos;
+  OFF_T current_pos;
+  OFF_T end_pos;
   tls_state_t *tls_state;
   int fd;
   int pty_max_bytes;
-  Ibyte eof_char;
-  int starting_pos;
-  int current_pos;
-  int end_pos;
   int chars_sans_newline;
+  Ibyte eof_char;
   unsigned int blocking_error_p :1;
 };
 
 #define FILEDESC_STREAM_DATA(stream) LSTREAM_TYPE_DATA (stream, filedesc)
 
-DEFINE_LSTREAM_IMPLEMENTATION ("filedesc", filedesc);
+DECLARE_LSTREAM_IMPLEMENTATION (filedesc);
 
 /* Make a stream that reads from or writes to a file descriptor FILEDESC.
    OFFSET is the offset from the *current* file pointer that the reading
@@ -1487,6 +1560,57 @@ filedesc_seekable_p (Lstream *stream)
     }
 }
 
+static Boolint
+filedesc_seek_from_end (Lstream *stream, OFF_T offset)
+{
+  struct filedesc_stream *str = FILEDESC_STREAM_DATA (stream);
+  OFF_T lsought;
+
+  assert (offset < 0);
+
+  if (str->end_pos != -1)
+    {
+      if (str->end_pos + offset < str->starting_pos)
+        {
+          lsought = lseek (FILEDESC_STREAM_DATA (stream)->fd,
+			   str->starting_pos, SEEK_SET);
+        }
+      else
+	{
+	  offset += str->end_pos;
+          lsought = lseek (FILEDESC_STREAM_DATA (stream)->fd, offset,
+						 SEEK_SET);
+	}
+
+      if (lsought == -1)
+        {
+          return 0;
+        }
+
+      str->current_pos = lsought;
+      return 1;
+    }
+
+  lsought = lseek (FILEDESC_STREAM_DATA (stream)->fd, offset, SEEK_END);
+
+  if (str->starting_pos > 0)
+    {
+      if (lsought < str->starting_pos)
+        {
+          lsought = lseek (FILEDESC_STREAM_DATA (stream)->fd,
+                           str->starting_pos, SEEK_SET);
+        }
+    }
+
+  if (lsought == -1)
+    {
+      return 0;
+    }
+
+  str->current_pos = lsought;
+  return 1;
+}
+
 static int
 filedesc_closer (Lstream *stream)
 {
@@ -1563,7 +1687,7 @@ static const struct memory_description lisp_string_lstream_description[] = {
   { XD_END }
 };
 
-DEFINE_LSTREAM_IMPLEMENTATION_WITH_DATA ("lisp-string", lisp_string);
+DECLARE_LSTREAM_IMPLEMENTATION (lisp_string);
 
 Lisp_Object
 make_lisp_string_input_stream (Lisp_Object string, Bytecount offset,
@@ -1637,7 +1761,7 @@ lisp_string_rewinder (Lstream *stream)
 /* Much of the implementation of this is in lstream.h, since we need to
    stack-allocate a fixed_buffer_lstream. */
 
-DEFINE_LSTREAM_IMPLEMENTATION ("fixed-buffer", fixed_buffer);
+DECLARE_LSTREAM_IMPLEMENTATION (fixed_buffer);
 
 Lisp_Object
 make_fixed_buffer_input_stream (const void *buf, Bytecount size)
@@ -1727,7 +1851,7 @@ static const struct memory_description resizing_buffer_lstream_description[] = {
   { XD_END }
 };
 
-DEFINE_LSTREAM_IMPLEMENTATION_WITH_DATA ("resizing-buffer", resizing_buffer);
+DECLARE_LSTREAM_IMPLEMENTATION (resizing_buffer);
 
 Lisp_Object
 make_resizing_buffer_output_stream (void)
@@ -1762,10 +1886,14 @@ resizing_buffer_write_with_extents (Lstream *stream, Lisp_Object string,
 
   DO_REALLOC (str->buf, str->allocked, str->stored + len, Ibyte);
 
-  memcpy (str->buf + str->stored, XSTRING_DATA (string) + position, len);
+  if (len)
+    {
+      memcpy (str->buf + str->stored, XSTRING_DATA (string) + position, len);
+      str->stored += len;
+    }
+
   copy_string_extents (wrap_lstream (stream), string, start, position, len);
 
-  str->stored += len;
   return len;
 }
 
@@ -1814,7 +1942,7 @@ resizing_buffer_to_lisp_string (Lstream *stream)
   return result;
 }
 
-DEFUN ("make-string-output-stream", Fmake_string_output_stream, 0, MANY, 0, /*
+DEFUN ("make-string-output-stream", Fmake_string_output_stream, 0, KEYWORDS, 0, /*
 Return an output stream for use with `print', and `write-sequence'.
 
 This output stream stores the characters written to it, for later access using
@@ -1826,7 +1954,7 @@ arguments: (&key (ELEMENT-TYPE character))
 */
         (int nargs, Lisp_Object *args))
 {
-  PARSE_KEYWORDS (Fmake_string_output_stream, nargs, args, 1, (element_type),
+  PARSE_KEYWORDS (Fmake_string_output_stream, (element_type),
                   (element_type = Qcharacter));
 
   if (!EQ (element_type, Qcharacter))
@@ -1878,7 +2006,7 @@ struct dynarr_stream
   unsigned_char_dynarr *dyn;
 };
 
-DEFINE_LSTREAM_IMPLEMENTATION ("dynarr", dynarr);
+DECLARE_LSTREAM_IMPLEMENTATION (dynarr);
 
 Lisp_Object
 make_dynarr_output_stream (unsigned_char_dynarr *dyn)
@@ -1936,7 +2064,7 @@ static const struct memory_description lisp_buffer_lstream_description[] = {
   { XD_END }
 };
 
-DEFINE_LSTREAM_IMPLEMENTATION_WITH_DATA ("lisp-buffer", lisp_buffer);
+DECLARE_LSTREAM_IMPLEMENTATION (lisp_buffer);
 
 static Lisp_Object
 make_lisp_buffer_stream_1 (struct buffer *buf, Charbpos start, Charbpos end,
@@ -2151,7 +2279,12 @@ OUTPUT-STREAM defaults to standard-output.
 void
 syms_of_lstream (void)
 {
-  INIT_LISP_OBJECT (lstream);
+  DEFINE_NODUMP_SIZABLE_LISP_OBJECT ("stream", lstream, print_lstream,
+                                     finalize_lstream,
+                                     0, 0, /* no equal or hash */
+                                     lstream_description, sizeof_lstream,
+                                     Lstream);
+  OBJECT_HAS_PREMETHOD (lstream, disksave);
 
   DEFKEYWORD (Q_element_type);
   DEFSYMBOL (Qstreamp);
@@ -2163,47 +2296,59 @@ syms_of_lstream (void)
 }
 
 void
-lstream_objects_create (void)
+vars_of_lstream (void)
 {
-  OBJECT_HAS_PREMETHOD (lstream, disksave);
-}
+  static Lisp_Object Vdump_time_lstream_free_list;
 
-void
-lstream_type_create (void)
-{
+  Vdump_time_lstream_free_list = make_vector (countof (lstream_types), Qnil);
+  staticpro_nodump (&Vdump_time_lstream_free_list);
+  lstream_free_list = XVECTOR_DATA (Vdump_time_lstream_free_list);
+
+  Vlstream_free_list = make_vector (countof (lstream_types), Qnil);
+  staticpro (&Vlstream_free_list);
+
+  DEFINE_LSTREAM_IMPLEMENTATION ("stdio", stdio);
   LSTREAM_HAS_METHOD (stdio, reader);
   LSTREAM_HAS_METHOD (stdio, writer);
   LSTREAM_HAS_METHOD (stdio, rewinder);
   LSTREAM_HAS_METHOD (stdio, seekable_p);
+  LSTREAM_HAS_METHOD (stdio, seek_from_end);
   LSTREAM_HAS_METHOD (stdio, flusher);
   LSTREAM_HAS_METHOD (stdio, closer);
 
+  DEFINE_LSTREAM_IMPLEMENTATION ("filedesc", filedesc);
   LSTREAM_HAS_METHOD (filedesc, reader);
   LSTREAM_HAS_METHOD (filedesc, writer);
   LSTREAM_HAS_METHOD (filedesc, was_blocked_p);
   LSTREAM_HAS_METHOD (filedesc, rewinder);
   LSTREAM_HAS_METHOD (filedesc, seekable_p);
+  LSTREAM_HAS_METHOD (filedesc, seek_from_end);
   LSTREAM_HAS_METHOD (filedesc, closer);
   LSTREAM_HAS_METHOD (filedesc, tls_p);
   LSTREAM_HAS_METHOD (filedesc, tls_negotiater);
 
+  DEFINE_LSTREAM_IMPLEMENTATION_WITH_DATA ("lisp-string", lisp_string);
   LSTREAM_HAS_METHOD (lisp_string, reader);
   LSTREAM_HAS_METHOD (lisp_string, rewinder);
 
+  DEFINE_LSTREAM_IMPLEMENTATION ("fixed-buffer", fixed_buffer);
   LSTREAM_HAS_METHOD (fixed_buffer, reader);
   LSTREAM_HAS_METHOD (fixed_buffer, writer);
   LSTREAM_HAS_METHOD (fixed_buffer, rewinder);
 
+  DEFINE_LSTREAM_IMPLEMENTATION_WITH_DATA ("resizing-buffer", resizing_buffer);
   LSTREAM_HAS_METHOD (resizing_buffer, writer);
   LSTREAM_HAS_METHOD (resizing_buffer, write_with_extents);
   LSTREAM_HAS_METHOD (resizing_buffer, extent_info);
   LSTREAM_HAS_METHOD (resizing_buffer, rewinder);
   LSTREAM_HAS_METHOD (resizing_buffer, closer);
 
+  DEFINE_LSTREAM_IMPLEMENTATION ("dynarr", dynarr);
   LSTREAM_HAS_METHOD (dynarr, writer);
   LSTREAM_HAS_METHOD (dynarr, rewinder);
   LSTREAM_HAS_METHOD (dynarr, closer);
 
+  DEFINE_LSTREAM_IMPLEMENTATION_WITH_DATA ("lisp-buffer", lisp_buffer);
   LSTREAM_HAS_METHOD (lisp_buffer, reader);
   LSTREAM_HAS_METHOD (lisp_buffer, writer);
   LSTREAM_HAS_METHOD (lisp_buffer, write_with_extents);
@@ -2211,6 +2356,7 @@ lstream_type_create (void)
 }
 
 void
-vars_of_lstream (void)
+reinit_vars_of_lstream (void)
 {
+  lstream_free_list = XVECTOR_DATA (Vlstream_free_list);
 }
