@@ -1,6 +1,6 @@
 /* Generic stream implementation -- header file.
    Copyright (C) 1995 Free Software Foundation, Inc.
-   Copyright (C) 1996, 2001, 2002 Ben Wing.
+   Copyright (C) 1996, 2001, 2002, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -24,6 +24,10 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #ifndef INCLUDED_lstream_h_
 #define INCLUDED_lstream_h_
 
+#include "tls.h"
+
+struct extent_info;
+
 /************************************************************************/
 /*                     definition of Lstream object                     */
 /************************************************************************/
@@ -32,9 +36,16 @@ DECLARE_LISP_OBJECT (lstream, struct lstream);
 #define XLSTREAM(x) XRECORD (x, lstream, struct lstream)
 #define wrap_lstream(p) wrap_record (p, lstream)
 #define LSTREAMP(x) RECORDP (x, lstream)
-/* #define CHECK_LSTREAM(x) CHECK_RECORD (x, lstream)
-   Lstream pointers should never escape to the Lisp level, so
-   functions should not be doing this. */
+/* Can't use the usual CONCHECK_RECORD() macros, since the type is lstream in
+   C and stream in Lisp. */
+#define CHECK_LSTREAM(x) do {                           \
+ if (!RECORD_TYPEP (x, lrecord_type_lstream))		\
+   dead_wrong_type_argument (Qstreamp, x);		\
+ } while (0)
+#define CONCHECK_LSTREAM(x) do {			\
+ if (!RECORD_TYPEP (x, lrecord_type_lstream))		\
+   x = wrong_type_argument (Qstreamp, x);		\
+}  while (0)
 
 #ifndef EOF
 #define EOF (-1)
@@ -125,11 +136,17 @@ typedef enum lstream_buffering
 
    Rewind semantics are generally easy to implement, so we do provide
    a rewind method.  Even rewind() may not be available on a stream,
-   however -- e.g. on process output. */
+   however -- e.g. on process output.
+
+     I have just added a seek_from_end() method, needed in the bowels of
+     coding detection for examining any coding cookies in the last page, and
+     only implemented for filedesc and stdio streams. I have not attempted to
+     add general support for seek()/tell(). Aidan Kehoe, Sa 10. Mai 18:12:59
+     IST 2025. */
 
 typedef struct lstream_implementation
 {
-  const char *name;
+  const Ascbyte *name;
   Bytecount size; /* Number of additional bytes to be
 		     allocated with this stream.  Access this
 		     data using Lstream_data(). */
@@ -170,6 +187,12 @@ typedef struct lstream_implementation
      This function can be NULL if the stream is input-only. */
   Bytecount (*writer) (Lstream *stream, const unsigned char *data,
 		       Bytecount size);
+
+  /* Like WRITER, but take the data from RELOC, a Lisp string, and copy any
+     extent information to the other end of the stream. */
+  Bytecount (*write_with_extents) (Lstream *stream, Lisp_Object reloc,
+                                   Bytecount position, Bytecount length);
+
   /* Return non-zero if the last write operation on the stream resulted
      in an attempt to block (EWOULDBLOCK). If this method does not
      exists, the implementation returns 0 */
@@ -181,6 +204,15 @@ typedef struct lstream_implementation
      method.  If this method is not present, the result is determined
      by whether a rewind method is present. */
   int (*seekable_p) (Lstream *stream);
+
+  /* Seek to OFFSET from the end of the underlying stream. Return 1 if
+     successful, 0 if this stream is not seekable or if unsuccessful for other
+     reasons. */
+  Boolint (*seek_from_end) (Lstream *stream, OFF_T offset);
+
+  /* Return the number of complete characters read so far. Respects
+     buffering and unget. Returns -1 if unknown or not implemented. */
+  Charcount (*character_tell) (Lstream *stream);
   /* Perform any additional operations necessary to flush the
      data in this stream. */
   int (*flusher) (Lstream *stream);
@@ -197,34 +229,60 @@ typedef struct lstream_implementation
      open at this point, the finalizer is called after calling
      Lstream_close().  Called only once (NOT called at disksave time). */
   void (*finalizer) (Lstream *stream);
-  /* Mark this object for garbage collection.  Same semantics as
-     a standard Lisp_Object marker.  This function can be NULL. */
-  Lisp_Object (*marker) (Lisp_Object lstream);
+  /* Return nonzero if this stream is using a TLS connection */
+  int (*tls_p) (Lstream *stream);
+  /* Perform STARTTLS negotiation on a pair of streams, one for input and one
+     for output.  Both are transformed if negotiation is successful. */
+  int (*tls_negotiater) (Lstream *instream, Lstream *outstream,
+			 const Extbyte *host, Lisp_Object keylist);
+
+  /* Return the extent info associated with the stream, or NULL if none. */ 
+  struct extent_info *(*extent_info)(Lstream *stream);
 } Lstream_implementation;
 
-#define DEFINE_LSTREAM_IMPLEMENTATION(name, c_name)	\
-  Lstream_implementation lstream_##c_name[1] =		\
-    { { (name), sizeof (struct c_name##_stream),	\
-      &lstream_empty_extra_description } }
+#define DECLARE_LSTREAM_IMPLEMENTATION(c_name)	\
+  Lstream_implementation *lstream_##c_name
 
-#define DEFINE_LSTREAM_IMPLEMENTATION_WITH_DATA(name, c_name)		      \
-  static const struct sized_memory_description c_name##_lstream_description_0 \
-  = {									      \
-    sizeof (struct c_name##_stream),					      \
-    c_name##_lstream_description					      \
-  };									      \
-  Lstream_implementation lstream_##c_name[1] =				      \
-    { { (name), sizeof (struct c_name##_stream),			      \
-      &c_name##_lstream_description_0 } }
+#define DEFINE_LSTREAM_IMPLEMENTATION_WITH_DATA(string_name, c_name)          \
+  do                                                                          \
+    {                                                                         \
+      static const struct sized_memory_description                            \
+        c_name##_lstream_description_0                                        \
+        = {                                                                   \
+        sizeof (struct c_name##_stream),                                      \
+        c_name##_lstream_description                                          \
+      };                                                                      \
+      lstream_##c_name = xnew_and_zero (Lstream_implementation);              \
+      lstream_##c_name->name = string_name;                                   \
+      lstream_##c_name->size = sizeof (struct c_name##_stream);               \
+      lstream_##c_name->extra_description = &(c_name##_lstream_description_0);\
+      dump_add_root_block_ptr (&lstream_##c_name,                             \
+                               &lstream_implementation_description);          \
+    } while (0)
+
+#define DEFINE_LSTREAM_IMPLEMENTATION(string_name, c_name)                    \
+  do                                                                          \
+    {                                                                         \
+      lstream_##c_name = xnew_and_zero (Lstream_implementation);              \
+      lstream_##c_name->name = string_name;                                   \
+      lstream_##c_name->size = sizeof (struct c_name##_stream);               \
+      lstream_##c_name->extra_description = &lstream_empty_extra_description; \
+      dump_add_root_block_ptr (&lstream_##c_name,                             \
+                               &lstream_implementation_description);          \
+    } while (0)
 
 #define DECLARE_LSTREAM(c_name) \
-  extern Lstream_implementation lstream_##c_name[]
+  extern Lstream_implementation *lstream_##c_name
 
-#define LSTREAM_FL_IS_OPEN		1
-#define LSTREAM_FL_READ			2
-#define LSTREAM_FL_WRITE		4
-#define LSTREAM_FL_NO_PARTIAL_CHARS	8
-#define LSTREAM_FL_CLOSE_AT_DISKSAVE	16
+/* Flags that can be passed to Lstream_new(). */
+#define LSTR_READ		(1 << 0)
+#define LSTR_WRITE		(1 << 1)
+#define LSTR_NO_PARTIAL_CHARS	(1 << 2)
+#define LSTR_CLOSE_AT_DISKSAVE	(1 << 3)
+#define LSTR_NO_SQUIRREL	(1 << 4)
+
+/* Internal flags that can also be set on the stream. */
+#define LSTR_IS_OPEN	        (1 << 5)
 
 struct lstream
 {
@@ -250,15 +308,23 @@ struct lstream
      similarly has to push the data on backwards. */
   unsigned char *unget_buffer; /* holds characters pushed back onto input */
   Bytecount unget_buffer_size; /* allocated size of buffer */
-  Bytecount unget_buffer_ind; /* pointer to next buffer spot
-					  to write a character */
+  Bytecount unget_buffer_ind; /* Next buffer spot to write a character */
+
+  Charcount unget_character_count; /* Count of complete characters ever ungot. */
 
   Bytecount byte_count;
   int flags;
+  /* Whether an error occurred during the most recent operation */
+  unsigned int error_occurred_p:1;
+  /* Whether an error has occurred since the user last reset the
+     error-occurred flag.  This is the value returned by
+     Lstream_error_occurred_p(). */
+  unsigned int public_error_occurred_p:1;
   max_align_t data[1];
 };
 
 extern const struct sized_memory_description lstream_empty_extra_description;
+extern const struct sized_memory_description lstream_implementation_description;
 
 #define LSTREAM_TYPE_P(lstr, type) \
   ((lstr)->imp == lstream_##type)
@@ -287,8 +353,7 @@ error_check_lstream_type (struct lstream *stream,
   (lstream_##type->m = type##_##m)
 
 
-Lstream *Lstream_new (const Lstream_implementation *imp,
-		      const char *mode);
+Lstream *Lstream_new (const Lstream_implementation *imp, int flags);
 void Lstream_reopen (Lstream *lstr);
 void Lstream_set_buffering (Lstream *lstr, Lstream_buffering buffering,
 			    int buffering_size);
@@ -297,19 +362,32 @@ int Lstream_flush_out (Lstream *lstr);
 int Lstream_fputc (Lstream *lstr, int c);
 int Lstream_fgetc (Lstream *lstr);
 void Lstream_fungetc (Lstream *lstr, int c);
-Bytecount Lstream_read (Lstream *lstr, void *data,
-				 Bytecount size);
-int Lstream_write (Lstream *lstr, const void *data,
-		   Bytecount size);
+Bytecount Lstream_read (Lstream *lstr, void *data, Bytecount size);
+Charcount Lstream_character_tell (Lstream *);
+Bytecount Lstream_write (Lstream *lstr, const void *data, Bytecount size);
+Bytecount Lstream_write_with_extents (Lstream *lstr, Lisp_Object object,
+                                      Bytexpos position, Bytecount len);
+int Lstream_errno (Lstream *lstr);
 int Lstream_was_blocked_p (Lstream *lstr);
 void Lstream_unread (Lstream *lstr, const void *data, Bytecount size);
 int Lstream_rewind (Lstream *lstr);
+Boolint Lstream_seek_from_end (Lstream *lstr, OFF_T offset);
 int Lstream_seekable_p (Lstream *lstr);
 int Lstream_close (Lstream *lstr);
+int Lstream_close_noflush (Lstream *lstr);
+
+int Lstream_tls_p (Lstream *lstr);
+int Lstream_tls_negotiate (Lstream *instr, Lstream *outstr,
+			   const Extbyte *host, Lisp_Object keylist);
 
 void Lstream_delete (Lstream *lstr);
 void Lstream_set_character_mode (Lstream *str);
 void Lstream_unset_character_mode (Lstream *lstr);
+int Lstream_error_occurred_p (Lstream *lstr);
+void Lstream_clear_error_occurred_p (Lstream *lstr);
+void Lstream_set_error_occurred_p (Lstream *lstr);
+
+int Lstream_is_type (Lstream *lstr, const Lstream_implementation *imp);
 
 /* Lstream_putc: Write out one byte to the stream.  This is a macro
    and so it is very efficient.  The C argument is only evaluated once
@@ -352,29 +430,41 @@ void Lstream_unset_character_mode (Lstream *lstr);
    reverse order they were pushed back -- most recent first. (This is
    necessary for consistency -- if there are a number of bytes that
    have been unread and I read and unread a byte, it needs to be the
-   first to be read again.) This is a macro and so it is very
-   efficient.  The C argument is only evaluated once but the STREAM
-   argument is evaluated more than once.
- */
+   first to be read again.) */
 
-#define Lstream_ungetc(stream, c)					\
-/* Add to the end if it won't overflow buffer; otherwise call the	\
-   function equivalent */						\
-  ((stream)->unget_buffer_ind >= (stream)->unget_buffer_size ?		\
-   Lstream_fungetc (stream, c) :					\
-   (void) ((stream)->byte_count--,					\
-   ((stream)->unget_buffer[(stream)->unget_buffer_ind++] =		\
-    (unsigned char) (c))))
+DECLARE_INLINE_HEADER (
+void
+Lstream_ungetc (Lstream *lstr, int c)
+)
+{
+  /* Add to the end if it won't overflow buffer; otherwise call the
+     function equivalent */
+  if (lstr->unget_buffer_ind >= lstr->unget_buffer_size)
+    {
+      Lstream_fungetc (lstr, c);
+    }
+  else
+    {
+      lstr->byte_count--;
+      lstr->unget_buffer[lstr->unget_buffer_ind] = (unsigned char) (c);
+      lstr->unget_character_count
+        += valid_ibyteptr_p (lstr->unget_buffer + lstr->unget_buffer_ind);
+      lstr->unget_buffer_ind++;
+    }
+}
 
-#define Lstream_data(stream) ((void *) ((stream)->data))
+/* Rawbyte *, not void *, access through void * is undefined under
+   strict-aliasing rules. */
+#define Lstream_data(stream) ((Rawbyte *) ((stream)->data))
 #define Lstream_byte_count(stream) ((stream)->byte_count)
 
+struct extent_info *Lstream_extent_info (Lstream *stream);
+
 
 /************************************************************************/
 /*             working with an Lstream as a stream of Ichars           */
 /************************************************************************/
 
-#ifdef MULE
 
 DECLARE_INLINE_HEADER (
 Ichar
@@ -410,14 +500,6 @@ Lstream_unget_ichar (Lstream *stream, Ichar ch)
   else
     Lstream_funget_ichar (stream, ch);
 }
-#else /* not MULE */
-
-# define Lstream_get_ichar(stream) Lstream_getc (stream)
-# define Lstream_put_ichar(stream, ch) Lstream_putc (stream, ch)
-# define Lstream_unget_ichar(stream, ch) Lstream_ungetc (stream, ch)
-
-#endif /* not MULE */
-
 
 /************************************************************************/
 /*                        Lstream implementations                       */
@@ -426,27 +508,29 @@ Lstream_unget_ichar (Lstream *stream, Ichar ch)
 /* Flags we can pass to the filedesc and stdio streams. */
 
 /* If set, close the descriptor or FILE * when the stream is closed. */
-#define LSTR_CLOSING 1
+#define LSTR_CLOSING		(1 << 16)
 
 /* If set, allow quitting out of the actual I/O. */
-#define LSTR_ALLOW_QUIT 2
+#define LSTR_ALLOW_QUIT		(1 << 17)
 
 /* If set and filedesc_stream_set_pty_flushing() has been called
    on the stream, do not send more than pty_max_bytes on a single
    line without flushing the data out using the eof_char. */
-#define LSTR_PTY_FLUSHING 4
+#define LSTR_PTY_FLUSHING	(1 << 18)
 
 /* If set, an EWOULDBLOCK error is not treated as an error but
    simply causes the write function to return 0 as the number
    of bytes written out. */
-#define LSTR_BLOCKED_OK 8
+#define LSTR_BLOCKED_OK		(1 << 19)
 
 Lisp_Object make_stdio_input_stream (FILE *stream, int flags);
 Lisp_Object make_stdio_output_stream (FILE *stream, int flags);
-Lisp_Object make_filedesc_input_stream (int filedesc, int offset, int count,
-					int flags);
-Lisp_Object make_filedesc_output_stream (int filedesc, int offset, int count,
-					 int flags);
+Lisp_Object make_filedesc_input_stream (int filedesc, OFF_T offset,
+                                        Bytecount count, int flags,
+                                        tls_state_t *state);
+Lisp_Object make_filedesc_output_stream (int filedesc, OFF_T offset,
+                                         Bytecount count, int flags,
+                                         tls_state_t *state);
 void filedesc_stream_set_pty_flushing (Lstream *stream,
 				       int pty_max_bytes,
 				       Ibyte eof_char);
@@ -458,18 +542,73 @@ Lisp_Object make_fixed_buffer_input_stream (const void *buf,
 					    Bytecount size);
 Lisp_Object make_fixed_buffer_output_stream (void *buf,
 					     Bytecount size);
-const unsigned char *fixed_buffer_input_stream_ptr (Lstream *stream);
-unsigned char *fixed_buffer_output_stream_ptr (Lstream *stream);
+const Ibyte *fixed_buffer_input_stream_ptr (Lstream *stream);
+Ibyte *fixed_buffer_output_stream_ptr (Lstream *stream);
 Lisp_Object make_resizing_buffer_output_stream (void);
-unsigned char *resizing_buffer_stream_ptr (Lstream *stream);
+const Ibyte *resizing_buffer_stream_ptr (Lstream *stream);
 Lisp_Object resizing_buffer_to_lisp_string (Lstream *stream);
 Lisp_Object make_dynarr_output_stream (unsigned_char_dynarr *dyn);
-#define LSTR_SELECTIVE 1
-#define LSTR_IGNORE_ACCESSIBLE 2
+
+EXFUN (Fget_output_stream_string, 1);
+
+/* Flags we can pass to lisp buffer streams. */
+
+#define LSTR_SELECTIVE		(1 << 16)
+#define LSTR_IGNORE_ACCESSIBLE	(1 << 17)
+
 Lisp_Object make_lisp_buffer_input_stream (struct buffer *buf, Charbpos start,
 					   Charbpos end, int flags);
 Lisp_Object make_lisp_buffer_output_stream (struct buffer *buf, Charbpos pos,
 					    int flags);
 Charbpos lisp_buffer_stream_startpos (Lstream *stream);
 
+#ifdef EXPOSE_FIXED_BUFFER_INTERNALS
+
+/* These internals are exposed for the sake of emacs_vsnprintf (), which needs
+   to function as well as possible when the heap and/or Lisp object allocation
+   are no longer working. As such, it creates its lstream on the stack. No
+   other code should be doing this. */
+
+extern Lstream_implementation *lstream_fixed_buffer;
+
+struct fixed_buffer_stream
+{
+  const Ibyte *inbuf;
+  Ibyte *outbuf;
+  Bytecount size;
+  Bytecount offset;
+};
+
+#define FIXED_BUFFER_STREAM_DATA(stream) \
+  LSTREAM_TYPE_DATA (stream, fixed_buffer)
+
+#define DECLARE_STACK_FIXED_BUFFER_LSTREAM(lname)                       \
+  union                                                                 \
+  {                                                                     \
+    struct lstream l;                                                   \
+    /* Make sure we have enough stack space for the type-specific       \
+       data. */                                                         \
+    Rawbyte s[offsetof (struct lstream, data) +                         \
+              sizeof (struct fixed_buffer_stream) ];                    \
+  } lname##u;                                                           \
+  /* Error early on attempt to write to this before initialization: */  \
+  Lisp_Object lname = Qunbound                                          \
+
+#define INIT_STACK_FIXED_BUFFER_OUTPUT_STREAM(lname, buf, bsize) do     \
+    {                                                                   \
+      memset (lname##u.s, 0, max (sizeof (lname##u.s),                  \
+                                  sizeof (lname##u.l)));                \
+      set_lheader_implementation ((struct lrecord_header *)&(lname##u.l), \
+                                  LRECORD_IMPLEMENTATION (lstream));    \
+      lname##u.l.imp = lstream_fixed_buffer;                            \
+      Lstream_set_buffering (&(lname##u.l), LSTREAM_UNBUFFERED, 0);     \
+      lname##u.l.flags = LSTR_IS_OPEN;                                  \
+      lname##u.l.flags |= LSTR_WRITE;                                   \
+      FIXED_BUFFER_STREAM_DATA (&(lname##u.l))->outbuf = buf;           \
+      FIXED_BUFFER_STREAM_DATA (&(lname##u.l))->size = bsize;           \
+      lname = wrap_pointer_1 (&(lname##u.l));                           \
+    } while (0)
+
+#endif
+
 #endif /* INCLUDED_lstream_h_ */
