@@ -1,22 +1,20 @@
 /* R2 AIX machine/system dependent defines
    Copyright (C) 1988 Free Software Foundation, Inc.
 
-This file is part of GNU Emacs.
+This file is part of XEmacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
-GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: FSF 19.31. */
 
@@ -34,34 +32,6 @@ Boston, MA 02111-1307, USA.  */
 #define CANNOT_DUMP
 #endif
 
-#ifndef UNEXEC
-#define UNEXEC "unexaix.o"
-#endif
-
-/* Define addresses, macros, change some setup for dump */
-
-#define NO_REMAP
-
-#ifndef USG5_4
-#define TEXT_START 0x10000000
-#ifdef __64BIT__
-#define DATA_START 0x110000000 /* empirically determined */
-#else
-#define DATA_START 0x20000000
-#endif
-#endif
-
-#define N_BADMAG(x) BADMAG(x)
-#define N_TXTOFF(x) A_TEXTPOS(x)
-#define N_SYMOFF(x) A_SYMPOS(x)
-#define A_TEXT_OFFSET(HDR) sizeof(HDR)
-/* #define ADJUST_EXEC_HEADER \
-    unexec_text_start += sizeof(hdr); \
-    unexec_data_start = ohdr.a_dbase
-*/
-#undef ADDR_CORRECT
-#define ADDR_CORRECT(x) ((int)(x))
-
 /* Specify the font for X to use.
    This used to be Rom14.500; that's nice on the X server shipped with
    the RS/6000, but it's not available on other servers.  */
@@ -69,10 +39,6 @@ Boston, MA 02111-1307, USA.  */
 
 /* Here override various assumptions in ymakefile */
 
-#ifdef AIXHFT
-#define OBJECTS_MACHINE "hftctl.o"
-#endif
-
 #ifndef USG5_4
 /* XEmacs change -- commented out: dkeller@vnet.ibm.com */
 /* #define C_SWITCH_MACHINE -D_BSD */
