@@ -1,5 +1,6 @@
 /* machine description file template.
    Copyright (C) 1985, 1986 Free Software Foundation, Inc.
+   Copyright (C) 2010 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -27,6 +28,8 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
    Ones defined so far include vax, m68000, ns16000, pyramid,
    orion, tahoe, APOLLO and many others */
 
+#ifdef ENABLE_SM_FILE_DECLS_OF_LOADAVG_STUFF
+
 /* Data type of load average, as read out of kmem.  */
 
 #define LOAD_AVE_TYPE long
@@ -35,12 +38,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 #define LOAD_AVE_CVT(x) (int) (((double) (x)) * 100.0 / FSCALE)
 
-/* Define NO_REMAP if memory segmentation makes it not work well
-   to change the boundary between the text section and data section
-   when Emacs is dumped.  If you define this, the preloaded Lisp
-   code will not be sharable; but that's better than failing completely.  */
-
-#define NO_REMAP
+#endif /* ENABLE_SM_FILE_DECLS_OF_LOADAVG_STUFF */
 
 /* After adding support for a new system, modify the large case
    statement in the `configure' script to recognize reasonable
