@@ -645,7 +645,7 @@ x_initialize_frame_menubar (struct frame *f)
 }
 
 
-static LWLIB_ID last_popup_menu_selection_callback_id;
+static LWLIB_ID last_popup_menu_selection_callback_id = (LWLIB_ID) -1;
 
 static void
 popup_menu_selection_callback (Widget widget, LWLIB_ID id,
@@ -699,13 +699,13 @@ make_dummy_xbutton_event (XEvent *dummy, Widget daddy, Lisp_Event *eev)
       {
 	Widget shell = XtParent (daddy);
 
-	Xt_SET_ARG (al [0], XtNx, &shellx);
-	Xt_SET_ARG (al [1], XtNy, &shelly);
+	XtSetArg (al [0], XtNx, &shellx);
+	XtSetArg (al [1], XtNy, &shelly);
 	XtGetValues (shell, al, 2);
       }
 #endif
-      Xt_SET_ARG (al [0], XtNx, &framex);
-      Xt_SET_ARG (al [1], XtNy, &framey);
+      XtSetArg (al [0], XtNx, &framex);
+      XtSetArg (al [1], XtNy, &framey);
       XtGetValues (daddy, al, 2);
       btn->x_root = shellx + framex + btn->x;
       btn->y_root = shelly + framey + btn->y;
@@ -1375,12 +1375,6 @@ console_type_create_menubar_x (void)
   CONSOLE_HAS_METHOD (x, popup_menu);
 }
 
-void
-reinit_vars_of_menubar_x (void)
-{
-  last_popup_menu_selection_callback_id = (LWLIB_ID) -1;
-}
-
 void
 vars_of_menubar_x (void)
 {
