@@ -285,9 +285,6 @@ static const struct memory_description charset_description[] = {
   { XD_END }
 };
 
-DEFINE_DUMPABLE_LISP_OBJECT ("charset", charset, print_charset, 0,
-			     0, 0, charset_description, Lisp_Charset);
-
 #ifndef UNICODE_INTERNAL
 
 static int
@@ -1741,18 +1738,14 @@ charset_memory_usage (Lisp_Object charset, struct generic_usage_stats *gustats)
 /************************************************************************/
 
 void
-mule_charset_objects_create (void)
+syms_of_mule_charset (void)
 {
+  DEFINE_DUMPABLE_LISP_OBJECT ("charset", charset, print_charset, 0,
+                               0, 0, charset_description, Lisp_Charset);
 #ifdef MEMORY_USAGE_STATS
   OBJECT_HAS_METHOD (charset, memory_usage);
 #endif
   OBJECT_HAS_PREMETHOD (charset, disksave);
-}
-
-void
-syms_of_mule_charset (void)
-{
-  INIT_LISP_OBJECT (charset);
 
   DEFSUBR (Fcharsetp);
   DEFSUBR (Ffind_charset);
@@ -1870,8 +1863,7 @@ vars_of_mule_charset (void)
   int i, j, k;
 
 #ifdef MEMORY_USAGE_STATS
-  OBJECT_HAS_PROPERTY
-    (charset, memusage_stats_list, list2 (Qfrom_unicode, Qto_unicode));
+  INIT_MEMORY_USAGE_STATS (charset, list2 (Qfrom_unicode, Qto_unicode));
 #endif /* MEMORY_USAGE_STATS */
 
   chlook = xnew_and_zero (struct charset_lookup); /* zero for Purify. */
@@ -2098,7 +2090,7 @@ void
 init_mule_charset (void)
 {
   /* Retrieve all the charsets by name.  Do this after loadup, when they
-     were created. */
+     were created. See disksave_object_finalization(). */
   Vcharset_latin_iso8859_2 = Fget_charset (Qlatin_iso8859_2);
   Vcharset_latin_iso8859_3 = Fget_charset (Qlatin_iso8859_3);
   Vcharset_latin_iso8859_4 = Fget_charset (Qlatin_iso8859_4);
