@@ -4,10 +4,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.*/
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -41,7 +39,7 @@ nt_play_sound_file (Lisp_Object path, int UNUSED (volume))
   Lisp_Object fname = Ffile_name_nondirectory (path);
   Extbyte *fnameext;
 
-  LISP_STRING_TO_TSTR (fname, fnameext);
+  fnameext = LISP_STRING_TO_TSTR (fname);
 
   if (qxeSearchPath (NULL, fnameext, NULL, 0, NULL, NULL) == 0)
     {
@@ -66,7 +64,7 @@ nt_play_sound_file (Lisp_Object path, int UNUSED (volume))
       if (retry_read (ofd, data, size) != size)
 	{
 	  retry_close (ofd);
-	  xfree (data, Binbyte *);
+	  xfree (data);
 	  return;
 	}
       retry_close (ofd);
@@ -88,7 +86,7 @@ play_sound_data_1 (Binbyte *data, int length, int UNUSED (volume),
   if (sound_data)
     {
       qxePlaySound (NULL, NULL, flags);
-      xfree (sound_data, Binbyte *);
+      xfree (sound_data);
       sound_data = 0;
     }
 
