@@ -27,22 +27,78 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 static mp_exp_t float_print_min, float_print_max;
 gmp_randstate_t random_state;
 
+#define GMP_GET_AND_SHIFT_LIMB(bn, n)                           \
+  (((unsigned long long) (mpz_getlimbn (bn, n) & GMP_NUMB_MASK) \
+    << (GMP_LIMB_BITS * n)))
+
+/* Return the lower-order ((SIZEOF_LONG_LONG * BITS_PER CHAR) - 1) bits of
+   bignum B as a signed long long, with the sign reflecting B's sign rather
+   than the corresponding bit within the bignum's value. */
 long long
 bignum_to_llong (const bignum b)
 {
-  long long l;
-
-  mpz_export (&l, NULL, 1, sizeof (l), 0, 0U, b);
-  return (mpz_sgn (b) < 0) ? -l : l;
+  unsigned long long sign_mask
+    = bignum_sign (b) < 0 ? ((signed long long) -1) : LLONG_MAX;
+  return
+    (
+#if (GMP_NUMB_BITS * 9) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+#error "Unimplemented" /* But easy to implement! */
+#endif
+#if (GMP_NUMB_BITS * 8) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+     GMP_GET_AND_SHIFT_LIMB (b, 7) |
+#endif
+#if (GMP_NUMB_BITS * 7) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+     GMP_GET_AND_SHIFT_LIMB (b, 6) |    
+#endif
+#if (GMP_NUMB_BITS * 6) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+     GMP_GET_AND_SHIFT_LIMB (b, 5) |    
+#endif
+#if (GMP_NUMB_BITS * 5) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+     GMP_GET_AND_SHIFT_LIMB (b, 4) |    
+#endif
+#if (GMP_NUMB_BITS * 4) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+     GMP_GET_AND_SHIFT_LIMB (b, 3) |    
+#endif
+#if (GMP_NUMB_BITS * 3) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+     GMP_GET_AND_SHIFT_LIMB (b, 2) |
+#endif
+#if (GMP_NUMB_BITS * 2) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+     GMP_GET_AND_SHIFT_LIMB (b, 1) |
+#endif
+     GMP_GET_AND_SHIFT_LIMB (b, 0)) & sign_mask;
 }
 
+/* Return the lower-order (SIZEOF_LONG_LONG * BITS_PER CHAR) bits of bignum B
+   as an unsigned long long. */
 unsigned long long
 bignum_to_ullong (const bignum b)
 {
-  unsigned long long l;
-
-  mpz_export (&l, NULL, 1, sizeof (l), 0, 0U, b);
-  return l;
+  return 
+#if (GMP_NUMB_BITS * 9) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+#error "Unimplemented"
+#endif
+#if (GMP_NUMB_BITS * 8) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+    GMP_GET_AND_SHIFT_LIMB (b, 7) |
+#endif
+#if (GMP_NUMB_BITS * 7) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+    GMP_GET_AND_SHIFT_LIMB (b, 6) |    
+#endif
+#if (GMP_NUMB_BITS * 6) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+    GMP_GET_AND_SHIFT_LIMB (b, 5) |    
+#endif
+#if (GMP_NUMB_BITS * 5) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+    GMP_GET_AND_SHIFT_LIMB (b, 4) |    
+#endif
+#if (GMP_NUMB_BITS * 4) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+    GMP_GET_AND_SHIFT_LIMB (b, 3) |    
+#endif
+#if (GMP_NUMB_BITS * 3) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+    GMP_GET_AND_SHIFT_LIMB (b, 2) |
+#endif
+#if (GMP_NUMB_BITS * 2) <= (SIZEOF_LONG_LONG * BITS_PER_CHAR)
+    GMP_GET_AND_SHIFT_LIMB (b, 1) |
+#endif
+    GMP_GET_AND_SHIFT_LIMB (b, 0);
 }
 
 void
@@ -64,11 +120,12 @@ bignum_set_llong (bignum b, long long l)
 CIbyte *
 bigfloat_to_string (mpf_t f, int base)
 {
-  mp_exp_t expt;
-  CIbyte *str = mpf_get_str (NULL, &expt, base, 0, f);
+  mp_exp_t expt_gmp;
+  CIbyte *str = mpf_get_str (NULL, &expt_gmp, base, 0, f);
   const int sign = mpf_sgn (f);
   const int neg = (sign < 0) ? 1 : 0;
-  int len = strlen (str) + 1;  /* Count the null terminator */
+  Bytecount len = strlen (str) + 1;  /* Count the null terminator */
+  EMACS_INT expt = expt_gmp; /* So we have a predictable type. */
 
   if (sign == 0)
     {
@@ -80,7 +137,7 @@ bigfloat_to_string (mpf_t f, int base)
       if (expt < 0)
 	{
 	  /* We need room for a radix point and leading zeroes */
-	  const int space = -expt + 2;
+	  const EMACS_INT space = -expt + 2;
 	  XREALLOC_ARRAY (str, CIbyte, len + space);
 	  memmove (&str[space + neg], &str[neg], len - neg);
 	  memset (&str[neg], '0', space);
@@ -120,7 +177,9 @@ bigfloat_to_string (mpf_t f, int base)
 	    memmove (&str[neg + 2], &str[neg + 1], len - neg);
 	    str[neg + 1] = '.';
 	  }
-	sprintf (&str[len + point - 1], "E%ld", expt);
+	emacs_snprintf_ascbyte (&str[len + point - 1],
+                                (len + space) - (len + point - 1),
+                                "E%zd", expt);
       }
     }
   return str;
