@@ -156,9 +156,6 @@ static const struct memory_description bignum_description[] = {
   { XD_END }
 };
 
-DEFINE_DUMPABLE_FROB_BLOCK_LISP_OBJECT ("bignum", bignum, bignum_print,
-					0, bignum_equal, bignum_hash,
-					bignum_description, Lisp_Bignum); 
 #endif /* HAVE_BIGNUM */
 
 /********************************** Ratios **********************************/
@@ -199,11 +196,6 @@ static const struct memory_description ratio_description[] = {
   { XD_OPAQUE_PTR, offsetof (Lisp_Ratio, data) },
   { XD_END }
 };
-
-DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("ratio", ratio, ratio_print, 0,
-				      ratio_equal, ratio_hash,
-				      ratio_description, Lisp_Ratio);
-
 #endif /* HAVE_RATIO */
 
 
@@ -245,10 +237,6 @@ static const struct memory_description bigfloat_description[] = {
   { XD_END }
 };
 
-DEFINE_DUMPABLE_FROB_BLOCK_LISP_OBJECT ("bigfloat", bigfloat, bigfloat_print,
-					0, bigfloat_equal, bigfloat_hash,
-					bigfloat_description, Lisp_Bigfloat);
-
 extern Lisp_Object float_to_bigfloat (const Ascbyte *, Lisp_Object,
                                       unsigned long);
 
@@ -356,6 +344,7 @@ Return the canonical form of NUMBER.
 */
        (number))
 {
+  CHECK_NUMBER (number);
   /* The tests should go in order from larger, more expressive, or more
      complex types to smaller, less expressive, or simpler types so that a
      number can cascade all the way down to the simplest type if
@@ -458,7 +447,7 @@ internal_coerce_number (Lisp_Object number, enum number_type type,
       switch (type)
 	{
 	case FIXNUM_T:
-	  return make_fixnum (bignum_to_long (XBIGNUM_DATA (number)));
+	  return make_fixnum (bignum_to_emacs_int (XBIGNUM_DATA (number)));
 	case BIGNUM_T:
 	  return number;
 	case RATIO_T:
@@ -492,7 +481,7 @@ internal_coerce_number (Lisp_Object number, enum number_type type,
 	case FIXNUM_T:
 	  bignum_div (scratch_bignum, XRATIO_NUMERATOR (number),
 		      XRATIO_DENOMINATOR (number));
-	  return make_fixnum (bignum_to_long (scratch_bignum));
+	  return make_fixnum (bignum_to_emacs_int (scratch_bignum));
 	case BIGNUM_T:
 	  bignum_div (scratch_bignum, XRATIO_NUMERATOR (number),
 		      XRATIO_DENOMINATOR (number));
@@ -655,6 +644,7 @@ cases; the information is silently lost.
        (number, type, USED_IF_BIGFLOAT (precision)))
 {
   CHECK_SYMBOL (type);
+  CHECK_NUMBER (number);
   if (EQ (type, Qfixnum))
     return internal_coerce_number (number, FIXNUM_T, 0UL);
   else if (EQ (type, Qinteger))
@@ -701,7 +691,7 @@ cases; the information is silently lost.
     }
 #endif /* HAVE_BIGFLOAT */
 
-  Fsignal (Qunsupported_type, type);
+  Fsignal (Qunsupported_type, list2 (type, number));
   /* NOTREACHED */
   return Qnil;
 }
@@ -711,13 +701,19 @@ void
 syms_of_number (void)
 {
 #ifdef HAVE_BIGNUM
-  INIT_LISP_OBJECT (bignum);
+  DEFINE_DUMPABLE_FROB_BLOCK_LISP_OBJECT ("bignum", bignum, bignum_print,
+                                          0, bignum_equal, bignum_hash,
+                                          bignum_description, Lisp_Bignum); 
 #endif
 #ifdef HAVE_RATIO
-  INIT_LISP_OBJECT (ratio);
+  DEFINE_NODUMP_FROB_BLOCK_LISP_OBJECT ("ratio", ratio, ratio_print, 0,
+                                        ratio_equal, ratio_hash,
+                                        ratio_description, Lisp_Ratio);
 #endif
 #ifdef HAVE_BIGFLOAT
-  INIT_LISP_OBJECT (bigfloat);
+  DEFINE_DUMPABLE_FROB_BLOCK_LISP_OBJECT ("bigfloat", bigfloat, bigfloat_print,
+                                          0, bigfloat_equal, bigfloat_hash,
+                                          bigfloat_description, Lisp_Bigfloat);
 #endif
 
   /* Functions */
