@@ -27,16 +27,14 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
    the argument to record_unwind_protect()).  Once created in C,
    opaque objects cannot be resized.
 
-   OPAQUE OBJECTS SHOULD NEVER ESCAPE TO THE LISP LEVEL.  Some code
-   depends on this.  As such, opaque objects are a generalization
-   of the Qunbound marker.
- */
+   OPAQUE OBJECTS SHOULD NEVER ESCAPE TO THE LISP LEVEL.  Much code depends on
+   this, particularly since Qunbound is implemented as a Lisp_Opaque.  */
 
 #include <config.h>
 #include "lisp.h"
 #include "opaque.h"
 
-Lisp_Object Vopaque_ptr_free_list;
+Lisp_Object Qunbound;
 
 /* Should never, ever be called. (except by an external debugger) */
 static void
@@ -111,11 +109,6 @@ static const struct memory_description opaque_description[] = {
   { XD_END }
 };
 
-DEFINE_DUMPABLE_SIZABLE_LISP_OBJECT ("opaque", opaque, print_opaque, 0,
-				     equal_opaque, hash_opaque,
-				     opaque_description,
-				     sizeof_opaque, Lisp_Opaque);
-
 /* stuff to handle opaque pointers */
 
 /* Should never, ever be called. (except by an external debugger) */
@@ -148,14 +141,10 @@ static const struct memory_description opaque_ptr_description[] = {
   { XD_END }
 };
 
-DEFINE_NODUMP_LISP_OBJECT ("opaque-ptr", opaque_ptr, print_opaque_ptr, 0,
-			   equal_opaque_ptr, hash_opaque_ptr,
-			   opaque_ptr_description, Lisp_Opaque_Ptr);
-
 Lisp_Object
 make_opaque_ptr (void *val)
 {
-  Lisp_Object res = alloc_managed_lcrecord (Vopaque_ptr_free_list);
+  Lisp_Object res = ALLOC_NORMAL_LISP_OBJECT (opaque_ptr);
   set_opaque_ptr (res, val);
   return res;
 }
@@ -164,24 +153,29 @@ make_opaque_ptr (void *val)
    free_cons() apply. */
 
 void
-free_opaque_ptr (Lisp_Object ptr)
+free_opaque_ptr (Lisp_Object obj)
 {
-  free_managed_lcrecord (Vopaque_ptr_free_list, ptr);
-}
-
-void
-reinit_opaque_early (void)
-{
-  Vopaque_ptr_free_list = make_lcrecord_list (sizeof (Lisp_Opaque_Ptr),
-					      &lrecord_opaque_ptr);
-  staticpro_nodump (&Vopaque_ptr_free_list);
+  free_normal_lisp_object (obj);
 }
 
 void
 init_opaque_once_early (void)
 {
-  INIT_LISP_OBJECT (opaque);
-  INIT_LISP_OBJECT (opaque_ptr);
+  DEFINE_DUMPABLE_SIZABLE_LISP_OBJECT ("opaque", opaque, print_opaque, 0,
+                                       equal_opaque, hash_opaque,
+                                       opaque_description,
+                                       sizeof_opaque, Lisp_Opaque);
 
-  reinit_opaque_early ();
+  DEFINE_NODUMP_LISP_OBJECT ("opaque-ptr", opaque_ptr, print_opaque_ptr, 0,
+                             equal_opaque_ptr, hash_opaque_ptr,
+                             opaque_ptr_description, Lisp_Opaque_Ptr);
+
+  Qunbound = make_opaque (OPAQUE_CLEAR, 0);
+  SET_C_READONLY (Qunbound);
+  dump_add_root_lisp_object (&Qunbound);
+  /* No need to have this around as a GC root after pdump_load(), it's
+     C-readonly and won't be traversed in any event. */
+  staticpro_nodump (&Qunbound);
 }
+
+/* opaque.c ends here. */
