@@ -781,7 +781,7 @@ arguments: (SEQUENCE &optional STREAM &key (START 0) END)
   Bytecount bstart = 0, blen = 0;
   Elemcount ii = 0;
 
-  PARSE_KEYWORDS_8 (Qwrite_sequence, nargs, args, 2, (start, end), 
+  PARSE_KEYWORDS_7 (Qwrite_sequence, nargs, args, (start, end), 
                     (start = Qzero), 2, 0);
 
   CHECK_SEQUENCE (sequence);
@@ -1586,9 +1586,13 @@ printing_unreadable_lisp_object (Lisp_Object obj, const Ibyte *name)
     XRECORD_LHEADER_IMPLEMENTATION (obj);
 
   if (name)
-    printing_unreadable_object_fmt ("#<%s %s 0x%x>", imp->name, name, header->uid);
+    printing_unreadable_object_fmt ("#<%s %s 0x%x>",
+                                    LRECORD_IMPLEMENTATION_IBYTE_NAME (imp),
+                                    name, header->uid);
   else
-    printing_unreadable_object_fmt ("#<%s 0x%x>", imp->name, header->uid);
+    printing_unreadable_object_fmt ("#<%s 0x%x>",
+                                    LRECORD_IMPLEMENTATION_IBYTE_NAME (imp),
+                                    header->uid);
 }
 
 void
@@ -1602,7 +1606,9 @@ external_object_printer (Lisp_Object obj, Lisp_Object printcharfun,
   if (print_readably)
     printing_unreadable_lisp_object (obj, 0);
 
-  write_fmt_string (printcharfun, "#<%s 0x%x>", imp->name, header->uid);
+  write_fmt_string (printcharfun, "#<%s 0x%x>",
+                    LRECORD_IMPLEMENTATION_IBYTE_NAME (imp),
+                    header->uid);
 }
 
 void
@@ -1612,14 +1618,16 @@ internal_object_printer (Lisp_Object obj, Lisp_Object printcharfun,
   if (print_readably)
     printing_unreadable_object_fmt
       ("#<INTERNAL OBJECT (XEmacs bug?) (%s) 0x%x>",
-       XRECORD_LHEADER_IMPLEMENTATION (obj)->name, LISP_OBJECT_UID (obj));
+       LRECORD_IMPLEMENTATION_IBYTE_NAME
+       (XRECORD_LHEADER_IMPLEMENTATION (obj)), LISP_OBJECT_UID (obj));
 
   /* Internal objects shouldn't normally escape to the Lisp level;
      that's why we say "XEmacs bug?".  This can happen, however, when
      printing backtraces. */
   write_fmt_string (printcharfun,
 		    "#<INTERNAL OBJECT (XEmacs bug?) (%s) 0x%x>",
-		    XRECORD_LHEADER_IMPLEMENTATION (obj)->name,
+                    LRECORD_IMPLEMENTATION_IBYTE_NAME
+		    (XRECORD_LHEADER_IMPLEMENTATION (obj)),
 		    LISP_OBJECT_UID (obj));
 }
 
@@ -2825,7 +2833,8 @@ debug_p4 (Lisp_Object obj)
 	debug_out ("<< bad object type=%d %p>>", header->type, header);
       else
 	debug_out ("#<%s addr=0x%zx uid=0x%zx>",
-		   LHEADER_IMPLEMENTATION (header)->name,
+                   LRECORD_IMPLEMENTATION_IBYTE_NAME
+                   (LHEADER_IMPLEMENTATION (header)),
 		   (EMACS_INT) header,
 		   (EMACS_INT) ((struct lrecord_header *) header)->uid);
     }
@@ -3045,12 +3054,6 @@ syms_of_print (void)
   DEFSUBR (Fwith_output_to_temp_buffer);
 }
 
-void
-reinit_vars_of_print (void)
-{
-  alternate_do_pointer = 0;
-}
-
 void
 vars_of_print (void)
 {
