@@ -67,6 +67,50 @@ Lisp_Object Qprocessp, Qprocess_live_p, Qprocess_readable_p;
 /* Process methods */
 struct process_methods the_process_methods;
 
+static const struct memory_description process_methods_description_1[] = {
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   print_process_data) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   finalize_process_data) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   alloc_process_data) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   init_process_io_handles) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   create_process) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   tooltalk_connection_p) },
+#ifdef HAVE_SOCKETS
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   open_network_stream) },
+#ifdef HAVE_MULTICAST
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   open_multicast_group) },
+#endif /* HAVE_MULTICAST */
+#endif /* HAVE_SOCKETS */
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   canonicalize_host_name) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   set_window_size) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   send_process) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   reap_exited_processes) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   update_status_if_terminated) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   kill_child_process) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   kill_process_by_pid) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   process_send_eof) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   deactivate_process) },
+  { XD_FUNCTION_POINTER, offsetof (struct process_methods,
+                                   init_process) },
+  { XD_END }
+};
+
 /* a process object is a network connection when its pid field a cons
    (name of name of port we are connected to . foreign host name) */
 
@@ -86,7 +130,7 @@ Lisp_Object Qmulticast; /* Will be used for occasional warnings */
 Lisp_Object Vprocess_connection_type;
 
 /* Read comments to DEFVAR of this */
-int windowed_process_io;
+Boolint windowed_process_io;
 
 #ifdef PROCESS_IO_BLOCKING
 /* List of port numbers or port names to set a blocking I/O mode.
@@ -101,7 +145,7 @@ volatile int process_tick;
 static int update_tick;
 
 /* Nonzero means delete a process right away if it exits.  */
-int delete_exited_processes;
+Boolint delete_exited_processes;
 
 /* Weak list mapping Lisp_Object USIDs as returned by create_io_streams_cb to
    process objects. Processes are not GC-protected through this! */
@@ -116,6 +160,7 @@ Lisp_Object Vdefault_process_coding_system;
 Lisp_Object Vdefault_network_coding_system;
 Lisp_Object Qprocess_error;
 Lisp_Object Qnetwork_error;
+Lisp_Object Qtls_error;
 Fixnum debug_process_io;
 Lisp_Object Vshell_file_name;
 Lisp_Object Vprocess_environment;
@@ -229,10 +274,6 @@ finalize_process (Lisp_Object obj)
       p->process_data = 0;
     }
 }
-
-DEFINE_NODUMP_LISP_OBJECT ("process", process, print_process,
-			   finalize_process, 0, 0, process_description,
-			   Lisp_Process);
 
 /************************************************************************/
 /*                       basic process accessors                        */
@@ -2528,16 +2569,6 @@ eputenv (const CIbyte *var, const CIbyte *value)
 }
 
 
-void
-reinit_process_early (void)
-{
-  OBJECT_HAS_METHOD (process, getprop);
-  OBJECT_HAS_METHOD (process, putprop);
-  OBJECT_HAS_METHOD (process, remprop);
-  OBJECT_HAS_METHOD (process, plist);
-  OBJECT_HAS_METHOD (process, setplist);
-}
-
 /* This is not named init_process in order to avoid a conflict with NS 3.3 */
 void
 init_xemacs_process (void)
@@ -2608,14 +2639,19 @@ init_xemacs_process (void)
 
     Vshell_file_name = build_istring (shell);
   }
-
-  reinit_process_early ();
 }
 
 void
 syms_of_process (void)
 {
-  INIT_LISP_OBJECT (process);
+  DEFINE_NODUMP_LISP_OBJECT ("process", process, print_process, 
+                             finalize_process, 0, 0, process_description,
+                             Lisp_Process);
+  OBJECT_HAS_METHOD (process, getprop);
+  OBJECT_HAS_METHOD (process, putprop);
+  OBJECT_HAS_METHOD (process, remprop);
+  OBJECT_HAS_METHOD (process, plist);
+  OBJECT_HAS_METHOD (process, setplist);
 
   DEFSYMBOL (Qprocessp);
   DEFSYMBOL (Qprocess_live_p);
@@ -2636,6 +2672,7 @@ syms_of_process (void)
 
   DEFERROR_STANDARD (Qprocess_error, Qio_error);
   DEFERROR_STANDARD (Qnetwork_error, Qio_error);
+  DEFERROR (Qtls_error, "TLS error", Qerror);
 
   DEFSUBR (Fprocessp);
   DEFSUBR (Fprocess_live_p);
@@ -2799,6 +2836,8 @@ The value takes effect when `open-network-stream-internal' is called.
 *File name to load inferior shells from.
 Initialized from the SHELL environment variable.
 */ );
+  Vshell_file_name = Qnil;
+  dump_add_nil_lisp_object (&Vshell_file_name);
 
   /* ben? thinks the format of this variable is "semi-bogus".
      sjt doesn't agree, since it captures a restriction that is
@@ -2809,10 +2848,15 @@ Each element should be a string of the form ENVVARNAME=VALUE.
 The environment which Emacs inherits is placed in this variable
 when Emacs starts.
 */ );
+  Vprocess_environment = Qnil;
+  dump_add_nil_lisp_object (&Vprocess_environment);
 
   Vlisp_EXEC_SUFFIXES = build_ascstring (EXEC_SUFFIXES);
   staticpro (&Vlisp_EXEC_SUFFIXES);
 
   Vusid_to_process = make_weak_list (WEAK_LIST_VALUE_ASSOC);
   staticpro (&Vusid_to_process);
+
+  dump_add_root_block (&the_process_methods, sizeof (the_process_methods),
+                       process_methods_description_1);
 }
