@@ -283,11 +283,6 @@ static const struct memory_description range_table_description[] = {
   { XD_END }
 };
 
-DEFINE_DUMPABLE_LISP_OBJECT ("range-table", range_table, print_range_table,
-			     finalize_range_table,
-			     range_table_equal, range_table_hash,
-			     range_table_description,
-			     Lisp_Range_Table);
 
 /************************************************************************/
 /*                        Range table operations                        */
@@ -904,7 +899,7 @@ rangetab_instantiate (Lisp_Object plist)
    range table that need to be marked for garbage collection.
    Good candidates for objects that can go into a range table are
 
-   -- numbers and characters (do not need to be marked)
+   -- fixnums and characters (do not need to be marked)
    -- nil, t (marked elsewhere)
    -- charsets and coding systems (automatically marked because
 				   they are in a marked list,
@@ -1080,17 +1075,16 @@ unified_range_table_get_range (void *unrangetab, int offset,
 /************************************************************************/
 /*                            Initialization                            */
 /************************************************************************/
-void
-rangetab_objects_create (void)
-{
-  OBJECT_HAS_METHOD (range_table, print_preprocess);
-  OBJECT_HAS_METHOD (range_table, nsubst_structures_descend);
-}
 
 void
 syms_of_rangetab (void)
 {
-  INIT_LISP_OBJECT (range_table);
+  DEFINE_DUMPABLE_LISP_OBJECT ("range-table", range_table, print_range_table,
+                               finalize_range_table, range_table_equal,
+                               range_table_hash, range_table_description,
+                               Lisp_Range_Table);
+  OBJECT_HAS_METHOD (range_table, print_preprocess);
+  OBJECT_HAS_METHOD (range_table, nsubst_structures_descend);
 
   DEFSYMBOL_MULTIWORD_PREDICATE (Qrange_tablep);
   DEFSYMBOL (Qrange_table);
