@@ -1,6 +1,6 @@
 /* Synchronize redisplay structures and output changes.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 1995, 1996, 2002, 2003 Ben Wing.
+   Copyright (C) 1995, 1996, 2002, 2003, 2005 Ben Wing.
    Copyright (C) 1996 Chuck Thompson.
    Copyright (C) 1999, 2002 Andy Piper.
    Copyright (C) 2010 Didier Verna
@@ -195,7 +195,7 @@ sync_display_line_structs (struct window *w, int line, int do_blocks,
 static int
 compare_runes (struct window *w, struct rune *crb, struct rune *drb)
 {
-  /* Do not compare the values of charbpos and endpos.  They do not
+  /* Do not compare the values of bytepos and endpos.  They do not
      affect the display characteristics. */
 
   /* Note: (hanoi 6) spends 95% of its time in redisplay, and about
@@ -556,6 +556,7 @@ compare_display_blocks (struct window *w, struct display_line *cdl,
       cdl->ascent != ddl->ascent ||
       cdl->descent != ddl->descent ||
       cdl->clip != ddl->clip ||
+      cdl->clear_findex != ddl->clear_findex ||
       force)
     {
       start_pos = 0;
@@ -788,7 +789,8 @@ output_display_line (struct window *w, display_line_dynarr *cdla,
 			   cdl->ascent != ddl->ascent ||
 			   cdl->descent != ddl->descent ||
 			   cdl->top_clip != ddl->top_clip ||
-			   cdl->clip != ddl->clip)))
+			   cdl->clip != ddl->clip ||
+			   cdl->clear_findex != ddl->clear_findex)))
 		{
 		  int x, y, width, height;
 		  face_index findex;
@@ -807,8 +809,8 @@ output_display_line (struct window *w, display_line_dynarr *cdla,
 		    }
 		  else if (x < ddl->bounds.right_in)
 		    {
-		      findex = (ddl->default_findex >= DEFAULT_INDEX) ?
-			ddl->default_findex
+		      findex = (ddl->clear_findex >= DEFAULT_INDEX) ?
+			ddl->clear_findex
 			: DEFAULT_INDEX;
 		    }
 		  else if (x < ddl->bounds.right_out)
@@ -942,11 +944,12 @@ output_display_line (struct window *w, display_line_dynarr *cdla,
  boolean indicating success or failure.
  ****************************************************************************/
 
-#define ADJ_CHARPOS (rb->charpos + dl->offset)
+#define ADJ_BYTEPOS (rb->bytepos + dl->offset) /* This can be zero! */
 #define ADJ_ENDPOS (rb->endpos + dl->offset)
 
-int
-redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
+Boolint
+redisplay_move_cursor (struct window *w, Bytebpos new_point,
+                       Boolint no_output_end)
 {
   struct frame *f = XFRAME (w->frame);
   struct device *d = XDEVICE (f->device);
@@ -982,14 +985,17 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
 
   if (rb->cursor_type == CURSOR_OFF)
     return 0;
-  else if (ADJ_CHARPOS == new_point
-	   || (ADJ_ENDPOS && (new_point >= ADJ_CHARPOS)
+  else if (ADJ_BYTEPOS == new_point
+	   || (ADJ_ENDPOS && (new_point >= ADJ_BYTEPOS)
 	       && (new_point <= ADJ_ENDPOS)))
     {
       w->last_point_x[CURRENT_DISP] = x;
       w->last_point_y[CURRENT_DISP] = y;
-      Fset_marker (w->last_point[CURRENT_DISP], make_int (ADJ_CHARPOS),
-		   w->buffer);
+      /* Not _restricted! */
+      set_marker_byte_position (w->last_point[CURRENT_DISP],
+                                buffer_or_string_clip_to_absolute_byte
+                                (w->buffer, ADJ_BYTEPOS),
+                                w->buffer);
       dl->cursor_elt = x;
       return 1;
     }
@@ -1026,8 +1032,8 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
     {
       w->last_point_x[CURRENT_DISP] = 0;
       w->last_point_y[CURRENT_DISP] = y;
-      Fset_marker (w->last_point[CURRENT_DISP], Qzero, w->buffer);
-
+      set_marker_byte_position (w->last_point[CURRENT_DISP],
+                                BYTE_BUF_BEG (w->buffer), w->buffer);
       rb = Dynarr_begin (db->runes);
       rb->cursor_type = CURSOR_ON;
       dl->cursor_elt = 0;
@@ -1047,7 +1053,7 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
       int first = 0;
       int cur_dl, up;
 
-      if (ADJ_CHARPOS < new_point)
+      if (ADJ_BYTEPOS < new_point)
 	{
 	  up = 1;
 	  cur_rb = x + 1;
@@ -1085,9 +1091,9 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
 
 	      if (rb->cursor_type != IGNORE_CURSOR
 		  && rb->cursor_type != NO_CURSOR &&
-		  (ADJ_CHARPOS == new_point
-		   || (ADJ_ENDPOS && (new_point >= ADJ_CHARPOS)
-		       && (new_point <= ADJ_CHARPOS))))
+		  (ADJ_BYTEPOS == new_point
+		   || (ADJ_ENDPOS && (new_point >= ADJ_BYTEPOS)
+		       && (new_point <= ADJ_ENDPOS))))
 		{
 		  rb->cursor_type = CURSOR_ON;
 		  dl->cursor_elt = cur_rb;
@@ -1098,9 +1104,12 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
 
 		  w->last_point_x[CURRENT_DISP] = cur_rb;
 		  w->last_point_y[CURRENT_DISP] = cur_dl;
-		  Fset_marker (w->last_point[CURRENT_DISP],
-			       make_int (ADJ_CHARPOS), w->buffer);
-
+                  /* Not _restricted! */
+                  set_marker_byte_position
+                    (w->last_point[CURRENT_DISP],
+                     buffer_or_string_clip_to_absolute_byte (w->buffer,
+                                                             ADJ_BYTEPOS),
+                     w->buffer);
 		  if (!no_output_end)
 		    {
 		      MAYBE_DEVMETH (d, window_output_end, (w));
@@ -1124,7 +1133,7 @@ redisplay_move_cursor (struct window *w, Charbpos new_point, int no_output_end)
     }
   return 0;
 }
-#undef ADJ_CHARPOS
+#undef ADJ_BYTEPOS
 #undef ADJ_ENDPOS
 
 /*****************************************************************************
@@ -1414,7 +1423,6 @@ redisplay_output_layout (Lisp_Object domain,
 {
   Lisp_Image_Instance *p = XIMAGE_INSTANCE (image_instance);
   Lisp_Object rest, window = DOMAIN_WINDOW (domain);
-  Ichar_dynarr *buf;
   struct window *w = XWINDOW (window);
   struct device *d = DOMAIN_XDEVICE (domain);
   int layout_height, layout_width;
@@ -1431,8 +1439,6 @@ redisplay_output_layout (Lisp_Object domain,
   if (!redisplay_normalize_glyph_area (db, dga))
     return;
 
-  buf = Dynarr_new (Ichar);
-
   /* Highly dodgy optimization. We want to only output the whole
      layout if we really have to. */
   if (!IMAGE_INSTANCE_OPTIMIZE_OUTPUT (p)
@@ -1474,13 +1480,13 @@ redisplay_output_layout (Lisp_Object domain,
 	    style = EDGE_ETCHED_OUT;
 	  else if (EQ (IMAGE_INSTANCE_LAYOUT_BORDER (p), Qbevel_in))
 	    style = EDGE_BEVEL_IN;
-	  else if (INTP (IMAGE_INSTANCE_LAYOUT_BORDER (p)))
+	  else if (FIXNUMP (IMAGE_INSTANCE_LAYOUT_BORDER (p)))
 	    {
 	      style = EDGE_ETCHED_IN;
 	      if (edges & EDGE_TOP)
 		{
-		  ypos += XINT (IMAGE_INSTANCE_LAYOUT_BORDER (p));
-		  height -= XINT (IMAGE_INSTANCE_LAYOUT_BORDER (p));
+		  ypos += XFIXNUM (IMAGE_INSTANCE_LAYOUT_BORDER (p));
+		  height -= XFIXNUM (IMAGE_INSTANCE_LAYOUT_BORDER (p));
 		}
 	    }
 	  else
@@ -1556,20 +1562,14 @@ redisplay_output_layout (Lisp_Object domain,
 			struct display_line dl;	/* this is fake */
 			Lisp_Object string =
 			  IMAGE_INSTANCE_TEXT_STRING (childii);
-			unsigned char charsets[NUM_LEADING_BYTES];
-			struct face_cachel *cachel
-			  = WINDOW_FACE_CACHEL (w, findex);
+			struct face_cachel *cachel =
+			  WINDOW_FACE_CACHEL (w, findex);
 
 			DISPLAY_LINE_INIT (dl);
 
-			find_charsets_in_ibyte_string (charsets,
-						       XSTRING_DATA (string),
-						       XSTRING_LENGTH (string));
-			ensure_face_cachel_complete (cachel, window, charsets);
-
-			convert_ibyte_string_into_ichar_dynarr
-			  (XSTRING_DATA (string), XSTRING_LENGTH (string),
-			   buf);
+			ensure_face_cachel_complete (cachel, window,
+                                                     XSTRING_DATA (string),
+                                                     XSTRING_LENGTH (string));
 
 			redisplay_normalize_display_box (&cdb, &cdga);
 			/* Offsets are now +ve again so be careful
@@ -1587,11 +1587,11 @@ redisplay_output_layout (Lisp_Object domain,
 			   add the offset to the width so that we
 			   output the full string. */
 			MAYBE_DEVMETH (d, output_string,
-				       (w, &dl, buf, cdb.xpos,
+				       (w, &dl, XSTRING_DATA (string),
+                                        XSTRING_LENGTH (string), cdb.xpos,
 					cdga.xoffset, cdb.xpos,
 					cdga.width + cdga.xoffset,
 					findex, 0, 0, 0, 0));
-			Dynarr_reset (buf);
 		      }
 		  }
 		  break;
@@ -1634,8 +1634,6 @@ redisplay_output_layout (Lisp_Object domain,
   /* Update any display properties. I'm not sure whether this actually
      does anything for layouts except clear the changed flags. */
   redisplay_subwindow (image_instance);
-
-  Dynarr_free (buf);
 }
 
 /****************************************************************************
@@ -1787,12 +1785,12 @@ redisplay_clear_region (Lisp_Object locale, face_index findex, int x, int y,
     {
       if (w)
 	{
-	  fcolor = WINDOW_FACE_CACHEL_FOREGROUND (w, findex);
+	  fcolor = WINDOW_FACE_CACHEL_FOREBACK (w, findex);
 	  bcolor = WINDOW_FACE_CACHEL_BACKGROUND (w, findex);
 	}
       else
 	{
-	  fcolor = FACE_FOREGROUND (Vdefault_face, locale);
+	  fcolor = FACE_FOREBACK (Vdefault_face, locale);
 	  bcolor = FACE_BACKGROUND (Vdefault_face, locale);
 	}
     }
@@ -1807,7 +1805,7 @@ redisplay_clear_region (Lisp_Object locale, face_index findex, int x, int y,
   if (UNBOUNDP (background_pixmap))
     background_pixmap = Qnil;
 
-  DEVMETH (d, clear_region, (locale, d, f, findex, x, y, width, height,
+  DEVMETH (d, clear_region, (locale, f, findex, x, y, width, height,
 			     fcolor, bcolor, 
 			     background_pixmap, background_placement));
 }
@@ -2266,10 +2264,10 @@ redisplay_update_line (struct window *w, int first_line, int last_line,
 
   while (first_line <= last_line)
     {
-      Charcount old_len = (Dynarr_atp (cdla, first_line)->end_charpos -
-			   Dynarr_atp (cdla, first_line)->charpos);
-      Charcount new_len = (Dynarr_atp (ddla, first_line)->end_charpos -
-			   Dynarr_atp (ddla, first_line)->charpos);
+      Bytecount old_len = (Dynarr_atp (cdla, first_line)->end_bytepos -
+			   Dynarr_atp (cdla, first_line)->bytepos);
+      Bytecount new_len = (Dynarr_atp (ddla, first_line)->end_bytepos -
+			   Dynarr_atp (ddla, first_line)->bytepos);
 
       assert (Dynarr_length (cdla) == Dynarr_length (ddla));
 
@@ -2288,11 +2286,9 @@ redisplay_update_line (struct window *w, int first_line, int last_line,
 	    }
 	}
 
-      /* Update the window_end_pos and other settings. */
+      /* Update last_point_x and last_point_y. */
       if (update_values)
 	{
-	  w->window_end_pos[CURRENT_DISP] -= (new_len - old_len);
-
 	  if (Dynarr_atp (ddla, first_line)->cursor_elt != -1)
 	    {
 	      w->last_point_x[CURRENT_DISP] = w->last_point_x[DESIRED_DISP];
@@ -2316,7 +2312,11 @@ redisplay_update_line (struct window *w, int first_line, int last_line,
 	  struct display_line *dl = Dynarr_atp (ddla, line);
 
 	  if (!dl->modeline)
-	    w->max_line_len = max (dl->num_chars, w->max_line_len);
+            {
+              assert (WINDOW_TRUNCATEP (w) ||
+                      dl->num_chars == ((Charcount) 0xDEADBEEF));
+              w->max_line_len = max (dl->num_chars, w->max_line_len);
+            }
 
 	  line++;
 	}
@@ -2324,10 +2324,12 @@ redisplay_update_line (struct window *w, int first_line, int last_line,
 
   w->last_modified[CURRENT_DISP] = w->last_modified[DESIRED_DISP];
   w->last_facechange[CURRENT_DISP] = w->last_facechange[DESIRED_DISP];
-  Fset_marker (w->last_point[CURRENT_DISP],
-	       Fmarker_position (w->last_point[DESIRED_DISP]), w->buffer);
-  Fset_marker (w->last_start[CURRENT_DISP],
-	       Fmarker_position (w->last_start[DESIRED_DISP]), w->buffer);
+  set_marker_byte_position (w->last_point[CURRENT_DISP],
+                            marker_byte_position (w->last_point[DESIRED_DISP]),
+                            w->buffer);
+  set_marker_byte_position (w->last_start[CURRENT_DISP],
+                            marker_byte_position (w->last_start[DESIRED_DISP]),
+                            w->buffer);
 
   /* We don't bother updating the vertical scrollbars here.  This
      gives us a performance increase while having minimal loss of
@@ -2425,7 +2427,8 @@ redisplay_output_window (struct window *w)
 	  else if (cdl->ypos != ddl->ypos ||
 		   cdl->ascent != ddl->ascent ||
 		   cdl->descent != ddl->descent ||
-		   cdl->clip != ddl->clip)
+		   cdl->clip != ddl->clip ||
+		   cdl->clear_findex != ddl->clear_findex)
 	    need_to_clear_bottom = 1;
 
 	  /* #### This kludge is to make sure the modeline shadows get
@@ -2496,19 +2499,23 @@ redisplay_output_window (struct window *w)
 					need_to_clear_end);
     }
 
-  w->window_end_pos[CURRENT_DISP] = w->window_end_pos[DESIRED_DISP];
-  Fset_marker (w->start[CURRENT_DISP],
-	       make_int (marker_position (w->start[DESIRED_DISP])),
-	       w->buffer);
-  Fset_marker (w->pointm[CURRENT_DISP],
-	       make_int (marker_position (w->pointm[DESIRED_DISP])),
-	       w->buffer);
+  set_marker_byte_position (w->start[CURRENT_DISP],
+                            marker_byte_position (w->start[DESIRED_DISP]),
+                            w->buffer);
+  set_marker_byte_position (w->pointm[CURRENT_DISP],
+                            marker_byte_position (w->pointm[DESIRED_DISP]),
+                            w->buffer);
+  set_marker_byte_position (w->end_pos[CURRENT_DISP],
+                            marker_byte_position (w->end_pos[DESIRED_DISP]),
+                            w->buffer);
   w->last_modified[CURRENT_DISP] = w->last_modified[DESIRED_DISP];
   w->last_facechange[CURRENT_DISP] = w->last_facechange[DESIRED_DISP];
-  Fset_marker (w->last_start[CURRENT_DISP],
-	       Fmarker_position (w->last_start[DESIRED_DISP]), w->buffer);
-  Fset_marker (w->last_point[CURRENT_DISP],
-	       Fmarker_position (w->last_point[DESIRED_DISP]), w->buffer);
+  set_marker_byte_position (w->last_start[CURRENT_DISP],
+                            marker_byte_position (w->last_start[DESIRED_DISP]),
+                            w->buffer);
+  set_marker_byte_position (w->last_point[CURRENT_DISP],
+                            marker_byte_position (w->last_point[DESIRED_DISP]),
+                            w->buffer);
   w->last_point_x[CURRENT_DISP] = w->last_point_x[DESIRED_DISP];
   w->last_point_y[CURRENT_DISP] = w->last_point_y[DESIRED_DISP];
   w->shadow_thickness_changed = 0;
@@ -2641,10 +2648,10 @@ redisplay_redraw_exposed_area_1 (Lisp_Object arg)
 {
   assert (!in_display);
   redisplay_redraw_exposed_area (XFRAME (X1ST (arg)),
-				 XINT (X2ND (arg)),
-				 XINT (X3RD (arg)),
-				 XINT (X4TH (arg)),
-				 XINT (X5TH (arg)));
+				 XFIXNUM (X2ND (arg)),
+				 XFIXNUM (X3RD (arg)),
+				 XFIXNUM (X4TH (arg)),
+				 XFIXNUM (X5TH (arg)));
   free_list (arg);
 }
 
@@ -2664,9 +2671,9 @@ redisplay_redraw_exposed_area (struct frame *f, int x, int y, int width,
     {
       /* Not safe to do it now, so delay it */
       register_post_redisplay_action (redisplay_redraw_exposed_area_1,
-				      list5 (wrap_frame (f), make_int (x),
-					     make_int (y), make_int (width),
-					     make_int (height)));
+				      list5 (wrap_frame (f), make_fixnum (x),
+					     make_fixnum (y), make_fixnum (width),
+					     make_fixnum (height)));
       return;
     }
 
@@ -2718,7 +2725,7 @@ bevel_modeline (struct window *w, struct display_line *dl)
   y = dl->ypos - dl->ascent - shadow_thickness;
   height = dl->ascent + dl->descent + 2 * shadow_thickness;
 
-  if (XINT (w->modeline_shadow_thickness) < 0)
+  if (XFIXNUM (w->modeline_shadow_thickness) < 0)
     {
       style = EDGE_BEVEL_IN;
     }
