@@ -294,7 +294,8 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 	  else if (rb->type == RUNE_BLANK)
 	    {
 	      int size = rb->width;
-              Ibyte linebuf[size * ichar_len (' ')], *lbufp = linebuf;
+              Ibyte *linebuf = alloca_ibytes (size * ichar_len (' '));
+              Ibyte *lbufp = linebuf;
 
 	      while (size--)
                 {
@@ -322,7 +323,8 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
           else if (rb->type == RUNE_HLINE)
             {
 	      int size = rb->width;
-              Ibyte linebuf[size * ichar_len ('-')], *lbufp = linebuf;
+              Ibyte *linebuf = alloca_ibytes (size * ichar_len ('-'));
+              Ibyte *lbufp = linebuf;
 
 	      while (size--)
                 {
@@ -607,16 +609,13 @@ substitute_in_dynamic_color_string (Lisp_Object spec, Lisp_Object string)
 #endif
 
 static void
-set_foreground_to (struct console *c, Lisp_Object sym)
+set_foreground_to (struct console *c, Lisp_Object esc_seq)
 {
-  Lisp_Object result;
   Ibyte *escseq = 0;
   Bytecount escseqlen = 0;
 
-  result = assq_no_quit (sym, Vtty_color_alist);
-  if (!NILP (result))
+  if (!NILP (esc_seq))
     {
-      Lisp_Object esc_seq = XCAR (XCDR (result));
       escseq    = XSTRING_DATA   (esc_seq);
       escseqlen = XSTRING_LENGTH (esc_seq);
     }
@@ -637,16 +636,13 @@ set_foreground_to (struct console *c, Lisp_Object sym)
 }
 
 static void
-set_background_to (struct console *c, Lisp_Object sym)
+set_background_to (struct console *c, Lisp_Object esc_seq)
 {
-  Lisp_Object result;
   Ibyte *escseq = 0;
   Bytecount escseqlen = 0;
 
-  result = assq_no_quit (sym, Vtty_color_alist);
-  if (!NILP (result))
+  if (!NILP (esc_seq))
     {
-      Lisp_Object esc_seq = XCDR (XCDR (result));
       escseq    = XSTRING_DATA   (esc_seq);
       escseqlen = XSTRING_LENGTH (esc_seq);
     }
@@ -716,12 +712,12 @@ tty_turn_on_face_1 (struct console *c, int highlight_p,
 
   if (COLOR_INSTANCEP (cinst_fore)
       && !EQ (cinst_fore, Vthe_null_color_instance))
-    set_foreground_to (c, COLOR_INSTANCE_TTY_SYMBOL
+    set_foreground_to (c, COLOR_INSTANCE_TTY_ESCAPE_FORE
 		       (XCOLOR_INSTANCE (cinst_fore)));
 
   if (COLOR_INSTANCEP (cinst_back)
       && !EQ (cinst_back, Vthe_null_color_instance))
-    set_background_to (c, COLOR_INSTANCE_TTY_SYMBOL
+    set_background_to (c, COLOR_INSTANCE_TTY_ESCAPE_BACK
 		       (XCOLOR_INSTANCE (cinst_back)));
 }
 
