@@ -46,9 +46,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #include "window.h"
 #include "text.h"
 
-#ifdef MULE
 #include "mule-ccl.h"
-#endif
 #include "charset.h"
 
 #define NEED_GCCACHE_H
@@ -189,7 +187,7 @@ XLIKE_output_display_block (struct window *w, struct display_line *dl,
 			    int cursor_start, int cursor_width,
 			    int cursor_height)
 {
-#ifndef USE_XFT
+#ifndef XEMACS_USE_XFT
   struct frame *f = XFRAME (w->frame);
 #endif
   Ibyte *buffer, *bufp;
@@ -202,8 +200,7 @@ XLIKE_output_display_block (struct window *w, struct display_line *dl,
   int elt = start;
   face_index findex;
   int xpos, width = 0;
-  Lisp_Object charset = Qunbound; /* Qnil is a valid charset when
-				     MULE is not defined */
+  Lisp_Object charset = Qunbound; /* Gives a better error message than Qnil. */
 
   window = wrap_window (w);
   rb = Dynarr_atp (rba, start);
@@ -411,7 +408,7 @@ XLIKE_output_display_block (struct window *w, struct display_line *dl,
 
   if (dl->modeline
       && !EQ (Qzero, w->modeline_shadow_thickness)
-#ifndef USE_XFT
+#ifndef XEMACS_USE_XFT
       /* This optimization doesn't work right with some Xft fonts, which
 	 leave antialiasing turds at the boundary.  I don't know if this
 	 is an Xft bug or not, but I think it is.   See x_output_string. */
