@@ -4,20 +4,18 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
-XEmacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* based on cygwin32.h by Andy Piper <andy@xemacs.org> */
 
@@ -27,6 +25,10 @@ Boston, MA 02111-1307, USA.  */
 
 #define MINGW
 
+/* We are using Cygwin-style headers in /usr/include, also used by
+   Cygwin itself */
+#define CYGWIN_HEADERS
+
 #ifndef ORDINARY_LINK
 #define ORDINARY_LINK
 #endif
@@ -35,10 +37,6 @@ Boston, MA 02111-1307, USA.  */
 #define LIBS_SYSTEM "-mno-cygwin -mwindows -lwinmm -lwsock32"
 #define WIN32_LEAN_AND_MEAN
 
-#define TEXT_START -1
-#define HEAP_IN_DATA
-#define UNEXEC "unexcw.o"
-
 #define TIME_ONESHOT 0
 #define TIME_PERIODIC 1
 #define LOCALE_USE_CP_ACP 0x40000000
@@ -53,7 +51,7 @@ Boston, MA 02111-1307, USA.  */
 /* #ifndef HAVE_SOCKETS */
 #define HAVE_SOCKETS
 /* #endif */
-#define OBJECTS_SYSTEM nt.o ntheap.o ntproc.o dired-msw.o
+#define OBJECTS_SYSTEM nt.o ntproc.o dired-msw.o
 
 #undef MAIL_USE_SYSTEM_LOCK
 #define HAVE_MSW_C_DIRED
