@@ -84,11 +84,6 @@ static const struct memory_description scrollbar_instance_description [] = {
   { XD_END }
 };
 
-
-DEFINE_NODUMP_INTERNAL_LISP_OBJECT ("scrollbar-instance", scrollbar_instance,
-				    scrollbar_instance_description,
-				    struct scrollbar_instance);
-
 static void
 free_scrollbar_instance (struct scrollbar_instance *instance,
 			 struct frame *frame)
@@ -423,6 +418,69 @@ scrollbar_point (struct window *w, Boolint flag)
     return sb_pos;
 }
 
+extern Bytebpos force_valid_bytebpos_or_zero (struct window *, Bytebpos);
+
+/* Given POSITION, which has come from the scrollbar data of one of the
+   toolkits, return a Lisp object that reflects a valid buffer position,
+   ideally close to where was intended with POSITION.  Since we don't store
+   markers or extents in the toolkit data, we have no guarantee that such
+   positions are actually valid (Lisp may have inserted or deleted text such
+   that the position is outside of the visible area or that it may reflect a
+   partial character). This function also takes into account that the
+   scrollbar is either line-based or buffer-position based, something that
+   update_scrollbar_instance() decides based on buffer size.  */
+Lisp_Object
+scrollbar_slider_position_to_lisp (struct scrollbar_instance *instance,
+                                   int position)
+{
+  Lisp_Object win = real_window (instance->mirror, 1);
+  Lisp_Object buf;
+  struct buffer *b;
+
+  if (NILP (win) || NILP (XWINDOW (win)->buffer))
+    {
+      return Qnil;
+    }
+
+  buf = get_buffer (XWINDOW (win)->buffer, 0);
+  if (NILP (buf))
+    {
+      return Qnil;
+    }
+
+  b = XBUFFER (buf);
+
+  if (instance->line_oriented_scrolling_p)
+    {
+      Bytebpos pos = byte_beginning_of_line (b, position, 1, 1);
+
+      if (pos < 0)
+        {
+          return make_fixnum (position);
+        }
+
+      return make_fixnum (bytebpos_to_charbpos (b, pos));
+    }
+  else
+    {
+      Charbpos cpos;
+      Bytebpos pos = force_valid_bytebpos_or_zero (XWINDOW (win), position);
+
+      if (pos > 0)
+        {
+          cpos = bytebpos_to_charbpos (b, pos);
+          return make_fixnum (cpos);
+        }
+
+      if (position > BYTE_BUF_ZV (b))
+        {
+          return make_fixnum (BUF_ZV (b));
+        }
+
+      return make_fixnum (BUF_BEGV (b));
+    }
+}
+
 /*
  * Update a window's horizontal or vertical scrollbar.
  */
@@ -549,10 +607,19 @@ update_scrollbar_instance (struct window *w, int vertical,
           if (WINDOW_HEIGHT (w)
               < ((BYTE_BUF_ZV (b) - BYTE_BUF_BEGV (b)) / 256))
             {
-              new_minimum = BYTE_BUF_BEGV (b);
-              new_maximum = BYTE_BUF_ZV (b);
+              Bytebpos end_pos;
+
               new_slider_position = scrollbar_point (w, 0);
-              new_slider_size = window_displayed_height (w) * 512;
+
+              end_pos = max (marker_byte_position (w->end_pos[CURRENT_DISP]),
+                             new_slider_position + 1);
+
+              new_minimum = BYTE_BUF_BEGV (b);
+              new_maximum = max (BYTE_BUF_ZV (b), new_minimum + 1);
+              new_slider_size = min ((end_pos - new_slider_position),
+                                     (new_maximum - new_minimum));
+
+              instance->line_oriented_scrolling_p = 0;
             }
           else
             {
@@ -565,6 +632,7 @@ update_scrollbar_instance (struct window *w, int vertical,
               new_minimum = 0;
               new_slider_position
                 = buffer_line_number (b, scrollbar_point (w, 0), 1, 1);
+              instance->line_oriented_scrolling_p = 1;
             }
 
           /* The end position must be strictly greater than the start
@@ -1004,16 +1072,14 @@ This ensures that VALUE is in the proper range for the horizontal scrollbar.
 /************************************************************************/
 
 void
-scrollbar_objects_create (void)
+syms_of_scrollbar (void)
 {
+  DEFINE_NODUMP_INTERNAL_LISP_OBJECT ("scrollbar-instance", scrollbar_instance,
+                                      scrollbar_instance_description,
+                                      struct scrollbar_instance);
 #ifdef MEMORY_USAGE_STATS
   OBJECT_HAS_METHOD (scrollbar_instance, memory_usage);
 #endif
-}
-void
-syms_of_scrollbar (void)
-{
-  INIT_LISP_OBJECT (scrollbar_instance);
 
   DEFSYMBOL (Qscrollbar_line_up);
   DEFSYMBOL (Qscrollbar_line_down);
@@ -1050,9 +1116,8 @@ void
 vars_of_scrollbar (void)
 {
 #ifdef MEMORY_USAGE_STATS
-  OBJECT_HAS_PROPERTY
-    (scrollbar_instance, memusage_stats_list,
-     list1 (intern ("device-data")));
+  INIT_MEMORY_USAGE_STATS (scrollbar_instance,
+                           list1 (intern ("device-data")));
 #endif /* MEMORY_USAGE_STATS */
 
   DEFVAR_LISP ("scrollbar-pointer-glyph", &Vscrollbar_pointer_glyph /*
