@@ -43,8 +43,7 @@ static Lisp_Object Vretrieved_selection;
 static gboolean waiting_for_selection;
 Lisp_Object Vgtk_sent_selection_hooks;
 
-extern int lisp_to_time (Lisp_Object, time_t *);
-extern Lisp_Object time_to_lisp (time_t);
+Lisp_Object Qgtk_sent_selection_hooks;
 
 static GdkAtom
 symbol_to_gtk_atom (struct device *UNUSED (d), Lisp_Object sym,
@@ -55,11 +54,11 @@ symbol_to_gtk_atom (struct device *UNUSED (d), Lisp_Object sym,
   if (EQ (sym, QPRIMARY))	return GDK_SELECTION_PRIMARY;
   if (EQ (sym, QSECONDARY))	return GDK_SELECTION_SECONDARY;
 
-  {
-    const Extbyte *nameext;
-    nameext = LISP_STRING_TO_EXTERNAL (Fsymbol_name (sym), Qctext);
-    return gdk_atom_intern (nameext, only_if_exists ? TRUE : FALSE);
-  }
+  /* GDK maintains its own atoms independent of X11, UTF-8 is appropriate
+     for the encoding. */
+  return gdk_atom_intern (LISP_STRING_TO_EXTERNAL (Fsymbol_name (sym),
+                                                   Qutf_8),
+                          only_if_exists ? TRUE : FALSE);
 }
 
 static Lisp_Object
@@ -74,9 +73,9 @@ atom_to_symbol (struct device *UNUSED (d), GdkAtom atom)
 
     if (! str) return Qnil;
 
-    intstr = EXTERNAL_TO_ITEXT (str, Qctext);
+    intstr = EXTERNAL_TO_ITEXT (str, Qutf_8);
     g_free (str);
-    return intern_istring (intstr);
+    return intern ((const CIbyte *) intstr);
   }
 }
 
@@ -148,8 +147,8 @@ emacs_gtk_selection_handle (GtkWidget *UNUSED (widget),
 
   GCPRO2 (converted_selection, target_symbol);
 
-  selection_symbol = atom_to_symbol (d, selection_data->selection);
-  target_symbol = atom_to_symbol (d, selection_data->target);
+  selection_symbol = atom_to_symbol (d, gtk_selection_data_get_selection (selection_data));
+  target_symbol = atom_to_symbol (d, gtk_selection_data_get_target (selection_data));
 
 #if 0 /* #### MULTIPLE doesn't work yet */
   if (EQ (target_symbol, QMULTIPLE))
@@ -166,7 +165,7 @@ emacs_gtk_selection_handle (GtkWidget *UNUSED (widget),
       goto DONE_LABEL;
     }
 
-  local_selection_time = * (guint32 *) XOPAQUE_DATA (temp_obj);
+  local_selection_time = lisp_to_UINT_32_BIT (temp_obj);
 
   if (time_stamp != GDK_CURRENT_TIME &&
       local_selection_time > time_stamp)
@@ -198,20 +197,26 @@ emacs_gtk_selection_handle (GtkWidget *UNUSED (widget),
 			 make_opaque_ptr (cl));
 
   {
-    Rawbyte *data;
-    Bytecount size;
-    int format;
+    Rawbyte *data; /* The selection data to transfer. */
+    Elemcount len; /* The number of *elements*, each of size FORMAT bits, in
+                      the array at address DATA. */
+    gint format, format_octets;
     GdkAtom type;
-    lisp_data_to_selection_data (d, converted_selection,
-				 &data, &type, &size, &format);
 
+    lisp_data_to_selection_data (d, converted_selection, &data, &type, &len,
+                                 &format);
+    format_octets = format / 8;
     gtk_selection_data_set (selection_data, type, format, data,
-			    /* #### is this right? */
-			    (unsigned int) size);
+                            format_octets * len);
     successful_p = Qt;
     /* Tell x_selection_request_lisp_error() it's cool. */
     cl->successful = TRUE;
-    xfree (data);
+    /* Data need not have been allocated; cf. select-convert-to-delete in
+       lisp/select.el . */
+    if (data)
+      {
+        xfree (data);
+      }
   }
 
   unbind_to (count);
@@ -224,18 +229,8 @@ emacs_gtk_selection_handle (GtkWidget *UNUSED (widget),
   UNGCPRO;
 
   /* Let random lisp code notice that the selection has been asked for. */
-  {
-    Lisp_Object val = Vgtk_sent_selection_hooks;
-    if (!UNBOUNDP (val) && !NILP (val))
-      {
-	Lisp_Object rest;
-	if (CONSP (val) && !EQ (XCAR (val), Qlambda))
-	  for (rest = val; !NILP (rest); rest = Fcdr (rest))
-	    call3 (Fcar (rest), selection_symbol, target_symbol, successful_p);
-	else
-	  call3 (val, selection_symbol, target_symbol, successful_p);
-      }
-  }
+  va_run_hook_with_args (Qgtk_sent_selection_hooks, 3, selection_symbol,
+                         target_symbol, successful_p);
 }
 
 
@@ -259,7 +254,7 @@ emacs_gtk_selection_clear_event_handle (GtkWidget *UNUSED (widget),
   if (NILP (local_selection_time_lisp))
     return;
 
-  local_selection_time = *(guint32 *) XOPAQUE_DATA (local_selection_time_lisp);
+  local_selection_time = lisp_to_UINT_32_BIT (local_selection_time_lisp);
 
   /* This SelectionClear is for a selection that we no longer own, so we can
      disregard it.  (That is, we have reasserted the selection since this
@@ -291,17 +286,17 @@ emacs_gtk_selection_received (GtkWidget *UNUSED (widget),
 
   signal_fake_event ();
 
-  if (selection_data->length < 0)
+  if (gtk_selection_data_get_length (selection_data) < 0)
     {
       return;
     }
 
   Vretrieved_selection =
     selection_data_to_lisp_data (NULL,
-				 selection_data->data,
-				 selection_data->length,
-				 selection_data->type,
-				 selection_data->format);
+				 gtk_selection_data_get_data (selection_data),
+				 gtk_selection_data_get_length (selection_data),
+				 gtk_selection_data_get_data_type (selection_data),
+				 gtk_selection_data_get_format (selection_data));
 }
 
 static int
@@ -342,7 +337,7 @@ gtk_get_foreign_selection (Lisp_Object selection_symbol,
   /* add a timeout handler */
   if (gtk_selection_timeout > 0)
     {
-      Lisp_Object id = Fadd_timeout (make_int (x_selection_timeout),
+      Lisp_Object id = Fadd_timeout (make_fixnum (x_selection_timeout),
 				     Qx_selection_reply_timeout_internal,
 				     Qnil, Qnil);
       record_unwind_protect (Fdisable_timeout, id);
@@ -422,19 +417,9 @@ gtk_own_selection (Lisp_Object selection_name,
   CHECK_SYMBOL (selection_name);
   selection_atom = symbol_to_gtk_atom (d, selection_name, 0);
 
-  gtk_selection_owner_set (selecting_window,
-			   selection_atom,
-			   thyme);
+  gtk_selection_owner_set (selecting_window, selection_atom, thyme);
 
-  /* [[ We do NOT use time_to_lisp() here any more, like we used to.
-     That assumed equivalence of time_t and Time, which is not
-     necessarily the case (e.g. under OSF on the Alphas, where
-     Time is a 64-bit quantity and time_t is a 32-bit quantity).
-
-     Opaque pointers are the clean way to go here. ]] 
-
-     See my comment on the same issue in select-x.c -- Aidan. */
-  return make_opaque (&thyme, sizeof (thyme));
+  return UINT_32_BIT_to_lisp (thyme);
 }
 
 static void
@@ -477,6 +462,7 @@ gtk_selection_exists_p (Lisp_Object selection,
 void
 syms_of_select_gtk (void)
 {
+  DEFSYMBOL (Qgtk_sent_selection_hooks);
 }
 
 void
@@ -508,5 +494,5 @@ to convert into a type that we don't know about or that is inappropriate.
 This hook doesn't let you change the behavior of emacs's selection replies,
 it merely informs you that they have happened.
 */ );
-  Vgtk_sent_selection_hooks = Qunbound;
+  Vgtk_sent_selection_hooks = Qnil;
 }
