@@ -229,7 +229,6 @@ mswindows_own_selection (Lisp_Object selection_name,
 {
   HGLOBAL 	hValue = NULL;
   UINT		cfType;
-  int		is_X_type = FALSE;
   Lisp_Object	cfObject;
   Lisp_Object	data = Qnil;
   int		size;
@@ -246,17 +245,8 @@ mswindows_own_selection (Lisp_Object selection_name,
   if (NILP (selection_type) || x_sym_p (selection_type))
     {
       /* Should COMPOUND_TEXT map to CF_UNICODETEXT? */
-      if (XEUNICODE_P)
-	{
-	  cfType = CF_UNICODETEXT;
-	  cfObject = QCF_UNICODETEXT;
-	}
-      else
-	{
-	  cfType = CF_TEXT;
-	  cfObject = QCF_TEXT;
-	}
-      is_X_type = TRUE;
+      cfType = CF_UNICODETEXT;
+      cfObject = QCF_UNICODETEXT;
     }
   else
     {
@@ -491,7 +481,6 @@ mswindows_get_foreign_selection (Lisp_Object selection_symbol,
   HGLOBAL	hValue = NULL;
   UINT		cfType;
   Lisp_Object	cfObject = Qnil, ret = Qnil, value = Qnil;
-  int		is_X_type = FALSE;
   int		size;
   void		*data;
   struct frame  *f = NULL;
@@ -507,17 +496,8 @@ mswindows_get_foreign_selection (Lisp_Object selection_symbol,
   if (NILP (target_type) || x_sym_p (target_type))
     {
       /* Should COMPOUND_TEXT map to CF_UNICODETEXT? */
-      if (XEUNICODE_P)
-	{
-	  cfType = CF_UNICODETEXT;
-	  cfObject = QCF_UNICODETEXT;
-	}
-      else
-	{
-	  cfType = CF_TEXT;
-	  cfObject = QCF_TEXT;
-	}
-      is_X_type = TRUE;
+      cfType = CF_UNICODETEXT;
+      cfObject = QCF_UNICODETEXT;
     }
   else
     {
