@@ -80,7 +80,7 @@ Lisp_Object Qx_sent_selection_hooks;
 Fixnum x_selection_timeout;
 
 /* Enable motif selection optimizations. */
-int x_selection_strict_motif_ownership;
+Boolint x_selection_strict_motif_ownership;
 
 
 /* Utility functions */
@@ -293,7 +293,6 @@ hack_motif_clipboard_selection (Atom selection_atom,
       const Ibyte *data  = XSTRING_DATA (selection_value);
       Bytecount bytes = XSTRING_LENGTH (selection_value);
 
-#ifdef MULE
       {
 	enum { ASCII, LATIN_1, WORLD } chartypes = ASCII;
 	const Ibyte *ptr = data, *end = ptr + bytes;
@@ -327,9 +326,11 @@ hack_motif_clipboard_selection (Atom selection_atom,
 	    encoding = "COMPOUND_TEXT";
 	  }
       }
-#endif /* MULE */
 
-      fmh = XmStringCreateLtoR ("Clipboard", XmSTRING_DEFAULT_CHARSET);
+      /* XmStringCreateLtoR() (through its implementation,
+         XmStringLtoRCreate()) copies TEXT but does not modify it, this cast
+         is safe. */
+      fmh = XmStringCreateLtoR ((char *) "Clipboard", XmSTRING_DEFAULT_CHARSET);
       while (ClipboardSuccess !=
 	     XmClipboardStartCopy (display, selecting_window, fmh, thyme,
 #ifdef MOTIF_INCREMENTAL_CLIPBOARDS_WORK
@@ -341,7 +342,8 @@ hack_motif_clipboard_selection (Atom selection_atom,
 	;
       XmStringFree (fmh);
       while (ClipboardSuccess !=
-	     XmClipboardCopy (display, selecting_window, itemid, encoding,
+	     XmClipboardCopy (display, selecting_window, itemid,
+                              (char *) encoding,
 #ifdef MOTIF_INCREMENTAL_CLIPBOARDS_WORK
 			      /* O'Reilly examples say size can be 0,
 				 but this clearly is not the case. */
@@ -1352,10 +1354,8 @@ Set the value of the named CUTBUFFER (typically CUT_BUFFER0) to STRING.
   const Ibyte *data;
   Bytecount bytes, bytes_remaining;
   Bytecount max_bytes = SELECTION_QUANTUM (display);
-#ifdef MULE
   const Ibyte *ptr, *end;
   enum { ASCII, LATIN_1, WORLD } chartypes = ASCII;
-#endif
 
   if (max_bytes > MAX_SELECTION_QUANTUM)
     max_bytes = MAX_SELECTION_QUANTUM;
@@ -1375,7 +1375,6 @@ Set the value of the named CUTBUFFER (typically CUT_BUFFER0) to STRING.
      The ICCCM requires that this be so, and other clients assume it,
      as we do ourselves in initialize_cut_buffers.  */
 
-#ifdef MULE
   /* Optimize for the common ASCII case */
   for (ptr = data, end = ptr + bytes; ptr <= end; )
     {
@@ -1400,7 +1399,6 @@ Set the value of the named CUTBUFFER (typically CUT_BUFFER0) to STRING.
     LISP_STRING_TO_SIZED_EXTERNAL (string, data, bytes, Qbinary);
   else if (chartypes == WORLD)
     LISP_STRING_TO_SIZED_EXTERNAL (string, data, bytes, Qctext);
-#endif /* MULE */
 
   bytes_remaining = bytes;
 
@@ -1492,16 +1490,6 @@ console_type_create_select_x (void)
   CONSOLE_HAS_METHOD (x, selection_exists_p);
 }
 
-void
-reinit_vars_of_select_x (void)
-{
-  reading_selection_reply = 0;
-  reading_which_selection = 0;
-  selection_reply_timed_out = 0;
-  for_whom_the_bell_tolls = 0;
-  prop_location_tick = 0;
-}
-
 void
 vars_of_select_x (void)
 {
