@@ -167,13 +167,8 @@ It defaults to the selected device.
     invalid_constant ("`how-to-add' must be nil, append, replace_all, "
 		      "replace_existing or t", how_to_add);
 
-#ifdef MULE
   if (NILP (data_type))
     data_type = QCOMPOUND_TEXT;
-#else
-  if (NILP (data_type))
-    data_type = QSTRING;
-#endif
 
   /* Examine the how-to-add argument */
   if (EQ (how_to_add, Qreplace_all) || NILP (how_to_add))
@@ -433,7 +428,7 @@ nil is the same as PRIMARY, and t is the same as SECONDARY.)
   if      (EQ (selection, Qnil)) selection = QPRIMARY;
   else if (EQ (selection, Qt))   selection = QSECONDARY;
 
-  return NILP (Fassq (selection, Vselection_alist)) ? Qnil : Qt;
+  return NILP (assq_no_quit (selection, Vselection_alist)) ? Qnil : Qt;
 }
 
 DEFUN ("selection-exists-p", Fselection_exists_p, 0, 3, 0, /*
@@ -506,13 +501,8 @@ TEXT or COMPOUND_TEXT, it will be decoded as Compound Text.
   if (NILP (device))
     device = Fselected_device (Qnil);
 
-#ifdef MULE
   if (NILP (target_type))
     target_type = QCOMPOUND_TEXT;
-#else
-  if (NILP (target_type))
-    target_type = QSTRING;
-#endif
 
 #if 0 /* #### MULTIPLE doesn't work yet and probably never will */
   if (CONSP (target_type) &&
