@@ -93,10 +93,6 @@ struct specifier_methods
   /* Create method: Initialize specifier data. Optional. */
   void (*create_method) (Lisp_Object specifier);
 
-  /* Mark method: Mark any lisp object within specifier data
-     structure. Not required if no specifier data are Lisp_Objects. */
-  void (*mark_method) (Lisp_Object specifier);
-
   /* Equal method: Compare two specifiers. This is called after
      ensuring that the two specifiers are of the same type, and have
      the same specs.  Quit is inhibited during the call so it is safe
@@ -129,8 +125,9 @@ struct specifier_methods
      If this function is not present, then Fcopy_tree is used. */
   Lisp_Object (*copy_instantiator_method) (Lisp_Object instantiator);
 
-  /* Validate-matchspec method: Given a matchspec, verify that it's
-     valid for this specifier type.  If not, signal an error.
+  /* Validate-matchspec method: Given a matchspec (see
+     Fspecifier_matching_instance), verify that it's valid for this
+     specifier type.  If not, signal an error.
 
      If this function is not present, *no* matchspecs are considered
      valid.  Note that this differs from validate_method(). */
@@ -150,8 +147,9 @@ struct specifier_methods
      name specifier_instance) to avoid creating "external"
      specification loops.
 
-     NO_FALLBACK indicates that the method should not try the fallbacks
-     (and thus simply return Qunbound) in case of a failure to instantiate.
+     NO_FALLBACK non-nil indicates that the method should not try the
+     fallbacks (and thus simply return Qunbound) in case of a failure to
+     instantiate.
 
      This method must presume that both INSTANTIATOR and MATCHSPEC are
      already validated by the corresponding validate_* methods, and
@@ -168,7 +166,7 @@ struct specifier_methods
 				     Lisp_Object domain,
 				     Lisp_Object instantiator,
 				     Lisp_Object depth,
-				     int no_fallback);
+				     Lisp_Object no_fallback);
 
   /* Going-to-add method: Called when an instantiator is about
      to be added to a specifier.  This function can specify
@@ -221,11 +219,6 @@ struct Lisp_Specifier
   NORMAL_LISP_OBJECT_HEADER header;
   struct specifier_methods *methods;
 
-  /* we keep a chained list of all current specifiers, for GC cleanup
-     purposes.  Do NOT mark through this, or specifiers will never
-     be GC'd. */
-  Lisp_Object next_specifier;
-
   /* This is a straight list of instantiators. */
   Lisp_Object global_specs;
 
@@ -330,16 +323,12 @@ static const struct sized_memory_description type##_specifier_description_0 \
   type##_specifier_methods->name = obj_name;				    \
   type##_specifier_methods->extra_description =				    \
     &specifier_empty_extra_description;					    \
-  defsymbol_nodump (&type##_specifier_methods->predicate_symbol, pred_sym); \
+  defsymbol (&type##_specifier_methods->predicate_symbol, pred_sym);	    \
   add_entry_to_specifier_type_list (Q##type, type##_specifier_methods);	    \
   dump_add_root_block_ptr (&type##_specifier_methods,			    \
 			    &specifier_methods_description);		    \
 } while (0)
 
-#define REINITIALIZE_SPECIFIER_TYPE(type) do {				\
-  staticpro_nodump (&type##_specifier_methods->predicate_symbol);	\
-} while (0)
-
 #define INITIALIZE_SPECIFIER_TYPE_WITH_DATA(type, obj_name, pred_sym)	\
 do {									\
   INITIALIZE_SPECIFIER_TYPE (type, obj_name, pred_sym);			\
@@ -425,9 +414,6 @@ enum spec_add_meth
 
 struct specifier_caching
 {
-#ifdef NEW_GC
-  NORMAL_LISP_OBJECT_HEADER header;
-#endif /* NEW_GC */
   int offset_into_struct_window;
   void (*value_changed_in_window) (Lisp_Object specifier, struct window *w,
 				   Lisp_Object oldval);
@@ -437,18 +423,6 @@ struct specifier_caching
   int always_recompute;
 };
 
-#ifdef NEW_GC
-DECLARE_LISP_OBJECT (specifier_caching, struct specifier_caching);
-#define XSPECIFIER_CACHING(x) \
-  XRECORD (x, specifier_caching, struct specifier_caching)
-#define wrap_specifier_caching(p) \
-  wrap_record (p, specifier_caching)
-#define SPECIFIER_CACHINGP(x) RECORDP (x, specifier_caching)
-#define CHECK_SPECIFIER_CACHING(x) \
-  CHECK_RECORD (x, specifier_caching)
-#define CONCHECK_SPECIFIER_CACHING(x) \
-  CONCHECK_RECORD (x, specifier_caching)
-#endif /* NEW_GC */
 
 /* #### get image instances out of domains! */
 
@@ -466,17 +440,28 @@ DECLARE_LISP_OBJECT (specifier_caching, struct specifier_caching);
   : (DEVICEP (obj) ? obj					\
   : (IMAGE_INSTANCEP (obj) ? image_instance_device (obj)	\
   : Qnil))))
+#define DOMAIN_XDEVICE(obj)			\
+  (XDEVICE (DOMAIN_DEVICE (obj)))
 
 #define DOMAIN_FRAME(obj)				\
    (WINDOWP (obj) ? WINDOW_FRAME (XWINDOW (obj))	\
   : (FRAMEP  (obj) ? obj				\
   : (IMAGE_INSTANCEP (obj) ? image_instance_frame (obj)	\
   : Qnil)))
+#define DOMAIN_XFRAME(obj)			\
+  (XFRAME (DOMAIN_FRAME (obj)))
 
 #define DOMAIN_WINDOW(obj)					\
    (WINDOWP (obj) ? obj						\
   : (IMAGE_INSTANCEP (obj) ? image_instance_window (obj)	\
   : Qnil))
+#define DOMAIN_XWINDOW(obj)			\
+  (XWINDOW (DOMAIN_WINDOW (obj)))
+
+Lisp_Object MAYBE_DOMAIN_BUFFER (Lisp_Object obj);
+Lisp_Object DOMAIN_BUFFER (Lisp_Object obj);
+
+#define DOMAIN_XBUFFER(obj) XBUFFER (DOMAIN_BUFFER (obj))
 
 #define DOMAIN_LIVE_P(obj)					\
    (WINDOWP (obj) ? WINDOW_LIVE_P (XWINDOW (obj))		\
@@ -485,13 +470,6 @@ DECLARE_LISP_OBJECT (specifier_caching, struct specifier_caching);
   : (IMAGE_INSTANCEP (obj) ? image_instance_live_p (obj)	\
   : 0))))
 
-#define DOMAIN_XDEVICE(obj)			\
-  (XDEVICE (DOMAIN_DEVICE (obj)))
-#define DOMAIN_XFRAME(obj)			\
-  (XFRAME (DOMAIN_FRAME (obj)))
-#define DOMAIN_XWINDOW(obj)			\
-  (XWINDOW (DOMAIN_WINDOW (obj)))
-
 EXFUN (Fcopy_specifier, 6);
 EXFUN (Fmake_specifier, 1);
 EXFUN (Fset_specifier_dirty_flag, 1);
@@ -547,7 +525,7 @@ DECLARE_SPECIFIER_TYPE (generic);
 #define CONCHECK_GENERIC_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, generic)
 
 DECLARE_SPECIFIER_TYPE (integer);
-#define XINTEGER_SPECIFIER(x) XSPECIFIER_TYPE (x, integer)
+#define XFIXNUMEGER_SPECIFIER(x) XSPECIFIER_TYPE (x, integer)
 #define INTEGER_SPECIFIERP(x) SPECIFIER_TYPEP (x, integer)
 #define CHECK_INTEGER_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, integer)
 #define CONCHECK_INTEGER_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, integer)
