@@ -92,23 +92,8 @@
 
 #include <stdio.h>
 #include <sys/types.h>
-#if defined(TM_IN_SYS_TIME) || (!defined(HAVE_TM_ZONE) && !defined(HAVE_TZNAME))
-#include <sys/time.h>
-#else
-#include <time.h>
-#endif
 
-#ifndef STDC_HEADERS
-time_t mktime ();
-#endif
-
-#if defined(WIN32_NATIVE) || defined(CYGWIN)
-#include <time.h>
-#else
-#if defined(HAVE_TZNAME)
-extern char *tzname[2];
-#endif
-#endif /* WIN32_NATIVE */
+#include "systime.h"
 
 #ifdef emacs
 #define strftime emacs_strftime
@@ -204,14 +189,14 @@ add_str (char *to, const char *from, int max)
 static int
 add_num_time_t (char *string, int max, time_t num)
 {
-  /* This buffer is large enough to hold the character representation
-     (including the trailing NUL) of any unsigned decimal quantity
-     whose binary representation fits in 128 bits.  */
-  char buf[40];
+  Ascbyte buf[DECIMAL_PRINT_SIZE (num) + 1];
 
-  if (sizeof (num) > 16)
+#ifdef ERROR_CHECK_ANY
+  if (sizeof (num) > sizeof (EMACS_UINT))
     ABORT ();
-  sprintf (buf, "%lu", (unsigned long) num);
+#endif
+
+  emacs_snprintf_ascbyte (buf, sizeof (buf), "%zu", (EMACS_UINT) num);
   return add_str (string, buf, max);
 }
 
@@ -509,7 +494,7 @@ strftime (char *string, size_t max, const char *format, const struct tm *tm)
                 add_char ((offset < 0 ? '-' : '+'));
                 
                 if ( minutes / 600 != 0 )
-                  add_char (minutes / 600 + '0');
+                  add_char ((char) ((minutes / 600) + '0'));
                 else if ( pad != none )
                   add_char ((pad == zero ? '0' : ' '));
                 
