@@ -1,13 +1,13 @@
 /* Definitions of symbol-value forwarding for XEmacs Lisp interpreter.
    Copyright (C) 1985, 1986, 1987, 1992, 1993 Free Software Foundation, Inc.
-   Copyright (C) 2000, 2001, 2002 Ben Wing.
+   Copyright (C) 2000, 2001, 2002, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -31,12 +29,18 @@ BEGIN_C_DECLS
 
 enum symbol_value_type
 {
-  /* The following tags use the 'symbol_value_forward' structure
+  /* The following tags use the 'symbol_value_forward_fixnum' structure
      and are strictly for variables DEFVARed on the C level. */
   SYMVAL_FIXNUM_FORWARD,	/* Forward C "Fixnum", really "EMACS_INT" */
   SYMVAL_CONST_FIXNUM_FORWARD,	/* Same, but can't be set */
-  SYMVAL_BOOLEAN_FORWARD,	/* Forward C boolean ("int") */
+
+  /* The following tags use the 'symbol_value_forward_boolint' structure, also
+     only for variables DEFVARed in C. */
+  SYMVAL_BOOLEAN_FORWARD,	/* Forward C boolean ("Boolint") */
   SYMVAL_CONST_BOOLEAN_FORWARD,	/* Same, but can't be set */
+
+  /* The following tags use the 'symbol_value_forward_object' structure, also
+     for variables DEFVARed in C. */
   SYMVAL_OBJECT_FORWARD,	/* Forward C Lisp_Object */
   SYMVAL_CONST_OBJECT_FORWARD,	/* Same, but can't be set */
   SYMVAL_CONST_SPECIFIER_FORWARD, /* Same, can't be set, but gives a
@@ -53,58 +57,86 @@ enum symbol_value_type
   SYMVAL_CONST_SELECTED_CONSOLE_FORWARD, /* Forward Lisp_Object into
 					    Vselected_console,
 					    can't be set */
-  SYMVAL_UNBOUND_MARKER,	/* Only Qunbound actually has this tag */
 
-  /* The following tags use the 'symbol_value_buffer_local' structure */
+  /* The following tags use the 'symbol_value_buffer_local' structure and can
+     be created from Lisp.  */
   SYMVAL_BUFFER_LOCAL,		/* make-variable-buffer-local */
   SYMVAL_SOME_BUFFER_LOCAL,	/* make-local-variable */
 
-  /* The following tag uses the 'symbol_value_lisp_magic' structure */
+  /* The following tag uses the 'symbol_value_lisp_magic' structure, ditto. */
   SYMVAL_LISP_MAGIC,		/* Forward to lisp callbacks */
 
-  /* The following tag uses the 'symbol_value_varalias' structure */
+  /* The following tag uses the 'symbol_value_varalias' structure, ditto. */
   SYMVAL_VARALIAS		/* defvaralias */
-
-#if 0
-  /* NYI */
-  SYMVAL_CONSTANT_SYMBOL,	/* Self-evaluating symbol */
-  /* NYI */
-#endif
 };
 
-/* Underlying C type used to implement DEFVAR_INT */
-typedef EMACS_INT Fixnum;
-
 struct symbol_value_magic
 {
-  struct LCRECORD_HEADER header;
-  void *value;
+  NORMAL_LISP_OBJECT_HEADER header; /* This is a struct old_lcrecord_header,
+				       with a NEXT pointer after the struct
+				       lrecord_header. This is abused in the
+				       implementation of
+				       symbol_value_forward_boolint and
+				       symbol_value_forward_fixnum, which are
+				       c_readonly and never GCed, so there is
+				       no need for a next pointer. */
   enum symbol_value_type type;
 };
 #define SYMBOL_VALUE_MAGIC_P(x)						\
 (LRECORDP (x) &&							\
  XRECORD_LHEADER (x)->type <= lrecord_type_max_symbol_value_magic)
-#define XSYMBOL_VALUE_MAGIC_TYPE(v) \
-	(((struct symbol_value_magic *) XPNTR (v))->type)
+
+#ifdef ERROR_CHECK_TYPES
+DECLARE_INLINE_HEADER (
+struct symbol_value_magic *
+error_check_symbol_value_magic (Lisp_Object obj, const Ascbyte *file,
+				int line)
+)
+{
+  assert_at_line (SYMBOL_VALUE_MAGIC_P (obj), file, line);
+  return (struct symbol_value_magic *) XPNTR (obj);
+}
+
+#define XSYMBOL_VALUE_MAGIC(x) \
+  error_check_symbol_value_magic (x, __FILE__, __LINE__)
+
+DECLARE_INLINE_HEADER (
+Lisp_Object
+wrap_symbol_value_magic_1 (const void *ptr, const Ascbyte *file, int line)
+)
+{
+  Lisp_Object obj = wrap_pointer_1 (ptr);
+
+  assert_at_line (SYMBOL_VALUE_MAGIC_P (obj), file, line);
+  return obj;
+}
+
+#define wrap_symbol_value_magic(x) \
+  wrap_symbol_value_magic_1(x, __FILE__, __LINE__)
+#else
+#define XSYMBOL_VALUE_MAGIC(obj) ((struct symbol_value_magic *) XPNTR (obj))
 #define wrap_symbol_value_magic(p) wrap_pointer_1 (p)
+#endif /* ERROR_CHECK_TYPES */
+
+#define XSYMBOL_VALUE_MAGIC_TYPE(v) (XSYMBOL_VALUE_MAGIC (v)->type)
+
 void print_symbol_value_magic (Lisp_Object, Lisp_Object, int);
 
 /********** The various different symbol-value-magic types ***********/
 
-/* 1. symbol-value-forward */
+/* 1a. symbol-value-forward-object */
 
 /* This type of symbol-value-magic is used for variables declared
-   DEFVAR_LISP, DEFVAR_INT, DEFVAR_BOOL, DEFVAR_BUFFER_LOCAL,
-   DEFVAR_BUFFER_DEFAULTS, DEFVAR_SPECIFIER, and for Qunbound.
+   DEFVAR_LISP, DEFVAR_CONST_LISP, DEFVAR_SPECIFIER.
 
    Note that some of these types of variables can be made buffer-local.
    Then, the symbol's value field contains a symbol-value-buffer-local,
-   whose CURRENT-VALUE field then contains a symbol-value-forward.
- */
+   whose CURRENT-VALUE field then contains a symbol-value-forward. */
 
-struct symbol_value_forward
+struct symbol_value_forward_object
 {
   struct symbol_value_magic magic;
+  Lisp_Object *value;
 
   /* `magicfun' is a function controlling the magic behavior of this
       forward variable.
@@ -141,17 +173,101 @@ struct symbol_value_forward
   int (*magicfun) (Lisp_Object sym, Lisp_Object *val, Lisp_Object in_object,
 		   int flags);
 };
-DECLARE_LRECORD (symbol_value_forward, struct symbol_value_forward);
-#define XSYMBOL_VALUE_FORWARD(x) \
-	XRECORD (x, symbol_value_forward, struct symbol_value_forward)
-#define symbol_value_forward_forward(m) ((void *)((m)->magic.value))
-#define symbol_value_forward_magicfun(m) ((m)->magicfun)
+DECLARE_LISP_OBJECT (symbol_value_forward_object,
+		     struct symbol_value_forward_object);
+#define SYMBOL_VALUE_FORWARD_OBJECTP(x) RECORDP (x, symbol_value_forward_object)
+#define XSYMBOL_VALUE_FORWARD_OBJECT(x) \
+	XRECORD (x, symbol_value_forward_object,\
+		 struct symbol_value_forward_object)
+#define symbol_value_forward_object_forward(m) ((m)->value)
+#define symbol_value_forward_object_magicfun(m) ((m)->magicfun)
+#define XSYMBOL_VALUE_FORWARD_OBJECT_FORWARD(x) \
+  symbol_value_forward_object_forward (XSYMBOL_VALUE_FORWARD_OBJECT (x))
+#define XSYMBOL_VALUE_FORWARD_OBJECT_MAGICFUN(x) \
+  symbol_value_forward_object_magicfun (XSYMBOL_VALUE_FORWARD_OBJECT (x))
+
+/* 1b. symbol-value-forward-fixnum.
+
+   This type of symbol-value-magic is used for variables declared DEFVAR_INT,
+   DEFVAR_CONST_INT and DEFVAR_INT_MAGIC. Commentary above as for
+   symbol_value_forward_object also applies.
+
+   In contrast to symbol_value_forward_object, GC does not have to mark through
+   symbol_value_forward_fixnum. This means we can mark these objects as
+   c_readonly and lisp_readonly, and use what would otherwise be the NEXT
+   pointer of the NORMAL_LISP_OBJECT_HEADER as the pointer to the C variable of
+   interest. */
+
+/* Underlying C type used to implement DEFVAR_INT */
+typedef EMACS_INT Fixnum;
+
+struct symbol_value_forward_fixnum_magic
+{
+  FROB_BLOCK_LISP_OBJECT_HEADER header;
+  Fixnum *value;
+  enum symbol_value_type type;
+};
+
+struct symbol_value_forward_fixnum
+{
+  struct symbol_value_forward_fixnum_magic magic;
+  int (*magicfun) (Lisp_Object sym, Lisp_Object *val, Lisp_Object in_object,
+		   int flags);
+};
+DECLARE_LISP_OBJECT (symbol_value_forward_fixnum,
+		     struct symbol_value_forward_fixnum);
+#define SYMBOL_VALUE_FORWARD_FIXNUMP(x) RECORDP (x, symbol_value_forward_fixnum)
+#define XSYMBOL_VALUE_FORWARD_FIXNUM(x) \
+	XRECORD (x, symbol_value_forward_fixnum,\
+		 struct symbol_value_forward_fixnum)
+#define symbol_value_forward_fixnum_forward(m) ((m)->magic.value)
+#define symbol_value_forward_fixnum_magicfun(m) ((m)->magicfun)
+#define XSYMBOL_VALUE_FORWARD_FIXNUM_FORWARD(x) \
+  symbol_value_forward_fixnum_forward (XSYMBOL_VALUE_FORWARD_FIXNUM (x))
+#define XSYMBOL_VALUE_FORWARD_FIXNUM_MAGICFUN(x) \
+  symbol_value_forward_fixnum_magicfun (XSYMBOL_VALUE_FORWARD_FIXNUM (x))
+
+/* 1c. symbol-value-forward-boolint.
+
+   This type of symbol-value-magic is used for variables declared DEFVAR_BOOL,
+   DEFVAR_CONST_BOOL and DEFVAR_BOOL_MAGIC. Commentary above as for
+   symbol_value_forward_fixnum also applies. */
+
+struct symbol_value_forward_boolint_magic
+{
+  FROB_BLOCK_LISP_OBJECT_HEADER header;
+  Boolint *value;
+  enum symbol_value_type type;
+};
+
+struct symbol_value_forward_boolint
+{
+  struct symbol_value_forward_boolint_magic magic;
+  /* `magicfun' is a function controlling the magic behavior of this
+     forward variable. Details as for symbol_value_forward_object. */
+  int (*magicfun) (Lisp_Object sym, Lisp_Object *val, Lisp_Object in_object,
+		   int flags);
+};
+DECLARE_LISP_OBJECT (symbol_value_forward_boolint,
+		     struct symbol_value_forward_boolint);
+#define SYMBOL_VALUE_FORWARD_BOOLINTP(x) RECORDP (x, symbol_value_forward_boolint)
+#define XSYMBOL_VALUE_FORWARD_BOOLINT(x) \
+	XRECORD (x, symbol_value_forward_boolint,\
+		 struct symbol_value_forward_boolint)
+#define symbol_value_forward_boolint_forward(m) ((m)->magic.value)
+#define symbol_value_forward_boolint_magicfun(m) ((m)->magicfun)
+#define XSYMBOL_VALUE_FORWARD_BOOLINT_FORWARD(x) \
+  symbol_value_forward_boolint_forward (XSYMBOL_VALUE_FORWARD_BOOLINT (x))
+#define XSYMBOL_VALUE_FORWARD_BOOLINT_MAGICFUN(x) \
+  symbol_value_forward_boolint_magicfun(XSYMBOL_VALUE_FORWARD_BOOLINT (x))
 
 /* 2. symbol-value-buffer-local */
 
 struct symbol_value_buffer_local
 {
-  struct symbol_value_magic magic;
+  struct symbol_value_magic magic; /* The NEXT pointer of the
+				      old_lcrecord_header is used here, by
+				      ALLOC_NORMAL_LISP_OBJECT(). */
   /* Used in a symbol value cell when the symbol's value is per-buffer.
 
      The type of the symbol-value-magic will be either
@@ -228,7 +344,7 @@ struct symbol_value_buffer_local
   Lisp_Object current_buffer;
   Lisp_Object current_alist_element;
 };
-DECLARE_LRECORD (symbol_value_buffer_local, struct symbol_value_buffer_local);
+DECLARE_LISP_OBJECT (symbol_value_buffer_local, struct symbol_value_buffer_local);
 #define XSYMBOL_VALUE_BUFFER_LOCAL(x) \
 	XRECORD (x, symbol_value_buffer_local, struct symbol_value_buffer_local)
 #define SYMBOL_VALUE_BUFFER_LOCAL_P(x) RECORDP (x, symbol_value_buffer_local)
@@ -253,7 +369,7 @@ struct symbol_value_lisp_magic
   Lisp_Object harg[MAGIC_HANDLER_MAX];
   Lisp_Object shadowed;
 };
-DECLARE_LRECORD (symbol_value_lisp_magic, struct symbol_value_lisp_magic);
+DECLARE_LISP_OBJECT (symbol_value_lisp_magic, struct symbol_value_lisp_magic);
 #define XSYMBOL_VALUE_LISP_MAGIC(x) \
 	XRECORD (x, symbol_value_lisp_magic, struct symbol_value_lisp_magic)
 #define SYMBOL_VALUE_LISP_MAGIC_P(x) RECORDP (x, symbol_value_lisp_magic)
@@ -266,7 +382,7 @@ struct symbol_value_varalias
   Lisp_Object aliasee;
   Lisp_Object shadowed;
 };
-DECLARE_LRECORD (symbol_value_varalias,	struct symbol_value_varalias);
+DECLARE_LISP_OBJECT (symbol_value_varalias,	struct symbol_value_varalias);
 #define XSYMBOL_VALUE_VARALIAS(x) \
 	XRECORD (x, symbol_value_varalias, struct symbol_value_varalias)
 #define SYMBOL_VALUE_VARALIAS_P(x) RECORDP (x, symbol_value_varalias)
@@ -275,67 +391,26 @@ DECLARE_LRECORD (symbol_value_varalias,	struct symbol_value_varalias);
 
 /* To define a Lisp primitive function using a C function `Fname', do this:
    DEFUN ("name, Fname, ...); // at top level in foo.c
-   DEFSUBR (Fname);           // in syms_of_foo();
-*/
-#ifdef NEW_GC
-MODULE_API void defsubr (Lisp_Subr *);
-#define DEFSUBR_MC_ALLOC(Fname)						\
-  S##Fname= (struct Lisp_Subr *) mc_alloc (sizeof (struct Lisp_Subr));	\
-  set_lheader_implementation (&S##Fname->lheader, &lrecord_subr);	\
-									\
-  S##Fname->min_args = MC_ALLOC_S##Fname.min_args;			\
-  S##Fname->max_args = MC_ALLOC_S##Fname.max_args;			\
-  S##Fname->prompt = MC_ALLOC_S##Fname.prompt;				\
-  S##Fname->doc = MC_ALLOC_S##Fname.doc;				\
-  S##Fname->name = MC_ALLOC_S##Fname.name;				\
-  S##Fname->subr_fn = MC_ALLOC_S##Fname.subr_fn;			\
-  MARK_LRECORD_AS_LISP_READONLY (S##Fname);
-
-
-#define DEFSUBR(Fname)				\
-do {						\
-  DEFSUBR_MC_ALLOC (Fname);			\
-  defsubr (S##Fname);				\
-} while (0)
+   DEFSUBR (Fname);           // in syms_of_foo(); */
+#define DEFSUBR(Fname) (defsubr_##Fname)(0)
 
 /* To define a Lisp primitive macro using a C function `Fname', do this:
    DEFUN ("name, Fname, ...); // at top level in foo.c
    DEFSUBR_MACRO (Fname);     // in syms_of_foo();
 */
-MODULE_API void defsubr_macro (Lisp_Subr *);
-#define DEFSUBR_MACRO(Fname)			\
-do {						\
-  DEFSUBR_MC_ALLOC (Fname);			\
-  defsubr_macro (S##Fname);			\
-} while (0)
-
-#else /* not NEW_GC */
-/* To define a Lisp primitive function using a C function `Fname', do this:
-   DEFUN ("name, Fname, ...); // at top level in foo.c
-   DEFSUBR (Fname);           // in syms_of_foo();
-*/
-MODULE_API void defsubr (Lisp_Subr *);
-#define DEFSUBR(Fname) defsubr (&S##Fname)
-
-/* To define a Lisp primitive macro using a C function `Fname', do this:
-   DEFUN ("name, Fname, ...); // at top level in foo.c
-   DEFSUBR_MACRO (Fname);     // in syms_of_foo();
-*/
-MODULE_API void defsubr_macro (Lisp_Subr *);
-#define DEFSUBR_MACRO(Fname) defsubr_macro (&S##Fname)
-#endif /* not NEW_GC */
+#define DEFSUBR_MACRO(Fname) (defsubr_##Fname) (1)
 
 MODULE_API void defsymbol_massage_name (Lisp_Object *location,
-					const char *name);
+					const Ascbyte *name);
 MODULE_API void defsymbol_massage_name_nodump (Lisp_Object *location,
-					       const char *name);
+					       const Ascbyte *name);
 MODULE_API void defsymbol_massage_multiword_predicate (Lisp_Object *location,
-						       const char *name);
+						       const Ascbyte *name);
 MODULE_API void
 defsymbol_massage_multiword_predicate_nodump (Lisp_Object *location,
-					      const char *name);
-MODULE_API void defsymbol (Lisp_Object *location, const char *name);
-MODULE_API void defsymbol_nodump (Lisp_Object *location, const char *name);
+					      const Ascbyte *name);
+MODULE_API void defsymbol (Lisp_Object *location, const Ascbyte *name);
+MODULE_API void defsymbol_nodump (Lisp_Object *location, const Ascbyte *name);
 
 /* Defining symbols:
 
@@ -370,18 +445,18 @@ MODULE_API void defsymbol_nodump (Lisp_Object *location, const char *name);
 #define DEFSYMBOL_MULTIWORD_PREDICATE_NO_DUMP(name) \
   defsymbol_massage_multiword_predicate_nodump (&name, #name)
 
-MODULE_API void defkeyword (Lisp_Object *location, const char *name);
+MODULE_API void defkeyword (Lisp_Object *location, const Ascbyte *name);
 MODULE_API void defkeyword_massage_name (Lisp_Object *location,
-					 const char *name);
+					 const Ascbyte *name);
 #define DEFKEYWORD(name) defkeyword_massage_name (&name, #name)
 
-MODULE_API void deferror (Lisp_Object *symbol, const char *name,
-			  const char *message, Lisp_Object inherits_from);
-MODULE_API void deferror_massage_name (Lisp_Object *symbol, const char *name,
-				       const char *message,
+MODULE_API void deferror (Lisp_Object *symbol, const Ascbyte *name,
+			  const Ascbyte *message, Lisp_Object inherits_from);
+MODULE_API void deferror_massage_name (Lisp_Object *symbol, const Ascbyte *name,
+				       const Ascbyte *message,
 				       Lisp_Object inherits_from);
 MODULE_API void deferror_massage_name_and_message (Lisp_Object *symbol,
-						   const char *name,
+						   const Ascbyte *name,
 						   Lisp_Object inherits_from);
 #define DEFERROR(name, message, inherits_from) \
   deferror_massage_name (&name, #name, message, inherits_from)
@@ -390,79 +465,63 @@ MODULE_API void deferror_massage_name_and_message (Lisp_Object *symbol,
 #define DEFERROR_STANDARD(name, inherits_from) \
   deferror_massage_name_and_message (&name, #name, inherits_from)
 
-/* Macros we use to define forwarded Lisp variables.
-   These are used in the syms_of_FILENAME functions.  */
+MODULE_API void defvar_magic (const Ascbyte *symbol_name, Lisp_Object magic);
 
-MODULE_API void defvar_magic (const char *symbol_name,
-			      const struct symbol_value_forward *magic);
+#define ASSERT_OK_FOR_MAGIC(forward_type) do {                          \
+    enum symbol_value_type aofmft = (forward_type);                     \
+    structure_checking_assert                                           \
+      (aofmft == SYMVAL_FIXNUM_FORWARD           ||                     \
+       aofmft == SYMVAL_BOOLEAN_FORWARD          ||                     \
+       aofmft == SYMVAL_OBJECT_FORWARD           ||                     \
+       aofmft == SYMVAL_CURRENT_BUFFER_FORWARD   ||                     \
+       aofmft == SYMVAL_SELECTED_CONSOLE_FORWARD ||                     \
+       aofmft == SYMVAL_BUFFER_LOCAL             ||                     \
+       aofmft == SYMVAL_SOME_BUFFER_LOCAL);                             \
+  } while (0)
 
-#ifdef NEW_GC
-#define DEFVAR_SYMVAL_FWD(lname, c_location, forward_type, magic_fun)	\
+#define DEFVAR_SYMVAL_FWD_NON_LISP(lname, c_location, forward_type, magic_fun, subtype) \
 do									\
 {									\
-  struct symbol_value_forward *I_hate_C =				\
-    alloc_lrecord_type (struct symbol_value_forward,			\
-		        &lrecord_symbol_value_forward);			\
-  /*  mcpro ((Lisp_Object) I_hate_C);*/					\
+  struct symbol_value_forward_##subtype *symbol_value_forward =		\
+    XRECORD (ALLOC_C_READONLY_LISP_OBJECT				\
+	     (symbol_value_forward_##subtype),				\
+	     symbol_value_forward_##subtype,				\
+	     struct symbol_value_forward_##subtype);			\
 									\
-  MARK_LRECORD_AS_LISP_READONLY (I_hate_C);				\
-									\
-  I_hate_C->magic.value = c_location;					\
-  I_hate_C->magic.type = forward_type;					\
-  I_hate_C->magicfun = magic_fun;					\
-									\
-  defvar_magic ((lname), I_hate_C);					\
-} while (0)
-#else /* not NEW_GC */
-#define DEFVAR_SYMVAL_FWD(lname, c_location, forward_type, magicfun)	\
-do									\
-{									\
-  static const struct symbol_value_forward I_hate_C =			\
-  { /* struct symbol_value_forward */					\
-    { /* struct symbol_value_magic */					\
-      { /* struct old_lcrecord_header */				\
-	{ /* struct lrecord_header */					\
-	  lrecord_type_symbol_value_forward, /* lrecord_type_index */	\
-	  1, /* mark bit */						\
-	  1, /* c_readonly bit */					\
-	  1, /* lisp_readonly bit */					\
-          0  /* unused */                                               \
-	},								\
-	0, /* next */							\
-	0, /* uid  */							\
-	0  /* free */							\
-      },								\
-      c_location,							\
-      forward_type							\
-    },									\
-    magicfun								\
-  };									\
-  defvar_magic ((lname), &I_hate_C);					\
-} while (0)
-#endif /* not NEW_GC */
-#define DEFVAR_SYMVAL_FWD_INT(lname, c_location, forward_type, magicfun) \
-do									 \
-{									 \
-  DEFVAR_SYMVAL_FWD (lname, c_location, forward_type, magicfun);	 \
-  dump_add_opaque_int (c_location);					 \
+  symbol_value_forward->magic.type = forward_type;			\
+  symbol_value_forward->magic.value = c_location;			\
+  symbol_value_forward->magicfun = magic_fun;				\
+  if (symbol_value_forward->magicfun)					\
+    {									\
+      ASSERT_OK_FOR_MAGIC (symbol_value_forward->magic.type);		\
+    }									\
+      									\
+  defvar_magic (lname, wrap_symbol_value_magic (symbol_value_forward));	\
 } while (0)
 
 #define DEFVAR_SYMVAL_FWD_FIXNUM(lname, c_location, forward_type, magicfun) \
-do									    \
-{									    \
-  DEFVAR_SYMVAL_FWD (lname, c_location, forward_type, magicfun);	    \
-  dump_add_opaque_fixnum (c_location);					    \
-} while (0)
+DEFVAR_SYMVAL_FWD_NON_LISP(lname, c_location, forward_type, magicfun, fixnum)
 
-#define DEFVAR_SYMVAL_FWD_OBJECT(lname, c_location, forward_type, magicfun) \
-do									    \
-{									    \
-  DEFVAR_SYMVAL_FWD (lname, c_location, forward_type, magicfun);	    \
-  {									    \
-    Lisp_Object *DSF_location = c_location; /* Type check */		    \
-    staticpro (DSF_location);						    \
-    if (EQ (*DSF_location, Qnull_pointer)) *DSF_location = Qnil;	    \
-  }									    \
+#define DEFVAR_SYMVAL_FWD_BOOL(lname, c_location, forward_type, magicfun) \
+DEFVAR_SYMVAL_FWD_NON_LISP(lname, c_location, forward_type, magicfun, boolint)
+
+#define DEFVAR_SYMVAL_FWD_OBJECT(lname, c_location, forward_type, magic_fun) \
+do									\
+{									\
+  struct symbol_value_forward_object *symbol_value_forward =		\
+    XSYMBOL_VALUE_FORWARD_OBJECT (ALLOC_NORMAL_LISP_OBJECT		\
+				  (symbol_value_forward_object));	\
+  /* Don't set this as readonly, otherwise GC of *VALUE */		\
+  /* doesn't happen. */							\
+  symbol_value_forward->magic.type = forward_type;			\
+  symbol_value_forward->value = c_location;				\
+  symbol_value_forward->magicfun = magic_fun;				\
+  if (symbol_value_forward->magicfun)					\
+    {									\
+      ASSERT_OK_FOR_MAGIC (symbol_value_forward->magic.type);		\
+    }									\
+									\
+  defvar_magic (lname, wrap_symbol_value_magic (symbol_value_forward)); \
 } while (0)
 
 #define DEFVAR_LISP(lname, c_location) \
@@ -471,23 +530,102 @@ do									    \
 	DEFVAR_SYMVAL_FWD_OBJECT (lname, c_location, SYMVAL_CONST_OBJECT_FORWARD, 0)
 #define DEFVAR_SPECIFIER(lname, c_location) \
 	DEFVAR_SYMVAL_FWD_OBJECT (lname, c_location, SYMVAL_CONST_SPECIFIER_FORWARD, 0)
+#define DEFVAR_LISP_MAGIC(lname, c_location, magicfun) \
+	DEFVAR_SYMVAL_FWD_OBJECT (lname, c_location, SYMVAL_OBJECT_FORWARD, magicfun)
+
 #define DEFVAR_INT(lname, c_location) \
 	DEFVAR_SYMVAL_FWD_FIXNUM (lname, c_location, SYMVAL_FIXNUM_FORWARD, 0)
 #define DEFVAR_CONST_INT(lname, c_location) \
 	DEFVAR_SYMVAL_FWD_FIXNUM (lname, c_location, SYMVAL_CONST_FIXNUM_FORWARD, 0)
-#define DEFVAR_BOOL(lname, c_location) \
-	DEFVAR_SYMVAL_FWD_INT (lname, c_location, SYMVAL_BOOLEAN_FORWARD, 0)
-#define DEFVAR_CONST_BOOL(lname, c_location) \
-	DEFVAR_SYMVAL_FWD_INT (lname, c_location, SYMVAL_CONST_BOOLEAN_FORWARD, 0)
-#define DEFVAR_LISP_MAGIC(lname, c_location, magicfun) \
-	DEFVAR_SYMVAL_FWD_OBJECT (lname, c_location, SYMVAL_OBJECT_FORWARD, magicfun)
 #define DEFVAR_INT_MAGIC(lname, c_location, magicfun) \
 	DEFVAR_SYMVAL_FWD_FIXNUM (lname, c_location, SYMVAL_FIXNUM_FORWARD, magicfun)
+
+#define DEFVAR_BOOL(lname, c_location) \
+	DEFVAR_SYMVAL_FWD_BOOL (lname, c_location, SYMVAL_BOOLEAN_FORWARD, 0)
+#define DEFVAR_CONST_BOOL(lname, c_location) \
+	DEFVAR_SYMVAL_FWD_BOOL (lname, c_location, SYMVAL_CONST_BOOLEAN_FORWARD, 0)
 #define DEFVAR_BOOL_MAGIC(lname, c_location, magicfun) \
-	DEFVAR_SYMVAL_FWD_INT (lname, c_location, SYMVAL_BOOLEAN_FORWARD, magicfun)
+	DEFVAR_SYMVAL_FWD_BOOL (lname, c_location, SYMVAL_BOOLEAN_FORWARD, magicfun)
 
 void flush_all_buffer_local_cache (void);
 
+struct multiple_value {
+  NORMAL_LISP_OBJECT_HEADER header;
+  Elemcount count;
+  Elemcount allocated_count; 
+  Elemcount first_desired;
+  Lisp_Object contents[1];
+};
+typedef struct multiple_value multiple_value;
+
+DECLARE_LISP_OBJECT (multiple_value, multiple_value);
+#define MULTIPLE_VALUEP(x) RECORDP (x, multiple_value)
+
+#define XMULTIPLE_VALUE(x) XRECORD (x, multiple_value, multiple_value)
+#define wrap_multiple_value(p) wrap_record (p, multiple_value)
+
+#define CHECK_MULTIPLE_VALUE(x) CHECK_RECORD (x, multiple_value)
+#define CONCHECK_MULTIPLE_VALUE(x) CONCHECK_RECORD (x, multiple_value)
+
+#define multiple_value_count(x) ((x)->count)
+#define multiple_value_allocated_count(x) ((x)->allocated_count)
+#define multiple_value_first_desired(x) ((x)->first_desired)
+#define multiple_value_contents(x) ((x)->contents)
+
+#define XMULTIPLE_VALUE_COUNT(x) multiple_value_count (XMULTIPLE_VALUE (x))
+#define XMULTIPLE_VALUE_ALLOCATED_COUNT(x) \
+  multiple_value_allocated_count (XMULTIPLE_VALUE (x))
+#define XMULTIPLE_VALUE_FIRST_DESIRED(x) \
+  multiple_value_first_desired (XMULTIPLE_VALUE(x))
+#define XMULTIPLE_VALUE_CONTENTS(x) multiple_value_contents (XMULTIPLE_VALUE(x))
+
+Lisp_Object multiple_value_call (int nargs, Lisp_Object *args);
+Lisp_Object multiple_value_list_internal (int nargs, Lisp_Object *args);
+
+/* It's slightly ugly to expose this here, but it does cut down the amount
+   of work the bytecode interpreter has to do substantially. */
+extern int multiple_value_current_limit;
+
+/* Bind the multiple value limits that #'values and #'values-list pay
+   attention to. Used by bytecode and interpreted code. */
+int bind_multiple_value_limits (int first, int upper);
+
+Lisp_Object multiple_value_aref (Lisp_Object, Elemcount);
+void multiple_value_aset (Lisp_Object, Elemcount, Lisp_Object);
+
+Lisp_Object values2 (Lisp_Object first, Lisp_Object second);
+
+DECLARE_INLINE_HEADER (
+Lisp_Object 
+ignore_multiple_values (Lisp_Object obj)
+)
+{
+  return MULTIPLE_VALUEP (obj) ? multiple_value_aref (obj, 0) : obj;
+}
+
+#ifdef ERROR_CHECK_MULTIPLE_VALUES
+
+DECLARE_INLINE_HEADER (
+Lisp_Object
+ignore_multiple_values_1 (Lisp_Object obj)
+)
+{
+  if (1 == multiple_value_current_limit)
+    {
+      assert (!MULTIPLE_VALUEP (obj));
+      return obj;
+    }
+
+  return ignore_multiple_values (obj);
+}
+
+#define IGNORE_MULTIPLE_VALUES(X) ignore_multiple_values_1 (X)
+
+#else 
+#define IGNORE_MULTIPLE_VALUES(X) (multiple_value_current_limit == 1 ? (X) \
+                                   : ignore_multiple_values (X))
+#endif
+
 END_C_DECLS
 
 #endif /* INCLUDED_symeval_h_ */
