@@ -33,13 +33,13 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 #define ST_STRING_STYLE  0x102
 
 Lisp_Object Qsyntax_table;
-int lookup_syntax_properties;
+Boolint lookup_syntax_properties;
 
 Lisp_Object Qsyntax_table_p;
 
-int words_include_escapes;
+Boolint words_include_escapes;
 
-int parse_sexp_ignore_comments;
+Boolint parse_sexp_ignore_comments;
 
 /* Tell the regex routines not to QUIT.  Normally there is a QUIT each
    iteration in re_search_2().  We do it this way rather than change
@@ -383,9 +383,10 @@ init_syntax_cache (struct syntax_cache *cache,  /* xzero'ed memory */
    be called repeatedly on the same cache.  The last three are for internal
    use by the syntax setup code and buffer initialization. */
 
-struct syntax_cache *		/* return CACHE or the cache of OBJECT */
-setup_syntax_cache (struct syntax_cache *cache,	/* may be NULL only if
-						   OBJECT is a buffer */
+struct syntax_cache *		/* Return CACHE. */
+setup_syntax_cache (struct syntax_cache *cache,	/* Never NULL; if OBJECT is a
+                                                   buffer, always that
+                                                   buffer's syntax cache. */
 		    Lisp_Object object,	 	/* the object (if any) cache
 						   is associated with */
 		    struct buffer *buffer,	/* the buffer to use as source
@@ -398,7 +399,7 @@ setup_syntax_cache (struct syntax_cache *cache,	/* may be NULL only if
      else make it valid for the whole object. */
   if (BUFFERP (object))
     {
-      cache = XBUFFER (object)->syntax_cache;
+      structure_checking_assert (cache == XBUFFER (object)->syntax_cache);
       if (!lookup_syntax_properties)
 	reset_syntax_cache_range (cache, 1);
     }
@@ -435,7 +436,8 @@ setup_syntax_cache (struct syntax_cache *cache,	/* may be NULL only if
 struct syntax_cache *
 setup_buffer_syntax_cache (struct buffer *buffer, Bytebpos from, int count)
 {
-  return setup_syntax_cache (NULL, wrap_buffer (buffer), buffer, from, count);
+  return setup_syntax_cache (buffer->syntax_cache, wrap_buffer (buffer),
+                             buffer, from, count);
 }
 
 /* 
@@ -547,26 +549,8 @@ update_syntax_cache (struct syntax_cache *cache, Bytexpos pos,
     }
 }
 
-/* buffer-specific APIs used in buffer.c
+/* buffer-specific API used in buffer.c
    #### Really unclean; the syntax cache should just be a LISP object. */
-
-void
-mark_buffer_syntax_cache (struct buffer *buf)
-{
-  struct syntax_cache *cache = buf->syntax_cache;
-  if (!cache) /* Vbuffer_defaults and such don't have caches */
-    return;
-  mark_object (cache->object);
-  if (cache->buffer)
-    mark_object (wrap_buffer (cache->buffer));
-  mark_object (cache->syntax_table);
-#ifdef MIRROR_TABLE
-  mark_object (cache->mirror_table);
-#endif /* MIRROR_TABLE */
-  mark_object (cache->start);
-  mark_object (cache->end);
-}
-
 void
 init_buffer_syntax_cache (struct buffer *buf)
 {
@@ -674,7 +658,10 @@ The chars are indexed by their internal syntax codes, starting at 0.
 
 DEFUN ("char-syntax", Fchar_syntax, 1, 2, 0, /*
 Return the syntax code of CHARACTER, designated by a character.
-Optional SYNTAX-TABLE defaults to the current buffer's syntax table.
+
+SYNTAX-TABLE, if non-nil, is a syntax-table which will be used to look up the
+CHARACTER's syntax, or a cons, the raw syntax descriptor to be used (in which
+case CHARACTER is ignored).  It defaults to the current buffer's syntax table.
 See `modify-syntax-entry' for the designators of the defined syntax codes.
 */
        (character, syntax_table))
@@ -683,17 +670,29 @@ See `modify-syntax-entry' for the designators of the defined syntax codes.
     {
       character = make_char ('\000');
     }
-  CHECK_CHAR_COERCE_INT (character);
-  syntax_table = check_syntax_table (syntax_table,
-				     current_buffer->syntax_table);
+  if (!CONSP (syntax_table))
+    {
+      CHECK_CHAR_COERCE_INT (character);
+      syntax_table = check_syntax_table (syntax_table,
+                                         current_buffer->syntax_table);
 #ifdef MIRROR_TABLE
-  return make_char
-    (syntax_code_spec[(int) SYNTAX (XCHAR_TABLE (syntax_table)->mirror_table,
-				    XCHAR (character))]);
+      return make_char
+        (syntax_code_spec[(int) SYNTAX (XCHAR_TABLE (syntax_table)->mirror_table,
+                                        XCHAR (character))]);
 #else
-  return make_char (syntax_code_spec[(int) SYNTAX (syntax_table,
-						   XCHAR (character))]);
+      return make_char (syntax_code_spec[(int) SYNTAX (syntax_table,
+                                                       XCHAR (character))]);
 #endif /* MIRROR_TABLE */
+    }
+  else
+    {
+      enum syntaxcode int_code = SYNTAX_FROM_CODE (XFIXNUM (XCAR
+                                                            (syntax_table)));
+
+      if (int_code >= Smax)
+        signal_error (Qargs_out_of_range, "Invalid syntax code", syntax_table);
+      return make_char (syntax_code_spec[(int) int_code]);
+    }
 }
 
 Lisp_Object
@@ -733,7 +732,6 @@ Optional SYNTAX-TABLE defaults to the current buffer's syntax table.
 
 
 
-#ifdef MULE
 /* Return 1 if there is a word boundary between two word-constituent
    characters C1 and C2 if they appear in this order, else return 0.
    There is no word boundary between two word-constituent ASCII
@@ -741,9 +739,6 @@ Optional SYNTAX-TABLE defaults to the current buffer's syntax table.
 #define WORD_BOUNDARY_P(buf, c1, c2)		\
   (!(ichar_ascii_p (c1) && ichar_ascii_p (c2))	\
    && word_boundary_p (buf, c1, c2))
-#else
-#define WORD_BOUNDARY_P(buf, c1, c2) 0
-#endif
 
 Charbpos
 scan_words (struct buffer *buf, Charbpos from, EMACS_INT count)
@@ -900,7 +895,7 @@ the documentation for this variable for more details.
 static void scan_sexps_forward (struct buffer *buf,
 				struct lisp_parse_state *,
 				Bytebpos from, Bytebpos end,
-				int targetdepth, int stopbefore,
+				EMACS_INT targetdepth, int stopbefore,
 				Lisp_Object oldstate,
 				int commentstop);
 
@@ -1093,7 +1088,10 @@ find_start_of_comment (struct buffer *buf, Bytebpos from, Bytebpos stop,
       else
 	/* We can't grok this as a comment; scan it normally.  */
 	from = comment_end;
-      UPDATE_SYNTAX_CACHE_FORWARD (scache, prev_bytebpos (buf, from));
+      if (from > 1)
+        {
+          UPDATE_SYNTAX_CACHE_FORWARD (scache, prev_bytebpos (buf, from));
+        }
     }
   return from;
 }
@@ -1919,7 +1917,7 @@ Optional BUFFER defaults to the current buffer.
 static void
 scan_sexps_forward (struct buffer *buf, struct lisp_parse_state *stateptr,
 		    Bytebpos from, Bytebpos end,
-		    int targetdepth, int stopbefore,
+		    EMACS_INT targetdepth, int stopbefore,
 		    Lisp_Object oldstate,
 		    int commentstop)
 {
