@@ -1,7 +1,7 @@
 /* toolbar implementation -- "Generic" (X or GTK) redisplay interface.
    Copyright (C) 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 1995, 1996, 2002, 2010 Ben Wing.
+   Copyright (C) 1995, 1996, 2002, 2005, 2010 Ben Wing.
    Copyright (C) 1996 Chuck Thompson.
 
 This file is part of XEmacs.
@@ -52,7 +52,7 @@ along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 */
 #if defined(HAVE_GTK)
 #include "console-gtk-impl.h"
-#define __INTERNAL_MAPPED_P(f) GTK_WIDGET_REALIZED (FRAME_GTK_TEXT_WIDGET (f))
+#define __INTERNAL_MAPPED_P(f) gtk_widget_get_realized (FRAME_GTK_TEXT_WIDGET (f))
 #define __INTERNAL_FLUSH(d) gdk_flush()
 #define __INTERNAL_APPROPRIATENESS_CHECK(f) assert(FRAME_GTK_P (f))
 #elif defined(HAVE_X_WINDOWS)
@@ -273,13 +273,9 @@ xlike_output_toolbar_button (struct frame *f, Lisp_Object button)
 	}
       else if (IMAGE_INSTANCE_TYPE (p) == IMAGE_TEXT)
 	{
-	  /* #### We need to make the face used configurable. */
-	  struct face_cachel *cachel =
-	    WINDOW_FACE_CACHEL (w, DEFAULT_INDEX);
+	  struct face_cachel *cachel = WINDOW_FACE_CACHEL (w, button_findex);
 	  struct display_line dl;
 	  Lisp_Object string = IMAGE_INSTANCE_TEXT_STRING (p);
-	  unsigned char charsets[NUM_LEADING_BYTES];
-	  Ichar_dynarr *buf;
 	  struct font_metric_info fm;
 
 	  /* This could be true if we were called via the Expose event
@@ -291,13 +287,10 @@ xlike_output_toolbar_button (struct frame *f, Lisp_Object button)
 	      MARK_TOOLBAR_CHANGED;
 	      return;
 	    }
-	  buf = Dynarr_new (Ichar);
-	  convert_ibyte_string_into_ichar_dynarr
-	    (XSTRING_DATA (string), XSTRING_LENGTH (string), buf);
-	  find_charsets_in_ichar_string (charsets, Dynarr_begin (buf),
-					  Dynarr_length (buf));
-	  ensure_face_cachel_complete (cachel, window, charsets);
-	  face_cachel_charset_font_metric_info (cachel, charsets, &fm);
+
+	  face_cachel_char_font_metric_info (cachel, window,
+                                             XSTRING_DATA (string),
+                                             XSTRING_LENGTH (string), &fm);
 
 	  dl.ascent = fm.ascent;
 	  dl.descent = fm.descent;
@@ -314,9 +307,9 @@ xlike_output_toolbar_button (struct frame *f, Lisp_Object button)
 	    }
 
 	  MAYBE_DEVMETH (d, output_string,
-			 (w, &dl, buf, tb->x + x_offset, 0, 0, width,
-			  button_findex, 0, 0, 0, 0));
-	  Dynarr_free (buf);
+			 (w, &dl, XSTRING_DATA (string),
+                          XSTRING_LENGTH (string), tb->x + x_offset, 0, 0,
+                          width, button_findex, 0, 0, 0, 0));
 	}
 
       /* We silently ignore the image if it isn't a pixmap or text. */
