@@ -95,18 +95,6 @@ allocate_toolbar_button (struct frame *f, int pushright)
   return wrap_toolbar_button (tb);
 }
 
-DEFINE_NODUMP_INTERNAL_LISP_OBJECT ("toolbar-button", toolbar_button,
-				    toolbar_button_description,
-				    struct toolbar_button);
-
-DEFUN ("toolbar-button-p", Ftoolbar_button_p, 1, 1, 0, /*
-Return non-nil if OBJECT is a toolbar button.
-*/
-       (object))
-{
-  return TOOLBAR_BUTTONP (object) ? Qt : Qnil;
-}
-
 /* Only query functions are provided for toolbar buttons.  They are
    generated and updated from a toolbar description list.  Any
    directly made changes would be wiped out the first time the toolbar
@@ -1192,17 +1180,6 @@ toolbar_after_change (Lisp_Object UNUSED (specifier),
   MARK_TOOLBAR_CHANGED;
 }
 
-DEFUN ("toolbar-specifier-p", Ftoolbar_specifier_p, 1, 1, 0, /*
-Return non-nil if OBJECT is a toolbar specifier.
-
-See `make-toolbar-specifier' for a description of possible toolbar
-instantiators.
-*/
-       (object))
-{
-  return TOOLBAR_SPECIFIERP (object) ? Qt : Qnil;
-}
-
 
 /*
   Helper for invalidating the real specifier when default
@@ -1325,7 +1302,9 @@ toolbar_shadows_changed (Lisp_Object UNUSED (specifier),
 void
 syms_of_toolbar (void)
 {
-  INIT_LISP_OBJECT (toolbar_button);
+  DEFINE_NODUMP_INTERNAL_LISP_OBJECT ("toolbar-button", toolbar_button,
+                                      toolbar_button_description,
+                                      struct toolbar_button);
 
   DEFSYMBOL_MULTIWORD_PREDICATE (Qtoolbar_buttonp);
   DEFSYMBOL (Q2D);
@@ -1335,7 +1314,6 @@ syms_of_toolbar (void)
   DEFKEYWORD (Q_size);
 
   DEFSYMBOL (Qinit_toolbar_from_resources);
-  DEFSUBR (Ftoolbar_button_p);
   DEFSUBR (Ftoolbar_button_callback);
   DEFSUBR (Ftoolbar_button_help_string);
   DEFSUBR (Ftoolbar_button_enabled_p);
@@ -1343,7 +1321,6 @@ syms_of_toolbar (void)
   DEFSUBR (Fcheck_toolbar_button_syntax);
   DEFSUBR (Fset_default_toolbar_position);
   DEFSUBR (Fdefault_toolbar_position);
-  DEFSUBR (Ftoolbar_specifier_p);
 }
 
 void
@@ -1366,12 +1343,6 @@ specifier_type_create_toolbar (void)
   SPECIFIER_HAS_METHOD (toolbar, after_change);
 }
 
-void
-reinit_specifier_type_create_toolbar (void)
-{
-  REINITIALIZE_SPECIFIER_TYPE (toolbar);
-}
-
 void
 specifier_vars_of_toolbar (void)
 {
