@@ -169,10 +169,6 @@ print_tooltalk_message (Lisp_Object obj, Lisp_Object printcharfun,
 		    (EMACS_INT) (p->m), LISP_OBJECT_UID (obj));
 }
 
-DEFINE_NODUMP_LISP_OBJECT ("tooltalk-message", tooltalk_message,
-			   print_tooltalk_message, 0, 0, 0, 
-			   tooltalk_message_description,
-			   Lisp_Tooltalk_Message);
 
 static Lisp_Object
 make_tooltalk_message (Tt_message m)
@@ -236,11 +232,6 @@ print_tooltalk_pattern (Lisp_Object obj, Lisp_Object printcharfun,
 		    (EMACS_INT) (p->p), LISP_OBJECT_UID (obj));
 }
 
-DEFINE_NODUMP_LISP_OBJECT ("tooltalk-pattern", tooltalk_pattern,
-			   print_tooltalk_pattern, 0, 0, 0, 
-			   tooltalk_pattern_description,
-			   Lisp_Tooltalk_Pattern);
-
 static Lisp_Object
 make_tooltalk_pattern (Tt_pattern p)
 {
@@ -1205,8 +1196,6 @@ init_tooltalk (void)
   /* This function can GC */
   char *retval;
   Lisp_Object lp;
-  Lisp_Object fil;
-
 
   /* tt_open() messes with our signal handler flags (at least when no
      ttsessions is running on the machine), therefore we save the
@@ -1242,8 +1231,7 @@ init_tooltalk (void)
     {
       /* Don't ask the user for confirmation when exiting Emacs */
       Fprocess_kill_without_query (lp, Qnil);
-      fil = GET_DEFUN_LISP_OBJECT (Freceive_tooltalk_message);
-      set_process_filter (lp, fil, 1, 0);
+      set_process_filter (lp, Qreceive_tooltalk_message, 1, 0);
     }
   else
     {
@@ -1286,8 +1274,15 @@ Returns t if successful, nil otherwise.
 void
 syms_of_tooltalk (void)
 {
-  INIT_LISP_OBJECT (tooltalk_message);
-  INIT_LISP_OBJECT (tooltalk_pattern);
+  DEFINE_NODUMP_LISP_OBJECT ("tooltalk-message", tooltalk_message,
+                             print_tooltalk_message, 0, 0, 0, 
+                             tooltalk_message_description,
+                             Lisp_Tooltalk_Message);
+
+  DEFINE_NODUMP_LISP_OBJECT ("tooltalk-pattern", tooltalk_pattern,
+                             print_tooltalk_pattern, 0, 0, 0, 
+                             tooltalk_pattern_description,
+                             Lisp_Tooltalk_Pattern);
 
   DEFSYMBOL_MULTIWORD_PREDICATE (Qtooltalk_messagep);
   DEFSUBR (Ftooltalk_message_p);
