@@ -534,12 +534,6 @@ syms_of_undo (void)
   DEFSYMBOL (Qinhibit_read_only);
 }
 
-void
-reinit_vars_of_undo (void)
-{
-  inside_undo = 0;
-}
-
 void
 vars_of_undo (void)
 {
