@@ -646,9 +646,12 @@
 (Assert (equal (format "%hx" #x12345) "2345")) ;; field size no longer ignored
 (Assert (equal (format "%hhx" #x123) "23")) ;; Truncate to 8 bits
 (Assert (equal (format "%lx" #x123) "123")) ;; To 32
+(Assert (equal (format "%zx" #x123) "123")) ;; To 32
 (when (featurep 'bignum)
   (Assert (equal (format "%lx" (read "#x123456789a")
 			 "3456789a")) ;; Really, to 32
+  (Assert (equal (format "%zx" (read "#x123456789a")
+			 "3456789a")) ;; Really, to 32
   (macrolet
       ((very-positive-bignum () (1- (expt 2 96)))
        (very-negative-bignum () (- (1- (expt 2 96)))))
@@ -659,60 +662,71 @@
     (Assert (equal (format "%hhx" (very-positive-bignum)) "-1"))
     (Assert (equal (format "%hx" (very-positive-bignum))  "-1"))
     (Assert (equal (format "%lx" (very-positive-bignum))  "-1"))
+    (Assert (equal (format "%zx" (very-positive-bignum))  "-1"))
     (Assert (equal (format "%llx" (very-positive-bignum)) "-1"))
     (Assert (equal (format "%x" (very-positive-bignum))
                    "ffffffffffffffffffffffff"))
     (Assert (equal (format "%hhux" (very-negative-bignum)) "ff"))
     (Assert (equal (format "%hux" (very-negative-bignum))  "ffff"))
     (Assert (equal (format "%lux" (very-negative-bignum))  "ffffffff"))
+    (Assert (equal (format "%zux" (very-negative-bignum))  "ffffffff"))
     (Assert (equal (format "%llux" (very-negative-bignum)) "ffffffffffffffff"))
     (Assert (equal (format "%hhx" (very-negative-bignum)) "-1"))
     (Assert (equal (format "%hx" (very-negative-bignum))  "-1"))
     (Assert (equal (format "%lx" (very-negative-bignum))  "-1"))
+    (Assert (equal (format "%zx" (very-negative-bignum))  "-1"))
     (Assert (equal (format "%llx" (very-negative-bignum)) "-1"))
     (Assert (equal (format "%x" (very-negative-bignum))
                    "-ffffffffffffffffffffffff"))
     (Assert (equal (format "%hhud" (very-positive-bignum)) "255"))
     (Assert (equal (format "%hud" (very-positive-bignum)) "65535"))
     (Assert (equal (format "%lud" (very-positive-bignum)) "4294967295"))
+    (Assert (equal (format "%zud" (very-positive-bignum)) "4294967295"))    
     (Assert (equal (format "%llud" (very-positive-bignum))
                    "18446744073709551615"))
     (Assert (equal (format "%hhd" (very-positive-bignum)) "-1"))
     (Assert (equal (format "%hd" (very-positive-bignum)) "-1"))
     (Assert (equal (format "%ld" (very-positive-bignum)) "-1"))
+    (Assert (equal (format "%zd" (very-positive-bignum)) "-1"))
     (Assert (equal (format "%lld" (very-positive-bignum)) "-1"))
     (Assert (equal (format "%d" (very-positive-bignum))
                    "79228162514264337593543950335"))
     (Assert (equal (format "%hhud" (very-negative-bignum)) "255"))
     (Assert (equal (format "%hud" (very-negative-bignum)) "65535"))
     (Assert (equal (format "%lud" (very-negative-bignum)) "4294967295"))
+    (Assert (equal (format "%zud" (very-negative-bignum)) "4294967295"))
     (Assert (equal (format "%llud" (very-negative-bignum))
                    "18446744073709551615"))
     (Assert (equal (format "%hhd" (very-negative-bignum)) "-1"))
     (Assert (equal (format "%hd" (very-negative-bignum)) "-1"))
     (Assert (equal (format "%ld" (very-negative-bignum)) "-1"))
+    (Assert (equal (format "%zd" (very-negative-bignum)) "-1"))
     (Assert (equal (format "%lld" (very-negative-bignum)) "-1"))
     (Assert (equal (format "%d" (very-negative-bignum))
                    "-79228162514264337593543950335"))
     (Assert (equal (format "%hhuo" (very-positive-bignum)) "377"))
     (Assert (equal (format "%huo" (very-positive-bignum))  "177777"))
     (Assert (equal (format "%luo" (very-positive-bignum))  "37777777777"))
+    (Assert (equal (format "%zuo" (very-positive-bignum))  "37777777777"))
     (Assert (equal (format "%lluo" (very-positive-bignum))
 		   "1777777777777777777777"))
     (Assert (equal (format "%hho" (very-positive-bignum))  "-1"))
     (Assert (equal (format "%ho" (very-positive-bignum))   "-1"))
     (Assert (equal (format "%lo" (very-positive-bignum))   "-1"))
+    (Assert (equal (format "%zo" (very-positive-bignum))   "-1"))
     (Assert (equal (format "%llo" (very-positive-bignum))  "-1"))
     (Assert (equal (format "%o" (very-positive-bignum))
 		   "77777777777777777777777777777777"))
     (Assert (equal (format "%hhuo" (very-negative-bignum)) "377"))
     (Assert (equal (format "%huo" (very-negative-bignum))  "177777"))
     (Assert (equal (format "%luo" (very-negative-bignum))  "37777777777"))
+    (Assert (equal (format "%zuo" (very-negative-bignum))  "37777777777"))
     (Assert (equal (format "%lluo" (very-negative-bignum))
 		   "1777777777777777777777"))
     (Assert (equal (format "%hho" (very-negative-bignum))  "-1"))
     (Assert (equal (format "%ho" (very-negative-bignum))   "-1"))
     (Assert (equal (format "%lo" (very-negative-bignum))   "-1"))
+    (Assert (equal (format "%zo" (very-negative-bignum))  "37777777777"))
     (Assert (equal (format "%llo" (very-negative-bignum))  "-1"))
     (Assert (equal (format "%o" (very-negative-bignum))
 		   "-77777777777777777777777777777777"))
@@ -721,6 +735,8 @@
                    "1111111111111111"))
     (Assert (equal (format "%lub" (very-positive-bignum))
                    "11111111111111111111111111111111"))
+    (Assert (equal (format "%zub" (very-positive-bignum))
+                   "11111111111111111111111111111111"))
     (Assert
      (equal
       (format "%llub" (very-positive-bignum))
@@ -728,6 +744,7 @@
     (Assert (equal (format "%hhb" (very-positive-bignum))  "-1"))
     (Assert (equal (format "%hb" (very-positive-bignum))   "-1"))
     (Assert (equal (format "%lb" (very-positive-bignum))   "-1"))
+    (Assert (equal (format "%zb" (very-positive-bignum))   "-1"))
     (Assert (equal (format "%llb" (very-positive-bignum))  "-1"))
     (Assert
      (equal
@@ -738,12 +755,15 @@
                    "1111111111111111"))
     (Assert (equal (format "%lub" (very-negative-bignum))
                    "11111111111111111111111111111111"))
+    (Assert (equal (format "%zub" (very-negative-bignum))
+                   "11111111111111111111111111111111"))
     (Assert
      (equal (format "%llub" (very-negative-bignum))
             "1111111111111111111111111111111111111111111111111111111111111111"))
     (Assert (equal (format "%hhb" (very-negative-bignum))  "-1"))
     (Assert (equal (format "%hb" (very-negative-bignum))   "-1"))
     (Assert (equal (format "%lb" (very-negative-bignum))   "-1"))
+    (Assert (equal (format "%zb" (very-negative-bignum))   "-1"))
     (Assert (equal (format "%llb" (very-negative-bignum))  "-1"))
     (Assert
      (equal
