@@ -335,6 +335,43 @@ with control-1"
                      (getf (string-char-byte-conversion-info latin1)
                            'ascii-begin))))))
 
+  (let ((big-string (make-string 8200 ?a))
+        (medium-string
+         (make-string 8192 (make-char 'japanese-jisx0213-2 126 126)))
+        (small-string (make-string 1 ?a)))
+    ;; Test resize_string ().
+    ;; 1. Big string resized to big string, growing.
+    (fill big-string (make-char 'japanese-jisx0213-2 126 126))
+    (garbage-collect)
+    (Assert (eq (aref big-string 0) (make-char 'japanese-jisx0213-2 126 126)))
+    ;; 2. Big string resized to big string, shrinking.
+    (fill big-string ?z)
+    (garbage-collect)
+    (Assert (eq (aref big-string 0) ?z))
+    ;; 3. Big string resized to small string (has to be shrinking).
+    (fill medium-string ?a)
+    (garbage-collect)
+    (Assert (eq (aref medium-string 0) ?a))
+    ;; 4. Small string changes size but allocation size does not (given
+    ;; alignment constraints).
+    (aset small-string 0 (make-char 'latin-iso8859-1 127))
+    (garbage-collect)
+    (Assert (eq (aref small-string 0) (make-char 'latin-iso8859-1 127)))
+
+    ;; 5. Small string resized to big string (medium string is now a small
+    ;; string).
+    (fill medium-string (make-char 'japanese-jisx0213-2 126 126))
+    (garbage-collect)
+    (Assert (eq (aref medium-string 0)
+                (make-char 'japanese-jisx0213-2 126 126)))
+
+    ;; 6. Small string remaining small.
+    (fill small-string (make-char 'japanese-jisx0213-2 126 126))
+    (garbage-collect)
+    (Assert (eq (aref small-string 0)
+                (make-char 'japanese-jisx0213-2 126 126))))
+
+
   ;;---------------------------------------------------------------
   ;; Test coding system functions
   ;;---------------------------------------------------------------
@@ -1187,4 +1224,72 @@ with control-1"
         "xex\346\227\245\346\234\254\350\252\236")))
   ) ; end of tests that require MULE built in.
 
+;; Test detection of coding cookies.
+(save-excursion
+  (let ((test-file-name
+         (make-temp-file (expand-file-name "nn724rW" (temp-directory))))
+        revert-buffer-function
+        kill-buffer-hook		; paranoia
+        (current-no-conversion-category
+         (coding-category-system 'no-conversion)))
+    (unwind-protect
+         (progn
+           (find-file test-file-name 'no-conversion-unix)
+           (erase-buffer)
+           (insert "-*- coding: koi8-r -*-\n")
+           (Silence-Message (save-buffer 0))
+           (kill-buffer (current-buffer))
+           (Assert (equal (find-coding-system-magic-cookie-in-file
+                           test-file-name)
+                          "koi8-r"))
+           (find-file test-file-name)
+           (Assert (eq buffer-file-coding-system 'koi8-r-unix))
+           (erase-buffer)
+           (setq buffer-file-coding-system 'utf-8-unix)
+           (insert "Der Vogelfänger bin ich ja Local variables: stets lustig
+Der Vogelfänger bin ich ja coding: iso-8859-2 stets lustig
+Der Vogelfänger bin ich ja end: stets lustig\n")
+           (Silence-Message (save-buffer 0))
+           (kill-buffer (current-buffer))
+           (Assert (equal (find-coding-system-magic-cookie-in-file
+                           test-file-name) "iso-8859-2"))
+           (find-file test-file-name)
+           (Assert (eq buffer-file-coding-system 'iso-8859-2-unix))
+           (erase-buffer)
+           (setq buffer-file-coding-system 'utf-8-unix)
+
+           ;; Invalid syntax, coding detection should give nil.
+           (insert "Der Vogelfänger bin ich ja Local variables: stets lustig
+Der Vogelfänger bin ich ja coding: iso-8859-2
+Der Vogelfänger bin ich ja end: stets lustig\n")
+           (Silence-Message (save-buffer 0))
+           (kill-buffer (current-buffer))
+           (set-coding-category-system 'no-conversion 'iso-8859-7)
+           (Assert (eq (find-coding-system-magic-cookie-in-file
+                        test-file-name) nil))
+           (flet ((hack-local-variables-last-page (&optional force)))
+             (find-file test-file-name))
+           (Assert (not (eq buffer-file-coding-system 'iso-8859-2-unix)))
+           (erase-buffer)
+           (setq buffer-file-coding-system 'utf-8-unix)
+
+           ;; Last page after the local variables specificiation, coding
+           ;; detection should give nil.
+           (insert "Der Vogelfänger bin ich ja Local variables: stets lustig
+Der Vogelfänger bin ich ja coding: iso-8859-2
+Der Vogelfänger bin ich ja end: stets lustig
+Ich Vogelfänger bin bekannt, bei alt und jung in ganzem Land\n")
+           (Silence-Message (save-buffer 0))
+           (kill-buffer (current-buffer))
+           (Assert (eq (find-coding-system-magic-cookie-in-file
+                        test-file-name) nil))
+           (flet ((hack-local-variables-last-page (&optional force)))
+             (find-file test-file-name))
+           (Assert (not (eq buffer-file-coding-system 'iso-8859-2-unix)))
+           (Silence-Message (save-buffer 0))
+           (kill-buffer (current-buffer)))
+      (progn
+        (delete-file test-file-name)
+        (set-coding-category-system 'no-conversion
+                                    current-no-conversion-category)))))
 ;;; end of mule-tests.el
