@@ -326,6 +326,52 @@ baaaa
 
 ; (test-regex-charset-mule-paranoid)
 
+;; Check that the regexp implementation does not choke when a range
+;; includes integer values that are not valid Ichars. Both ends of the
+;; range must, however, be valid characters. First test is most
+;; relevant for non-unicode-internal builds:
+(Assert
+ (eql (string-match-p (format "[%c-%c]"
+			      (make-char 'japanese-jisx0208-1978 125 126)
+			      (make-char 'japanese-jisx0208-1978 126 33))
+		      (string
+		       (make-char 'japanese-jisx0208-1978 126 33)))
+      0))
+
+;; And the following tests the same feature for unicode-internal builds:
+(if (featurep 'unicode-internal)
+    (Assert (eql (string-match-p "[\uD7ff-\uE000]" "\uD7FF") 0))
+  (if (eq (char-charset ?\uD7ff) (char-charset ?\uE000))
+      (Assert (eql (string-match-p "[\uD7ff-\uE000]" "\uD7FF") 0))
+    ;; It is possible on non-unicode-internal that these codepoints
+    ;; will be assigned to differing just in time character sets.
+    (Check-Error invalid-regexp
+		 (Assert (eql (string-match-p "[\uD7ff-\uE000]" "\uD7FF")
+			      0)))))
+
+;; Check that the unicode-error-default-translation-table table is
+;; correct, and that all the actual error sequences are caught by
+;; unicode-invalid-sequence-regexp-range:
+(with-temp-buffer
+  (loop
+    for i from ?\x00 to ?\xFF
+    with to-check = (make-string 20 ?\x20) 
+    with regexp = (concat "[" unicode-invalid-sequence-regexp-range "]")
+    do 
+    (delete-region (point-min) (point-max))
+    (insert to-check)
+    (goto-char 10)
+    (insert (decode-coding-string (format "\xd8\x00\x00%c" i)
+                                  'utf-16-be))
+    (backward-char)
+    (Assert (eql i (get-char-table (char-after (point)) 
+				   unicode-error-default-translation-table))
+            (format "Char ?\\x%x not the expected error sequence!"
+		    (char-after (point))))
+    (goto-char (point-min))
+    (Assert (re-search-forward regexp)
+            (format "Could not find char ?\\x%x in buffer" i))))
+
 ;; Test that replace-match does not clobber registers after a failed match
 (with-temp-buffer
   (insert "This is a test buffer.")
@@ -370,6 +416,33 @@ baaaa
 (Assert (not (string-match "\\> " " ")))
 (Assert (not (string-match "a\\<" "a")))
 (Assert (not (string-match "\\>a" "a")))
+
+;; Test symbol boundaries
+(Assert (eql (string-match "\\_<a" " a") 1))
+(Assert (eql (string-match "\\_<=" " =") 1))
+(Assert (eql (string-match "\\_<a" " a") 1))
+(Assert (eql (string-match "\\_<=" " =") 1))
+(Assert (eql (string-match "a\\_>" "a ") 0))
+(Assert (eql (string-match "=\\_>" "= ") 0))
+;; should work at target boundaries
+(Assert (eql (string-match "\\_<a" "a") 0))
+(Assert (eql (string-match "\\_<=" "=") 0))
+(Assert (eql (string-match "a\\_>" "a") 0))
+(Assert (eql (string-match "=\\_>" "=") 0))
+;; Check for weirdness
+(Assert (not (string-match " \\_> " "  ")))
+(Assert (not (string-match " \\_< " "  ")))
+
+;; but not if the "symbol" would be on the null side of the boundary!
+(Assert (not (string-match "\\_<" "")))
+(Assert (not (string-match "\\_>" "")))
+(Assert (not (string-match " \\_<" " ")))
+(Assert (not (string-match "\\_> " " ")))
+(Assert (not (string-match "a\\_<" "a")))
+(Assert (not (string-match "=\\_<" "=")))
+(Assert (not (string-match "\\_>a" "a")))
+(Assert (not (string-match "\\_>=" "=")))
+
 ;; Added Known-Bug 2002-09-09 sjt
 ;; Fixed bug 2003-03-21 sjt
 (Assert (not (string-match "\\b" "")))
@@ -652,19 +725,55 @@ appropriately, ASCII digits" limit)))))
  work, %s" limit script))))))))
   (probe-backref-limits 1 9 10 50 100 150 200 255 500))
 
-(Assert (eql (string-match-p (apply #'concat (make-list 260 "\\(?:\\)")) "hello")
-	     0)
+(Assert (eql (string-match-p (apply #'concat (make-list 260 "\\(?:\\)"))
+                             "hello")
+             0)
 	"checking more than 266 shy groups supported")
 
-(Known-Bug-Expect-Error
- invalid-regexp
- (Assert (eql 
-	  ;; Shy groups shouldn't need to take up an internal register
-	  ;; number. They currently do in our implementation but not with
-	  ;; GNU's.
-	  (string-match-p (apply #'concat (make-list (/ 65536 9) "\\(?:\\)"))
-			  "hello")
-	  0)))
+(Assert (eql (string-match-p (apply #'concat (make-list 65536 "\\(?:\\)"))
+                             "hello")
+             0)
+        "checking #x10000 shy groups supported")
+
+(Assert (eql (string-match 
+              (get-output-stream-string
+               (let ((stream (make-string-output-stream)))
+                 (dotimes (i #x20000)
+                   (write-sequence "\\(?:\\)" stream))
+                 (write-sequence "\\(hello\\)" stream)
+                 stream))
+              "hello")
+             0)
+        "checking #x20000 shy groups supported with one non-shy group")
+
+(Assert (equal (match-string 1 "hello") "hello")
+        "checking match data correct, #x20000 shy groups with one non-shy \
+group")
+
+(macrolet
+    ((check-charset-not (&rest alist)
+       (cons
+	'progn
+	(mapcar (function*
+		 (lambda ((pattern . name))
+		    `(Assert (equal (string-match-p ,pattern ,name)
+			            ,(search (subseq pattern 0
+						     (position ?/ pattern))
+					     name)))))
+		alist))))
+  (check-charset-not
+   ("unicode/mule-ucs/chinese-cns11643-[^/]*\\.txt$"
+    . "21.5-loads-of-shy-groups/etc/unicode/mule-ucs/chinese-cns11643-1.txt")
+   ("unicode/unicode-consortium/ISO8859/ISO-[^/]*\\.TXT$"
+    . "21.5-loads-of-shy-groups/etc/unicode/unicode-consortium/ISO8859/ISO-8859-1.TXT")
+   ("unicode/unicode-consortium/VENDORS/MICSFT/[^/]*/CP[^/]*\\.TXT$"
+    . "21.5-loads-of-shy-groups/etc/unicode/unicode-consortium/VENDORS/MICSFT/EBCDIC/CP037.TXT")
+   ("unicode/mule-ucs/chinese-cns11643-[^\u00A1/]*\\.txt$"
+    . "21.5-loads-of-shy-groups/etc/unicode/mule-ucs/chinese-cns11643-1.txt")
+   ("unicode/unicode-consortium/ISO8859/ISO-[^\u00A1/]*\\.TXT$"
+    . "21.5-loads-of-shy-groups/etc/unicode/unicode-consortium/ISO8859/ISO-8859-1.TXT")
+   ("unicode/unicode-consortium/VENDORS/MICSFT/[^\u00A1/]*/CP[^/]*\\.TXT$"
+    . "21.5-loads-of-shy-groups/etc/unicode/unicode-consortium/VENDORS/MICSFT/EBCDIC/CP037.TXT")))
 
 (let ((log-warning-suppressed-classes (cons 'regex log-warning-suppressed-classes)))
   (Assert
@@ -1344,4 +1453,13 @@ vento")
   (Assert (null (re-search-forward "[[:alnum:]]" nil t))
           "checking that a bug with dirty syntax table caches has been fixed"))
   
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;     Tests of `regexp-opt'                         ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(Assert (equal (regexp-opt '("foo" "bar")) "bar\\|foo"))
+(Assert (equal (regexp-opt '("foo" "bar") t) "\\(bar\\|foo\\)"))
+(Assert (equal (regexp-opt '("foo" "bar") 'words) "\\<\\(bar\\|foo\\)\\>"))
+(Assert (equal (regexp-opt '("foo" "bar") 'symbols) "\\_<\\(bar\\|foo\\)\\_>"))
+
 ;;; end of regexp-tests.el
