@@ -103,6 +103,67 @@
   (goto-char (point-max))
   (Assert (not (search-backward "" nil t 6))))
 
+(with-temp-buffer
+  (insert "one two-three four")
+  (let ((old-syntax-table (syntax-table)))
+    (set-syntax-table emacs-lisp-mode-syntax-table)
+    (unwind-protect
+	(progn
+	  (goto-char (point-min))
+	  (Assert (eq 1 (re-search-forward "\\_<" nil t)))
+	  (Assert (eq 4 (re-search-forward "\\_>" nil t)))
+	  (Assert (eq 6 (re-search-forward "\\_<t" nil t)))
+	  (Assert (null (re-search-forward "\\_<t" nil t))) ; Compare "\\<".
+	  (Assert (eq 19 (re-search-forward "\\_<four\\_>" nil t)))
+	  (goto-char (point-min))
+	  (Check-Error-Message error "Premature end of regular expression"
+			       (re-search-forward "\\_" nil t))
+	  (Check-Error-Message error "Invalid regular expression"
+			       (re-search-forward "\\_a" nil t))
+	  (Assert (null (re-search-forward "\\_<ne two" nil t)))
+	  (Assert (null (re-search-forward "three \\_>" nil t)))
+
+	  (goto-char (point-max))
+	  (Assert (eq 19 (re-search-backward "\\_>" nil t)))
+	  (Assert (eq 15 (re-search-backward "\\_<f" nil t)))
+	  (Assert (null (re-search-backward "\\_<-t" nil t)))
+	  (Assert (eq 5 (re-search-backward "\\_<t" nil t)))
+	  (Assert (eq 1 (re-search-backward "\\_<one\\>" nil t))))
+      (set-syntax-table old-syntax-table))))
+
+(with-temp-buffer
+  (insert "template<class T>")
+  (let ((lookup-syntax-properties t))
+    (goto-char (point-min))
+    (search-forward "<")
+    (set-extent-properties
+     (make-extent (1- (point)) (point))
+     (list 'syntax-table '(4. ?>)))	; Open-paren, matching with >
+    (Assert (looking-at "\\_<class\\_>"))))
+
+(with-temp-buffer
+  (insert "foo<bar")
+  (let ((lookup-syntax-properties t))
+    (goto-char (point-min))
+    (search-forward "<")
+    (backward-char)
+    (set-extent-properties
+     (make-extent (point) (1+ (point)))
+     (list 'syntax-table '(4 . ?>))) ; Open-paren, matching with >
+    (Assert (looking-at "\\>"))))
+
+(with-temp-buffer
+  (insert "foo_<bar")
+  (let ((lookup-syntax-properties t))
+    (goto-char (point-min))
+    (search-forward "<")
+    (backward-char)
+    (set-extent-properties
+     (make-extent (point) (1+ (point)))
+     (list 'syntax-table '(4 . ?>))) ; Open-paren, matching with >
+    (backward-char)		     ; to the _
+    (Assert (not (looking-at "\\_>")))))
+
 (when (featurep 'mule)
   (let* ((hiragana-a (make-char 'japanese-jisx0208 36 34))
 	 (a-diaeresis ?)
